/**********************************************************************
 *
 *   uhelp.c - Help screen & license routines for uhfa.c - DSP-10
 *                        Version 3.34
 *
 * NOTICE: Copyright (c) 2000-2005 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm
 *
 * 3.30 Transferred from UTIL.C, reduce file size for compiler. RSL 21 Nov 2004
 * 3.34 Moved help screens to UHELP_1.TXT to allow customization. RSL 2 Mar 05
 */

#include <alloc.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <graphics.h>
#include <string.h>
#include <dos.h>
#include <io.h>
#include <time.h>
#include "u_code.h"
#include "moonsun.h"
#include "util.h"
#include "uhfa.h"
#include "lbox.h"
#include "async.h"

 void show_help(void);
 void license_info(void);
 void license(void);
 void warranty(void);

/******************  Functions ***********************/

/* show_help()  -  Creates an open area on the DSP-10 screen
 * for help instructions. Prints the file UHELP_1.TXT, a screen
 * at a time. Then prints current EME locations and a
 * signature.
 *
 * Before each screen is a line of numbers that helps count the
 * number of characters, showing the 63 character limit.
 * This line starts with the string "12345678901" which is used to
 * show new screen start.
 *
 * This file uses the 8-bit member ASCII set.  CR (13) and LF (10)
 * are taken as end of line, either or both. Empty lines are ignored
 * and a blank line should include at least one space.
 * 
 * The last screen should be 8 lines, or less, allowing for EME
 * location data at the bottom.
 *
 * Lines preceedinfg the first "number line" are comments and ignored.
 */
void show_help(void)
	{
	char cc1[7], cc2[33], sch;
	int i, nl, hstate;
	FILE *help_file;

	if (NULL == ( help_file=fopen("UHELP_1.TXT", "r") ))
	    {
	    do_t_error(57, "E57 Help File", 500);
	    return;
	    }
	setcolor(c_text_bold);
	settextjustify(0,2);		/* Left and top */
	/* Works out to be 224 pixels high: */
	setviewport(X2L+1, Y2T+96, X2R-1, Y2B-12, DO_CLIP);
	getimage(0,  0, 510, 160, big_area_ptr1);
	getimage(0, 161, 510, 224, big_area_ptr2);

	/* hstate==0 means initial look for start of new screen
	 * hstate==1 means reading lines and building screens
	 */
	nl=0; hstate=0;
	/* First read lines until "12345678901..." is found */
	while (fgets(cc, 256, help_file))  /* Read a line */
	    {
	    sch=cc[11];
	    cc[11]=0;			/* Temp end to test number string */
	    /* First, read past any header, looking for number string */
	    if (hstate==0 && strcmp(cc, "12345678901") == 0)
		{  /* Start of file line is number string */
		hstate=1;		/* Start looking for 22 lines */
		clearviewport();
		line(1, 0, 509, 0);
		line(1, 223, 509, 223);
		nl=0;			/* Top of screen */
		}
	    else if (hstate==1)		/* Past header, reading lines */
		{
		if(strcmp(cc, "12345678901") != 0) /* Not new screen */
		    {
		    if (nl < 22)	/* OK to print line */
			{
			cc[11]=sch;	/* Repair the line */
			cc[63]=0;	/* Truncate line if more than 63 */
			/* Remove CR and LF, replace with nulls */
			for (i=0; i<63; i++)
			    if (cc[i]==10 || cc[i]==13)  cc[i]=0;
			moveto(3, 10*nl+3);
			if(cc[0] != 0)  /* Ignore empty lines*/
			    {
			    outtext(cc); /* Print */
			    nl++;
			    }
			}
			/* If more than 22, but not new screen, no print:  */
		    else
			{
			do_t_error(17, "W17 Help long", 0);
			setviewport(X2L+1, Y2T+96, X2R-1, Y2B-12, DO_CLIP);
			setcolor(c_text_bold);
			settextjustify(0,2);		/* Left and top */
			nl++;
			}
		    }			/* End, line not number line */
		else			/* Line is a num, start of new scrn */
		    {
		    nl=0;		/* Back to top */
		    for(;;)		/* Wait for keyboard hit */
			if ( kbhit() )
			    {
			    _bios_keybrd(_KEYBRD_READ);
			    break;		/* Break the for() */
			    }
		    clearviewport();	/* Have hit, clear screen */
		    line(1, 0, 509, 0);
		    line(1, 223, 509, 223);
		    }
		}			/* End, read a line past header */
	    }				/* End, read a line */
	fclose(help_file);		/* EOF, no further screens */
	/* Add last lines, EME locations, plus signature */
	if (nl > 8)			/* Need more room */
	    {
	    for(;;)			/* Wait for keyboard hit */
		if ( kbhit() )
		    {_bios_keybrd(_KEYBRD_READ);  break; }  // Break the for()
	    clearviewport();
	    line(1, 0, 509, 0);
	    line(1, 223, 509, 223);
	    }
	moveto(3, 83);			/* Line 9 */
	outtext("             -----  MOON/SUN LOCATIONS  -----");
	for(i=0; i<=9; i++)
	    {
	    if (loc[i].time_zone != 999)
		{
		sprintf(cc, "%d %6.6s %31.31s %10.5lf %10.5lf",
		    i,
		    left_justify(loc[i].nm, cc1, 6, SPACE),
		    left_justify(loc[i].name, cc2, 31, HYPHEN),
		    r2d(loc[i].lat), r2d(loc[i].lon));
		moveto(3, 93+10*i);
		outtext(cc);
		}
	    else
		{
		sprintf(cc, "%d", i);
		moveto(3, 93+10*i);
		outtext(cc);
		}
	    }
	moveto(3, 193);		/* Line 20 */
	outtext("");
	moveto(3, 203);		/* Line 21 */
	outtext("       DSP-10 Program by W7PUA - boblark@proaxis.com");
	moveto(3, 213);		/* Line 22 */
	outtext("       Program Copyright (c) Robert Larkin 1996-2005");

	for(;;)			/* Wait for keyboard hit */
	    if ( kbhit() )
		{_bios_keybrd(_KEYBRD_READ);  break; }
	clearviewport();
	putimage(0,   0, big_area_ptr1, COPY_PUT);
	putimage(0, 161, big_area_ptr2, COPY_PUT);
	}

void license_info(void)
    {
    clrscr();
    print_stars();
    printf("\n\n\t     2-Meter Transceiver DSP-10 (uhfa.exe) v%s\n\n",version);
    printf("\t\t\t         14 Nov 2004\n\n");
    printf("\t\tCopyright (c) 1996-2004 Robert S. Larkin\n\n");
    print_stars();  printf("\n\n");

 printf("\nDSP-10 (uhfa.exe) comes with ABSOLUTELY NO WARRANTY; for details type `1'.");
 printf("\nThis is free software, and you are welcome to redistribute it");
 printf("\nunder certain conditions; type `2' for details, or alternatively, read");
 printf("\nthe plain text file GNU_GPL.txt that is in the same directory with");
 printf("\nthis executable file uhfa.exe. Be sure to read it.\n");
 printf("\nHit the 'Enter' key (Return key) to continue with the DSP-10 program.");
    }

void warranty(void)
    {
    clrscr();
 printf("\n\n                 NO WARRANTY\n");
 printf("\nBECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY");
 printf("\nFOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN");
 printf("\nOTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES");
 printf("\nPROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED");
 printf("\nOR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF");
 printf("\nMERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS");
 printf("\nTO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE");
 printf("\nPROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,");
 printf("\nREPAIR OR CORRECTION.\n");
 printf("\nIN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING");
 printf("\nWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR");
 printf("\nREDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,");
 printf("\nINCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING");
 printf("\nOUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED");
 printf("\nTO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY");
 printf("\nYOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER");
 printf("\nPROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE");
 printf("\nPOSSIBILITY OF SUCH DAMAGES.\n");
 printf("\nAfter reading, hit Enter (Return) to continue");
    getch();
    }

void license(void)
    {
    char cline[90];
    int nl;

    nl=0;
    clrscr();
    if (NULL == ( license_file=fopen("GNU_GPL.txt", "r") ))
	    {
	    printf("\nNo GNU_GPL.txt license file was found. This must be in");
	    printf(" the same directory\nas UHFA.EXE file. Exiting program.");
	    exit(1);
	    }
    while (fgets(cline, 80, license_file))
	{
	printf("%s", cline);
	if (nl++ >= 20)
	    {
	    nl=0;
	    printf("\n\n(After reading, hit any key to continue)");
	    getch();
	    clrscr();
	    }
	}
    fclose(license_file);
    printf("\n\n(After reading, hit any key to continue)");
    getch();
    clrscr();
    }
