/**********************************************************************
 *
 *   ustate.c  -  Support routines for saving and recalling the DSP-10 state
 *                        Version 3.31
 *
 * NOTICE: Copyright (c) 2000-2005 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm  and,
 *
 * 3.31 Broke from util.c, added memory modal boxes.  RSL 1 Feb 05
 * 3.33 Fixed memory bugs.
 *
 */

#include <alloc.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <bios.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <graphics.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <dos.h>
#include <io.h>
#include <direct.h>
#include "util.h"
#include "moonsun.h"
#include "uhfa.h"
#include "lbox.h"
#include "u_code.h"
#include "umatrix.h"

#define CFG 1
#define MEM 0

 void wmem_modalbox(void);
 void rmem_modalbox(void);
 int  read_config_file(void);
 int  rcfg_mem(FILE*, int);
 void save_state(void);
 void wcfg_mem(FILE*, int);

/* write_mem_file(int nm)  -  Writes a memory channel file containing
 * parameters to determine transceiver operation. These will overwrite the
 * previous memory channel nm if it exists.  The categories of variables to be
 * written are set by the memch_xxx variables which are written at the top
 * of the file.
 *
 *   memch_fr_md  Frequency and mode, same as a VFO
 *   memch_radio  Radio settings, special modes
 *   memch_tvtr   Transverter definitions
 *   memch_vfo    VFO (freq and mode) definitions
 *   memch_stn    Station definition, such as code_id, call_str
 *   memch_disp   Display character, except colors
 *   memch_bcn    Beacon operation
 *   memch_fil    Filters, equalizers
 *   memch_sys    System parameters, such as startup_detail, file paths
 *   memch_color  Display colors
 *   memch_kbd    Keyboard definitions
 *   memch_hdw    Hardware, such as ezkit_dfreq, and pc clock
 *
 * If all the memch_xxx are set to 1, it is equivalent to a new UHFA.CFG
 * and can completely redine the radio.
 *
 * File names are "UMxxxx.MEM" where xxxx is 0001 to 9999
 * Returns:	 0  Non-fatal error.
 *		 1  Successful file write.
 */
int write_mem_file(int nm)
	{
	char cms[50];
	FILE* mem_file;

	cms[0]='U'; cms[1]='M';			/* Always start with UM */
	sprintf(&cms[2], "%4.4d.MEM", nm); /* Rest of file name */
	if (NULL==( mem_file=fopen(cms, "wt") ))
	    {
	    do_t_error(56, "E56 MEM File", 200);
	    return 0;
	    }
	else
	    {
	    rewind(mem_file);
	    fprintf(mem_file, "UHFAMEM2\n");

	    /* Before writing the MEM file, move the last frequency into the
	     * current VFO     */
	    vfo[cur_vfo].last_tfreq=(double)tfreqL;

	    fprintf(mem_file, "cur_mem %d\n", cur_mem);
	    fprintf(mem_file, "mem_name  |%s| \n", mem_name);
	    fprintf(mem_file, "mem_descr  |%s| \n", mem_descr);
	    /* The VFO related properties come from the current VFO, but they
	     * will be returned to vfo 21.
	     * Saves the basic set of i-f frequency, frequency
	     * offset (FM), mode and transverter. This is the same set of data
	     * that defines a "VFO," and replaces that function.
	     */
	    if (memch_fr_md)
		{
		fprintf(mem_file, "mem_incl_fr_md 1\n");
		fprintf(mem_file,

 /*		  "vfo 21 %d %d %.1f %d\n",  vfo[cur_vfo].tvtr,
		   vfo[cur_vfo].tr_offset, vfo[cur_vfo].last_tfreq,
		   vfo[cur_vfo].last_mode);
  */
		    /* Rev 3.31 */
		    "vfo 21 %d %d %.1lf %d %d %.1lf %d %d %d %d\n",
		    vfo[cur_vfo].tvtr, vfo[cur_vfo].tr_offset,
		    vfo[cur_vfo].last_tfreq, vfo[cur_vfo].last_mode,
		    vfo[cur_vfo].t_fm_use, vfo[cur_vfo].t_fm_freq,
		    vfo[cur_vfo].t_fm_dur, vfo[cur_vfo].t_fm_level,
		    vfo[cur_vfo].par1, vfo[cur_vfo].par2);
		}
	    else
		fprintf(mem_file, "mem_incl_fr_md 0\n");

  /*	    if(memch_fr_md)  fprintf(mem_file, "m_fr_md\n");  */
	    if(memch_radio)  fprintf(mem_file, "m_radio\n");
	    if(memch_tvtr)  fprintf(mem_file, "m_tvtr\n");
	    if(memch_vfo)  fprintf(mem_file, "m_vfo\n");
	    if(memch_stn)  fprintf(mem_file, "m_stn\n");
	    if(memch_disp)  fprintf(mem_file, "m_disp\n");
	    if(memch_bcn)  fprintf(mem_file, "m_bcn\n");
	    if(memch_fil)  fprintf(mem_file, "m_fil\n");
	    if(memch_sys)  fprintf(mem_file, "m_sys\n");
	    if(memch_color)  fprintf(mem_file, "m_color\n");
	    if(memch_kbd)  fprintf(mem_file, "m_kbd\n");
	    if(memch_hdw)  fprintf(mem_file, "m_hdw\n");

	    wcfg_mem(mem_file, MEM);	/* Write appropriate parts of vars */
	    if (fclose(mem_file) != NULL)
		{
		do_t_error(56, "E56 MEM File", 200);
		return 0;
		}
	    }
	return 1;
	}

/*************************************************************************
 * A modal dialog box for writing the current VFO to a memory file.
 *
 * The box structure is lb_ut4.
 **************************************************************************/
void wmem_modalbox(void)
	{
	static struct edit_info   el0, el1, el2;
	static struct check_info
	    ck0, ck1, ck2, ck3, ck4, ck5, ck6, ck7, ck8, ck9, ck10, ck11;
	char eline0[12], eline1[12], eline2[25];
	int sl;
	unsigned int edresult, ckui;
	struct lbox_info  *lb_ut4_ptr;

	/* Dynamic allocation of memory for box data */
	if ((lb_ut4_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 	lb_create(lb_ut4_ptr, "VFO -> Memory",
			 160, 175, 370, 440, 0, MODAL);
	lb_open(lb_ut4_ptr, 1);

	/* Fixed text in modal box:     */
	lb_line(lb_ut4_ptr, 1, BLANK, "", 0);
	lb_line(lb_ut4_ptr, 3, BLANK, "", 0);
	lb_line(lb_ut4_ptr, 4, BLANK, "", 0);
	lb_line(lb_ut4_ptr, 6, BOLD,  "Parameter areas to allow", 0);
	lb_line(lb_ut4_ptr, 7, BOLD,  "modification with memory:", 0);
	lb_line(lb_ut4_ptr, 21, BOLD, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut4_ptr, 22, BOLD, "SPACE changes check boxes", 0);
	lb_line(lb_ut4_ptr, 23, BOLD, "\030 and \031 Move to Next Item", 0);
	lb_line(lb_ut4_ptr, 24, BOLD, "ENTER=Close", 0);

	/* An edit field for Memory Number, 1 to 9999 */
	sprintf(eline0,"");  /* The field to be edited */
	sl=strlen(eline0);
	lb_create_el(&el0, 5, 151, &eline0[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut4_ptr, 0, EDIT, "Memory 1 to 9999", &el0);

	/* An edit field for Memory Name, 7 characters max */
	sprintf(eline1, "");             /* The field to be edited */
	sl=strlen(eline1);
	lb_create_el(&el1, 8, 127, &eline1[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut4_ptr, 2, EDIT, "Memory Name", &el1);

	/* An edit field for Memory Description, 18 characters max */
	sprintf(eline2,"Description"); /* The field to be edited */
	sl=strlen(eline2);
	lb_create_el(&el2, 24, 0, &eline2[0], ED_INACTIVE, 0, sl);
	lb_line(lb_ut4_ptr, 4, EDIT, "", &el2);

	/* Parameters in call to lb_create_check(....)
	 *   struct check_info    One data structure per check mark
	 *   int                  Check position in pixels from left edge
	 *   unsigned int         State of check, checked=TRUE
	 *   int                  Text position in pixels from left edge
	 */

	/* Frequency and mode check box:     */
	if (memch_fr_md)		/* Show box checked */
	    lb_create_check(&ck0, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck0, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 8, CHECK, "Freq and Mode", &ck0);

	/* Radio information check box:     */
	if (memch_radio)			/* Show box checked */
	    lb_create_check(&ck1, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck1, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 9, CHECK, "Radio settings", &ck1);

	/* Beacon information check box:     */
	if (memch_bcn)			/* Show box checked */
	    lb_create_check(&ck2, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck2, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 10, CHECK, "Beacon", &ck2);

	/* Display parameters check box:     */
	if (memch_disp)			/* Show box checked */
	    lb_create_check(&ck3, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck3, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 11, CHECK, "Display except color", &ck3);

	/* Station Information check box:     */
	if (memch_stn)			/* Show box checked */
	    lb_create_check(&ck4, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck4, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 12, CHECK, "Station, EME Locs", &ck4);

	/* Filter check box:     */
	if (memch_fil)			/* Show box checked */
	    lb_create_check(&ck5, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck5, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 13, CHECK, "Filters & Equalizers", &ck5);

	/* Current VFO definitions check box:     */
	if (memch_vfo)			/* Show box checked */
	    lb_create_check(&ck6, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck6, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 14, CHECK, "Current VFO Defs", &ck6);

	/* Transverter information check box:     */
	if (memch_tvtr)			/* Show box checked */
	    lb_create_check(&ck7, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck7, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 15, CHECK, "Transverter Defs", &ck7);

	/* System information check box:     */
	if (memch_sys)			/* Show box checked */
	    lb_create_check(&ck8, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck8, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 16, CHECK, "System, file paths", &ck8);

	/* Color parameters check box:     */
	if (memch_disp)			/* Show box checked */
	    lb_create_check(&ck9, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck9, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 17, CHECK, "Screen colors", &ck9);

	/* Keyboard parameters check box:     */
	if (memch_kbd)			/* Show box checked */
	    lb_create_check(&ck10, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck10, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 18, CHECK, "Keyboard defs", &ck10);

	/* Hardware parameters check box:     */
	if (memch_hdw)			/* Show box checked */
	    lb_create_check(&ck11, 0, TRUE, 20);
	else				/* Show box unchecked */
	    lb_create_check(&ck11, 0, FALSE, 20);
	lb_line(lb_ut4_ptr, 19, CHECK, "Hardware settings", &ck11);

	lb_line(lb_ut4_ptr, 20, BLANK, "", 0);

	lb_update(lb_ut4_ptr);

        /* ----------- Loop ---------------- */
	/* Following is a loop for handling user keyboard input */
 e0:	edresult=lb_edit_line(lb_ut4_ptr, &el0);
	if (edresult==CUR_UP)   goto ck19;
	else if (edresult==ENTER) goto lb_ut4end;

 e2:	edresult=lb_edit_line(lb_ut4_ptr, &el1);
	if (edresult==CUR_UP)   goto e0;
	else if (edresult==ENTER) goto lb_ut4end;

 e4:	edresult=lb_edit_line(lb_ut4_ptr, &el2);
	if (edresult==CUR_UP)   goto e2;
	else if (edresult==ENTER) goto lb_ut4end;

 ck8:	ckui=lb_check_mark(lb_ut4_ptr, &ck0);
	if     (ckui==CUR_UP)  goto e4;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck9:	ckui=lb_check_mark(lb_ut4_ptr, &ck1);
	if     (ckui==CUR_UP)  goto ck8;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck10:	ckui=lb_check_mark(lb_ut4_ptr, &ck2);
	if     (ckui==CUR_UP)  goto ck9;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck11:	ckui=lb_check_mark(lb_ut4_ptr, &ck3);
	if     (ckui==CUR_UP)  goto ck10;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck12:	ckui=lb_check_mark(lb_ut4_ptr, &ck4);
	if     (ckui==CUR_UP)  goto ck11;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck13:	ckui=lb_check_mark(lb_ut4_ptr, &ck5);
	if     (ckui==CUR_UP)  goto ck12;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck14:	ckui=lb_check_mark(lb_ut4_ptr, &ck6);
	if     (ckui==CUR_UP)  goto ck13;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck15:	ckui=lb_check_mark(lb_ut4_ptr, &ck7);
	if     (ckui==CUR_UP)  goto ck14;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck16:	ckui=lb_check_mark(lb_ut4_ptr, &ck8);
	if     (ckui==CUR_UP)  goto ck15;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck17:	ckui=lb_check_mark(lb_ut4_ptr, &ck9);
	if     (ckui==CUR_UP)  goto ck16;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck18:	ckui=lb_check_mark(lb_ut4_ptr, &ck10);
	if     (ckui==CUR_UP)  goto ck17;
	else if(ckui==ENTER)     goto lb_ut4end;

 ck19:	ckui=lb_check_mark(lb_ut4_ptr, &ck11);
	if     (ckui==CUR_UP)  goto ck18;
	else if(ckui==ENTER)     goto lb_ut4end;

	goto e0;

 lb_ut4end:
	delay(200);
	/* As we exit, the variables must be changed to agree with the
	 * indicated states of the buttons and edit fields:
	 */
	if (ck0.state & CHECK_MARK)
	    memch_fr_md=1;
	else
	    memch_fr_md=0;
	if (ck1.state & CHECK_MARK)
	    memch_radio=1;
	else
	    memch_radio=0;
	if (ck2.state & CHECK_MARK)
	    memch_bcn=1;
	else
	    memch_bcn=0;
	if (ck3.state & CHECK_MARK)
	    memch_disp=1;
	else
	    memch_disp=0;
	if (ck4.state & CHECK_MARK)
	    memch_stn=1;
	else
	    memch_stn=0;
	if (ck5.state & CHECK_MARK)
	    memch_fil=1;
	else
	    memch_fil=0;
	if (ck6.state & CHECK_MARK)
	    memch_vfo=1;
	else
	    memch_vfo=0;
	if (ck7.state & CHECK_MARK)
	    memch_tvtr=1;
	else
	    memch_tvtr=0;
	if (ck8.state & CHECK_MARK)
	    memch_sys=1;
	else
	    memch_sys=0;
	if (ck9.state & CHECK_MARK)
	    memch_color=1;
	else
	    memch_color=0;
	if (ck10.state & CHECK_MARK)
	    memch_kbd=1;
	else
	    memch_kbd=0;
	if (ck11.state & CHECK_MARK)
	    memch_hdw=1;
	else
	    memch_hdw=0;

	cur_mem=atoi(eline0);
	if (cur_mem<1 || cur_mem>9999)
	    {
	    lb_msg(375, 225, "Memory Error", "Invalid memory number",
		"No memory write occuring");
	    lb_close(lb_ut4_ptr, 1);
	    free(lb_ut4_ptr);
	    return;
	    }
	strcpy(mem_name, eline1);
	strcpy(mem_descr, eline2);

	lb_close(lb_ut4_ptr, 1);
	free(lb_ut4_ptr);
	if(write_mem_file(cur_mem)==0)
	    {
	    lb_msg(375, 225, "Memory Error", "Write error to file", "");
	    lb_close(lb_ut4_ptr, 1);
	    free(lb_ut4_ptr);
	    return;
	    }
	}

/*************************************************************************
 * A modal dialog box for reading a memory file.
 * The memory is identified by an index number, 1 to 9999. This establishes
 * the name of the file containing the memory data.
 * This modal box enters the number, shows the pertinant data and allows
 * moving on to the memory channel, or canellation.
 * The box structure is lb_ut5.
 **************************************************************************/
void rmem_modalbox(void)
	{
	static struct edit_info   el1, el2;
	char eline1[12], eline2[12];
	int sl;
	unsigned int edresult;
	struct lbox_info  *lb_ut5_ptr;
	char cms[50], ln1[25], ln2[25], ln3[25];
	char mname[25], mdescr[50];		/* Temporary copies */
	FILE *mem_file;
	static char entrysep[]=" \t,=\n\r";	/* Allow sp, tab, comma,
						   equal, lf, cr  */
	char *ci, *cd;
	char cline[260], cline2[260];
	int mn;					/* Temp mem number */
	int mincl_fr_md;			/* Temp freq/mode info */

	/* Dynamic allocation of memory for box data */
	if ((lb_ut5_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
	/* Box 10 lines high (0,9) */
 	lb_create(lb_ut5_ptr, "Memory Channel Load",
			 160, 175, 370, 290, 0, MODAL);
	lb_open(lb_ut5_ptr, 1);

	lb_line(lb_ut5_ptr, 0, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 2, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 3, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 4, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 5, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 6, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 7, BLANK, "", 0);
	lb_line(lb_ut5_ptr, 8, BLANK, "", 0);

	/* Fixed text in modal box:     */
	lb_line(lb_ut5_ptr, 9, BOLD, "TEXT: Backspace & Type", 0);
	lb_line(lb_ut5_ptr, 10, BOLD, "ENTER=Close", 0);

 rmem0:	/* An edit field for Memory Number, 1 to 9999 */
	sprintf(eline1,"");		/* The field to be edited */
	sl=strlen(eline1);
	lb_create_el(&el1, 7, 135, &eline1[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut5_ptr, 1, EDIT, "Memory 1 to 9999", &el1);
	lb_update(lb_ut5_ptr);

        /* ----------- Loop ---------------- */
	/* Following is a loop for handling user keyboard input */
 e0:	edresult=lb_edit_line(lb_ut5_ptr, &el1);
	if (edresult==ENTER) goto lb_ut5end;
	goto e0;

 lb_ut5end:
	cms[0]='U'; cms[1]='M';			/* Always start with UM */
	sprintf(&cms[2], "%4.4d.MEM", atoi(eline1)); /* Rest of file name */
        if (NULL == ( mem_file=fopen(cms, "rt") ))
	    {
	    lb_msg(375, 225, "Memory Error", "Memory file not present",
		"No memory will be loaded");
	    fclose(mem_file);		/* Close the file */
	    lb_close(lb_ut5_ptr, 1);	/* Close the dialog box */
	    free(lb_ut5_ptr);
	    return;
	    }
	if (fgets(cline, 256, mem_file) == NULL)
	    {
	    lb_msg(375, 225, "Memory Error", "Memory file insufficient",
		"No memory will be loaded");
	    fclose(mem_file);
	    lb_close(lb_ut5_ptr, 1);
	    free(lb_ut5_ptr);
	    return;
	    }
 	ci = strtok(cline, entrysep);	/* Should be file identifier line */
	if (strcmp(ci, "UHFAMEM2"))
	    {
	    lb_msg(375, 225, "Memory Error", "Memory file illegal format",
		"No memory will be loaded");
	    fclose(mem_file);
	    lb_close(lb_ut5_ptr, 1);
	    free(lb_ut5_ptr);
	    return;
	    }

	/* Read the file body to get the memory parameters. This gets the
	 * minimum set to make a go/no-go decision. If go, they will
	 * all be read with rcfg_mem(), below. Various parameters are put
	 * into temp variables, until the go decision.    */
	while (fgets(cline, 256, mem_file))
	    {
	    strcpy(cline2, cline);	/* Extra for when string token first */
	    /* Get identifier and skip return:   */
	    if ((ci = strtok(cline, entrysep)) == NULL)  break;
	    /* Get a token to cd */
	    if ((cd = strtok(NULL, entrysep)) == NULL)  strcpy(cd, "BLANK");
	    if (!stricmp(ci, "cur_mem")) mn=atoi(cd);
	    if (!stricmp(ci, "mem_name"))
		{
		cd=strtok(cline2, "|");		/* Go to string */
		cd=strtok(NULL, "|");		/* Get string */
		strcpy(mname, cd);
		mname[7]=0;			/* Max len is 7 */
		}
	    if (!stricmp(ci, "mem_descr"))
		{
		cd=strtok(cline2, "|");		/* Go to fscreen_path string */
		cd=strtok(NULL, "|");		/* Get string */
		strcpy(mdescr, cd);
		mdescr[23]=0;			/* Max len is 23 */
		}
	    if (!stricmp(ci, "mem_incl_fr_md")) mincl_fr_md=atoi(cd);
	    }

	if (mn != atoi(eline1))
	    {
	    lb_msg(375, 225, "Memory Error", "Inconsistent memory number",
		"No memory will be loaded");
	    fclose(mem_file);
	    lb_close(lb_ut5_ptr, 1);
	    free(lb_ut5_ptr);
	    return;
	    }

	sprintf(ln2, "\"%s\"", mname);
	lb_line(lb_ut5_ptr, 3, BOLD, ln2, 0);
	sprintf(ln3, "%s", mdescr);
	lb_line(lb_ut5_ptr, 4, BOLD, ln3, 0);
	if(mincl_fr_md)
	    lb_line(lb_ut5_ptr, 5, BOLD, "MEM changes freq/mode", 0);
	else
	    lb_line(lb_ut5_ptr, 5, BOLD, "This MEM leaves freq/mode", 0);
	lb_line(lb_ut5_ptr, 6, BLANK, "", 0);

	sprintf(eline2,"Y");		/* The field to be edited */
	sl=strlen(eline2);
	lb_create_el(&el2, 2, 173, &eline2[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut5_ptr, 7, EDIT, "'Y' Enter to continue", &el2);
	lb_update(lb_ut5_ptr);

	edresult=lb_edit_line(lb_ut5_ptr, &el2);
	if(eline2[0]!='y' && eline2[0]!='Y')
	    {
	    lb_msg(375, 225, "Memory Error", "Memory Load Aborted",
		"No memory will be loaded");
	    fclose(mem_file);
	    lb_close(lb_ut5_ptr, 1);
	    free(lb_ut5_ptr);		/* Close the dialog box */
	    return;
	    }

	lb_close(lb_ut5_ptr, 1);
	free(lb_ut5_ptr);		/* Close the dialog box */

	/* Read the contents of the memory file, setting variables 	 */
	rewind(mem_file);
	rcfg_mem(mem_file, MEM);
	fclose(mem_file);

	/* There are two groups of parameters. Thoses associated with a VFO and
	 * everything else. If mincl_fr_md==0 we are not changing VFO params
	 * and should not show a "M:xxxx", nor change cur_mem.
	 * Now arrange the transceiver state to look like a memory. */

	if(mincl_fr_md)			/* This means vfo[21] is present */
	    {
	    new_vfo(21);		/* Set mode, tfreqL, select tvtr[] */
	    cur_mem=mn;			/* These 4 were temp copies */
	    strcpy(mem_name, mname);	/*    ...make permanent */
	    strcpy(mem_descr, mdescr);
	    mem_incl_fr_md=mincl_fr_md;
	    }
	/* Make all the parameters active     */
	setallpalette(&upalette);
	setbkcolor(c_background);
	initialize(2);
	}

/* rcfg_mem( ) reads the main body of either a .CFG file or a .MEM file.
 *   state_file is a pointer to a structure FILE
 *   ftype is either CFG or MEM
 */
int rcfg_mem(FILE* state_file, int ftype)
	{
	int i;
	static char entrysep[]=" \t,=\n\r";	/* Allow sp, tab, comma,
						   equal, lf, cr  */
	char *ci, *cd;
	char cline[260], cline2[260];
	char *zptr;				/* Dummy for strtol() */

	while (fgets(cline, 256, state_file))
	    {
	    strcpy(cline2, cline);	/* Extra for when string token first */
	    /* Get identifier and skip return:   */
	    if ((ci = strtok(cline, entrysep)) == NULL)  break;
	    /* Get a token to cd */
	    if ((cd = strtok(NULL, entrysep)) == NULL)  strcpy(cd, "BLANK");
	    if (!stricmp(ci, "v_m")) v_m=atoi(cd);
	    if (!stricmp(ci, "cur_vfo"))  cur_vfo=atoi(cd);
	    if (!stricmp(ci, "cur_mem")) cur_mem=atoi(cd);
	    if (!stricmp(ci, "mem_incl_fr_md")) mem_incl_fr_md=atoi(cd);
	    if (!stricmp(ci, "memch_stn")) memch_stn=atoi(cd);
	    if (!stricmp(ci, "memch_fr_md")) memch_fr_md=atoi(cd);
	    if (!stricmp(ci, "memch_radio")) memch_radio=atoi(cd);
	    if (!stricmp(ci, "memch_tvtr")) memch_tvtr=atoi(cd);
	    if (!stricmp(ci, "memch_vfo")) memch_vfo=atoi(cd);
	    if (!stricmp(ci, "memch_disp")) memch_disp=atoi(cd);
	    if (!stricmp(ci, "memch_bcn")) memch_bcn=atoi(cd);
	    if (!stricmp(ci, "memch_fil")) memch_fil=atoi(cd);
	    if (!stricmp(ci, "memch_sys")) memch_sys=atoi(cd);
	    if (!stricmp(ci, "memch_color")) memch_color=atoi(cd);
	    if (!stricmp(ci, "memch_kbd")) memch_kbd=atoi(cd);
	    if (!stricmp(ci, "memch_hdw")) memch_hdw=atoi(cd);
	    if (!stricmp(ci, "contrast")) contrast=atoi(cd);
	    if (!stricmp(ci, "bright"))  bright=atoi(cd);
	    if (!stricmp(ci, "afgain"))  afgain=atoi(cd);
	    if (!stricmp(ci, "fm_afgain"))  fm_afgain=atoi(cd);
	    if (!stricmp(ci, "rfgain"))  rfgain=atoi(cd);
	    if (!stricmp(ci, "rfgdb"))
		{
		rfg[1].dbgain=atof(cd);
		for (i=2; i<=6; i++)
		    {
		    if ((cd=strtok(NULL, entrysep))==0)
			{
			printf("Missing rfgdb entry in UHFA.CFG (need 6)\n");
			return 0;
			}
		    else
			{
			rfg[i].dbgain=atof(cd);
			}
		    }
		}
	    if (!stricmp(ci, "gain_adj"))  gain_adj=atof(cd);
	    if (!stricmp(ci, "cw_speed"))  cw_speed=atoi(cd);
	    if (!stricmp(ci, "cw_longkey"))  cw_longkey=atoi(cd);
	    if (!stricmp(ci, "agc_decay"))
		{agc_decay=atoi(cd);
		 last_agc=agc_decay;	/* Get started right */
		}
	    if (!stricmp(ci, "mike_gain"))  mike_gain=atoi(cd);
	    if (!stricmp(ci, "xmit_power"))  xmit_power=atoi(cd);
	    if (!stricmp(ci, "fm_squelch"))  fm_squelch=atoi(cd);
	    if (!stricmp(ci, "fm_dev"))
		{fm_dev=atof(cd);
		 if (fm_dev < 0.0)    fm_dev=0.0;
		 if (fm_dev > 9.99)   fm_dev=9.99;
		}
	    if (!stricmp(ci, "cwoffset"))  cwoffset=atoi(cd);
	    if (!stricmp(ci, "eme_doppler")) eme_doppler=atoi(cd);
	    if (!stricmp(ci, "sidetone_lev"))  sidetone_lev=atoi(cd);
	    if (!stricmp(ci, "noncohave"))
		{noncohave=atoi(cd);
		 if (noncohave<1) noncohave=1;
		}
	    if (!stricmp(ci, "do_window"))  do_window=atoi(cd);
	    if (!stricmp(ci, "fft_bw"))  { fft_bw=atoi(cd);
						fft_bw_save=fft_bw;}
	    if (!stricmp(ci, "knoise"))  knoise=atof(cd);
	    if (!stricmp(ci, "kfilt"))  kfilt=atof(cd);
	    if (!stricmp(ci, "fft_fcn"))  fft_fcn=atoi(cd);
	    if (!stricmp(ci, "do_af_filt"))  do_af_filt=atoi(cd);
	    if (!stricmp(ci, "af_coef"))  af_coef=atoi(cd);
	    if (!stricmp(ci, "do_LMS"))  do_LMS=atoi(cd);
	    if (!stricmp(ci, "denoise_level"))  denoise_level=atoi(cd);
	    if (!stricmp(ci, "denoise_decay"))  denoise_decay=atoi(cd);
	    if (!stricmp(ci, "denoise_del_len")) denoise_del_len=atoi(cd);
	    if (!stricmp(ci, "autonotch_level")) autonotch_level=atoi(cd);
	    if (!stricmp(ci, "dot_type"))  dot_type=atoi(cd);
	    if (!stricmp(ci, "freq_low"))  freq_low=atof(cd);
	    if (!stricmp(ci, "db_div_set"))  db_div_set=atoi(cd);
	    if (!stricmp(ci, "koffset"))  koffset=atof(cd);
	    /* if (!stricmp(ci, "mode"))  mode=atoi(cd);  Ver 3.28 */
	    if (!stricmp(ci, "mode_mask"))  mode_mask=(1 | atoi(cd));
	    if (!stricmp(ci, "hardware"))  hardware=atoi(cd);
	    if (!stricmp(ci, "hardware_ptt"))  hardware_ptt=atoi(cd);
	    if (!stricmp(ci, "rit"))  rit=atoi(cd);
	    if (!stricmp(ci, "rit_on"))  rit_on=atoi(cd);
	    if (!stricmp(ci, "band"))  band=atoi(cd);
#if 0
	    if (!stricmp(ci, "tfreqL"))  tfreqL=atol(cd); /* No more 3.28 */
#endif
	    if (!stricmp(ci, "delf"))  delf=atol(cd);
	    if (!stricmp(ci, "delf_sh"))  delf_sh=atol(cd);
	    if (!stricmp(ci, "deldb"))  deldb=atoi(cd);
	    if (!stricmp(ci, "deldb_sh"))  deldb_sh=atoi(cd);
	    if (!stricmp(ci, "ref_offset"))  ref_offset=atof(cd);
	    if (!stricmp(ci, "ezkit_dfreq"))  ezkit_dfreq=atof(cd);
	    if (!stricmp(ci, "diag_mode"))  diag_mode=atoi(cd);
	    if (!stricmp(ci, "startup_detail")) startup_detail=atoi(cd);
	    if (!stricmp(ci, "waterfall"))  waterfall=atoi(cd);
	    if (!stricmp(ci, "shift_reg"))  shift_reg=atoi(cd);
	    if (!stricmp(ci, "eme2_te"))  eme2_te=atof(cd);
	    if (!stricmp(ci, "eme_freq_range"))  eme_freq_range=atof(cd);
	    if (!stricmp(ci, "code_nb_ratio"))  code_nb_ratio=atof(cd);
	    if (!stricmp(ci, "lterm_int_flow"))  lterm_int_flow=atof(cd);
	    if (!stricmp(ci, "lterm_int_fhigh")) lterm_int_fhigh=atof(cd);
	    if (!stricmp(ci, "lterm_int_center"))
		 lterm_int_center=atof(cd);
	    if (!stricmp(ci, "lti_freq_range")) lti_freq_range=atof(cd);
	    if (!stricmp(ci, "lhl_bins"))  lhl_bins=atoi(cd);
	    if (!stricmp(ci, "lhl_peak_ave")) lhl_peak_ave=atoi(cd);
	    if (!stricmp(ci, "lhl7_2line")) lhl7_2line=atoi(cd);
	    if (!stricmp(ci, "pua43_noisedecay"))
			 pua43_noisedecay=atof(cd);
	    if (!stricmp(ci, "pua43_dorandom")) pua43_dorandom=atoi(cd);
	    if (!stricmp(ci, "pua43_xstring"))
		{
		cd=strtok(cline2, "|");		/* Go to string */
		cd=strtok(NULL, "|");		/* Get string */
		cd[28]=0;			/* Eventual end */
		if (strlen(cd) < 28)
		    for (i=strlen(cd); i<28; i++)
			cd[i]=32;
		strcpy(pua43_xstring, cd);
		}
	    if (!stricmp(ci,"cwmsg") && (i=atoi(cd))>=0 && i<=9 )
		{
		cd=strtok(NULL, "|");		/* Move to string */
		cd=strtok(NULL, "|");		/* Get message string */
		cd[49]=0;			/* Prevent excessive length */
		strcpy(&cwmbuf[i*50], cd);
		}
	    if (!stricmp(ci, "pua14_28")) pua14_28=atoi(cd);
	    if (!stricmp(ci, "dly_ant2amp"))  dly_ant2amp=atoi(cd);
	    if (!stricmp(ci, "dly_amp2xmit"))  dly_amp2xmit=atoi(cd);
	    if (!stricmp(ci, "dly_xmit2amp"))  dly_xmit2amp=atoi(cd);
	    if (!stricmp(ci, "dly_amp2ant"))  dly_amp2ant=atoi(cd);
	    if (!stricmp(ci, "dly_emet2r"))  dly_emet2r=atoi(cd);
	    if (!stricmp(ci, "b_transmit_sec"))  b_transmit_sec=atoi(cd);
	    if (!stricmp(ci, "b_receive_sec"))  b_receive_sec=atoi(cd);
	    if (!stricmp(ci, "b_offset_sec"))  b_offset_sec=atoi(cd);
	    if (!stricmp(ci, "lt_b_save"))  lt_b_save=atoi(cd);
	    if (!stricmp(ci, "lt_b_save_bins"))  lt_b_save_bins=atoi(cd);
	    if (!stricmp(ci, "lt_b_clear"))  lt_b_clear=atoi(cd);
	    if (!stricmp(ci, "b_active1"))  b_active1=atoi(cd);
	    if (!stricmp(ci, "b_active2"))  b_active2=atoi(cd);
	    if (!stricmp(ci, "b_repeat60"))  b_repeat60=atoi(cd);
	    if (!stricmp(ci, "data_omit_tr"))  data_omit_tr=atoi(cd);
	    if (!stricmp(ci, "dut1"))  dut1=atof(cd);
	    if (!stricmp(ci, "time_type"))  time_type=atoi(cd);
	    if (!stricmp(ci, "show_secs"))  show_secs=atoi(cd);
	    if (!stricmp(ci, "time_str"))
		{
		cd=strtok(cline2, "|");		/* Go to time_str */
		cd=strtok(NULL, "|");		/* Get string */
		strcpy(time_str, cd);
		}
	    if (!stricmp(ci, "tick_time")) tick_time=atoi(cd);
	    if (!stricmp(ci, "pua43_tick")) pua43_tick=atoi(cd);
	    if (!stricmp(ci, "clock_speed"))  clock_cor_factor=atof(cd);
	    if (!stricmp(ci, "code_nx2")) code_nx2=atoi(cd);
	    if (!stricmp(ci, "code_id"))
		{
		cd=strtok(cline2, "|");		/* Go to code_id */
		cd=strtok(NULL, "|");		/* Get string */
		cd[9]=0;			/* For excessive length */
		strcpy(code_id, cd);
		}
	    if (!stricmp(ci, "call_str"))
		{
		cd=strtok(cline2, "|");		/* Go to call_str string */
		cd=strtok(NULL, "|");		/* Get string */
		strcpy(call_str, cd);
		}
	    if (!stricmp(ci, "file_ident"))
		{
		cd=strtok(cline2, "|");		/* Go to file_ident string */
		cd=strtok(NULL, "|");		/* Get string */
		file_ident[0]=cd[0]; file_ident[1]=0;
		}

	    if (!stricmp(ci, "quit_save_state")) quit_save_state=atoi(cd);
	    if (!stricmp(ci, "beep_on_error"))  beep_on_error=atoi(cd);
	    if (!stricmp(ci, "palette"))
		{
		upalette.colors[0]=atoi(cd);
		for (i=1; i<16; i++)
		    {
		    if ((cd=strtok(NULL, entrysep))==0)
			{
			printf("Missing color entry in UHFA.CFG\n");
			return 0;
			}
		    else
			{
			upalette.colors[i]=atoi(cd);
			}
		    }
		}
	    if (!stricmp(ci, "c_background"))  c_background=atoi(cd);
	    if (!stricmp(ci, "c_outline")) c_outline=atoi(cd);
	    if (!stricmp(ci, "c_outline_bold"))  c_outline_bold=atoi(cd);
	    if (!stricmp(ci, "c_outline_bright"))
		c_outline_bright=atoi(cd);
	    if (!stricmp(ci, "c_text"))  c_text=atoi(cd);
	    if (!stricmp(ci, "c_text_bold"))  c_text_bold=atoi(cd);
	    if (!stricmp(ci, "c_text_bright"))  c_text_bright=atoi(cd);
	    if (!stricmp(ci, "c_text_note"))  c_text_note=atoi(cd);
	    if (!stricmp(ci, "c_plot_bold"))  c_plot_bold=atoi(cd);
	    if (!stricmp(ci, "c_plot_bright"))  c_plot_bright=atoi(cd);
	    if (!stricmp(ci, "c_plot_marker"))  c_plot_marker=atoi(cd);
	    if (!stricmp(ci, "c_bar_dark"))  c_bar_dark=atoi(cd);
	    if (!stricmp(ci, "c_bar_low"))  c_bar_low=atoi(cd);
	    if (!stricmp(ci, "c_bar_medium"))  c_bar_medium=atoi(cd);
	    if (!stricmp(ci, "c_bar_high"))  c_bar_high=atoi(cd);
	    if (!stricmp(ci, "kbd_alt1"))  kbd_alt1=atoi(cd);
	    if (!stricmp(ci, "kbd_alt2"))  kbd_alt2=atoi(cd);
	    if (!stricmp(ci, "kbd_alt3"))  kbd_alt3=atoi(cd);
	    if (!stricmp(ci, "fscreen_path"))
		{
		cd=strtok(cline2, "|");		/* Go to fscreen_path string */
		cd=strtok(NULL, "|");		/* Get string */
		if (strlen(cd)>0)
		    {
		    if (strlen(cd) < 50)
			strcpy(fscreen_path, cd);
		    else
			printf("\nfscreen_path exceeds 50 ch; Not used");
		    }
		}
	    if (!stricmp(ci, "scsave"))  scsave=atoi(cd);
/* Rev 3.50 if (!stricmp(ci, "scs_key"))  scs_key=atoi(cd);  */
	    if (!stricmp(ci, "use_scs_box")) use_scs_box=atoi(cd);
	    if (!stricmp(ci, "fdata_path"))
		{
		cd=strtok(cline2, "|");		/* Go to fdata_path string */
		cd=strtok(NULL, "|");		/* Get string */
		if (strlen(cd)>0)
		    {
		    if (strlen(cd) < 50)
			strcpy(fdata_path, cd);
		    else
			printf("\nfdata_path exceeds 50 ch; Not used");
		    }
		}
	    if ( !stricmp(ci,"tvtr") && (i=atoi(cd))>=0 && i<=20 )
		{
		cd=strtok(NULL, "|");		/* Move to string */
		cd=strtok(NULL, "|");		/* Get name string */
		if(strlen(cd) > 8) *(cd+8)=0;   /* Chop it off */
		strcpy(tvtr[i].name, cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].lo_type=atoi(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].sb=atoi(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].lo_freq=atof(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].chain_mult=atoi(cd);
		strtok(NULL, "|");
		cd=strtok(NULL, "|");
		strcpy(tvtr[i].init_str, cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].gain_db=atof(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].control=atoi(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].bus_adr=atoi(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].last_tfreq=atof(cd);
		cd=strtok(NULL, entrysep);
		tvtr[i].last_mode=atoi(cd);
		}
	    if ( !stricmp(ci,"vfo") && (i=atoi(cd))>=0 && i<=21 )
		{
		cd=strtok(NULL, entrysep);
		vfo[i].tvtr=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].tr_offset=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].last_tfreq=atof(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].last_mode=atoi(cd);
 		/* New with ver 3.33 */
		cd=strtok(NULL, entrysep);
		if (cd==NULL)  goto oldvfo;  /* Early versions didn't have */
		vfo[i].t_fm_use=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].t_fm_freq=atof(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].t_fm_dur=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].t_fm_level=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].par1=atoi(cd);
		cd=strtok(NULL, entrysep);
		vfo[i].par2=atoi(cd);
		}
 oldvfo:
	    if ( !stricmp(ci,"loc") && (i=atoi(cd))>=0 && i<=9 )
		{
		loc[i].time_zone=0;		/* Mark as existant */
		cd=strtok(NULL, "|");		/* Move to string */
		cd=strtok(NULL, "|");		/* Get short name string */
		if (strlen(cd)>6)
		    {
		    printf("\nLoc[%d].nm exceeds 6 ch; Loc not used", i);
		    loc[i].time_zone=999;
		    }
		else
		    strcpy(loc[i].nm, cd);
		strtok(NULL, "|");
		cd=strtok(NULL, "|");		/* Get description */
		if (strlen(cd)>31)
		    {
		    printf("\nLoc[%d].name exceeds 31 ch; Loc not used", i);
		    loc[i].time_zone=999;
		    }
		else
		    strcpy(loc[i].name, cd);
		cd=strtok(NULL, entrysep);
		loc[i].lat=d2r(atof(cd));
		cd=strtok(NULL, entrysep);
		loc[i].lon=d2r(atof(cd));
		}
	    if (!stricmp(ci, "f_eme"))  f_eme=atof(cd);
	    if (!stricmp(ci, "cur_loc"))  cur_loc=atoi(cd);
	    if (!stricmp(ci, "delay_state"))  delay_state=atoi(cd);
	    if (!stricmp(ci, "delay_right"))  delay_right=atoi(cd);
	    if (!stricmp(ci, "xsnr"))  xsnr=atof(cd);
	    if (!stricmp(ci, "xfade_rate"))  xfade_rate=atof(cd);
	    if (!stricmp(ci, "xnoise"))  xnoise=atoi(cd);
	    if (!stricmp(ci, "xfade"))  xfade=atoi(cd);
	    if (!stricmp(ci, "lpt_port"))  lpt_port=atoi(cd);
	    if (!stricmp(ci, "v2f_noise"))  v2f_noise=atoi(cd);
	    if (!stricmp(ci, "v2f_pkave"))  v2f_pkave=atoi(cd);
	    if (!stricmp(ci, "v2f_tc"))  v2f_tc=atof(cd);
	    if (!stricmp(ci, "v2f_gain"))  v2f_gain=atof(cd);
	    if (!stricmp(ci, "v2f_basef"))  v2f_basef=atof(cd);
	    if (!stricmp(ci, "v2f_bin_low"))  v2f_bin_low=atoi(cd);
	    if (!stricmp(ci, "v2f_bin_high"))  v2f_bin_high=atoi(cd);
	    if (!stricmp(ci, "v2f_freq_low"))  v2f_freq_low=atof(cd);
	    if (!stricmp(ci, "v2f_freq_high"))  v2f_freq_high=atof(cd);
	    if (!stricmp(ci, "v2f_af_level"))  v2f_af_level=atoi(cd);
	    if (!stricmp(ci, "dtmf_level"))  dtmf_level=atoi(cd);
	    if (!stricmp(ci, "dtmf_length"))  dtmf_length=atoi(cd);
	    if (!stricmp(ci, "beep_use"))  beep_use=atoi(cd);
	    if (!stricmp(ci, "beep_freq"))  beep_freq=atof(cd);
	    if (!stricmp(ci, "beep_dur"))  beep_dur=atoi(cd);
	    if (!stricmp(ci, "beep_level"))  beep_level=atoi(cd);
	    if (!stricmp(ci, "filter_mask"))  filter_mask=atoi(cd);
	    /* Filter 8 is not available from .CFG as it is "custom" */
	    if ( !stricmp(ci,"fildat") && (i=atoi(cd))>=1 && i<=7 )
		{
		cd=strtok(NULL, "|");		/* Move to string */
		cd=strtok(NULL, "|");		/* Get name string */
		if (strlen(cd)>6)
		    {
		    printf("\nfildat[%d].name exceeds 6 ch; Data not used", i);
		    sprintf(filt[i-1].name, "FILT%d", i);
		    }
		else
		    strcpy(filt[i-1].name, cd);

		strtok(NULL, "|");
		cd=strtok(NULL, "|");
		if (strlen(cd)>30)
		    {
		    printf("\nfildat[%d].file_name > 30 ch; Data not used", i);
		    strcpy(filt[i-1].file_name, " ");
		    }
		else
		    strcpy(filt[i-1].file_name, cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].fil_type=atoi(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].param1=atof(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].param2=atof(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].param3=atoi(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].fclow=atoi(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].fchigh=atoi(cd);
		cd=strtok(NULL, entrysep);
		filt[i-1].fil_gain=atoi(cd);
		}
	    if (!stricmp(ci, "flow"))  flow=atof(cd);
	    if (!stricmp(ci, "fhigh"))  fhigh=atof(cd);
	    if (!stricmp(ci, "kaiser_db"))  kaiser_db=atof(cd);
	    if (!stricmp(ci, "fil8_use"))  fil8_use=atoi(cd);
	    if (!stricmp(ci, "fil8_name"))
		{
		cd=strtok(cline2, "|");		/* Move to string */
		cd=strtok(NULL, "|");		/* Get short name string */
		cd[6]=0;			/* Limit length */
		strcpy(fil8_name, cd);
		}
	    if (!stricmp(ci, "aeqrdb"))
		{
		aeqrdb[0]=atof(cd);
		for (i=1; i<NFRB-1; i++)
		    {
		    if ((cd=strtok(NULL, entrysep))==0)
			{
			printf("Missing aeqrdb equalizer entry in UHFA.CFG\n");
			return 0;
			}
		    else
			{
			aeqrdb[i]=atof(cd);
			}
		    }
		}
	    if (!stricmp(ci, "aeqdb_t"))
		{
		aeqdb_t[0]=atof(cd);
		for (i=1; i<NFTB-1; i++)
		    {
		    if ((cd=strtok(NULL, entrysep))==0)
			{
			printf("Missing aeqdb_t xmit eq entry in UHFA.CFG\n");
			return 0;
			}
		    else
			{
			aeqdb_t[i]=atof(cd);
			}
		    }
		}
	    /* Rev 3.23 add following for k_ variables */
	    if (!stricmp(ci, "k_freq_m1"))  k_freq_m1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p1"))  k_freq_p1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m10"))  k_freq_m10=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p10"))  k_freq_p10=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m100"))  k_freq_m100=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p100"))  k_freq_p100=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m1k"))  k_freq_m1k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p1k"))  k_freq_p1k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m10k"))  k_freq_m10k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p10k"))  k_freq_p10k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m100k"))  k_freq_m100k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p100k"))  k_freq_p100k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_m1mhz"))  k_freq_m1mhz=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_p1mhz"))  k_freq_p1mhz=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_gain_m"))  k_af_gain_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_gain_p"))  k_af_gain_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_gain_m1"))  k_af_gain_m1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_gain_p1"))  k_af_gain_p1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rf_gain_m"))  k_rf_gain_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rf_gain_p"))  k_rf_gain_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mike_gain_m"))  k_mike_gain_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mike_gain_p"))  k_mike_gain_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fm_squelch_m_c"))  k_fm_squelch_m_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fm_squelch_m_l"))  k_fm_squelch_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fm_squelch_p_c"))  k_fm_squelch_p_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fm_squelch_p_l"))  k_fm_squelch_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cw_speed_m"))  k_cw_speed_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cw_speed_p"))  k_cw_speed_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cw_buf_clr"))  k_cw_buf_clr=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_brite_m"))  k_brite_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_brite_p"))  k_brite_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_contrast_m"))  k_contrast_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_contrast_p"))  k_contrast_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_filter_t"))  k_af_filter_t=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_filter_s"))  k_af_filter_s=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_denoise_t"))  k_denoise_t=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_denoise_m"))  k_denoise_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_denoise_p"))  k_denoise_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_nc_ave_m"))  k_nc_ave_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_nc_ave_p"))  k_nc_ave_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_delay_t_l"))  k_delay_t_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_delay_t_c"))  k_delay_t_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_m_c"))  k_rit_m_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_m_l"))  k_rit_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_p_c"))  k_rit_p_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_p_l"))  k_rit_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_m1_c"))  k_rit_m1_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_m1_l"))  k_rit_m1_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_p1_c"))  k_rit_p1_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_p1_l"))  k_rit_p1_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_o_c"))  k_rit_o_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_o_l"))  k_rit_o_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_z_c"))  k_rit_z_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_rit_z_l"))  k_rit_z_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_norm_qck_c"))  k_norm_qck_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_norm_qck_l"))  k_norm_qck_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_norm_prc_c"))  k_norm_prc_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_norm_prc_l"))  k_norm_prc_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_color_agc_c"))  k_color_agc_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_color_agc_l"))  k_color_agc_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_gif_c"))  k_gif_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_gif_l"))  k_gif_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_beacon_c"))  k_beacon_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_beacon_l"))  k_beacon_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_record_c"))  k_record_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_record_l"))  k_record_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_window_t_c"))  k_window_t_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_window_t_l"))  k_window_t_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fftbw_c"))  k_fftbw_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fftbw_l"))  k_fftbw_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_emedop_c"))  k_emedop_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_emedop_l"))  k_emedop_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mode_s_c"))  k_mode_s_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mode_s_l"))  k_mode_s_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_binaural_t"))  k_binaural_t=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_disp_t_c"))  k_disp_t_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_disp_t_l"))  k_disp_t_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_db_div"))  k_db_div=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_offset_m"))  k_offset_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_offset_p"))  k_offset_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_freq_box"))  k_freq_box=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_save_cfg"))  k_save_cfg=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas1"))  k_cas1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas2"))  k_cas2=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas3"))  k_cas3=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas4"))  k_cas4=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas5"))  k_cas5=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas6"))  k_cas6=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas7"))  k_cas7=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas8"))  k_cas8=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas9"))  k_cas9=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cas0"))  k_cas0=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_dafgu"))  k_dafgu=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_dafgd"))  k_dafgd=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_dfreqd"))  k_dfreqd=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_dfrequ"))  k_dfrequ=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mute"))  k_mute=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sh_dafgu"))  k_sh_dafgu=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sh_dafgd"))  k_sh_dafgd=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sh_dfreqd"))  k_sh_dfreqd=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sh_dfrequ"))  k_sh_dfrequ=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_fil96"))  k_fil96=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_dos_visit"))  k_dos_visit=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_t_r_tog"))  k_t_r_tog=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_help"))  k_help=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_code_data_c"))  k_code_data_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_code_data_l"))  k_code_data_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_code_change_c"))  k_code_change_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_code_change_l"))  k_code_change_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_clr_lt_c"))  k_clr_lt_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_clr_lt_l"))  k_clr_lt_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_clr_ylw_c"))  k_clr_ylw_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_clr_ylw_l"))  k_clr_ylw_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_flat_c"))  k_flat_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_flat_l"))  k_flat_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_vcur1_c"))  k_vcur1_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_vcur1_l"))  k_vcur1_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_vcur2_c"))  k_vcur2_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_vcur2_l"))  k_vcur2_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_set_clock_c"))  k_set_clock_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_set_clock_l"))  k_set_clock_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_eme_prev"))  k_eme_prev=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_eme_next"))  k_eme_next=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_beacon_box"))  k_beacon_box=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_tone_box"))  k_tone_box=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_diag_s"))  k_diag_s=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_af_filter_design"))  k_af_filter_design=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_xmt_eq_design"))  k_xmt_eq_design=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sig_sim"))  k_sig_sim=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_pua_del"))  k_pua_del=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_agc_decay_m"))  k_agc_decay_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_agc_decay_p"))  k_agc_decay_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_agc_decay_m_l"))  k_agc_decay_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_agc_decay_p_l"))  k_agc_decay_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cwoffset_m"))  k_cwoffset_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cwoffset_p"))  k_cwoffset_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cwoffset_m_l"))  k_cwoffset_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_cwoffset_p_l"))  k_cwoffset_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sidetone_lev_m"))  k_sidetone_lev_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sidetone_lev_p"))  k_sidetone_lev_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sidetone_lev_m_l"))  k_sidetone_lev_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_sidetone_lev_p_l"))  k_sidetone_lev_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_xmt_power_m"))  k_xmt_power_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_xmt_power_p"))  k_xmt_power_p=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_xmt_power_m_l"))  k_xmt_power_m_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_xmt_power_p_l"))  k_xmt_power_p_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_do_quit"))  k_do_quit=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_warm_boot"))  k_warm_boot=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_50_a"))  k_band_50_a=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_144_b"))  k_band_144_b=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_220_c"))  k_band_220_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_432_d"))  k_band_432_d=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_902_9"))  k_band_902_9=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_1296_e"))  k_band_1296_e=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_2304_f"))  k_band_2304_f=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_3456_g"))  k_band_3456_g=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_5760_h"))  k_band_5760_h=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_10k_i"))  k_band_10k_i=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_24k_j"))  k_band_24k_j=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_47k_k"))  k_band_47k_k=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_75k_l"))  k_band_75k_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_119k_m"))  k_band_119k_m=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_145k_n"))  k_band_145k_n=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_audio_o"))  k_audio_o=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_memo_1"))  k_band_memo_1=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_memo_2"))  k_band_memo_2=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_memo_3"))  k_band_memo_3=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_memo_4"))  k_band_memo_4=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_band_memo_5"))  k_band_memo_5=strtol(cd, &zptr, 0);
	    /* k_v2f entries came with 3.30, Nov 04 - RSL */
	    if (!stricmp(ci, "k_v2f_tog_c"))  k_v2f_tog_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_v2f_tog_l"))  k_v2f_tog_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_v2f_rew_c"))  k_v2f_res_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_v2f_res_l"))  k_v2f_res_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_v2f_lbox_c"))  k_v2f_lbox_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_v2f_lbox_l"))  k_v2f_lbox_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memw_c"))  k_memw_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memw_l"))  k_memw_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memr_c"))  k_memr_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memr_l"))  k_memr_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mem2vfo_c"))  k_mem2vfo_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_mem2vfo_l"))  k_mem2vfo_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memlist_c"))  k_memlist_c=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "k_memlist_l"))  k_memlist_l=strtol(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_m1s"))
		  ky_clock_m1s=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_p1s"))
		  ky_clock_p1s=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_m10ms"))
		  ky_clock_m10ms=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_p10ms"))
		  ky_clock_p10ms=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_mrate"))
		  ky_clock_mrate=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_prate"))
		  ky_clock_prate=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_resync"))
		  ky_clock_resync=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    if (!stricmp(ci, "ky_clock_exit"))
		  ky_clock_exit=0XFF & (unsigned int)strtoul(cd, &zptr, 0);
	    /* Note: comments will fall through here */
	    }
	return 1;
	}				/* End rcfg_mem() */

/* read_config_file()  -  Restores transceivers to previous settings
 * using UHFA.CFG.
 * Returns:	 0  Non-fatal error, or no file found.
 *		 1  Successful file read.
 *
 * Replaced "else" with "short comment" to reduce stack load on compilers!
 * This makes the reading of the configuration file more laborous, but
 * this is not important, since it only happens once.
 * Thanks to KC7WW for this.   RSL 24 Aug 04
 */
int read_config_file(void)
	{
	int i;
	static char entrysep[]=" \t,=\n\r";	/* Allow sp, tab, comma,
						   equal, lf, cr  */
	char *ci;
	char cline[260];

        if (NULL == ( config_file=fopen("UHFA.CFG", "rt") ))
	    {
	    printf("No UHFA.CFG file.  Using default configuration\n");
	    return 0;
	    }

	if (fgets(cline, 256, config_file) == NULL)
	    {
	    printf("Empty UHFA.CFG file.  Using default configuration\n");
	    fclose(config_file);
	    return 0;
	    }
 	ci = strtok(cline, entrysep);	/* Should be file identifier line */
	if (strcmp(ci, "UHFACFG2"))
	    {
	    printf("Illegal config file, not UHFACFG2. Use default config\n");
	    fclose(config_file);
	    return 0;
	    }
	if(rcfg_mem(config_file, CFG)==0)
	    {
	    fclose(config_file);
	    return 0;	/* Read main body of file */
	    }
	fclose(config_file);

	/* The cfg file needs to allow for old data that does not include
	 * values for vfo[].  Scan the vfo[]list and fill in any frequencies
	 * and modes that have tvtr[] but not vfo[]. The magic code for not
	 * being used is 999 in vfo[i].tvtr or tvtr[i].lo_type.   V3.28
	 */
	for(i=0; i<=20; i++)
	    {
	    if(vfo[i].tvtr==999 && tvtr[i].lo_type < 999)
		{
		vfo[i].tvtr=i;		/* Default, can change later */
		vfo[i].last_tfreq=tvtr[i].last_tfreq;	/* Move over */
		vfo[i].last_mode=tvtr[i].last_mode;
		}
	    }

	/* The local frequency control, tfreqL, needs to track the value
	 * in the current VFO vfo[cur_vfo].  If values were not read from the
	 * CFG file, the default band and tvtr will be used. V3.28      */
	tfreqL=(long int)(0.5+vfo[cur_vfo].last_tfreq);

	/* The same for band (tvtr setting), and mode.     */
	band=vfo[cur_vfo].tvtr;
	mode=vfo[cur_vfo].last_mode;

	return 1;
	}

/* save_state()  -  UHFA.CFG is used to bring the state of the transceiver
 * back to last state.  This will replace uhfa.cfg file.
 * The first line must be "UHFACFG2".  The fprintf below summarizes the
 * the flexible format.  The special rules for strings come from strtok()
 * that gags on empty strings.  The use of the '|' allows spaces within the
 * string for titles like "432 MHz Tvtr".
 */
void save_state(void)
	{
	if (NULL==( config_file=fopen("UHFA.CFG", "wt") ))
	    do_t_error(42, "E42 CFG File", 200);
	else
	    {
	    rewind(config_file);
	    fprintf(config_file, "UHFACFG2\n");

	    /* Before writing the CFG file, move the last frequency into the
	     * current VFO. V3.28              */
	    vfo[cur_vfo].last_tfreq=(double)tfreqL;

	    fprintf(config_file,
		"// Note: Use double space around .name and .init_str\n");
	    fprintf(config_file,
		"// and demark each string with vertical bars '|'.\n");
	    fprintf(config_file,
		"// Empty strings should have at least one space in them.\n");
	    fprintf(config_file,
		"// Fields can be delimited by space, tab, ',' ';' or '='\n");
	    fprintf(config_file,
		"//  \n");

	    fprintf(config_file, "v_m %d\n",  v_m);
	    fprintf(config_file, "cur_vfo %d\n", cur_vfo);
	    fprintf(config_file, "cur_mem %d\n", cur_mem);

	    wcfg_mem(config_file, CFG);	/* Write appropriate parts of vars */
	    if (fclose(config_file) != NULL)
		do_t_error(42, "E42 CFG File", 200);
	    }
	}				/* End save_state()  */

/* wcfg_mem( ) writes the main body of either a .CFG file or a .MEM file.
 * state_file is a pointer to a structure FILE
 * ftype is either CFG or MEM
 */
void wcfg_mem(FILE* state_file, int ftype)
	{
	int i, imax;

	if(ftype==CFG || memch_stn)
	    {
	    fprintf(state_file,
		"// \n//*** Station definition (memch_stn) ***\n// \n");
	    fprintf(state_file,
		"// CW & LTI messages are entered as 'cwmsg i  |msg|'\n");
	    fprintf(state_file,
		"// where i=0 to 9 and msg is less than 50 characters.\n");
	    fprintf(state_file,
		"// e.g., message 3 might be 'cwmsg 3  |#CQ DE W7XYZ #|'\n");
	    for (i=0; i<10; i++)
	      {
	      if ( cwmbuf[i*50] != 0 )
	        fprintf(state_file, "cwmsg %d  |%s| \n", i, &cwmbuf[i*50]);
	      }
	    fprintf(state_file,
		"// Up  to 10 locations are given by:\n");
	    fprintf(state_file,
		"//     'loc i  |Name|  |Description|  lat lon'\n");
	    fprintf(state_file,
		"// where i is (0,9) and lat and lon are decimal degrees,\n");
	    fprintf(state_file,
		"// with West being negative longitude. i=0 is home loc.\n");
	    for (i=0; i<=9; i++)
	      {
	      if (loc[i].time_zone != 999)
		    {
		    if (strlen(loc[i].nm)==0)
			strcpy(loc[i].nm, " ");
		    if (strlen(loc[i].name)==0)
			strcpy(loc[i].name, " ");
		    fprintf(state_file,
			"loc %d  |%s|  |%s|  %.5lf %.5lf\n",
			i, loc[i].nm, loc[i].name,
			r2d(loc[i].lat), r2d(loc[i].lon));
		    }
	      }
	    fprintf(state_file, "cur_loc %d\n", cur_loc);
	    fprintf(state_file,
		"// code_id for CW ID during FSK code, e.g. |DE W7XYZ|\n");
	    if (strlen(code_id)==0)
		strcpy(code_id, "DE ?");
	    fprintf(state_file, "code_id  |%s|\n", code_id);
	    fprintf(state_file,
		"// call_str is a string printed at top of upper-left\n");
	    fprintf(state_file,
		"// identification box, e.g. |W7XYZ|\n");
	    if (strlen(call_str)==0)
		strcpy(call_str, ".");
	    fprintf(state_file, "call_str  |%s|\n", call_str);
	    fprintf(state_file,
		"// file_ident is a single letter that prefixes output");
	    fprintf(state_file,
		" files, e.g. |U|\n");
	    fprintf(state_file, "file_ident  |%s|\n", file_ident);
	    }

	if(ftype==CFG || memch_radio)
	    {
	    fprintf(state_file,
		"// \n//*** Transceiver operation (memch_radio) ***\n// \n");
	    fprintf(state_file, "afgain %d\n",  afgain);
	    fprintf(state_file, "fm_afgain %d\n",  fm_afgain);
	    fprintf(state_file, "rfgain %d\n",  rfgain);
	    fprintf(state_file, "cw_speed %d\n",  cw_speed);
	    fprintf(state_file, "cw_longkey %d\n", cw_longkey);
	    fprintf(state_file, "agc_decay %d\n",  agc_decay);
	    fprintf(state_file, "mike_gain %d\n",  mike_gain);
	    fprintf(state_file, "xmit_power %d\n",  xmit_power);
	    fprintf(state_file, "rit %d\n",  rit);
	    fprintf(state_file, "rit_on %d\n",  rit_on);
	    fprintf(state_file, "fm_squelch %d\n",  fm_squelch);
	    fprintf(state_file, "fm_dev %4.2lf\n",  fm_dev);
	    fprintf(state_file, "cwoffset %d\n",  cwoffset);
	    fprintf(state_file, "eme_doppler %d\n",  eme_doppler);
	    fprintf(state_file, "sidetone_lev %d\n",  sidetone_lev);
	    fprintf(state_file,
	      "//dtmf_level 0,100 dB; dtmf_length in milliseconds, FM only\n");
	    fprintf(state_file, "dtmf_level %d\n", dtmf_level);
	    fprintf(state_file, "dtmf_length %d\n", dtmf_length);
	    fprintf(state_file,
	      "//beep_xxx are for tone at end of SSB transmission\n");
	    fprintf(state_file,
	      "//  _freq in Hz, _dur in millisec, level in dB, 0 to 100 \n");
	    fprintf(state_file, "beep_use %d\n", beep_use);
	    fprintf(state_file, "beep_freq %lf\n", beep_freq);
	    fprintf(state_file, "beep_dur %d\n", beep_dur);
	    fprintf(state_file, "beep_level %d\n", beep_level);
	    fprintf(state_file, "delay_state %d\n", delay_state);
	    fprintf(state_file, "delay_right %d\n", delay_right);
	    fprintf(state_file,
		"// mode_mask is sum of values for desired modes:\n");
	    fprintf(state_file,
		"// 1=CW, 2=USB, 4=LSB, 8=FM, 16=LHL7,");
	    fprintf(state_file,
		" 32=PUA43, 64=EME2, 128=LTI\n");
	    fprintf(state_file, "mode_mask %d\n",  mode_mask);
	    /* fprintf(state_file, "band %d\n",  band); Out V3.28, VFO sets */
	    /* fprintf(state_file, "tfreqL %ld\n",  tfreqL); Out V3.28 */

	    /* Following was memch_wksigs */
	    fprintf(state_file, "eme2_te %lf\n", eme2_te);
	    fprintf(state_file, "eme_freq_range %lf\n", eme_freq_range);
	    fprintf(state_file, "code_nb_ratio %lf\n", code_nb_ratio);
	    fprintf(state_file, "lterm_int_flow %.1lf\n", lterm_int_flow);
	    fprintf(state_file, "lterm_int_fhigh %.1lf\n", lterm_int_fhigh);
	    fprintf(state_file, "lterm_int_center %.1lf\n", lterm_int_center);
	    fprintf(state_file, "lti_freq_range %.1lf\n", lti_freq_range);
	    fprintf(state_file, "lhl_bins %d\n",  lhl_bins);
	    fprintf(state_file, "lhl_peak_ave %d\n",  lhl_peak_ave);
	    fprintf(state_file, "lhl7_2line %d\n",  lhl7_2line);
	    fprintf(state_file, "pua43_noisedecay %.6lf\n", pua43_noisedecay);
	    fprintf(state_file, "pua43_dorandom %d\n", pua43_dorandom);
	    fprintf(state_file, "pua43_xstring  |%s|\n", pua43_xstring);
	    fprintf(state_file, "pua14_28 %d\n",  pua14_28);
	    fprintf(state_file, "code_nx2 %d\n", code_nx2);
	    fprintf(state_file, "tick_time %d\n", tick_time);
	    fprintf(state_file, "pua43_tick %d\n", pua43_tick);
	    fprintf(state_file, "v2f_noise %d\n", v2f_noise);
	    fprintf(state_file, "v2f_pkave %d\n", v2f_pkave);
	    fprintf(state_file, "v2f_tc %.5lf\n",  v2f_tc);
	    fprintf(state_file, "v2f_gain %.3lf\n",  v2f_gain);
	    fprintf(state_file, "v2f_basef %.3lf\n",  v2f_basef);
	    fprintf(state_file, "v2f_bin_low %d\n", v2f_bin_low);
	    fprintf(state_file, "v2f_bin_high %d\n", v2f_bin_high);
	    fprintf(state_file, "v2f_freq_low %.3lf\n",  v2f_freq_low);
	    fprintf(state_file, "v2f_freq_high %.3lf\n",  v2f_freq_high);
	    fprintf(state_file, "v2f_af_level %d\n", v2f_af_level);
	    }

	if(ftype==CFG || memch_tvtr)
	    {
	    fprintf(state_file,
		"// \n//*** Transverter definitions (memch_tvtr) ***\n// \n");
	    fprintf(state_file,
		"// f_eme in Hz, used for audio processing only\n");
	    fprintf(state_file, "f_eme %.1lf\n", f_eme);
	    for (i=0; i<=20; i++)
	      {
	      if (tvtr[i].lo_type < 999)
		{
		if (strlen(tvtr[i].name)==0)
			strcpy(tvtr[i].name, " ");
		if (strlen(tvtr[i].init_str)==0)
			strcpy(tvtr[i].init_str, " ");
	        fprintf(state_file,
		    "tvtr %d  |%s|  %d %d %.1f %d  |%s|  %.3f %d %d %.1f %d\n",
		    i, tvtr[i].name, tvtr[i].lo_type, tvtr[i].sb,
		    tvtr[i].lo_freq, tvtr[i].chain_mult, tvtr[i].init_str,
		    tvtr[i].gain_db, /*tvtr[i].control*/ 0, tvtr[i].bus_adr,
		    0.0, 0);
		    /* tvtr[i].last_tfreq, tvtr[i].last_mode); Out 3.28 */
		}
	      }
	    }
	if(ftype==CFG || memch_vfo)
	    {
	    fprintf(state_file,
		"// \n//*** VFO definitions (memch_vfo) ***\n// \n");
	    if (ftype==CFG)  imax=21;   else imax=20;
	    for (i=0; i<=imax; i++)	/* Added VFOs 3.28 */
	      {
	      if (vfo[i].tvtr < 999)
		{
	        fprintf(state_file,
		    "vfo %d %d %d %.1lf %d %d %.1lf %d %d %d %d\n",
		    i, vfo[i].tvtr, vfo[i].tr_offset, vfo[i].last_tfreq,
		    vfo[i].last_mode,		/* Rev 3.33 */
		    vfo[i].t_fm_use, vfo[i].t_fm_freq, vfo[i].t_fm_dur,
		    vfo[i].t_fm_level, vfo[i].par1, vfo[i].par2);
		}
	      }
	    }

	if(ftype==CFG || memch_disp)
	    {
	    fprintf(state_file,
		"// \n//*** Display character (memch_disp) ***\n// \n");
	    fprintf(state_file, "contrast %d\n", contrast);
	    fprintf(state_file, "bright %d\n",  bright);
	    fprintf(state_file, "dot_type %d\n",  dot_type);
	    fprintf(state_file, "// freq_low is lowest freq marker\n");
	    fprintf(state_file, "freq_low %lf\n",  freq_low);
	    fprintf(state_file, "db_div_set %d\n",  db_div_set);
	    fprintf(state_file, "koffset %lf\n",  koffset);
	    fprintf(state_file, "noncohave %d\n",  noncohave);
	    fprintf(state_file,
		"// do_window=0,1,2,3 for None, Tukey25, Hamming or Bh-92.\n");
	    fprintf(state_file, "do_window %d\n",  do_window);
	    fprintf(state_file,
		"// fft_bw=0,1,2,3 for 1200, 2400, 4800 4800W Hz.\n");
	    fprintf(state_file, "fft_bw %d\n",  fft_bw);
	    fprintf(state_file,
	     "// knoise:1.0 constant sig; 0.0 constant noise pwr, display\n");
	    fprintf(state_file, "knoise %lf\n",  knoise);
	    fprintf(state_file,
		"// time_type=0 is GMT display, 1=computer time\n");
	    fprintf(state_file, "show_secs %d\n", show_secs);
	    fprintf(state_file, "time_type %d\n", time_type);
	    if (strlen(time_str)==0)
		strcpy(time_str, ".");
	    fprintf(state_file, "time_str  |%s|\n", time_str);
	    }

	if(ftype==CFG || memch_bcn)
	    {
	    fprintf(state_file,
		"// \n//*** Beacon operation (memch_bcn) ***\n// \n");
	    fprintf(state_file, "b_transmit_sec %d\n",  b_transmit_sec);
	    fprintf(state_file, "b_receive_sec %d\n",  b_receive_sec);
	    fprintf(state_file, "b_offset_sec %d\n",  b_offset_sec);
	    fprintf(state_file, "lt_b_save %d\n",  lt_b_save);
	    fprintf(state_file, "lt_b_save_bins %d\n",  lt_b_save_bins);
	    fprintf(state_file, "lt_b_clear %d\n",  lt_b_clear);
	    fprintf(state_file, "// b_active1 & 2 set seconds during half hour that beacon is on\n");
	    fprintf(state_file, "b_active1 %d\n",  b_active1);
	    fprintf(state_file, "b_active2 %d\n",  b_active2);
	    fprintf(state_file, "b_repeat60 %d\n",  b_repeat60);
	    fprintf(state_file, "data_omit_tr %d\n",  data_omit_tr);
	    }

	if(ftype==CFG || memch_fil)
	    {
	    fprintf(state_file,
		"// \n//*** Filters, equalizers (memch_fil) ***\n// \n");
	    fprintf(state_file,
	      "//The fildat data is informational input to the PC program.\n");
	    fprintf(state_file,
	      "//This data does NOT design filters.\n");
	    for (i=0; i<7; i++)
		{
		if (strlen(filt[i].name)==0)
			strcpy(filt[i].name, " ");
		if (strlen(filt[i].file_name)==0)
			strcpy(filt[i].file_name, " ");
	        fprintf(state_file,
		    "fildat %d  |%s|  |%s|  %d %.3lf %.6f %d %d %d %d\n",
		    i+1, filt[i].name, filt[i].file_name,
		    filt[i].fil_type, filt[i].param1,
		    /* filt[i].param2 */   0.0,
		    /* filt[i].param3 */   0,
		    filt[i].fclow, filt[i].fchigh,
		    filt[i].fil_gain);
		}
	    fprintf(state_file,
		"// filter mask=sum[1=Filter1, 2=Filter2,...,128=Filter8]\n");
	    fprintf(state_file, "filter_mask %d\n", filter_mask);
	    fprintf(state_file, "flow %.1lf\n", flow);
	    fprintf(state_file, "fhigh %.1lf\n", fhigh);
	    fprintf(state_file, "kaiser_db %.1lf\n", kaiser_db);
	    fprintf(state_file, "fil8_use %d\n", fil8_use);
	    fprintf(state_file, "fil8_name  |%s|\n", fil8_name);
	    fprintf(state_file,
	     "// kfilt:1.0 constant sig; 0.0 constant noise pwr, FIR filt\n");
	    fprintf(state_file, "kfilt %lf\n",  kfilt);
	    fprintf(state_file, "fft_fcn %d\n",  fft_fcn);
	    fprintf(state_file, "do_af_filt %d\n",  do_af_filt);
	    fprintf(state_file, "af_coef %d\n",  af_coef);
	    fprintf(state_file, "do_LMS %d\n",  do_LMS);
	    fprintf(state_file, "denoise_level %d\n",  denoise_level);
	    fprintf(state_file, "denoise_decay %d\n",  denoise_decay);
	    fprintf(state_file,
		 "// denoise_del_len=0 for 31; =1 for 61 delay line length\n");
	    fprintf(state_file, "denoise_del_len %d\n",  denoise_del_len);
	    fprintf(state_file, "autonotch_level %d\n",  autonotch_level);
	    fprintf(state_file, "// Rcv Equalizer dB:\n");
	    fprintf(state_file, "aeqrdb ");
	    for (i=0; i<NFRB-1; i++)
		fprintf(state_file, "%6.1lf ", aeqrdb[i]);
	    fprintf(state_file, "\n");
	    fprintf(state_file,
                         "//     0      200    400    600    800   1000  ");
	    fprintf(state_file, "1300   1600   2000   2400   2800   3200");
	    fprintf(state_file, "   3600   4000   4400\n");
	    fprintf(state_file, "// Transmit Equalizer dB:\n");
	    fprintf(state_file, "aeqdb_t ");
	    for (i=0; i<NFTB-1; i++)
		fprintf(state_file, "%6.1lf ", aeqdb_t[i]);
	    fprintf(state_file, "\n");
	    fprintf(state_file,
                         "//     0      250    550    850   1200   1600   ");
	    fprintf(state_file, "2100   2800\n");
	    }

	if(ftype==CFG || memch_sys)
	    {
	    fprintf(state_file,
		"// \n//*** System parameters (memch_sys) ***\n// \n");
	    fprintf(state_file, "diag_mode %d\n",  diag_mode);
	    fprintf(state_file,
		"// startup_detail=1 for diagnostic text screens:\n");
	    fprintf(state_file, "startup_detail %d\n",  startup_detail);
	    fprintf(state_file,
		"// waterfall=1 for weak-signal waterfall,\n");
	    fprintf(state_file,
		"//          =0 for big dial & help summary screen\n");
	    fprintf(state_file, "waterfall %d\n",  waterfall);
	    fprintf(state_file, "xsnr %lf\n", xsnr);
	    fprintf(state_file, "xfade_rate %lf\n", xfade_rate);
	    fprintf(state_file, "xnoise %d\n", xnoise);
	    fprintf(state_file, "xfade %d\n", xfade);
	    fprintf(state_file,
		"// dut1 is the time difference UT1-UTC for eme Doppler\n");
	    fprintf(state_file, "dut1 %.3lf\n",  dut1);
	    fprintf(state_file, "quit_save_state %d\n",  quit_save_state);
	    fprintf(state_file, "fscreen_path  |%s|\n", fscreen_path);
	    fprintf(state_file, "scsave %d\n", scsave);
/* Rev 3.50 fprintf(state_file, "scs_key %d\n", scs_key); */
	    fprintf(state_file, "use_scs_box %d\n", use_scs_box);
	    fprintf(state_file, "fdata_path  |%s|\n", fdata_path);
	    fprintf(state_file, "beep_on_error %d\n", beep_on_error);
	    fprintf(state_file,
		"// The following memch_xxx define the contents of a\n");
	    fprintf(state_file,
		"// memory file. 1=include, 0=exclude.\n");
	    fprintf(state_file,
		"// memch_stn: Station definition: code_id, call_str, etc\n");
	    fprintf(state_file, "memch_stn %d\n",  memch_stn);
	    fprintf(state_file,
		"// memch_fr_md  Transceiver frequency and mode ");
	    fprintf(state_file,
		"(conventional memory)\n");
	    fprintf(state_file, "memch_fr_md %d\n",  memch_fr_md);
	    fprintf(state_file,
		"// memch_radio  Transceiver settings, special modes \n");
	    fprintf(state_file, "memch_radio %d\n",  memch_radio);
	    fprintf(state_file,
		"// memch_tvtr  Transverter definitions\n");
	    fprintf(state_file, "memch_tvtr %d\n",  memch_tvtr);
	    fprintf(state_file,
		"// memch_vfo  VFO definitions\n");
	    fprintf(state_file, "memch_vfo %d\n",  memch_vfo);
	    fprintf(state_file,
		"// memch_disp  Display character, except colors\n");
	    fprintf(state_file, "memch_disp %d\n",  memch_disp);
	    fprintf(state_file,
		"// memch_bcn   Beacon operation\n");
	    fprintf(state_file, "memch_bcn %d\n",  memch_bcn);
	    fprintf(state_file,
		"// memch_fil   Filters, equalizers\n");
	    fprintf(state_file, "memch_fil %d\n",  memch_fil);
	    fprintf(state_file,
		"// memch_sys   System: startup_detail, file paths, etc\n");
	    fprintf(state_file, "memch_sys %d\n",  memch_sys);
	    fprintf(state_file,
		"// memch_color Display colors\n");
	    fprintf(state_file, "memch_color %d\n",  memch_color);
	    fprintf(state_file,
		"// memch_kbd   Keyboard definitions\n");
	    fprintf(state_file, "memch_kbd %d\n",  memch_kbd);
	    fprintf(state_file,
		"// memch_hdw   Hardware: ezkit_dfreq, pc clock, etc\n");
	    fprintf(state_file, "memch_hdw %d\n",  memch_hdw);
	    }

	if(ftype==CFG || memch_color)
	    {
	    fprintf(state_file,
		"// \n//*** Display colors (memch_color) ***\n// \n");
	    fprintf(state_file, "// Following defines palette and screen:\n");
	    fprintf(state_file,
		"palette %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n", 
		 upalette.colors[0],  upalette.colors[1],  upalette.colors[2],
		 upalette.colors[3],  upalette.colors[4],  upalette.colors[5],
		 upalette.colors[6],  upalette.colors[7],  upalette.colors[8],
		 upalette.colors[9],  upalette.colors[10], upalette.colors[11],
		 upalette.colors[12], upalette.colors[13], upalette.colors[14],
		 upalette.colors[15]);
	    fprintf(state_file, "c_background %d\n", c_background);
	    fprintf(state_file, "c_outline %d\n", c_outline);
	    fprintf(state_file, "c_outline_bold %d\n", c_outline_bold);
	    fprintf(state_file, "c_outline_bright %d\n",c_outline_bright);
	    fprintf(state_file, "c_text %d\n", c_text);
	    fprintf(state_file, "c_text_bold %d\n", c_text_bold);
	    fprintf(state_file, "c_text_bright %d\n", c_text_bright);
	    fprintf(state_file, "c_text_note %d\n", c_text_note);
	    fprintf(state_file, "c_plot_bold %d\n", c_plot_bold);
	    fprintf(state_file, "c_plot_bright %d\n", c_plot_bright);
	    fprintf(state_file, "c_plot_marker %d\n", c_plot_marker);
	    fprintf(state_file, "c_bar_dark %d\n", c_bar_dark);
	    fprintf(state_file, "c_bar_low %d\n", c_bar_low);
	    fprintf(state_file, "c_bar_medium %d\n", c_bar_medium);
	    fprintf(state_file, "c_bar_high %d\n", c_bar_high);
	    }

	if(ftype==CFG || memch_hdw)
	    {
	    fprintf(state_file,
		"// \n//*** Hardware (memch_hdw) ***\n// \n");
	    fprintf(state_file,
		"// The following six entries are the actual attenuation\n");
	    fprintf(state_file,
		"// for rf gain settings 94, 88,...,64. Nominal values\n");
	    fprintf(state_file,
		"// of -6.0, -12.0,...,-36.0.\n");
	    fprintf(state_file,
		"rfgdb %.2lf %.2lf %.2lf %.2lf %.2lf %.2lf\n", 
		rfg[1].dbgain, rfg[2].dbgain, rfg[3].dbgain, 
		rfg[4].dbgain, rfg[5].dbgain, rfg[6].dbgain);
	    fprintf(state_file,
		"// gain_adj is small dB value to adjust power displays.\n");
	    fprintf(state_file, "gain_adj %.2lf\n", gain_adj);
	    fprintf(state_file,
	        "// hardware=1 normal DSP-10; hardware=0 audio processor \n");
	    fprintf(state_file, "hardware %d\n",  hardware);
	    fprintf(state_file,
	        "// hardware_ptt=1 fast PTT has been implemented; else 0 \n");
	    fprintf(state_file, "hardware_ptt %d\n",  hardware_ptt);
	    fprintf(state_file,
		"// ref_offset ratios the 10 MHz ref freq, like 1.0000002\n");
	    fprintf(state_file, "ref_offset %15.13lf\n",  ref_offset);
	    fprintf(state_file,
	        "// ezkit_dfreq is the Hz error in 3rd LO, 0.0 if unknown\n");
	    fprintf(state_file, "ezkit_dfreq %lf\n",  ezkit_dfreq);
	    fprintf(state_file, "clock_speed %.7lf\n", clock_cor_factor);
 /*>>>*/    fprintf(state_file, "shift_reg %d\n",  shift_reg);
	    fprintf(state_file,
	        "// dly_ant2amp Delay in millisec, R->T after ant relay\n");
	    fprintf(state_file, "dly_ant2amp %d\n",  dly_ant2amp);
	    fprintf(state_file,
	        "// dly_amp2xmit Delay in millisec, R->T after amp on\n");
	    fprintf(state_file, "dly_amp2xmit %d\n",  dly_amp2xmit);
	    fprintf(state_file,
	        "// dly_ant2amp Delay in millisec, T-R after RF off\n");
	    fprintf(state_file, "dly_xmit2amp %d\n",  dly_xmit2amp);
	    fprintf(state_file,
	        "// dly_ant2amp Delay in millisec, T->R after amp off\n");
	    fprintf(state_file, "dly_amp2ant %d\n",  dly_amp2ant);
	    fprintf(state_file, "dly_emet2r %d\n",  dly_emet2r);
	    fprintf(state_file,
		"// lpt_port=0 for none, 888 for LPT1 (0x378),\n");
	    fprintf(state_file,
		"// or 956 for LPT2 (0x3BC) (confirm your computer)\n");
	    fprintf(state_file, "lpt_port %d\n", lpt_port);
	    }

	if(ftype==CFG || memch_kbd)
	    {
	    fprintf(state_file,
		"// \n//*** Keyboard definitions (memch_kbd) ***\n// \n");
	    fprintf(state_file, "delf %ld\n",  delf);
	    fprintf(state_file, "delf_sh %ld\n",  delf_sh);
	    fprintf(state_file, "deldb %d\n",  deldb);
	    fprintf(state_file, "deldb_sh %d\n",  deldb_sh);
/*>>>>*/    fprintf(state_file, "kbd_alt1 %d\n", kbd_alt1);
	    fprintf(state_file, "kbd_alt2 %d\n", kbd_alt2);
	    fprintf(state_file, "kbd_alt3 %d\n", kbd_alt3);
	    /* Rev 3.23 Following added for k_ variables */
	    fprintf(state_file, "k_freq_m1 0X%lX\n",  k_freq_m1);
	    fprintf(state_file, "k_freq_p1 0X%lX\n",  k_freq_p1);
	    fprintf(state_file, "k_freq_m10 0X%lX\n",  k_freq_m10);
	    fprintf(state_file, "k_freq_p10 0X%lX\n",  k_freq_p10);
	    fprintf(state_file, "k_freq_m100 0X%lX\n",  k_freq_m100);
	    fprintf(state_file, "k_freq_p100 0X%lX\n",  k_freq_p100);
	    fprintf(state_file, "k_freq_m1k 0X%lX\n",  k_freq_m1k);
	    fprintf(state_file, "k_freq_p1k 0X%lX\n",  k_freq_p1k);
	    fprintf(state_file, "k_freq_m10k 0X%lX\n",  k_freq_m10k);
	    fprintf(state_file, "k_freq_p10k 0X%lX\n",  k_freq_p10k);
	    fprintf(state_file, "k_freq_m100k 0X%lX\n",  k_freq_m100k);
	    fprintf(state_file, "k_freq_p100k 0X%lX\n",  k_freq_p100k);
	    fprintf(state_file, "k_freq_m1mhz 0X%lX\n",  k_freq_m1mhz);
	    fprintf(state_file, "k_freq_p1mhz 0X%lX\n",  k_freq_p1mhz);
	    fprintf(state_file, "k_af_gain_m 0X%lX\n",  k_af_gain_m);
	    fprintf(state_file, "k_af_gain_p 0X%lX\n",  k_af_gain_p);
	    fprintf(state_file, "k_af_gain_m1 0X%lX\n",  k_af_gain_m1);
	    fprintf(state_file, "k_af_gain_p1 0X%lX\n",  k_af_gain_p1);
	    fprintf(state_file, "k_rf_gain_m 0X%lX\n",  k_rf_gain_m);
	    fprintf(state_file, "k_rf_gain_p 0X%lX\n",  k_rf_gain_p);
	    fprintf(state_file, "k_mike_gain_m 0X%lX\n",  k_mike_gain_m);
	    fprintf(state_file, "k_mike_gain_p 0X%lX\n",  k_mike_gain_p);
	    fprintf(state_file, "k_fm_squelch_m_c 0X%lX\n",  k_fm_squelch_m_c);
	    fprintf(state_file, "k_fm_squelch_m_l 0X%lX\n",  k_fm_squelch_m_l);
	    fprintf(state_file, "k_fm_squelch_p_c 0X%lX\n",  k_fm_squelch_p_c);
	    fprintf(state_file, "k_fm_squelch_p_l 0X%lX\n",  k_fm_squelch_p_l);
	    fprintf(state_file, "k_cw_speed_m 0X%lX\n",  k_cw_speed_m);
	    fprintf(state_file, "k_cw_speed_p 0X%lX\n",  k_cw_speed_p);
	    fprintf(state_file, "k_cw_buf_clr 0X%lX\n",  k_cw_buf_clr);
	    fprintf(state_file, "k_brite_m 0X%lX\n",  k_brite_m);
	    fprintf(state_file, "k_brite_p 0X%lX\n",  k_brite_p);
	    fprintf(state_file, "k_contrast_m 0X%lX\n",  k_contrast_m);
	    fprintf(state_file, "k_contrast_p 0X%lX\n",  k_contrast_p);
	    fprintf(state_file, "k_af_filter_t 0X%lX\n",  k_af_filter_t);
	    fprintf(state_file, "k_af_filter_s 0X%lX\n",  k_af_filter_s);
	    fprintf(state_file, "k_denoise_t 0X%lX\n",  k_denoise_t);
	    fprintf(state_file, "k_denoise_m 0X%lX\n",  k_denoise_m);
	    fprintf(state_file, "k_denoise_p 0X%lX\n",  k_denoise_p);
	    fprintf(state_file, "k_nc_ave_m 0X%lX\n",  k_nc_ave_m);
	    fprintf(state_file, "k_nc_ave_p 0X%lX\n",  k_nc_ave_p);
	    fprintf(state_file, "k_delay_t_l 0X%lX\n",  k_delay_t_l);
	    fprintf(state_file, "k_delay_t_c 0X%lX\n",  k_delay_t_c);
	    fprintf(state_file, "k_rit_m_c 0X%lX\n",  k_rit_m_c);
	    fprintf(state_file, "k_rit_m_l 0X%lX\n",  k_rit_m_l);
	    fprintf(state_file, "k_rit_p_c 0X%lX\n",  k_rit_p_c);
	    fprintf(state_file, "k_rit_p_l 0X%lX\n",  k_rit_p_l);
	    fprintf(state_file, "k_rit_m1_c 0X%lX\n",  k_rit_m1_c);
	    fprintf(state_file, "k_rit_m1_l 0X%lX\n",  k_rit_m1_l);
	    fprintf(state_file, "k_rit_p1_c 0X%lX\n",  k_rit_p1_c);
	    fprintf(state_file, "k_rit_p1_l 0X%lX\n",  k_rit_p1_l);
	    fprintf(state_file, "k_rit_o_c 0X%lX\n",  k_rit_o_c);
	    fprintf(state_file, "k_rit_o_l 0X%lX\n",  k_rit_o_l);
	    fprintf(state_file, "k_rit_z_c 0X%lX\n",  k_rit_z_c);
	    fprintf(state_file, "k_rit_z_l 0X%lX\n",  k_rit_z_l);
	    fprintf(state_file, "k_norm_qck_c 0X%lX\n",  k_norm_qck_c);
	    fprintf(state_file, "k_norm_qck_l 0X%lX\n",  k_norm_qck_l);
	    fprintf(state_file, "k_norm_prc_c 0X%lX\n",  k_norm_prc_c);
	    fprintf(state_file, "k_norm_prc_l 0X%lX\n",  k_norm_prc_l);
	    fprintf(state_file, "k_color_agc_c 0X%lX\n",  k_color_agc_c);
	    fprintf(state_file, "k_color_agc_l 0X%lX\n",  k_color_agc_l);
	    fprintf(state_file, "k_gif_c 0X%lX\n",  k_gif_c);
	    fprintf(state_file, "k_gif_l 0X%lX\n",  k_gif_l);
	    fprintf(state_file, "k_beacon_c 0X%lX\n",  k_beacon_c);
	    fprintf(state_file, "k_beacon_l 0X%lX\n",  k_beacon_l);
	    fprintf(state_file, "k_record_c 0X%lX\n",  k_record_c);
	    fprintf(state_file, "k_record_l 0X%lX\n",  k_record_l);
	    fprintf(state_file, "k_window_t_c 0X%lX\n",  k_window_t_c);
	    fprintf(state_file, "k_window_t_l 0X%lX\n",  k_window_t_l);
	    fprintf(state_file, "k_fftbw_c 0X%lX\n",  k_fftbw_c);
	    fprintf(state_file, "k_fftbw_l 0X%lX\n",  k_fftbw_l);
	    fprintf(state_file, "k_emedop_c 0X%lX\n",  k_emedop_c);
	    fprintf(state_file, "k_emedop_l 0X%lX\n",  k_emedop_l);
	    fprintf(state_file, "k_mode_s_c 0X%lX\n",  k_mode_s_c);
	    fprintf(state_file, "k_mode_s_l 0X%lX\n",  k_mode_s_l);
	    fprintf(state_file, "k_binaural_t 0X%lX\n",  k_binaural_t);
	    fprintf(state_file, "k_disp_t_c 0X%lX\n",  k_disp_t_c);
	    fprintf(state_file, "k_disp_t_l 0X%lX\n",  k_disp_t_l);
	    fprintf(state_file, "k_db_div 0X%lX\n",  k_db_div);
	    fprintf(state_file, "k_offset_m 0X%lX\n",  k_offset_m);
	    fprintf(state_file, "k_offset_p 0X%lX\n",  k_offset_p);
	    fprintf(state_file, "k_freq_box 0X%lX\n",  k_freq_box);
	    fprintf(state_file, "k_save_cfg 0X%lX\n",  k_save_cfg);
	    fprintf(state_file, "k_cas1 0X%lX\n",  k_cas1);
	    fprintf(state_file, "k_cas2 0X%lX\n",  k_cas2);
	    fprintf(state_file, "k_cas3 0X%lX\n",  k_cas3);
	    fprintf(state_file, "k_cas4 0X%lX\n",  k_cas4);
	    fprintf(state_file, "k_cas5 0X%lX\n",  k_cas5);
	    fprintf(state_file, "k_cas6 0X%lX\n",  k_cas6);
	    fprintf(state_file, "k_cas7 0X%lX\n",  k_cas7);
	    fprintf(state_file, "k_cas8 0X%lX\n",  k_cas8);
	    fprintf(state_file, "k_cas9 0X%lX\n",  k_cas9);
	    fprintf(state_file, "k_cas0 0X%lX\n",  k_cas0);
	    fprintf(state_file, "k_dafgu 0X%lX\n",  k_dafgu);
	    fprintf(state_file, "k_dafgd 0X%lX\n",  k_dafgd);
	    fprintf(state_file, "k_dfreqd 0X%lX\n",  k_dfreqd);
	    fprintf(state_file, "k_dfrequ 0X%lX\n",  k_dfrequ);
	    fprintf(state_file, "k_mute 0X%lX\n",  k_mute);
	    fprintf(state_file, "k_sh_dafgu 0X%lX\n",  k_sh_dafgu);
	    fprintf(state_file, "k_sh_dafgd 0X%lX\n",  k_sh_dafgd);
	    fprintf(state_file, "k_sh_dfreqd 0X%lX\n",  k_sh_dfreqd);
	    fprintf(state_file, "k_sh_dfrequ 0X%lX\n",  k_sh_dfrequ);
	    fprintf(state_file, "k_fil96 0X%lX\n",  k_fil96);
	    fprintf(state_file, "k_dos_visit 0X%lX\n",  k_dos_visit);
	    fprintf(state_file, "k_t_r_tog 0X%lX\n",  k_t_r_tog);
	    fprintf(state_file, "k_help 0X%lX\n",  k_help);
	    fprintf(state_file, "k_code_data_c 0X%lX\n",  k_code_data_c);
	    fprintf(state_file, "k_code_data_l 0X%lX\n",  k_code_data_l);
	    fprintf(state_file, "k_code_change_c 0X%lX\n",  k_code_change_c);
	    fprintf(state_file, "k_code_change_l 0X%lX\n",  k_code_change_l);
	    fprintf(state_file, "k_clr_lt_c 0X%lX\n",  k_clr_lt_c);
	    fprintf(state_file, "k_clr_lt_l 0X%lX\n",  k_clr_lt_l);
	    fprintf(state_file, "k_clr_ylw_c 0X%lX\n",  k_clr_ylw_c);
	    fprintf(state_file, "k_clr_ylw_l 0X%lX\n",  k_clr_ylw_l);
	    fprintf(state_file, "k_flat_c 0X%lX\n",  k_flat_c);
	    fprintf(state_file, "k_flat_l 0X%lX\n",  k_flat_l);
	    fprintf(state_file, "k_vcur1_c 0X%lX\n",  k_vcur1_c);
	    fprintf(state_file, "k_vcur1_l 0X%lX\n",  k_vcur1_l);
	    fprintf(state_file, "k_vcur2_c 0X%lX\n",  k_vcur2_c);
	    fprintf(state_file, "k_vcur2_l 0X%lX\n",  k_vcur2_l);
	    fprintf(state_file, "k_set_clock_c 0X%lX\n",  k_set_clock_c);
	    fprintf(state_file, "k_set_clock_l 0X%lX\n",  k_set_clock_l);
	    fprintf(state_file, "k_eme_prev 0X%lX\n",  k_eme_prev);
	    fprintf(state_file, "k_eme_next 0X%lX\n",  k_eme_next);
	    fprintf(state_file, "k_beacon_box 0X%lX\n",  k_beacon_box);
	    fprintf(state_file, "k_tone_box 0X%lX\n",  k_tone_box);
	    fprintf(state_file, "k_diag_s 0X%lX\n",  k_diag_s);
	    fprintf(state_file, "k_af_filter_design 0X%lX\n",  k_af_filter_design);
	    fprintf(state_file, "k_xmt_eq_design 0X%lX\n",  k_xmt_eq_design);
	    fprintf(state_file, "k_sig_sim 0X%lX\n",  k_sig_sim);
	    fprintf(state_file, "k_pua_del 0X%lX\n",  k_pua_del);
	    fprintf(state_file, "k_agc_decay_m 0X%lX\n",  k_agc_decay_m);
	    fprintf(state_file, "k_agc_decay_p 0X%lX\n",  k_agc_decay_p);
	    fprintf(state_file, "k_agc_decay_m_l 0X%lX\n",  k_agc_decay_m_l);
	    fprintf(state_file, "k_agc_decay_p_l 0X%lX\n",  k_agc_decay_p_l);
	    fprintf(state_file, "k_cwoffset_m 0X%lX\n",  k_cwoffset_m);
	    fprintf(state_file, "k_cwoffset_p 0X%lX\n",  k_cwoffset_p);
	    fprintf(state_file, "k_cwoffset_m_l 0X%lX\n",  k_cwoffset_m_l);
	    fprintf(state_file, "k_cwoffset_p_l 0X%lX\n",  k_cwoffset_p_l);
	    fprintf(state_file, "k_sidetone_lev_m 0X%lX\n",  k_sidetone_lev_m);
	    fprintf(state_file, "k_sidetone_lev_p 0X%lX\n",  k_sidetone_lev_p);
	    fprintf(state_file, "k_sidetone_lev_m_l 0X%lX\n",  k_sidetone_lev_m_l);
	    fprintf(state_file, "k_sidetone_lev_p_l 0X%lX\n",  k_sidetone_lev_p_l);
	    fprintf(state_file, "k_xmt_power_m 0X%lX\n",  k_xmt_power_m);
	    fprintf(state_file, "k_xmt_power_p 0X%lX\n",  k_xmt_power_p);
	    fprintf(state_file, "k_xmt_power_m_l 0X%lX\n",  k_xmt_power_m_l);
	    fprintf(state_file, "k_xmt_power_p_l 0X%lX\n",  k_xmt_power_p_l);
	    fprintf(state_file, "k_do_quit 0X%lX\n",  k_do_quit);
	    fprintf(state_file, "k_warm_boot 0X%lX\n",  k_warm_boot);
	    fprintf(state_file, "k_v2f_tog_c 0X%lX\n",  k_v2f_tog_c);
	    fprintf(state_file, "k_v2f_tog_l 0X%lX\n",  k_v2f_tog_l);
	    fprintf(state_file, "k_v2f_res_c 0X%lX\n",  k_v2f_res_c);
	    fprintf(state_file, "k_v2f_res_l 0X%lX\n",  k_v2f_res_l);
	    fprintf(state_file, "k_v2f_lbox_c 0X%lX\n", k_v2f_lbox_c);
	    fprintf(state_file, "k_v2f_lbox_l 0X%lX\n", k_v2f_lbox_l);
	    fprintf(state_file, "k_band_50_a 0X%lX\n",  k_band_50_a);
	    fprintf(state_file, "k_band_144_b 0X%lX\n",  k_band_144_b);
	    fprintf(state_file, "k_band_220_c 0X%lX\n",  k_band_220_c);
	    fprintf(state_file, "k_band_432_d 0X%lX\n",  k_band_432_d);
	    fprintf(state_file, "k_band_902_9 0X%lX\n",  k_band_902_9);
	    fprintf(state_file, "k_band_1296_e 0X%lX\n",  k_band_1296_e);
	    fprintf(state_file, "k_band_2304_f 0X%lX\n",  k_band_2304_f);
	    fprintf(state_file, "k_band_3456_g 0X%lX\n",  k_band_3456_g);
	    fprintf(state_file, "k_band_5760_h 0X%lX\n",  k_band_5760_h);
	    fprintf(state_file, "k_band_10k_i 0X%lX\n",  k_band_10k_i);
	    fprintf(state_file, "k_band_24k_j 0X%lX\n",  k_band_24k_j);
	    fprintf(state_file, "k_band_47k_k 0X%lX\n",  k_band_47k_k);
	    fprintf(state_file, "k_band_75k_l 0X%lX\n",  k_band_75k_l);
	    fprintf(state_file, "k_band_119k_m 0X%lX\n",  k_band_119k_m);
	    fprintf(state_file, "k_band_145k_n 0X%lX\n",  k_band_145k_n);
	    fprintf(state_file, "k_audio_o 0X%lX\n",  k_audio_o);
	    fprintf(state_file, "k_band_memo_1 0X%lX\n",  k_band_memo_1);
	    fprintf(state_file, "k_band_memo_2 0X%lX\n",  k_band_memo_2);
	    fprintf(state_file, "k_band_memo_3 0X%lX\n",  k_band_memo_3);
	    fprintf(state_file, "k_band_memo_4 0X%lX\n",  k_band_memo_4);
	    fprintf(state_file, "k_band_memo_5 0X%lX\n",  k_band_memo_5);
	    fprintf(state_file, "k_memw_c 0X%lX\n",  k_memw_c);
	    fprintf(state_file, "k_memw_l 0X%lX\n",  k_memw_l);
	    fprintf(state_file, "k_memr_c 0X%lX\n",  k_memr_c);
	    fprintf(state_file, "k_memr_l 0X%lX\n",  k_memr_l);
	    fprintf(state_file, "k_mem2vfo_c 0X%lX\n",  k_mem2vfo_c);
	    fprintf(state_file, "k_mem2vfo_l 0X%lX\n",  k_mem2vfo_l);
	    fprintf(state_file, "k_memlist_c 0X%lX\n",  k_memlist_c);
	    fprintf(state_file, "k_memlist_l 0X%lX\n",  k_memlist_l);
	    fprintf(state_file, "ky_clock_m1s 0X%X\n", ky_clock_m1s);
	    fprintf(state_file, "ky_clock_p1s 0X%X\n", ky_clock_p1s);
	    fprintf(state_file, "ky_clock_m10ms 0X%X\n", ky_clock_m10ms);
	    fprintf(state_file, "ky_clock_p10ms 0X%X\n", ky_clock_p10ms);
	    fprintf(state_file, "ky_clock_mrate 0X%X\n", ky_clock_mrate);
	    fprintf(state_file, "ky_clock_prate 0X%X\n", ky_clock_prate);
	    fprintf(state_file, "ky_clock_resync 0X%X\n", ky_clock_resync);
	    fprintf(state_file, "ky_clock_exit 0X%X\n", ky_clock_exit);
	    }
	}

/* um_dir()  writes a text file directory of the DSP-10 memory files.   */
/* Improvements: Check for missing name, descr, show types of data,
 * show "Writing File   Wait...", arrange data to columns */
void um_dir(void)
	{
	FILE *umd_file, *mem_file;
	int i, sl;
	char ct[20];
	char m_name[40];
	char m_descr[60];
	static char entrysep[]=" \t,=\n\r";	/* Allow sp, tab, comma,
						   equal, lf, cr  */
	char *ci, *cd;
	char cline[260], cline2[260];
	int err_format, err_name, err_descr;
	struct lbox_info  *lb_ut6_ptr;

	/* Dynamic allocation of memory for box data */
	if ((lb_ut6_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 	lb_create(lb_ut6_ptr, "Memory Directory",
			 160, 175, 370, 250, 0, MODAL);
	lb_open(lb_ut6_ptr, 1);

	/* Fixed text in modal box:     */
	lb_line(lb_ut6_ptr, 0, BLANK, "", 0);
	lb_line(lb_ut6_ptr, 1, BOLD, "Writing UMEM_DIR.TXT", 0);
	lb_line(lb_ut6_ptr, 2, BOLD, "list of memories.", 0);
	lb_line(lb_ut6_ptr, 3, BLANK, "", 0);
	lb_line(lb_ut6_ptr, 4, BOLD, "   Please wait...", 0);
	lb_line(lb_ut6_ptr, 5, BLANK, "", 0);
	lb_update(lb_ut6_ptr);

	if (NULL==( umd_file=fopen("UMEM_DIR.TXT", "wt") ))
	    do_t_error(42, "E42 CFG File", 200);
	else
	    {
	    rewind(config_file);
	    fprintf(config_file, "DSP-10 Memory Channels\n");
	    }
	cc[0]='U';  cc[1]='M';  cc[2]='0';  cc[3]='0';  cc[4]='0';
	cc[6]=' ';  cc[7]=' ';  cc[8]='.';
	cc[9]='M';  cc[10]='E';  cc[11]='M';  cc[12]=0;
	for(i=1; i<=9999; i++)
	    {
	    if (i<10)
		{itoa(i, &ct[0], 10);  cc[5]=ct[0];}
	    else if (i<100)
		{itoa(i, &ct[0], 10);  cc[4]=ct[0]; cc[5]=ct[1];}
	    else if (i<1000)
		{itoa(i, &ct[0], 10);  cc[3]=ct[0]; cc[4]=ct[1]; cc[5]=ct[2];}
	    else
		{itoa(i, &ct[0], 10);  cc[2]=ct[0];  cc[3]=ct[1]; cc[4]=ct[2];
		 cc[5]=ct[3];}

	    /* See if the memory file exists, and if so read the name and
	     * description and write to the file.                    */
	    if(access(cc, 0)==0)
		{
		mem_file=fopen(cc, "rt");
		ci = strtok(cline, entrysep); /* Should be file ident line */
		if (strcmp(ci, "UHFAMEM2"))  err_format=0; else err_format=1;

		/*  Read the file body to get the memory parameters */
		while (fgets(cline, 256, mem_file))
		    {
		    strcpy(cline2, cline); /* Extra for string token first */
		    /* Get identifier and skip return:   */
		    if ((ci = strtok(cline, entrysep)) == NULL)  break;
		    /* Get a token to cd */
		    if ((cd = strtok(NULL, entrysep)) == NULL)
			strcpy(cd, "BLANK");
 /*		    if (!stricmp(ci, "cur_mem")) mn=atoi(cd);  */
		    if (!stricmp(ci, "mem_name"))
			{
			cd=strtok(cline2, "|");	/* Go to string */
			cd=strtok(NULL, "|");	/* Get string */
			strcpy(m_name, cd);
			if ((sl=strlen(m_name))>7) /* strlen not incl null */
			    {
			    m_name[7]=0;	/* Max len is 7 */
			    err_name=1;
			    }
			else
			    err_name=0;
			while(7-sl>0)		/* Make string 7 Char */
			    { strcat(m_name, " ");  sl++; }
			}
		    if (!stricmp(ci, "mem_descr"))
			{
			cd=strtok(cline2, "|");
			cd=strtok(NULL, "|");	/* Get string */
			strcpy(m_descr, cd);

			if ((sl=strlen(m_descr))>23)	/* strlen not incl null */
			    {
			    m_descr[23]=0;	/* Max len is 23 */
			    err_descr=1;
			    }
			else
			    err_descr=0;
			while(23-sl>0)		/* Make string 23 Char */
			    { strcat(m_descr, " ");  sl++; }
			}
		    }
		fprintf(umd_file, "%4.4d  %s  %s   %d %d %d\n",
		    i, m_name, m_descr, err_format, err_name, err_descr);
		fclose(mem_file);
		}
	    }
	fclose(umd_file);
	lb_close(lb_ut6_ptr, 1);
	}

/* to_vfo()  -  Trans from current frequency and mode to any VFO. This can
 * transfer from a memory or from another VFO.  The only variables transferred
 * are the transverter, frequency, mode and FM offset.  Ver 3.31
 */
void to_vfo(void)
	{
	static char *mode_str[]={"CW", "USB", "LSB", "FM", "LHL7", "PUA43",
				"EME2", "LTI", "AM"};
	struct edit_info   el1, el2;
	char eline1[5], eline2[5];
	int sl;
	unsigned int edresult;
	struct lbox_info  *lb_ut7_ptr;
	char  ln2[25], ln3[25];
	int e, nw_vfo;

	/* Dynamic allocation of memory for box data */
	if ((lb_ut7_ptr = (void *)calloc(1,sizeof(struct lbox_info))) == NULL)
		{
		do_t_error(54, "E54 Mem lbox", 50);
		return;
		}
 	lb_create(lb_ut7_ptr, "Settings -> VFO",
			 160, 175, 370, 290, 0, MODAL);
	lb_open(lb_ut7_ptr, 1);

	/* Fixed text in modal box, init blank lines:     */
	lb_line(lb_ut7_ptr, 0, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 2, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 3, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 4, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 5, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 6, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 7, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 8, BLANK, "", 0);
	lb_line(lb_ut7_ptr, 9, BOLD, "TEXT: Backspace & Type", 0);

 tovfo0:	/* An edit field for Memory Number, 1 to 9999 */
	sprintf(eline1,"");		/* The field to be edited */
	sl=strlen(eline1);
	lb_create_el(&el1, 2, 155, &eline1[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 1, EDIT, "New VFO A-N, 1-5, 9", &el1);
	lb_update(lb_ut7_ptr);

        /* ----------- Loop ---------------- */
	/* Following is a loop for handling user keyboard input */
 e0:	edresult=lb_edit_line(lb_ut7_ptr, &el1);
	if (edresult==ENTER) goto lb_ut7end;
	goto e0;

 lb_ut7end:
	e=(int)eline1[0];
	if      (e>=65 && e<=68)   nw_vfo=e-65; // A to D
	else if (e>=97 && e<=100)  nw_vfo=e-97; // a to d
	else if (e==57)            nw_vfo=4;    // 9
	else if (e>=69 && e<=78)   nw_vfo=e-64; // E to N
	else if (e>=101 && e<=110) nw_vfo=e-96; // e to n
	else if (e>=49 && e<=53)   nw_vfo=e-33; // 1 to 5
	else
	    {
	    lb_msg(375, 225, "Error", "Illegal VFO Designation",
		"No VFO will be loaded");
	    lb_close(lb_ut7_ptr, 1);
	    free(lb_ut7_ptr);		/* Close the dialog box */
	    return;
	    }

	if(vfo[nw_vfo].tvtr < 21)
	    {
	    sprintf(ln2, "Was: %s %.0lf", mode_str[vfo[nw_vfo].last_mode],
		vfo[nw_vfo].last_tfreq+tvtr[vfo[nw_vfo].tvtr].lo_freq);
	    lb_line(lb_ut7_ptr, 4, BOLD, ln2, 0);
	    sprintf(ln3, "Tvtr: %s ", tvtr[vfo[nw_vfo].tvtr].name);
	    lb_line(lb_ut7_ptr, 5, BOLD, ln3, 0);
	    }
	else
	    {
	    sprintf(ln2, "This VFO currently");
	    lb_line(lb_ut7_ptr, 4, BOLD, ln2, 0);
	    sprintf(ln3, "undefined.");
	    lb_line(lb_ut7_ptr, 5, BOLD, ln3, 0);
	    }

	sprintf(eline2,"");		/* The field to be edited */
	sl=strlen(eline2);
	lb_create_el(&el2, 2, 155, &eline2[0], ED_ACTIVE, 0, sl);
	lb_line(lb_ut7_ptr, 7, EDIT, "Hit Y to continue", &el2);
	lb_update(lb_ut7_ptr);

	edresult=lb_edit_line(lb_ut7_ptr, &el2);
	if(eline2[0]!='y' && eline2[0]!='Y')
	    {
	    lb_msg(375, 225, "Error", "Transfer to VFO Aborted",
		"No VFO will be loaded");
	    lb_close(lb_ut7_ptr, 1);
	    free(lb_ut7_ptr);		/* Close the dialog box */
	    return;
	    }

	lb_close(lb_ut7_ptr, 1);
	free(lb_ut7_ptr);		/* Close the dialog box */

	/* Now move current data to VFO, and activate.
	 * The magic code for not
	 * being used is 999 in vfo[i].tvtr or tvtr[i].lo_type.   V3.28
	 */
	vfo[nw_vfo].tvtr=band;
	vfo[nw_vfo].tr_offset=vfo[cur_vfo].tr_offset;
	vfo[nw_vfo].last_tfreq=(double)tfreqL;
	vfo[nw_vfo].last_mode=mode;
	/* Rev 3.31 */
	vfo[nw_vfo].t_fm_use=tone_fm_use;
	vfo[nw_vfo].t_fm_freq=tone_fm_freq;
	vfo[nw_vfo].t_fm_dur=tone_fm_dur;
	vfo[nw_vfo].t_fm_level=tone_fm_level;
	vfo[nw_vfo].par1=0;
	vfo[nw_vfo].par2=0;
	cur_vfo=nw_vfo;
	/* Set mode, tfreqL, select transverter */
	new_vfo(cur_vfo);
	}
