/*************************************************************************** 
 *
 *                       uhfa.c Version 3.50
 *
 *            Control Panel for DSP-10 2-meter Transciever
 *
 *
 * NOTICE: Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 *         2005  by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm  and,
 * http://www.ao.com/~gnome/
 *
 * Compatable with EZKIT-Lite program DSP3 version 0X13 and later.
 *
 * RSL = Bob Larkin  W7PUA   Initial write 17 May 96 
 * 29 Oct 96  Added audio FIR, warm boot.  RSL
 * 16 Nov 96  Added audio input and 8/16 bit data option. RSL
 * 18 Nov 96  Added normalization and recording to disk. RSL
 * 10 Dec 96  Reorganized keyboard to allow CW. RSL
 * 22 Dec 96  Added screen-save recording of screen image.  RSL
 *  2 Jan 97  Added transverters and configuration file.  RSL
 * 20 Jan 97  Added CW transmit functions.  RSL
 * 27 June 97 Revised to updated hardware: RF gains, T/R sequencing RSL
 * 28 Sept 97 Added control for DAC for carrier cancellation
 * 11 Mar 98  Revised to crystal filter hdw. Removed i-q corrections. RSL
 * Ver1.3:
 * 21 Sept 98 Added experimental versions of LHL and PUA43 Code estimators
 * 25 Nov  98 Revised to a 5 kHz ref freq for 1st LO (v1.31)
 * Ver1.40
 * 17 Jan  99 Full review.  Fixed error 112 problems. Ver 1.4 RSL
 * 17 Apr  99 Update of audio processor unique elements.
 * 29 Apr  99 Corrections & added binaural. Ver 1.41
 *  2 Aug  99 Startup detail, Fd corrected.  v1.5  RSL
 * 12 Sept 99 Put under GNU GPL, corrected startup error.
 *            Removed GIF for current distribution.  V1.6, RSL
 * 12 Jan  00 Added LHL, PUA codes, 2 sec clock timing for codes.
 *
 * The ASYNC routines are used to provide buffered serial communications
 * with the DSP.  These are freeware from
 *	David Kessner
 *	david@kessner.denver.co.us
 *	1135 Fairfax
 *	Denver CO  80220
 * Many thanks for use of these routines.
 *
 ************************************************************************/

/* 1.62 Rearranged LHL Code. 1.63 Added CW key, 1.64 Space in LHL single tone
 * 1.67 Repeater Offsets, Programmable colors, CW text moved up.
 * 1.70 Added EME-2 mode
 * 1.71 Fixed clock x2 problem and Error 201 underflow problem.
 * 1.72 Fix lockup in PUA43
 * 1.80 Corrected timing in LHL7
 * 1.90 Added transmit, data box & tidyed up PUA43. Needs dsp v1.9
 *      to have correct DSP timing for all triggered modes.
 * 1.91 Update EME data in transmit for EME2, add &=1sec in CW, 1 Hz RIT
 * 1.92 Changed to dR/dt Doppler calculation
 * 1.93D Data Dump for EME2
 * 1.94 Less clock checking for slow computers 16 Aug 00 RSL
 * 1.95 Tukey25 window, Variable BW FFT, AutoNotch. ... 18 Oct 00 RSL
 * 1.96 Integrated var BW into EME2. Added eme doppler to SSB, CW. 30 Oct00 RSL
 * 1.97 Fixed PUA43 for narrower fft BW; Added audio equalizers,
 *      filter design 12 Nov 00 RSL
 * 1.98 Added long term integration to USB 9 Dec 00 RSL
 * 1.99 Converted to LTI, Added Filter 8 design. 22 Jan 00. RSL
 * 2.00 Bugs fixed, error messages.  9 Feb 01  RSL
 * 2.01 NB to PUA43, Top5 List on PUA43 6 May 01  RSL
 * 2.02 Worked over comments. Removed extra variables.  8 May 01 RSL
 * 2.03 Added noise generator on transmit for signal simulation.
 *      Fixed problems with audio processor.
 *      Increased update rate on Sun, cleared lines for LTI transmit,
 *      Added EME2, LHL7, PUA43, LTI help.   11 June 01 RSL
 * 2.04 Added PUA43 char clear and ezkit_dfreq correction. 30 July 01 RSL
 * 2.05 Corrected char clear problem in PUA43. 1 Aug 01
 * 2.06 Added beacon control for all modes. 12 Aug 01
 * 2.08 Update .cfg w/o shutdown, A: drive tests
 * 2.09 Removed pua43 test patches from 2.08 18 Mar 02
 * 2.10 Added ULT...DAT files for yellow trace save  4 June 02
 * 2.11 Added tick marks to waterfall for ULT saves, clear and straighten 16 Jn
 * 2.12 Corrected offset on ULT....DAT files 20 Jun 02
 * 2.14 Removed SCRL-F2 8/16 bit selection. LTI and EME-2 fixed with 16.
 *      Screen width always full except LTI & EME-2 where it is half.
 *      Added markers for CW, LTI, LHL-7, PUA43, EME-2. Added paths
 *      for data and screen files. Fixed shut-down in FM, removed warm boot.
 *	Removed .LTD file capability (Ctrl-Q).  RSL 12 Jan 03
 * 2.15 Fixed floppy door for .DAT, Added Scrl-F2 Beacon params  RSL 15Jan03
 * 2.16 Added microphone equalizer. RSL 17 Jan 02
 * 2.17 Added data_omit_tr to cfg to prevent "white lines." RSL 19 Jan 03
 *      Added .BMP somewhere! Not noted above.
 * 2.98 3.00 same - Fixed white and red marks on Ctrl-W and E.
 * 3.02 Experimental for PTT & floppy door problems. Added error_on_tr to
 *      initialize() to stop PLL error.
 * 3.04 Issued for general use. Removed experimental stuff.
 * 3.05 Reinstalled requirement for b_transmit_sec to be > 3 for transmit.
 *	Allows 0 value for no transmit at all. RSL 8 Apr 03
 * 3.06 Changed FM S-meter resolution to .001 dB & added Scrl-F3/F4 ave to FM
 *      S-meter readings.  RSL  21 June 03
 * 3.07 Added DOS-visit ALT-% and dos-exit dialog box. RSL 3 Jan 04
 * 3.08 Fixed PUA43 non-random bug caught by LHL.  RSL 12 Jan 04
 * 3.10 Added "data lost" marks on right edge. Source-code notes added.
 *      RSL  21 Jan 2004
 * 3.11 Added indicators for transmit audio clipping. Req V22 of DSP 2Feb04
 * 3.12 FM Freq, added FM dev control, corrected S-meter, Req DSP v24. 15Feb04
 * 3.13, 3.14 Fixed errors with serial from DSP. RSL 23 Feb 04
 * 3.15 Added PUA43 Tuning function RSL 8 Mar 04
 * 3.17 Added '*' to PUA43 for tuning data. RSL 11 June 04
 * 3.18 Fixed E1 on last char==Esc Char. Also lost data on Check==Esc Char
 *      RSL 21 June 04 
 * 3.19 Restored underline for PUA43 '*', fixed E1 program hang, RSL 23 Jun04
 * 3.20 Added pua43_tick and fixed '*' for 14 char PUA43 RSL 28 Jul 04
 * 3.21 Fixed LSB clip display RSL 6 Sept 04
 * 3.23 Put keyboard def into .CFG. See u_cmd.c.  RSL 22 Sept 04
 * 3.24 Added clock commands as ky_ variables, same as 3.23.  RSL 27 Sept 04
 * 3.25 Added control for V2F (Voltage to Frequency) feature. RSL 18 Oct 04
 * 3.26 Added base freq control to V2F. RSL 26 Oct 04
 * 3.27 Added tones to FM transmit, signal capability to v2f. This is all
 *      compatible with ver 2A of the DSP, that also fixes load problems.
 *      RSL 26 Nov 04.
 * 3.28 Corrected timing of PUA43 to eliminate double frequency problem.
 *	RSL 4 Jan 05
 * 3.29 More of 3.28!  RSL 9 Jan 05
 * 3.30 Fixed 1200 & 2400 widths for PUA43, broken in 3.29. RSL 10 Jan 05
 * 3.31 Added memory channel read, write, listing and transfer to VFO
 *	RSL 5 Feb 05
 * 3.32 Fixed bug that was keeping mode and freq from loading RSL 7 Feb 05
 * 3.33 Added CTCSS and beep for FM plus end-beep for SSB. Modal box for
 *      this in util.c. RSL 22 Feb 05
 * 3.34 Added control for side tone control on SSB end-beep.  RSL 27 Feb 05
 * 3.37 Added delays for T2R for audio processor. RSL 29 Sept 05
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <float.h>
#include <graphics.h>
#include <string.h>
#include <dos.h>
#include <bios.h>
#include <time.h>
#include <alloc.h>
#include <signal.h>
#include "util.h"
#include "ustate.h"
/* include "uhf_kbd.h" */
#include "lbox.h"
#include "u_code.h"
#include "async.h"
#include "moonsun.h"
#include "umatrix.h"
#include "ufilter.h"
#include "u_cmd.h"
#include "uhfa.h"
#include "gifsave.h"

/* KGAIN makes the S-Meter for non-FM correct and reflects the nominal gain
 * of the transceiver. Detailed corrections come from the >CFG variable
 * gain_adj and for external preamps/transverters from tvtr[ ].gain_db.
 */
#define KGAIN -162.8

/* LOG_OFFSET is sent to dsp to offset log data before
 * conversion to 8 bits. Every 4096 is 10 dB.
 */
#define LOG_OFFSET 12288
#define LOG_OFFSET_D 12288.0

#define UDUMP 0

 void update_data(void);
 void prep_display(void);
 void print_stars(void);
 void draw_grid(void);
 void sa_graph(void);
 void mark_frequency(int);
 void show_tr(int, int);		/* tr and color Rev 3.39 */
 void t_r_toggle(void);
 void freq_box(void);
 void new_freq(void);
 void set_afgain(int);
 void set_rfgain(void);
 void set_mike_gain(void);
 void set_fm_squelch(void);
 void set_cwoffset(void);
 void set_eme_doppler(void);
 void show_er(void);
 void set_fsk(int);
 void set_xmit_power(void);
 void set_contrast(void);
 void set_brightness(void);
 void set_window(void);
 void set_if_bw(int);
 void set_affilt_sw(void);
 void set_affilt_select(void);
 void set_delay_right(int);
 void filter_design(void);
 void select_denoise(void);
 void set_denoise(void);
 void annotate_af_filt(void);
 void set_dot(void);
 void set_display_width(void);
 void set_disp_offset(int);
 void set_db_div(int);
 void set_noncohave(int);
 void set_mode(int);
 void set_code_nx2(void);
 void set_8_16(int);	/* 0=set 8 bit, 1=toggle 8/16, 9=var bits_8_16 */
 void set_data_save(void);
 void v2f(int);
 void set_ttn(int);
 void disp_v2f(void);
 void disp_toggle(void);
 void normalize_plot(int);	/* 0=clear, 1=15 second, 2=10 min */
 void normalize(int);		/* Do the normalization calcs, 256 or 512 */
 void lt_normalize(void);
 void dofreqplot(int);
 void fsk2sec(void);		/* Sends 2 sec fsk tone */
 void set_clock(int);		/* Manual set of DSP clock (NOT PC clock) */
 long int  update_clock(int);	/* Set/get start of next second */
 void diagnostics(int);
 void freq_data(int, long int, long int, long int, long int, long int,
               unsigned int);
 void initialize(int);
 void presets(void);
 void do_transmit(void);
 void eme_timing(void);		/* Poles eme_state after times all */
 void lhl7_timing(void);
 void do_auto_disp(int k);
 int  read_config_file(void);
 void new_vfo(int);
 void new_band(int);
 void rit_toggle(void);
 void display_summary(char*, char*);
 void print_time(void);
 void send_shift_data(unsigned long);  /* Load shift regs and synth from dsp */
 int  near_int(double);
 int  get_kbd_int(void);
 int  keyident(void);
 void get_mem(void);
 void set_agc(int);
 void set_cw_speed(void);
 void draw_panel(void);
 void draw_s_meter(void);
 void draw_clip(void);
 double p2db(double);
 double upow10(double);
 int i2mfp(int i);
 void get_dsp_ptrs(void);
 void get_dsp_mem(int what, int* tbl, int item, int offset);
 void set_dsp_mem(int what, int adr, long int data);
 int  get_status_bytes(int type, double timeout);
 double update_moon(struct tm *ptm, int do_now);
 void update_sun(struct tm *ptm, int do_now);
 void set_cur_loc(int);
 void set_bins(void);
 void do_t_error(int, char*, int);

 void xx(int, int, int, int, int);

/******************   Global Variables ******************/
char far	*big_area_ptr1;
char far	*big_area_ptr2;
char		cc[256];		/* General purpose - use in any fcn */
char		version[10];		/* String, like "2.12" */
int		contrast=18;
double		dis_gain=0.0525;
int		bright=64;
double		dis_offset=0.0;
int		power_exp=10;	/* Power of 2 that dsp data is divided by
				   before integration; must be (0, 255).  */
int		scroll_line=0;
/* T/R is controlled either by PTT on rf board or by 'Home'. The following
   three variables allow either but not both at any time.   */
unsigned int	transmit=0;		/* 1=transmit, 0=receive */
unsigned int	ptt=0;			/* PTT: 1=transmit, 0=receive */
unsigned int	home_tr=0;		/* 'Home' key: 1=transmit, 0=receive */
unsigned int	cw_on=0;		/* 0=cw off, 1=cw on */
int	rfgain=0;			/* (0, 6) for 0, -6, ... -36 dB */
int	xmit_power=94;
int	mike_gain=95;
int	mike_mult=95;			/* Multiplier for mike gain */
int	mgain_shift =0X0080;		/* Bits 6,7 are amt of shift, set==2 */
int	afgain=25;
int	deldb=2;			/* For up/down cursor keys */
int	deldb_sh=10;			/*     and when shift key is down */
int	mute=FALSE;
int	fm_afgain=25;
int	bufcount=0;			/* CW, LHL7, etc, Char to be sent */
int	cwoffset=600;
int     fsk=0;				/* Xmtr tone LHL7, PUA43, EME2, LTI */
int	sidetone_lev=80;
int	fm_squelch=66;
double  fm_dev=5.00;			/* kHz, 0 to pk */
int	noncohave=2;
int	d_noncoh=0;			/* Communicate change -1 or +1 */
double	fft_per[4]={1.5, 3.0, 6.0, 8.0};  /* Equiv FFT per data set from DSP */

/* Relative noise bandwidths of the windowing functions */
double  noise_bw[4]={1.0, 1.10, 1.36, 2.0};

int     win_offset=0;			/* 3 dB per offset step, DSP spectrm*/
int	do_window=HAMMING;
int	do_window_save=HAMMING;		/* When changing modes */
/* fft_bw=3 is 4800 with pairs of bins added together to make 256 big bins */
int	fft_bw=2;			/* 0=1200, 1=2400, 2=4800 3=4800*2 */
int	fft_bw_save=2;
int	if_bw_save=1;			/* When changing modes */
/* Bit 10 of the fft_fcn is a software trigger of the FFT data collection.
 * If it is set to 1 and the "F" command is sent to the DSP it will trigger
 * the data collection. It is reset by the DSP and doesn't need to be set
 * explicitly to 0 by the control program. Software trigger is in effect
 * when bit 9 is 1.
 * Bit 8 = 1 forces a cancellation of current FFT data collection, and can
 * be combined with bits 9 and 10 both = 1 to stop current collection and
 * software trigger a new collection.
 */
unsigned int    fft_fcn=0X0044;		/* Free running, 4800 BW, 1024 point */

double  win_adj_loss=0.0;		/* Correction for window loss */
double  win_adj_residual=0.0;		/* After adjusing pwr_exp for win fcn*/
double  win_adj=0.0;			/* Applied to display and S-meter */
double	knoise=1.0;			/* Spectral correction for BW */
double  kfilt=1.0;			/* Gain correction in FIR's */
int     do_af_filt=FALSE;		/* 0=no filt, 1=af filt on */
int	af_coef=0;			/* 0=filt[0],... */
int     do_LMS=0;			/* 0=no LMS, 1=denoise, -1=auto-notch*/
int	denoise_level=40;
int	denoise_decay=70;
int	denoise_del_len=1;		/* 61 long */
int	autonotch_level=50;
int	delay_state=0;			/* 0=No binaural delay */
int	delay_right=9;			/* Power of 2 x 20.8usec on right ch */
int	agc_decay=5;
int	last_agc=5;			/* Used at init & by set_agc() */
int	dot_type=2;
int	display_width=FULL_FREQ;	/* Either 0 or 1 */
double	freq_low=200.0;			/* Lowest frequency for marker */
int	bin_low;			/* Derived from freq_low */
int	db_div_set=0;			/* 0=10db/div, 1=5db, 2=2db, 3=1db */
double	kdb_div=3.2;			/* 80 div/256 values */
double	koffset=0.0;			/* Display offset in dB */
/* mode is the actual transmitted mode. dsp_usb is the dsp I-F sideband select.
 * For 2-meters, this is the reverse of the actual s.b. because of the 
 * high side 2nd LO. But, this can again be reversed by a transverter and
 * for sure is the other way for audio input.  */
int	mode=USB;
int	dsp_usb=FALSE;
/* dsp_dph is the 15+/- kHz 3rd LO phase. Global to allow modification for
 * SSB end beep.     */
unsigned int dsp_dph=20480;		/* 15 kHz  */
int     mode_mask=255;			/* Sums of bits:
					 * 1=CW, 2=USB, 4=LSB, 8=FM, 16=LHL7
					 * 32=PUA43 64=EME2 128=LTI */
int	narrow=1;
int	binaural=1;
int	iq=1;
int	bits_8_16=0;			/* Start 8 bit */
					/* Next var allows for operating with
					   non-standard hardware. Set TRUE only
					   if std hardware available */
int	hardware=TRUE;			/* RF Hardware present */
int	hardware_ptt=FALSE;		/* Use RI of Serial Port for PTT */
int	normalize_on=FALSE;		/* 0=off, 1=on, >1 is countdown */
int	auto_disp_on=FALSE;		/* 0=off, 1=on  */
int	rit=0;
int	rit_save=0;
int	rit_on=FALSE;
int	v_m=1;				/* 1->Using VFOs, 0->memories */
int	cur_vfo=1;
int	band=1;				/* Band 1 = 144 MHz */
int	cur_mem=-1;			/* No memory, they are 1,9999 */
long	tfreqL=144200000;
long	delf=50;			/* For L/R cursor keys */
long	delf_sh=1000;			/* For Shift L/R cursor keys */
int     deflt_drv;			/* 0=A: 1=B: 999=HD */

/* The following are personality variables that are set through either the
 * default value, or if present, by the value in the .CFG file.
 * They cannot be set by direct keyboard commands */

/* COLORS:   */
struct  palettetype  savpalette;
struct	palettetype  upalette=
			{16,0,16,34,3,25,41,21,29,61,60,52,46,38,54,55,63};
int c_background=4;
int c_outline=11;
int c_outline_bold=15;
int c_outline_bright=8;
int c_text=11;
int c_text_bold=15;
int c_text_bright=13;
int c_text_note=9;
int c_plot_bold=15;
int c_plot_bright=13;
int c_plot_marker=9;
int c_bar_dark=1;
int c_bar_low=15;
int c_bar_medium=2;
int c_bar_high=9;

int kbd_alt1=FALSE;			/* 10 Hz/100Hz Steps */
int kbd_alt2=FALSE;			/* Increased Scrl locks */
int kbd_alt3=FALSE;			/* RF Gain to Scrl F9, F10 */

int	startup_detail=0;		/* 0=No detail, 1=Show diagnostics
					   Set =1 in case .cfg missing */
int	waterfall=1;			/* 0=Fancy freq dial, 1=Waterfall */
int	time_type=0;			/* 0=GMT, 1=local */
char	time_str[10]="GMT";		/* Change to "PST" etc from .CFG file*/
int	show_secs=FALSE;		/* Seconds on waterfall time */
char	call_str[10]="   ";		/* Enter from .CFG file */
char	file_ident[2]="U";		/* Prefix letter for output files */
char    fscreen_path[52]="";		/* For screen saves */
char    fdata_path[52]="";		/* For  data files */
unsigned char code_id[10]="DE ?";       /* ID when sending fsk code */
int	code_nx2=1;			/* Number of 2 sec per tone */
int	d_nx2=0;			/* Communications for code_nx2 change*/
int	ms_available=TRUE;
double  millisec;			/* Corrected count of millisec */
double  millisec_offset=0.0;		/* Used in clock setting to line up
					   ticks */
int	tick_time=10;			/* Audable tick for time fcns */
int	pua43_tick=1;			/* Do tick for each update */
double  clock_cor_factor=1.000000000;	/* Relative speed of PC clock */
long int  dsp_clock_base=0L;		/* Offset to make millisec valid */
long int  cl_ms;			/* Results from last clock() */
int	clock_been_set=FALSE;		/* Error for codes, if not set */
int	quit_save_state=TRUE;		/* TRUE=save state on quit() */
/* memch_xxx defines the classes of items to be saved in a "memory channel" */
int	memch_stn=0;    // Station definition, such as code_id, call_str
int	memch_fr_md=1;  // Special modes, weak signal modes
int	memch_radio=0;   // Transceiver operation, special modes
int	memch_bcn=0;    // Beacon operation
int	memch_tvtr=0;   // Transverter definitions
int	memch_vfo=0;    // VFO Definitions
int	memch_disp=0;   // Display character, except colors
int	memch_fil=0;    // Filters, equalizers
int	memch_sys=0;    // System parameters: startup_detail, file paths,etc
int	memch_color=0;  // Display colors
int	memch_kbd=0;    // Keyboard definitions
int	memch_hdw=0;    // Hardware, such as ezkit_dfreq, and pc clock

char	mem_name[8]="";		// Short name string ptr
char 	mem_descr[24]="";	// Description string ptr
int	mem_incl_fr_md=1; 	// Carries need for updating freq and mode

int	beep_on_error=TRUE;		/* When error codes are displayed */
int	error_on_tr=3;			/* Prevent error after T/R change */
					/* Don't know why 3 is needed! */
int	data_omit_tr=3;			/* Reset value (CFG) for error_on_tr*/
unsigned int errcode;
struct	transverter_info tvtr[21];
struct	vfo_info vfo[22];

/* Following 3 variables control the synthesizer, dsp 2nd lo and hardware
   setting of the shift registers.  This needs to be collected into
   a structure for neatness, some day!   */
unsigned int	dsp_deltap_lo=0XA6;	/* Phase incr dsp, low 8 bits */
unsigned int	dsp_deltap_hi=0X4A;	/* Phase incr dsp, high 8 bits */
/* shift_reg:  Bit 0=CW on, 1=trans, 2=recv, 3=PA enable, 4=Ant relay transmit,
   High 8 bits come from rfg[i].gainbyte    */
unsigned int	shift_reg=0X04;		/* Low 8 bit hardware control */

int	dsp_status[18];
int	dsp_xmit_status[10];
double	ave_spectral_pwr=0.0;		/* Ave if color agc on */
double	ref_offset=1.0;			/* Corr factor, freq ref, frm .cfg */
double	ezkit_dfreq=0.0;		/* Corr factor, ezkit osc, frm .cfg */
double	dff=1.0;			/* Ratio, derived from ezkit_dfreq */
int	integrate_count=0;		/* Varies from 0 to noncohave  */
int	contrast_line_count=0;		/* Display contrast line if >0 */
int	cw_speed=10;			/* Index of speed (0, 25) */
int	cw_longkey=10;			/* '~' does key down this many sec */
int	display_error_count=0;
int	data_lost=FALSE;		/* Flag to display event on w'fall */
int	diag_mode=2;			/* 0=none, 1=text, 2=bargraphs
					   3=Moon, 4=Sun data */
int	do_data_save=0;			/* 0=no save to file, 1=save */
int	text_area=FALSE;		/* TRUE if y=(0,32) for text */
unsigned int	com_port;
int	lpt_port=0;			/* Zero means 'not used'    */
					/* Bit 0 - Time ticks       */
					/* Bit 1 - T/R Ctrl Ant Ry  1=Xmit */
					/* Bit 2 - T/R Ctrl Amp     1=Xmit */
					/* Bit 3 - T/R Ctrl RF On   1=Xmit */
					/* Bit 4 - Noise Source Ctrl Future  */
int	lpt_dat=0;			/* Data sent on LPT */
unsigned int	ntau=200;		/* Count for 20 usec delay */
int	incount;
int far bar_color[]  ={15, 15, 15, 15, 15, 2, 2, 2, 2, 2, 9, 9, 9, 9, 9};
int far bar_color_o[]={15, 15, 15, 15, 15, 2, 2, 2, 2, 2, 2, 2, 2, 2, 9};
double far fgain[101];			/* 'Exact' gain value */
int far	fgainl[101];			/* Low 8 bits of gain value */
int far	fgainh[101];			/*   and high 8 bits */
int far	raw_data[515];
double*	dnorm;				/* [515] calloc() */
double* lt_dnorm;			/* Used in EME, etc [256] calloc() */

double min_lt_dd=1.6;			/* Yellow trace scale limit */

int    scsave=0;			/* Minutes between screen saves */
/* Rev 3.50 int    scs_key=4891;  */
int    use_scs_box=TRUE;		/* if TRUE, comment box for sc saves*/

/*  VVVVV No MORE */
/* int    lterm_int=FALSE; */			/* LTI available */

double lterm_int_flow=200.0;		/* Start freq for yellow trace */
double lterm_int_fhigh=700.0;		/* End freq, for endless integratn */
double lterm_int_center=500.0;		/* Ctr of 21 freq in "Alt A" box */
double lti_frandom=0.0;			/* Current random offset for LTI */
double lti_freq_range=0.0;		/* Total spread of the random freq */

double	normalize_num;
double  eme2_te=290.0;			/* Effective Noise Temp Kelvins */
double  lhl7_state=0.0;			/* 0=waiting for trig, 1=triggered */

double*	pwrdat;				/* An array of 515 for power from
						dsp fft.  Uses calloc() */
int	pua14_28=1;			/* 0=14 char, 1=28 char */
int	code_min=0;			/* Minute of day (0, 1439) */
int*	pua43_rnd;			/* Array of 1440 rand num (key) */
double	pua43_noisedecay=0.999;
int	pua43_dorandom=TRUE;
char	pua43_xstring[35]="                            "; /* For PUA43 (28) */
/* The following two parameters specify the PUA43 frequency arrangement
 * as a function of the fft_bw parameter.     */
double	df[4]={2.34375, 4.6875, 9.375, 18.75};	/* Bin spacing in Hz */
int	b0[4]={192, 96, 48, 24};	/* Bins of offset to first freq, 450 */
int	b_id43[4]={160, 80, 40, 20};	/* Bins for PUA43 ID/Tuning */
double* pua43_tune;			/* Array allocated in pua43_update() */
int ptune0[]={96,   48, 24, 12};	/* Bottom bin */
int ptune1[]={224, 112, 56, 28};	/* Top bin */
int ptunec[]={160,  80, 40, 20}; 	/* Center bin */
double pua43_tmin, pua43_tmax;

double  pave;
double	paveave=0.0;			/* Rng ave of pave in update_disp() */
double*	lt_pwrdat;			/* 'Perpetual' power accumulation
					   [256] from calloc() */
double *pd16;				/* Power for 16 bit data */
double	lt_max, lt_min;
double* power10;			/* 256 entry table for converting log
						to power. Uses calloc() */
int	dly_ant2amp=50;			/* T/R Sequencing delays */
int	dly_amp2xmit=75;
int	dly_xmit2amp=25;
int	dly_amp2ant=25;

FILE*	data_file;
FILE*   lt_data_file;
FILE*	config_file;
FILE*   license_file;
FILE*	lt_pave_file;

/* A table of pointers for important locations in DSP memory.
 * Table is downloaded from DSP at initialization by
 * void get_dsp_ptrs();
 * For the ADSP2181 without external memory, these are (0, 16383).
 * The relevant dsp symbolic addresses are:
 *   dsp_ptr[0]   ham_win
 *   dsp_ptr[1]   bh4_win
 *   dsp_ptr[2]   tukey_win
 *   dsp_ptr[3]   fir1_coefs
 *   dsp_ptr[4]   fir1a_coefs
 *   dsp_ptr[5]   wfir2_coefs
 *   dsp_ptr[6]   fir2_coefs
 *   dsp_ptr[7]   fir3_coefs0
 *   dsp_ptr[8]   fir3_coefs1
 *   dsp_ptr[9]   fir3_coefs1
 *   dsp_ptr[10]  fir3_coefs1
 *   dsp_ptr[11]  fir3_coefs1
 *   dsp_ptr[12]  fir3_coefs1
 *   dsp_ptr[13]  fir3_coefs1
 *   dsp_ptr[14]  fir3_coefs1
 *   dsp_ptr[15]  fir4_coefs
 *   dsp_ptr[16]  fir5_coefs
 *   dsp_ptr[17]  fir6_coefs
 *   dsp_ptr[18]  fir7_coefs
 *   dsp_ptr[19]  fir8_coefs
 *   dsp_ptr[20]  NU
 *   dsp_ptr[21]  NU
 *   dsp_ptr[22]  fir1200_coefs
 *   dsp_ptr[23]  fir2400_coefs
 *   dsp_ptr[24]  hilbert1_coeff
 *   dsp_ptr[25]  hilbert3_coeff
 *   dsp_ptr[26]  NU
 *   dsp_ptr[27]  NU
 *   dsp_ptr[28]  NU
 *   dsp_ptr[29]  NU
 *   dsp_ptr[30]  xmit_status_buf
 *   dsp_ptr[31]  cmd_buf
 *   dsp_ptr[32]  esc_a
 *   dsp_ptr[33]  filter_data
 *   dsp_ptr[34]  buf_len_table
 *   dsp_ptr[35]  NU
 *   dsp_ptr[36]  filter_id
 *   dsp_ptr[37]  0X123456 Check number
 */
int far dsp_ptr[38];
int dsp_data_index=0;			/* Keeps track of data from DSP */
int table_adr=0;			/* DSP address of pointer table, or
					 * =0 if error in getting table */

/* For signal simulation capability */
double  xsnr, xfade_rate;
int     xnoise=0;
int	xfade=0;
int	xnoise_disp_toggle=0;		/* Flashes 'N' as warning */

/* "Beacon" mode variables:  */
int b_offset_sec=0;
int b_transmit_sec=10;
int b_receive_sec=20;
int beacon=FALSE;
int lt_b_save=FALSE;			/* F=no save on beacon Xmit, T=save */
int lt_b_save_bins=30;			/* Number of points above/below saved*/
int lt_b_clear=FALSE;			/* F=no clear, T=clear on save */
int b_active1=0;			/* Period when beacon is active for */
int b_active2=0;			/*   trans, save, clear. Each hour */
int b_repeat60=FALSE;			/* F=30 min repeat, T=60 min repeat */

/* Variables to support the noise measurements in the FM mode. The DSP does
 * an RMS calculation on the full 12 kHz BW and sends results every 0.6 sec.
 * These are accumulated further by SpecAve update times, and then displayed.
 */
int	np_record=FALSE;
double	np1=0.0;
double	np2=0.0;
int	np_on=FALSE;			/* 0=OFF, 1=ON */
int	np_state=0;			/* 0 or 1 for two measurements */
int	np_delay=20;			/* Milliseconds of delay for relays */

/* Variables for the v2f noise and signal measuring. */
int	v2f_on=FALSE;
int	v2f_noise=1;			/* 1=using noise, 0=using signal */
int	v2f_pkave=0;			/* 0=ave sig, 1=peak sig */
double	v2f_tc=0.5;			/* Seconds */
double	v2f_gain=250.000;		/* Hz per dB */
double	v2f_basef=450.000;		/* Frequency for reset */
double	v2f_base_db=0.0;
int	v2f_bin_low=32;			/* Signal measurement 0,511 */
int	v2f_bin_high=65;		/* Signal measurement */
double	v2f_freq_low=295.312;		/* Signal measurement 0.0,4800.0 */
double	v2f_freq_high=604.688;		/* Signal measurement */
int	v2f_af_level=60;		/* AF level (0,99) */

int	tone_fm_use=0;			/* Transmit FM CTCSS or beep */
double	tone_fm_freq=100.0;
int	tone_fm_dur=9999;
int	tone_fm_level=80;
int	beep_use=2;			/* Tmt SSB beep, Bit 1=sidetone 0=On */
double	beep_freq=1000.0;
int	beep_dur=250;
int	beep_level=90;			/* Level rel to CW pwr 100=full CW */

int	ttn_control=4;			/* Mike audio, no tones */
double	ttn_freq1=100.0;
double	ttn_freq2=1700.0;
int	ttn_level1=70;
int	ttn_level2=0;
/* DTMF is available in FM transmit. The level is in dB (0,100) and the
 * time duration, dtmf_length, is in milliseconds.  This is only available if
 * continuous tone is not being used. We only have 2 tones! See tone_fm_use */
int	dtmf_level=82;
int	dtmf_length=100;

/* The following variables and the function pe() are debugging tools */
double dt1=1.0;
double dt2=2.0;
double dt3=3.0;
double dt4=4.0;
double dt5=5.0;
void pe(int);

struct rf_gn  rfg[] = {0.00,  0x88, 100,
			-6.0,  0x48, 94,
			-12.0, 0x28, 88,
			-18.0, 0x18, 82,
			-24.0, 0x14, 76,
			-30.0, 0x12, 70,
			-36.0, 0x11, 64,
			-99.0, 0x00, 00};
double	gain_adj=0.0;			/* dB factor to correct pwr disp */
/* Values for s-meter bar graph. First is for -140 dBm. Last is -27dBm */
int far smbar[114]={1, 1, 1, 1, 1, 1,			/* S1 */
		2, 2, 2, 2, 2, 2,			/* S2 */
		4, 4, 4, 4, 4, 4,			/* S3 */
		8, 8, 8, 8, 8, 8,
		16, 16, 16, 16, 16, 16,
		32, 32, 32, 32, 32, 32,
		64, 64, 64, 64, 64, 64,
		128, 128, 128, 128, 128, 128,
		256, 256, 256, 256, 256, 256,		/* S9 */
		512, 512, 512, 512, 512, 512, 512, 512, 512, 512,
		1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 1024, 
		2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048, 2048,
		4096, 4096, 4096, 4096, 4096, 4096, 4096, 4096, 4096, 4096,
		8192, 8192, 8192, 8192, 8192, 8192, 8192, 8192, 8192, 8192,
		16384, 16384, 16384, 16384, 16384,
		16384, 16384, 16384, 16384, 16384};

int	screen_save=TRUE;   /* FALSE; */

/* CW/LTI messages. Each a maximum of 49 characters. */
char* cwmbuf;				/* calloc to 500 total */

/* For moonsun we need the data on the home location and dx.  To allow multiple
 * dx stations there is provision for loc[0] which is always used
 * and up to 9 'dx' locations, one of which could be loc[0] for self echoes.
 * These are normally defined in the .cfg file.
 */
struct	location loc[10];
int	cur_loc=0;
struct	eenv	 cur_env;		/* Cur enviornment:day, nutation, etc*/
int	ut_available=0;			/* 0 if not avail, else 1 */
double	ut=0.0;				/* UT for moonsun calculations */
double  dut1=0.0;			/* UT1-UTC if known, else 0.0 */
struct  tm       tdt;			/* TDT for ephereris calculations */
double	f_eme=144200000.0;		/* Dopplr Fr (audio processing only) */

struct  lhl  lc0;			/* Info on LHL codes from u_code */
int	lhl_bins=1;			/* 1, 3, 5, 7 or 9 */
int	lhl_peak_ave=1;			/* Default to ave */
int	lhl7_2line=0;			/* 0 is 1 big line, 1 is two small */
int	sync_flag=FALSE;		/* For time aligning displays */

/* For EME2 and LTI modes we need:     */
int     lt_i1=75;
int     lt_i2=115;
int     lt_i3=125;
int     lt_i4=135;
int     lt_i5=175;
long int    eme_pulse_count=0;		/* The number of 5 second periods */
double      eme_freq_range=200.0;	/* Total random range, Hz */
double      eme_pwr_sum[21];		/* Long term pwr +/- 10 bins */
int         eme_freq_ctrl=1;		/* 0=fixed, 1=random */
long int    eme_millisec_base=0;	/* Starts at 0, 5000, 1000, etc */
int         eme_state=-1;		/* -1=standby
					    0=transmit, still processing data
					    1=transmit, display complete
					    2=switch to rcv, waiting for 2.5sec
					    3=trigger rcv in dsp, get data
					    4=special case- first xmit */
int         eme_record=FALSE;		/* FALSE=off, TRUE=21 freq / 5 sec */
double      code_nb_ratio=1.2589;	/* To throw out impulse noise */
int         dly_emet2r=400;		/* Millisec */
int         eme2_format=7;		/* Bit  0: 1=doppler correct
					 *      1: 1=use random freq
					 *      2: 1=use auto noise blanker */
/* Note: in the interest of  confusion, eme_doppler is used for all modes
 * except EME2.  */
int         eme_doppler=FALSE;		/* FALSE=no correct, TRUE=offset rcvr*/
double	    eme_dop1, eme_dop2;		/* Due to E rotation, E-M move */
double	    eme_dop3, eme_dop4;		/* Due to RA, Dec move */
long int    random_freqL=0L;		/* Used in codes to suppress birdies */

unsigned int mat_err;			/* Details on matrix errors go here */

double  flow=300.0;			/* Parameters for filter design */
double  fhigh=2600.0;
double  kaiser_db=50.0;
int     filter_mask=255;		/* Bit-by-bit for 8 filters */
int     fil8_use=FALSE;			/* Apply the filter */
char    fil8_name[7]="FIL8";		/* Name for screen */
/* Parameters for receive equalizer that is part of filter design: */
#define	NFRB	15			/* Break frequencies (not progrmabl) */
double feqr[NFRB]={0.0,			/* Equalizer break frequencies */
              0.02083,			/* Fractions of 9600 Hz Fs */
              0.04167,			/* 0, 200, 400 Hz */
	      0.06250,			/* 600 Hz */
	      0.08333,			/* 800 Hz */
	      0.10417,			/* 1000 Hz */
	      0.13542,			/* 1300 Hz */
	      0.16667,			/* 1600 Hz */
	      0.20833,			/* 2000 Hz */
	      0.25000,			/* 2400 Hz */
	      0.29167,			/* 2800 Hz */
	      0.33333,			/* 3200 Hz */
	      0.37500,			/* 3600 Hz */
	      0.41667,			/* 4000 Hz */
              0.45833};			/* 4400 Hz */
/* Relative equalizer gains in dB (program from .cfg file): */
double aeqrdb[NFRB-1]={-100.0,		/*    0 to 200 Hz */
	       0.0,			/*  200 to 400 */
	       0.0,			/*  400 to 600 */
               0.0,			/*  600 to 800 */
	       0.0,			/*  800 to 1000 */
               0.0,			/* 1000 to 1300 */
	       0.0,			/* 1300 to 1600 */
	       0.0,			/* 1600 to 2000 */
	       0.0,			/* 2000 to 2400 */
	       0.0,			/* 2400 to 2800 */
	       0.0,			/* 2800 to 3200 */
	       0.0,			/* 3200 to 3600 */
	       0.0,			/* 3600 to 4000 */
	       0.0}; 			/* 4000 to 4400 */

/* Transmitter equalizer  */
double feq_t[NFRB]={			/* Equalizer break frequencies
					 * Fractions of 48,000 Hz Fs */
              0.0,			/*   0 Hz */
	      0.0052083,		/* 250 Hz */
	      0.0114583,		/* 550 Hz */
	      0.0177083,		/* 850 Hz */
	      0.025,			/* 1200 Hz */
	      0.0333333,		/* 1600 Hz */
	      0.04375,			/* 2100 Hz */
	      0.0625};			/* 3000 Hz (insure getting to 2800) */
/* Relative equalizer gains in dB (program from .cfg file): */

double aeqdb_t[NFTB-1]={-100.0,		/*    0 to 250 */
	       0.0,			/*  250 to 500 */
	       0.0,			/*  500 to 750 */
               0.0,			/*  750 to 1000 */
	       0.0,			/* 1000 to 1400 */
               0.0,			/* 1400 to 1900 */
	       0.0}; 			/* 1900 to 2500 */

struct lbox_info lb_freq; 		/* Freq info box */
struct lbox_info lb_eme2;		/* EME2 data summary box */
struct lbox_info lb_lhl7;		/* and for LHL7 */
struct lbox_info lb_pua43;		/* and for PUA43 */
struct lbox_info lb_lterm_int;		/* and long term integration of USB */
struct lbox_info lb_newfreq;;
void* current_info_box=0;		/* 0 if no box, otherwise ptr */

/* Associated with the audio filters:   */
struct filter_info  filt[8];

/* Keyboard codes for .CFG programmable (ver 3.23) 
 * Values given are defaults. These can now be changed in UHFA.CFG file */
long int k_freq_m1=0XD7000;
long int k_freq_p1=0XD7100;
long int k_freq_m10=0X4300;
long int k_freq_p10=0X4400;
long int k_freq_m100=0X15C00;
long int k_freq_p100=0X15D00;
long int k_freq_m1k=0X87000;
long int k_freq_p1k=0X87100;
long int k_freq_m10k=0X46600;
long int k_freq_p10k=0X46700;
long int k_freq_m100k=0XC7000;
long int k_freq_p100k=0XC7100;
long int k_freq_m1mhz=0X56600;
long int k_freq_p1mhz=0X56700;
long int k_af_gain_m=0X4100;
long int k_af_gain_p=0X4200;
long int k_af_gain_m1=0X46400;
long int k_af_gain_p1=0X46500;
long int k_rf_gain_m=0X15A00;
long int k_rf_gain_p=0X15B00;
long int k_mike_gain_m=0X96E00;
long int k_mike_gain_p=0X96F00;
long int k_fm_squelch_m_c=0X91F00;
long int k_fm_squelch_m_l=0X81F00;
long int k_fm_squelch_p_c=0X92000;
long int k_fm_squelch_p_l=0X82000;
long int k_cw_speed_m=0X86E00;
long int k_cw_speed_p=0X86F00;
long int k_cw_buf_clr=0X15300;
long int k_brite_m=0X3F00;
long int k_brite_p=0X4000;
long int k_contrast_m=0X15800;
long int k_contrast_p=0X15900;
long int k_af_filter_t=0X3D00;
long int k_af_filter_s=0X3E00;
long int k_denoise_t=0X15600;
long int k_denoise_m=0X46000;
long int k_denoise_p=0X46100;
long int k_nc_ave_m=0X86A00;
long int k_nc_ave_p=0X86B00;
long int k_delay_t_l=0X81200;
long int k_delay_t_c=0X91200;
long int k_rit_m_c=0X91300;
long int k_rit_m_l=0X81300;
long int k_rit_p_c=0X91400;
long int k_rit_p_l=0X81400;
long int k_rit_m1_c=0XD1300;
long int k_rit_m1_l=0XC1300;
long int k_rit_p1_c=0XD1400;
long int k_rit_p1_l=0XC1400;
long int k_rit_o_c=0X91800;
long int k_rit_o_l=0X81800;
long int k_rit_z_c=0X92C00;
long int k_rit_z_l=0X82C00;
long int k_norm_qck_c=0X91000;
long int k_norm_qck_l=0X81000;
long int k_norm_prc_c=0X91900;
long int k_norm_prc_l=0X81900;
long int k_color_agc_c=0X92E00;
long int k_color_agc_l=0X82E00;
long int k_gif_c=0X92200;
long int k_gif_l=0X82200;
long int k_beacon_c=0X92300;
long int k_beacon_l=0X82300;
long int k_record_c=0X92100;
long int k_record_l=0X82100;
long int k_window_t_c=0X91100;
long int k_window_t_l=0X81100;
long int k_fftbw_c=0X92400;
long int k_fftbw_l=0X82400;
long int k_emedop_c=0X92600;
long int k_emedop_l=0X82600;
long int k_mode_s_c=0X93200;
long int k_mode_s_l=0X83200;
long int k_binaural_t=0X98000;
long int k_disp_t_c=0X91500;
long int k_disp_t_l=0X81500;
long int k_db_div=0X87800;
long int k_offset_m=0X87900;
long int k_offset_p=0X87A00;
long int k_freq_box=0X88000;
long int k_save_cfg=0X88100;
long int k_cas1=0XD7800;
long int k_cas2=0XD7900;
long int k_cas3=0XD7A00;
long int k_cas4=0XD7B00;
long int k_cas5=0XD7C00;
long int k_cas6=0XD7D00;
long int k_cas7=0XD7E00;
long int k_cas8=0XD7F00;
long int k_cas9=0XD8000;
long int k_cas0=0XD8100;
long int k_dafgu=0X4800;
long int k_dafgd=0X5000;
long int k_dfreqd=0X4B00;
long int k_dfrequ=0X4D00;
long int k_mute=0X5100;
long int k_sh_dafgu=0X14800;
long int k_sh_dafgd=0X15000;
long int k_sh_dfreqd=0X14B00;
long int k_sh_dfrequ=0X14D00;
long int k_fil96=0X97B00;
long int k_dos_visit=0X97C00;
long int k_t_r_tog=0X04700;
long int k_help=0X3B00;
long int k_code_data_c=0X91E00;
long int k_code_data_l=0X81E00;
long int k_code_change_c=0X93000;
long int k_code_change_l=0X83000;
long int k_clr_lt_c=0X51117;
long int k_clr_lt_l=0X41117;
long int k_clr_ylw_c=0XD1100;
long int k_clr_ylw_l=0XC1100;
long int k_flat_c=0X51205;
long int k_flat_l=0X41205;
long int k_vcur1_c=0X51E01;
long int k_vcur1_l=0X41E01;
long int k_vcur2_c=0X52004;
long int k_vcur2_l=0X42004;
long int k_set_clock_c=0X92500;
long int k_set_clock_l=0X82500;
long int k_eme_prev=0X1333C;
long int k_eme_next=0X1343E;
long int k_beacon_box=0X103C00;
long int k_tone_box=0X104100;		// SCRL-F7
long int k_diag_s=0X103D00;
long int k_af_filter_design=0X103E00;
long int k_xmt_eq_design=0X103F00;

long int k_sig_sim=0X104000;
long int k_pua_del=0X104400;
long int k_agc_decay_m=0X112247;
long int k_agc_decay_p=0X112348;
long int k_agc_decay_m_l=0X102267;
long int k_agc_decay_p_l=0X102368;
long int k_cwoffset_m=0X112E43;
long int k_cwoffset_p=0X112F56;
long int k_cwoffset_m_l=0X102E63;
long int k_cwoffset_p_l=0X102F76;
long int k_sidetone_lev_m=0X111F53;
long int k_sidetone_lev_p=0X112044;
long int k_sidetone_lev_m_l=0X101F73;
long int k_sidetone_lev_p_l=0X102064;
long int k_xmt_power_m=0X11184F;
long int k_xmt_power_p=0X111950;
long int k_xmt_power_m_l=0X10186F;
long int k_xmt_power_p_l=0X101970;
long int k_do_quit=0X186B00;
long int k_warm_boot=0X1C6B00;
long int k_band_50_a=0XC1E00;
long int k_band_144_b=0XC3000;
long int k_band_220_c=0XC2E00;
long int k_band_432_d=0XC2000;
long int k_band_902_9=0XC8000;
long int k_band_1296_e=0XC1200;
long int k_band_2304_f=0XC2100;
long int k_band_3456_g=0XC2200;
long int k_band_5760_h=0XC2300;
long int k_band_10k_i=0XC1700;
long int k_band_24k_j=0XC2400;
long int k_band_47k_k=0XC2500;
long int k_band_75k_l=0XC2600;
long int k_band_119k_m=0XC3200;
long int k_band_145k_n=0XC3100;
long int k_audio_o=0XC1800;
long int k_band_memo_1=0XC7800;
long int k_band_memo_2=0XC7900;
long int k_band_memo_3=0XC7A00;
long int k_band_memo_4=0XC7B00;
long int k_band_memo_5=0XC7C00;
long int k_v2f_tog_c=0X92F00;		/* Toggle v2f on/off, Alt-V */
long int k_v2f_tog_l=0X82F00;		/*                    Alt-v */
long int k_v2f_res_c=0X92D00;		/* Reset v2f base freq, Alt-X */
long int k_v2f_res_l=0X82D00;		/*                      Alt-x */
long int k_v2f_lbox_c=0X52F16;		/* V-to-F Dialog Box, Ctrl-V */
long int k_v2f_lbox_l=0X42F16;		/*                    Ctrl-v */
/* Next 8 from v 3.31  */
long int k_memw_c=0X93100;		// ALT-N
long int k_memw_l=0X83100;		// ALT-n
long int k_memr_c=0X5310E;		// CTRL-N
long int k_memr_l=0X4310E;		// CTRL-n
long int k_mem2vfo_c=0X11314E;		// SCRL-N
long int k_mem2vfo_l=0X10316E;		// SCRL-n
long int k_memlist_c=0X193100;		// SCRL-ALT-N
long int k_memlist_l=0X183100;		// SCRL-ALT-n

/* All keys for modal boxes are exclusive to that box, and can be any
 * key, such as even 'a', 'b', etc.  These have symbol names starting
 * with "ky_".
 */
/* Keystrokes for the clock setting modal box */
unsigned int ky_clock_m1s='(';
unsigned int ky_clock_p1s=')';
unsigned int ky_clock_m10ms='{';
unsigned int ky_clock_p10ms='}';
unsigned int ky_clock_mrate='<';
unsigned int ky_clock_prate='>';
unsigned int ky_clock_resync='^';
unsigned int ky_clock_exit='*';

/* The keys for controlling the V2F function are letters. They are specified
 * here as caps, and any lower case is converted to caps in u_cmd.c.  */
unsigned int ky_v2f_base='R';
unsigned int ky_v2f_gainp='G';
unsigned int ky_v2f_gainm='F';
unsigned int ky_v2f_tcp='T';
unsigned int ky_v2f_tcm='V';
unsigned int ky_v2f_freqp='Q';
unsigned int ky_v2f_freqm='Z';
unsigned int ky_v2f_afp='U';
unsigned int ky_v2f_afm='N';

/* ------------------------------------------------------------------------ */

void main(int argc, char *argv[])
	{
	int grdrv, grmode, colormax, i;
	int cx, new_char, last_char;
	int stop_now;
	int esc_count;
	int y1, y2;
	int check_sum_dsp;		/* Received from dsp */
	int check_sum_pc;		/* Calculated by pc frm data*/ 
	int xmax, ymax;
	char  *cptr;
	unsigned int is;
	time_t t0;
	struct tm *tp;
	long int tL;
	int ri, hsec;
	char *ptc;
	char ct[20];
	int ser_cnt;
	int drv_num;

	strcpy(version, "3.50");	
	cptr = &cc[0];
	stop_now=FALSE;
	/* Get command line arguments for com port */
	if (argc==1)			/* Should be 2 arguments */
	    {
	    display_summary(argv[0], version);
	    exit(0);
	    }
	sscanf(argv[1], "%d", &com_port);
	if (com_port<1 || com_port>4)
	    {
	    beep(150);
	    printf("COM port must be 1, 2, 3 or 4\n");
	    exit(0);
	    }
	if (com_port==1) com_port=COM1;
	if (com_port==2) com_port=COM2;
	if (com_port==3) com_port=COM3;
	if (com_port==4) com_port=COM4;

	/* Look at command arguments to see if using a floppy drive:  */
        if (*argv[0]=='A')
	    deflt_drv=0;
	else if (*argv[0]=='B')
	    deflt_drv=1; 
	else deflt_drv=999;		/* Hard Drive is default */

	/* NOTE: Read the license before deletion or modification to any
	 * of the following license related code:
	 */
        if (NULL == ( license_file=fopen("GNU_GPL.txt", "r") ))
	    {
	    printf("\nNo GNU_GPL.txt license file was found. This must be in");
	    printf(" the same directory\nas UHFA.EXE file. Exiting program.");
	    exit(1);
	    }
	fclose(license_file);
	license_info();
	for(;;)
	    {
	    if ( kbhit() )
		{		
		cx=getch() & 0X7F;
		if (cx==13)
		    {clrscr(); break;}
		else if (cx=='1')
		    {warranty(); license_info();}
		else if (cx=='2')
		    {license(); license_info();}
		else license_info();
	        }
	    }

	if ((ptc=getenv("TZ")) != 0)
	    {
	    ut_available=1;
	    tzset();
	    printf("\nGMT Conversion from 'SET TZ=%s'", ptc);
	    time(&t0);
	    tp=localtime(&t0);
	    strcpy(cc, asctime(tp));   /* Get copy to rearrange */
	    cc[24]=0;
	    printf("\nComputer time =  %s (local)", cc);
	    tp=gmtime(&t0);
	    strcpy(cc, asctime(tp));
	    cc[24]=0;
	    printf("\n   translates to %s GMT\n\n", cc);
	    }
	else
	    {
	    ut_available=0;
	    printf("\nMoon & Sun data unavailable, needs 'SET TZ=PST8', etc.");
	    time(&t0);
	    tp=localtime(&t0);		/* time_type==0 --> use GMT */
	    strcpy(cc, asctime(tp));   /* Get copy to rearrange */
	    cc[24]=0;
	    printf("\nComputer time=%s (local)", cc);
	    }
	if (clock()==(clock_t) -1) 	/* This may not be there */
	    {
	    startup_detail=TRUE;	/* They may want to see this */
	    printf("Milli-second clock is NOT AVAILABLE,\n");
	    printf("   therefore, modes LHL7, PUA43 unavailable.\n");
	    if (mode>=4)  mode=0;		/* i.e., unavalailable */
	    ms_available=FALSE;			/* Communicate it */
	    }
 
	get_mem();			/* Memory for big arrays */

	/* Initialize cw message buffers */
	for (i=0; i<10; i++)
	    cwmbuf[i*50]=0;		/* Make all buffers empty */

	presets();			/* These may be overwritten */
	if (read_config_file() == 0)	/* Get data from UHFA.CFG */
	    {
	    printf("\nNo uhfa.cfg present, using default values");
	    }

	if (mode_mask & 0X20)
	    pua43_update(0, -3);	/* Allocate memory for PUA43 */

	/* Open COM port, delays added v3.36 to fix problem in some PC's  */
	ASYNCINIT(com_port);
	delay(100);
	ASYNCSET(BAUD_RATE, COM_SETTINGS);
	delay(100);
	ASYNCCLEAR();
	delay(100);
 
	if (startup_detail)
	    printf("ASYNCSTAT(%d) = %.4X \n", com_port, ASYNCSTAT());

	/* See if PTT used & on; usually means no hardware connected! */
	if ( hardware_ptt && (0X4000 & ASYNCSTAT())==0 )
	    {
	    printf("Push-to-talk indicates TRANSMIT and the .CFG variable\n");
	    printf("'hardware_ptt' is non-zero, meaning PTT using Ring\n");
	    printf("Detect is being used. This often means the wrong\n");
	    printf("serial port was specified in the command line, or\n");
	    printf("the DSP-10 hardware serial plug is not connected.\n");
	    exit(0);
	    }

	/* Need a kludge here. If radio was shut down in FM, need to go
	 * to SSB before starting. Covers a problem in the DSP. */
	if (mode==FM)			/* Change to USB, FM later */
	    dsp_cmd('M', 0, 0, fgainl[mike_gain], fgainh[mike_gain]);

	/* Download the DSP pointer table */
	printf("\nDownloading DSP pointers...");
	get_dsp_ptrs();
 
	if (startup_detail)
	    printf("DSP Configuration = %.4X \n", dsp_ptr[35]);

	/* Download audio filter 8, if in use. */
	printf("\nDownloading Audio Filter #8...");
	init_filr();
	/* Download transmit audio equalizer */
	printf("\nDownloading Transmit Audio Equalizer...");
	init_filt();

	if(startup_detail)
	   {
	   grdrv=DETECT;	
	   detectgraph(&grdrv, &grmode);

	   printf("\nGraphics driver= %d, Mode %d\n", grdrv, grmode);

	   if (grdrv != 9)
	      {
	      beep(1000);
	      printf("If driver is not 9 (VGA) & Mode is not 2 (640x480) ");
	      printf("you may encounter\ngraphics driver incompatibilities. ");
	      printf("Hit ENTER to continue\n");
	      }
	   printf("Graphresult= %d,  Hit 'Enter' to proceed", graphresult() );
	   getch();
	   }

	grdrv=9;			/* Only allow VGA 640x480 */
	grmode=2;
	initgraph(&grdrv, &grmode, "");
	strcpy(cptr, getdrivername() );
	xmax=getmaxx() + 1;  ymax = getmaxy() + 1;  /* Global */
	colormax = getmaxcolor();
	is = imagesize(1, 10,  511, 179);

	if (signal(SIGINT, ctrlc_handler)==SIG_ERR)
	    {
	    printf("\nInvalid signal number for Ctrl-C handler\n");
	    exit(0);
	    }
	if (signal(SIGFPE, fpe_handler)==SIG_ERR)
	    {
	    printf("\nInvalid signal number for Floating Point ");
	    printf("error handler\n");
	    exit(0);
	    }

	if(startup_detail)
	    {
	    closegraph();
	    printf("Graphics is %s, Mode %d, %d x %d\n",
		cptr, grmode, xmax, ymax);
	    printf("Bottom image size (170 lines worth) is %u bytes\n",is);
	    printf("Max color is %d\n", colormax);
	    printf("Graphresult= %d\n\n", graphresult() );
	    printf("\nHit 'Enter' (with Scroll Lock off) to proceed.\n");
	    printf("Any other key will give 'key ident' for uhfa_kbd.h.\n");
	    printf("Exit program with Scroll Lock Alt F4.\n");
	    for(;;)
		{
		if ( kbhit() )
		    if ( keyident() )  break;
		}
	    initgraph(&grdrv, &grmode, "");
	    }
	getpalette(&savpalette);	/* Palette when we arrived */
	setallpalette(&upalette);
	if ( graphresult()==grError )
	     do_t_error(40, "E40 Graphics", 200);
	setbkcolor(c_background);
	draw_grid();
	initialize(1);			/* This needs to be after get_mem() */
	scroll_line = 0;
	if (lpt_port!=0 && hardware==0)	/* T/R control if no hardware */		
	    outp(lpt_port, 0);		/* All control Off */

	/* The following is the main control loop for the transceiver */
 pp:	/* If contrast or brightness was recently changed, do color bar: */
	if (--contrast_line_count>0)
		{setviewport(X1L+1, Y1T+1, X1R-1, Y1B-1, DO_CLIP);
		 for (i=1; i<16; i++)
		     {setfillstyle(1, i);
		      /* y1=bottom of bar */
		      y1=128-(int)((dis_offset+((double)i)/dis_gain)/2.0);
		      if (y1>127)  y1=127;
		      if (i==1)  y1=127;
		      y2=128-(int)((dis_offset+((double)(i+1))/dis_gain)/2.0);
		      if (y2<1) y2=1;
		      if (i==15)  y2=1;
		      bar(390, y2, 415, y1);
		     }
		 setcolor(c_text_bold);
		}
	else
		contrast_line_count=0;

	esc_count = 0;
	for(;;)				/* Wait for 1st character */
		{
		if ( ASYNCINSTAT() > 0 )	/* Have ch, break if ESC_CHAR*/
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR)  esc_count++;
			 break;			/* Break the for(;;) */
			}
		while ( kbhit() )		/* Check for commands */
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();
		    if (transmit && mode!=EME2) do_transmit();
		    }
		/* Check for PTT via fast serial RI line */
		if ( ((0X4000 & ASYNCSTAT())==0) && hardware_ptt
			 && hardware && mode!=EME2)
		    {
		    ptt=1;		/* Set to override 'Home' key */
		    t_r_toggle();		/* Go to transmit */
		    do_transmit();		/* And wait here */
		    }
		if(mode==EME2)
		    eme_timing();	/* Transmit, etc. */
		if(mode==LHL7 || mode==PUA43)
		    lhl7_timing();	/* Receive triggering */
		}
	for(;;)			/* Wait here for ESC_CHAR A to come */
		{last_char=new_char;
		 if(ASYNCINSTAT() > 0)
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR) esc_count++;	
			 if (esc_count==2)	/* Waiting for ESC_CHAR, 'A' */
				{esc_count=0;	/*    but got 2xESC_CHAR     */
				 goto pp;	/* So try again              */
				}
			 if ((last_char==ESC_CHAR) && (new_char=='A'))
				break;	/* The beginning of a new data set */
			}

		 /* Loop around waiting for a ch, but also for manual input: */
		 while ( kbhit() )
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();
		    if (transmit && mode!=EME2) do_transmit();
		    }
		/* Check for PTT via fast serial RI line */
		if ( ((0X4000 & ASYNCSTAT())==0) && hardware_ptt
			 && hardware && mode!=EME2)
		    {
		    ptt=1;		/* Set to override 'Home' key */
		    t_r_toggle();		/* Go to transmit */
		    do_transmit();		/* And wait here */
		    }
		if(mode==EME2)
		    eme_timing();	/* Transmit, etc. */
		if(mode==LHL7 || mode==PUA43)
		    lhl7_timing();	/* Receive triggering */
		}

	for(;;)				/* Wait for a buf full */
		{if ( ASYNCINSTAT() > 490 )	/* Almost full (was 520) */
			 break;
		 while ( kbhit() )
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();
		    if (transmit && mode!=EME2) do_transmit();
		    }
		/* Check for PTT via fast serial RI line */
		if ( ((0X4000 & ASYNCSTAT())==0) && hardware_ptt
			 && hardware && mode!=EME2)
		    {
		    ptt=1;		/* Set to override 'Home' key */
		    t_r_toggle();		/* Go to transmit */
		    do_transmit();		/* And wait here */
		    }
		if(mode==EME2)
		    eme_timing();	/* Transmit, etc. */
		if(mode==LHL7 || mode==PUA43)
		    lhl7_timing();	/* Receive triggering */
		}
	if (--display_error_count == 1)
	    {
	    do_t_error(9999,"", 0);	/* Clears any errors */
	    display_error_count=0;	/* Don't do it until another error */
	    }
	incount=-1;			/* incount is for diagnostic display */
	i=0;  check_sum_pc=0;  esc_count=0;

	for(;;)              /* Stay here until 512 data points are received */
	    {if (i>511 && esc_count!=1)
		break; /* Ver 3.18 moved up here*/
	     if (ser_cnt<1000)	 /* If buffer has a lot, no timing checks */
		{
		if(mode==EME2)
		    eme_timing(); /* Time eme */
		if(mode==LHL7 || mode==PUA43)
		    lhl7_timing();	/* Receive trig */
		}
	     if( (ser_cnt=ASYNCINSTAT()) > 0 )
		{new_char = ASYNCIN();
		 ++incount;
		 if (new_char==ESC_CHAR) esc_count++;
		 if (esc_count==2)
		    {esc_count=0;
		     goto skip_char;
		    }
		 if (i<=511)
		    raw_data[i]=new_char;
		 check_sum_pc += new_char & 0XFF;
		 i++;		/* Count of data buffer entries */
skip_char:	}
	    }			/* End - collecting 512 data points */

	for(;;)				/* Get check sum */
	    {if (ASYNCINSTAT() > 0)
		{check_sum_dsp=ASYNCIN();
		 break;
		}
	     if(mode==LHL7 || mode==PUA43)
		lhl7_timing();	/* Receive trig */
	    }
	check_sum_pc &= 0XFF;
	if (error_on_tr > 0)		/* Prevent error after T/R change */
	    error_on_tr--;

	/* Now get 18 dsp_status bytes */
	i=0; esc_count=0;
	for(;;)
	    {if(mode==LHL7 || mode==PUA43)
		lhl7_timing();	/* Receive trig */
	     if( ASYNCINSTAT() > 0 )
		{new_char = ASYNCIN();
		 /* First esc_char goes into buff, but not second */
		 if (new_char==ESC_CHAR) esc_count++;
		 if (esc_count==2)
		    {esc_count=0;
		     goto dstat1;
		    }
		 dsp_status[i] = new_char;
		 if (++i > 17) 
		    break;
dstat1:		}
	     }

	/* Hold up screen update until after the next xmit pulse
	 * has started   */
	if(mode==EME2 && transmit)
		{
		while (eme_state==3)
		eme_timing();	/* Transmit, etc. */
		}
	if (check_sum_pc==check_sum_dsp)
	    {
	    /* error_on_tr prevents update right after reception starts,
	     * but we don't want it to stop any EME2 data */
	    if (error_on_tr==0 || mode==EME2)   update_data();
	    }
	else
	    {
	    if (error_on_tr==0)
		do_t_error(1, "E1 DSP Data Lst", 20);
	    data_lost=TRUE;		/* Change pixel on w'fall */
	    goto pp;
	    }
	if(dsp_status[17]==2)		/* Valid status data? */
	    /* See if PTT because of ring detect in ASYNCSTAT() or because
	     * sent in dsp_status(). Status bytes require hardware. In EME2,
	     * T/R conttrol is automatic by eme2_timing().
	     * Rev 3.39: status byte control for voice only.
	     */
	    {
	    if (  ((((0X4000 & ASYNCSTAT())==0) && hardware_ptt)
			 || ( (dsp_status[16] & 0X10) && hardware &&
			 (mode==FM || mode==USB || mode==LSB)))
			 && mode!=EME2  )
		{ptt=1;			/* Set to override 'Home' key */
		 t_r_toggle();		/* Go to transmit */
		 do_transmit();		/* And wait here */
		}
	    if (beacon && (b_transmit_sec>=3 || eme_pulse_count>=7))
		{
		t0=update_clock(1) + dsp_clock_base - (long int)b_offset_sec;
		tp=gmtime(&t0);		/* Get struct that includes .tm_sec */
		tL=(long int)tp->tm_sec + 60L*(long int)tp->tm_min
			+ 3600L*(long int)tp->tm_hour;
		hsec=(int)(60*tp->tm_min + tp->tm_sec);
		/* The next line makes the period 30 minutes, instead of 1hr */
		if (!b_repeat60 && hsec>=1800) hsec -= 1800;
		ri = (int)(tL % (long int)(b_transmit_sec + b_receive_sec));
		/* (ri<=2) makes sure to catch clock, but requires transmit
		 * times >2 sec. Changed Rev 2.10 RSL */
		if (ri<=2)
		    {
		    /* The following saves part of the
		     * yellow trace. If 0, it is inactive */
		    if (mode==LTI && lt_b_save &&
				 ((b_active1<=hsec && b_active2>=hsec) ||
				  (b_active1==0 && b_active2==0)))
			{
			/* Add tick marks to show beacon .ULT saves */
			setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
			setcolor(c_plot_bold);
			line(2*lt_i1-7, scroll_line, 2*lt_i1-2, scroll_line);
			line(2*lt_i5+2, scroll_line, 2*lt_i5+7, scroll_line);

			cc[0]=file_ident[0]; cc[1]='L'; cc[2]='T';

			/* Remove offset to save real time */
			t0 += (long int)b_offset_sec;
			if (time_type==1)
			    tp=localtime(&t0);	/* time_type==1 for local */
			else
			    tp=gmtime(&t0);		/* ==0 for gmt */
			itoa( (1+tp->tm_mon), ct, 13);   /* Jan=1,..., Dec=C */
			cc[3]=ct[0];
			itoa(tp->tm_mday, ct, 10);
			if (tp->tm_mday < 10)
			    {
			    cc[4]='0';
			    cc[5]=ct[0];
			    }
			else
			    {
			    cc[4]=ct[0];
			    cc[5]=ct[1];
			    }
			itoa(tp->tm_year, ct, 10);  /* Year, 102 for 2002*/
			cc[6]='_';
			cc[7]=ct[2];	/* Last number of year */
			cc[8]='.'; cc[9]='D'; cc[10]='A'; cc[11]='T'; cc[12]=0;

			/* If path exists, hang file name on the end of it */
			if (strlen(fdata_path)>1)
			    {
			    strcpy(&cc[100], fdata_path);
			    strcat(&cc[100], &cc[0]);
			    }
			else
			    strcpy(&cc[100], &cc[0]);

			if (cc[100]=='A')        drv_num=0;
			else if (cc[100]=='B')   drv_num=1;
			else                     drv_num=deflt_drv;

		        /* See if floppy is being used and, if so, available */
		        if ( !floppy_door(drv_num) )
			    do_t_error(51, "E51 Disk Door", 50);
			else
			    {
			    /* Write to file */
			    if(NULL==(lt_data_file=fopen(&cc[100], "a+t") ))
			         do_t_error(47, "E47 DAT File", 200);
			    else
			  	{
				fprintf(lt_data_file,
				    "HL %d %d %d %02d%02d %d ",
				    tp->tm_year+1900, tp->tm_mon+1,
				    tp->tm_mday,
				    tp->tm_hour, tp->tm_min, tp->tm_sec);
				fprintf(lt_data_file, "%d %d %d %d %d ",
				    fft_bw, do_window, b_receive_sec, lt_i3,
				    eme_pulse_count);
				fprintf(lt_data_file, "%.6lf %d ",
				    0.000001*(tvtr[band].lo_freq-
				    ((double)tvtr[band].sb)*(double)tfreqL),
				    (int)eme2_te);
				for(i=lt_i3-lt_b_save_bins;
					 i<=lt_i3+lt_b_save_bins; i++)
			        fprintf(lt_data_file,"%.2lf ",
				    p2db(lt_pwrdat[i]));
				fprintf(lt_data_file, "\n");
				fclose(lt_data_file);
				}
			    }
		       }

		    /* Time to beacon transmit. Transmit time is
		     * 3 or more sec, so that we don't end up right back here.
		     * We are here only if beacon==TRUE.
		     */
		    if ( b_transmit_sec>=3 &&
			 ((mode==LTI && b_active1<=hsec && b_active2>=hsec) ||
			 (mode==LTI && b_active1==0 && b_active2==0) ||
			  mode!=LTI))
			{
			if (!transmit)		/* If receive, */
			    t_r_toggle();	/*   go to transmit */
			do_transmit();		/* And wait here */
			}
		    /* The following, if >0,  clears the yellow trace. */
		    if (mode==LTI && lt_b_clear>0)
			{
			lterm_int_update(-1);	/* Zero sums */
			paveave=0.0;
			eme_pulse_count=0;
			setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
			setcolor(c_plot_bold);
			line(34, scroll_line, 36, scroll_line);
			}
		    }
		}
	    diagnostics(1);		/* Update diagnostics */
	    }
	else
	    {
	    do_t_error(2, "E2 DSP Status", 0);
	    }
	print_time();
	goto pp;	
	}				/* End main() */

/*  UPDATE_DATA()  -  Called after 512 data points if check sum is OK.
 *  Converts log to power, adds to power sums and displays on both upper fft
 *  screen as well as lower spectral map.
 */
void update_data(void)
	{
	static time_line_count=0;	/* Decides when to do time print */
	static double auto_disp_base=0;	/* Base value for setting color agc */
	static double auto_disp_correct=0.0;
	double correction;
	int i, j;
	int im, ix, imax;
	int pclr;			/* Color for spectral map, 0 to 7 */
	int biggest;			/* Highest during sweep */
	int biggest_freq;
	int display_data;
	unsigned int p56ui;
	static int lt_display_data=0;	/* All lt_... are long term for eme */
	static double spandb=1.0;	/* Intermediate in scaling lt data */
	static double lt_dd=1.0;	/* db/div for lt data */
	static double lt_mid=0.0;	/* dB level for mid graph */
	double delta_f;
	double pua43_tscale, ptdb;
	double xi, p43_i2, p43_mom1, p43_slope;
	double pl, ph, pc;
	/* Next is FFT gain, rel to 9600 samp/sec: */
	static double db_fft[4]={6.02, 3.01, 0.0, -6.99};
	int newy;
	static int oldx=0;
	static int oldy=128;
	int lt_newy;
	static int lt_oldx=0;
	static int lt_oldy=48;
	double scale;
	double bar_pwr, big_bp;
	int    noise_blanked;
	time_t tnow;
	struct tm *tmnow;
	double lpave;
	static double pave_print=0.0;	/* Accumulated over noncohave periods*/
	double dbm_level;		/* in i-f, non-FM mode, fft power */
	double fm_log_pwr;		/* Measured log power in FM mode */
	double dfr;			/* Freq offset in FM, from DSP discr */
	double fmf;			/* Freq, MHz from discr */
	int pwr5n6;			/* Power from dsp_status[5] and [6] */
	double pwr5n6_dbm;		/* Converted to dbm */
	double tmp, tmp1, lt_tmp;	/* Temporary storage */
	int base;			/* For plotting */
	int lt_trace;			/* Yellow trace being used */
	static double dp5n6;		/* Power sum over multiple status */

	if(dsp_status[17]==2)		/* Valid status data? */
	    {
	    /* Look for serial receive errors at DSP  */
	    if (dsp_status[1])  do_t_error(3, "E3 DSP Cmd", 20);

	    /* Look for PLL "Not-locked" errors in DSP */
	    if (((0X08^dsp_status[16]) & 0X08) && hardware &&
			band!=AUDIO_BAND)
		do_t_error(30, "E30 PLL UnLock", 20);
	    }

	/* Catch freq changes here, they happen once per minute.
	 * Also update EME data.
	 */
	if (mode==LTI || eme_doppler)
	     new_freq();

	lt_trace = (mode==EME2 || mode==LTI);  /* Shorthand */

	if (text_area) base=94;
	else           base=126;

	if (lt_trace)  eme_pulse_count++;

	/* Convert log raw data to power and add powers for non-coherent
	   integration.  For 8 bit data we use a lookup table, power10[],
	   to do a fast conversion.  Table is too big for 16 bit data.  */
	big_bp=1.0e-200;
	pave=0.0;			/* Spectral power average for 1 set */
	if (bits_8_16 == 0)		/* If using 8 bit log data */
	    {
	    /* fft_bw==3 creates 18.75 Hz bins by adding the powers of two
	     * 9.375  Hz bins together. This creates 256 bins, and the next
	     * 256 are set to 0. Available only with 8-bit log data.
	     */
	    if (fft_bw==3)
		{
		for (i=0; i<256; i++)
		    {
		    bar_pwr=power10[raw_data[2*i]]+power10[raw_data[1+2*i]];
	 	    pwrdat[i] += bar_pwr;	/* Non-coherent integration */
		    if (bar_pwr > big_bp)   big_bp=bar_pwr;
		    if (i>15) pave += bar_pwr;
		    }
		for (i=256; i<512; i++)
		    pwrdat[i]=1.0e-100;   /* Rev 3.19 was 0 */
		}
	    else			/* if fft_bw != 3 */
		{
		for (i=0; i<512; i++)
		    {
		    bar_pwr=power10[ raw_data[i] ];
	 	    pwrdat[i] += bar_pwr;	/* Non-coherent integration */
		    if (bar_pwr > big_bp)   big_bp=bar_pwr;
		    if (i>30) pave += bar_pwr;
		    }
		}

	    /* Add up FM powers. The value in status[5] and [6] is
	       1000*log10(pwr) or 100 * power in dB  */
	    if (mode==FM)
		{
		p56ui = 256*(unsigned int)dsp_status[6] +
			 (unsigned int)dsp_status[5];
		if (p56ui <= 32767)
		   pwr5n6=p56ui;
		else
		   pwr5n6 = -(int)(65536-p56ui);
		dp5n6 += upow10(0.001*(double)pwr5n6);
		}

	    if (normalize_on > 1) normalize(512);

	    /* A kludge to take care of changing offsets in the log data from
	     * the DSP that results in paveave errors. This needs to be moved
	     * into pave calculation by including power_exp & other factors.
	     * The problem is that it is really slow to adjust paveave if pave
	     * is much less, and this prevents noise-blanking of yellow trace.
	     */
	    if (paveave > 2*pave) paveave=2*pave;

	    if (eme_pulse_count == 1)
		paveave=pave;
	    else
		paveave = 0.95*paveave + 0.05*pave;	/* Running ave */
	    if (mode==PUA43)		/* LTI & EME2 use 16-bit data */
		noise_blanked=(pave>code_nb_ratio*paveave);
	    else
		noise_blanked=FALSE;
	    }				/* End 8 bit non-co ave */
	else				/* using 16 bit log data */
	    {
	    big_bp=1.0e-20;
	    for (i=0; i<256; i++)	/* Pass 1 for 16 bit data */
		{
		/* The following makes the value of 16 bit data the
		    same as that of 8 bit */
		/* Note: LOG_OFFSET_D 12288.0    0.0078125=1/128
		 * and upow10(0.03125*x) is the same as the lookup power10[x].
		 * The 2 constants, 0.0078125*0.03125=0.0002441406 
		 */
		pd16[i]=bar_pwr=upow10(0.0002441406*
		   ((double)(raw_data[2*i]+256*raw_data[2*i+1])+LOG_OFFSET_D));
		if (bar_pwr > big_bp)   big_bp=bar_pwr;
	 	pwrdat[i] += bar_pwr;	/* Non-coherent integration */
		if (i>30)		/* Dodge hum at left */
		   pave += bar_pwr;	/* Noise ave over spectrum */
		pwrdat[i+256]=0.00001;	/* Dummy out unused spectrum */
		}			/* End first pass of 16 bit data */

	    /* A kludge to take care of changing offsets in the log data from
	     * the DSP that results in paveave errors. This needs to be moved
	     * into pave calculation by including power_exp & other factors.
	     * The problem is that it is really slow to adjust paveave if pave
	     * is much less, and this prevents noise-blanking of yellow trace.
	     */
	    if (paveave > 2*pave) paveave=2*pave;

	    if (eme_pulse_count == 1)
		paveave=pave;
	    else
		paveave = 0.95*paveave + 0.05*pave;	/* Running ave */
	    if ((eme2_format & 4) || mode==PUA43)
		noise_blanked=(pave>code_nb_ratio*paveave);
	    else
		noise_blanked=FALSE;

	    lt_max=0.0;  lt_min=1.0E300;

	    /* For either EME2 or LTI, get data */
	    if (lt_trace && !noise_blanked)
		{
		for (i=lt_i1; i<=lt_i5; i++)
		    {
		    lt_pwrdat[i] += pd16[i]; /* Very long term integration */
		    if (i>=lt_i2 && i<lt_i4)
			{
			if(lt_pwrdat[i] < lt_min)  lt_min=lt_pwrdat[i];
			if(lt_pwrdat[i] > lt_max)  lt_max=lt_pwrdat[i];
			}
		    }
		}
	    if (normalize_on > 1) normalize(256);
	    }				/* End 16 bit non-co ave */
	if(mode==LHL7)
	    {
	    lhl7_timing();	/* Receive trig */
	    lhl7_update(dsp_status[14], 2);
	    }
	if(mode==PUA43)
	    {
	    lhl7_timing();	/* Possible receive trig */
	    setviewport(600, 5, 638, 15, DO_CLIP);
	    clearviewport();
	    settextjustify(LEFT_TEXT, TOP_TEXT);
	    moveto(0, 0);
	    if (!noise_blanked)
		pua43_update(dsp_status[14], 2);
	    else
		outtext("*NB*");
	    eme_pulse_count++;
	    }
#if 0
  /* 3.28 */
	/* Adjust integrate count to line up with the top of the minute
	 * for LHL7 and PUA43, to stop leakage in bins for display
	 * when noncohave>1
	 */
	if (sync_flag && dsp_status[14]==2)
	    {
	    integrate_count=-1;		/* It will soon be ++ed to 0 */
	    sync_flag=FALSE;		/* Only once */
	    }
#endif

	pave_print += pave;

	/* If time for display...  EME2 and PUA43 update every time integrate
	 * count gets to noncohave for fft_bw==2 (3 groups of 12) but for
	 * the narrower fft_bw the data is sent only once per 2 seconds
	 * and the display is updated each time.
	 */
#if 0
	if ((++integrate_count >= noncohave)
	       || ((mode==PUA43 || mode==EME2) && (fft_bw==0 || fft_bw==1)))
#endif
	if ( ((++integrate_count >= noncohave) && mode!=PUA43 && mode!=EME2)
	       || ( (mode==PUA43 || mode==EME2) && 
		(   (fft_bw==2 && (dsp_status[14]&2)>0) || fft_bw!=2    )   ) )
	    {
	    if (auto_disp_on && (auto_disp_correct<-10.0 |
			auto_disp_correct>10.0) )
		do_t_error(10, "E10 AutoDisp>10", 0);
	    /* Switch the auto-disp stuff to pave or paveave ?? */

	    if (auto_disp_on == -1)	/* Establish base average level */
		{
		auto_disp_on=TRUE;
		auto_disp_base=0.0;
		for (i=0; i<(512-display_width*256); i++)
		    auto_disp_base += pwrdat[i];
		}
	    if ( (auto_disp_on) && (auto_disp_base>1.0e-100) )
		{
		for (i=FLOW; i<(512-display_width*256); i++) /* Cur ave lev */
		    auto_disp_correct += pwrdat[i];
		/* For display, remove dc term: */
		ave_spectral_pwr=auto_disp_correct - pwrdat[0] - pwrdat[1];
		auto_disp_correct /= auto_disp_base;  /* Make rel to base */
		auto_disp_correct = p2db(auto_disp_correct);
		}
	    integrate_count=0;
	    biggest=0; biggest_freq=0;
	    prep_display();
	    if (mode==FM && !transmit)
		{
		setviewport(160, 117, 420, 127, DO_CLIP); /* Corner of upper */
		clearviewport();
		settextjustify(LEFT_TEXT, TOP_TEXT);
		moveto(0, 1);
		dfr = -0.77975243*(-12824.58333 +
		    (double)(256*dsp_status[4]+dsp_status[3]));
		fmf=0.000001*(dfr + tvtr[band].lo_freq -
		     ((double)(tvtr[band].sb)*(double)tfreqL));
		sprintf(cc, "DF=%5.0lf Hz  Fr=%12.6lf MHz", dfr, fmf);
		outtext(cc);
		}

	    /* scale accounts for non coherent averaging */
	    scale=p2db( (double)noncohave );

	    if (lt_trace)	    
		oldx=2*(lt_i1-1);
	    else
		oldx=0;
            oldy=128;
	    lt_oldx=2*(lt_i1-1);   /* <<< Is this right?? */
	    /* -3*power_exp is a correction for data scaling in DSP */
	    correction = 32.0 - koffset + scale + auto_disp_correct
		+ knoise*db_fft[fft_bw]
		- win_adj - 3.0*(double)power_exp;

	    if(display_width==FULL_FREQ)
		{
		setcolor(c_plot_bold);
		for (i=0; i<512; i++)
		    {
		    im=i-1;
		    display_data = (int)
			( kdb_div*(p2db(pwrdat[i]) - dnorm[i]
			     - correction) );
		    if (text_area)
			setviewport(X1L+1, Y1T+33, X1R-1, Y1B-1, DO_CLIP);
		    else
			setviewport(X1L+1, Y1T+1, X1R-1, Y1B-1, DO_CLIP);
		    if (dot_type==0)
			putpixel(im, base-(display_data >> 1), c_plot_bold);
		    if (dot_type==1)
			point1(im, base-(display_data >> 1) );
		    if (dot_type==2)
			{
			newy = base-(display_data >> 1);
			line(oldx, oldy, im, newy );
			oldx=im;
			oldy=newy;
			}
		    if (waterfall)
			{
			setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
			pclr=(int)
			    (dis_gain*((double)display_data - dis_offset));
			if (pclr>15) pclr=15;
			if (pclr<1) pclr=1;
			putpixel(i, scroll_line, pclr);
			}
		    /* bin_low*9.375 is lowest freq considered for marker*/
		    if (display_data > biggest  &&  i>bin_low )
			{
			biggest=display_data;
			biggest_freq=i;
			}
		    }		/* End 512 times */
		}		/* End FULL_FREQ display */
	    else 		/* if (display_width==HALF_FREQ) */
		{
		/* Set the long term integration plot so that it occupies about
		   half of  the display area. lt_dd is dB/div.  There are 16
		   pixels, vertically, per major grid division.  */
		if (lt_trace)
		   {spandb=p2db(lt_max/lt_min);
		    if     (spandb<0.2)  lt_dd=320.0;	/* .05dB/16 pixels */
		    else if(spandb<0.4)  lt_dd=160.0;	/* .1dB/16  */
		    else if(spandb<0.8)  lt_dd=80.0;	/* .2dB/16  */
		    else if(spandb<2.0)  lt_dd=32.0;	/* .5dB/16  */
		    else if(spandb<4.0)  lt_dd=16.0;	/*  1dB/16  */
		    else if(spandb<8.0)  lt_dd=8.0;	/*  2dB/16  */
		    else if(spandb<20.0) lt_dd=3.2;	/*  5dB/16  */
		    else                 lt_dd=1.6;	/* 10dB/16  */
		    /* See if allowed to overrange display */
		    if (lt_dd<min_lt_dd)  lt_dd=min_lt_dd;

		    /* Set the mid-point of the display for long term
		     * integration at the average of noise in lt_i3+/-2.
		     * lt_mid, p2db() and lt_dnorm all in dB.
		     * lt_i3 is the index of the center bin for EME2, LTI.
		     */
		    lt_mid=0.5*( p2db(lt_pwrdat[lt_i3-2])-lt_dnorm[lt_i3-2]
			        +p2db(lt_pwrdat[lt_i3+2])-lt_dnorm[lt_i3+2] );
		    setviewport(150, 52, 230, 63, DO_CLIP);
		    if (noise_blanked)
			sprintf(cc, "*NB*");
		    else
			sprintf(cc, "%4.2lfdB/", 16.0/lt_dd);
		    setcolor(c_plot_bright);
		    moveto(1,1);
		    settextjustify(LEFT_TEXT, TOP_TEXT);
		    outtext(cc);
		    setcolor(c_text_bold);
		   }
		setcolor(c_plot_bold);
	if(mode==PUA43)    lhl7_timing();
		for (i=0; i<256; i++)
		    {
		    im=2*i-1;
		    /* EME2 & LTI only displays a subset of the data */
		    if (lt_trace && (i<lt_i1 || i>=lt_i5))
			goto end256;

		    if (!lt_trace)
		      {
		      display_data = (int)
			   (kdb_div*(p2db(pwrdat[i])-dnorm[i]-correction));
		      }
		    else  /* If lt_trace (EME2 or LTI)  */
		      {
		      /* Kinda a kluge (lt_i3-2);
		       * can we do better?? If random<eps otherwise don't
                       */
                      display_data = (int)
			  ( kdb_div*(p2db(pwrdat[i])-correction
                          -lt_dnorm[i]+lt_dnorm[lt_i3-2]) );
		      lt_display_data = (int)
			  ( lt_dd*(p2db(lt_pwrdat[i])-lt_dnorm[i]-lt_mid) );
		      }
		    if (text_area)
			setviewport(X1L+1, Y1T+33, X1R-1, Y1B-1, DO_CLIP);
		    else
			setviewport(X1L+1, Y1T+1, X1R-1, Y1B-1, DO_CLIP);

		    if (dot_type==0)
		       putpixel(im,base-(display_data >> 1), c_plot_bold);
		       putpixel(im+1,base-(display_data>>1), c_plot_bold);
		    if (dot_type==1)
			point1(im, base-(display_data >> 1) );
		    if (dot_type==2)
			    {
			    newy = base-(display_data >> 1);
			    line(oldx, oldy, im, newy );
			    oldx=im;
			    oldy=newy;
			    }
		    if (lt_trace && !noise_blanked)
			{
			/* Display the perpetual average in yellow */
			lt_newy = 48-lt_display_data;
			setcolor(c_plot_bright);
			if (i==lt_i1)
			    {
			    line(im, lt_newy, im, lt_newy );
			    lt_oldy=lt_newy;
			    }
			else
			    {
			    line(lt_oldx, lt_oldy, im, lt_newy );
			    lt_oldy=lt_newy;
			    }
			lt_oldx=im;
			setcolor(c_plot_bold);
			}
		    if (waterfall)
			{
			setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
			pclr=(int)
			    (dis_gain*((double)display_data - dis_offset));
			if (pclr>15) pclr=15;
			if (pclr<1) pclr=1;
			putpixel(2*i,   scroll_line, pclr);
			putpixel(2*i+1, scroll_line, pclr);
			}
			/* bin_low*9.375 is lowest freq for marker*/
			if (display_data>biggest  &&  i>bin_low )
			    {
			    biggest=display_data;
			    biggest_freq=i;
			    }
end256:
		    }			/* End of 256 times */

	        }			/* End of  else if (half width)


	    /* Show tuning data for PUA43 */
	    if(mode==PUA43 && pua43_tune[ptunec[fft_bw]]>1.0e-99)
	      {
  lhl7_timing();
	      /* Convert to dB and work on dB versions. Find slope */
	      p43_i2 = 0.0;    p43_mom1 = 0.0;
	      for (i=ptune0[fft_bw]; i<=ptune1[fft_bw]; i++)
		{
		lt_pwrdat[i] = p2db(pua43_tune[i]);	/* To dB */
		xi = (double)(i-ptunec[fft_bw]);
		p43_i2 += xi*xi;		/* See Hamming, p435 */
		p43_mom1 += xi*lt_pwrdat[i];
		}
	      p43_slope = p43_mom1/p43_i2;


  lhl7_timing();


	      /* Remove slope, find min and max:  */
	      pua43_tmin=2000.0; pua43_tmax=-2000.0;
	      for (i=ptune0[fft_bw]; i<=ptune1[fft_bw]; i++)
		{
		lt_pwrdat[i] -= (double)(i-ptunec[fft_bw])*p43_slope;
		/* Find min and max of dB versions to scale display */
		if(lt_pwrdat[i] > pua43_tmax)
		    {
		    pua43_tmax = lt_pwrdat[i];
		    imax = i;	/* Peak frequency */
		    }
		if(lt_pwrdat[i] < pua43_tmin)
		    pua43_tmin = lt_pwrdat[i];
		}
	      pc=pua43_tune[imax];
	      pl=pua43_tune[imax-1]; ph=pua43_tune[imax+1];
	      delta_f = df[fft_bw]*
		    ((double)(imax-ptunec[fft_bw])+0.5*(ph/pc - pl/pc));

	      /* tscale makes the vertical plot 64 pixels high */
	      pua43_tscale = 64.0/(pua43_tmax - pua43_tmin);

  lhl7_timing();


	      setviewport(X1L, Y1T+1, X1R-1, Y1B-1, DO_CLIP);
	      setcolor(c_plot_bright);	/* Yellow */
	      for (i=ptune0[fft_bw]; i<=ptune1[fft_bw]; i++)
		{
		lt_newy = 96-near_int(pua43_tscale*(lt_pwrdat[i]-pua43_tmin));
		if (i==ptune0[fft_bw])
		    line(i-1, lt_newy, i-1, lt_newy );
		else
		    line(i-1, lt_oldy, i, lt_newy );
		lt_oldy = lt_newy;
		}

	      /* Tick marks every 50 Hz */
	      for (i=1; i<=3; i++)
		{
		/* Each bin is a pixel, so 50/df[] is the number of pixels
		 * in 50 Hz */
		im = near_int((double)i*50.0/df[fft_bw]);
		line(ptunec[fft_bw]-im, 96, ptunec[fft_bw]-im, 100); 
		line(ptunec[fft_bw]+im, 96, ptunec[fft_bw]+im, 100); 
		}
	      setcolor(c_plot_marker);	
	      line(ptunec[fft_bw], 102, ptunec[fft_bw], 32); 

	      setcolor(c_plot_bright);	/* Yellow */
	      if(fft_bw < 2)
		setviewport(138, 101, 200, 121, DO_CLIP);
	      else			/* Not  enough room on left */
		setviewport(400, 101, 460, 121, DO_CLIP);
	      clearviewport();
	      settextjustify(LEFT_TEXT, TOP_TEXT);	      
	      moveto(1, 1);
	      sprintf(cc, "DF=%d", near_int(delta_f));
	      outtext(cc);

	      ptdb = pua43_tmax-pua43_tmin;   /* total dB (3.28) */
	      if (ptdb > 1.0)
		sprintf(cc, "SN=%4.1lf", ptdb);
	      else if (ptdb > 0.1)
		sprintf(cc, "SN=%4.2lf", ptdb);
	      else
		sprintf(cc, "SN=%4.3lf", ptdb);
	      moveto(1, 11);
	      outtext(cc);
  lhl7_timing();
	      setcolor(c_plot_bold);
	      }

	    if(mode==LHL7 || mode==PUA43)
		 lhl7_timing();	/* Receive trig */

	    /* Add marker at highest point in upper spectral display */
	    setcolor(c_plot_marker);
	    if (text_area)
		setviewport(X1L+1, Y1T+33, X1R-1, Y1B-1, DO_CLIP);
	    else
		setviewport(X1L+1, Y1T+1, X1R-1, Y1B-1, DO_CLIP);
	    if (!text_area && biggest > 250) biggest=250;
	    else if (text_area && biggest > 186) biggest=186;
	    point5((1+display_width)*biggest_freq-1, base-(biggest >> 1) );

	    if(mode==CW)
		{/* Add red line at cw offset */
		 i=(int)((double)cwoffset/df[fft_bw])-1;
		 line(i, base-6, i, base+2);
		}
	    else if(mode==LHL7)
		{i=16*(1+display_width);
		 line(4*i-1,  78, 4*i-1,  96);
		 line(5*i-1,  88, 5*i-1,  96);
		 line(6*i-1,  88, 6*i-1,  96);
		 line(7*i-1,  88, 7*i-1,  96);
		 line(8*i-1,  88, 8*i-1,  96);
		 line(9*i-1,  88, 9*i-1,  96);
		 line(10*i-1, 78, 10*i-1, 96);
		}
	    else if(mode==PUA43)	/* Red lines showing band */
		{
		/* Always FULL_WIDTH and band is 42*4=168 bins wide */
		i=b0[fft_bw]-1;
		line(i, 87, i, 94);
		line(i+168, 87, i+168, 94);
		line(i, 94, i+168, 94);
		}
	    else if(lt_trace)
		{
		 i=(1+display_width)*lt_i3-1;	/* Center freq */
		 line(i, 48, i, 128);
		 i=(1+display_width)*lt_i2-1;
		 j=(1+display_width)*lt_i4-1;
		 line(i, 120, i, 128);
		 line(j, 120, j, 128);
		 line(i, 126, j, 126);
		 if (current_info_box == &lb_eme2)
		    {eme2_update(1);
		     eme2_update(2);
		    }
		 if (eme_state > -1)  eme_state=1; /* Screen update finished */
		 if (current_info_box == &lb_lterm_int)
		    {lterm_int_update(1);
		     lterm_int_update(2);
		    }
		}			/* End lt_trace */

	    setcolor(c_text_bold);

	    /* Several things need tmnow, so do it here: */
	    if (ms_available)
		tnow=update_clock(1)+dsp_clock_base;
	    else
		(void)time(&tnow);
	    if (time_type)
		tmnow = localtime(&tnow);	/* time_type==1 for local */
	    else
		tmnow = gmtime(&tnow);		/*          ==0 for gmt */

	    if ( waterfall && (time_line_count++ >= 20) )  /* Mark the time: */
                {time_line_count = 0;
		 if (scroll_line > 9)	/* Not above box 2 */
		    {
		    setviewport(X2L+1, Y2T+scroll_line-7,
			X2L+33, Y2T+scroll_line+1, DO_CLIP);
		    clearviewport();
		    if (show_secs)
			{
		        tiny_time( (100*tmnow->tm_min+tmnow->tm_sec), 1, 8);
			}
		    else
			{
		        tiny_time( (100*tmnow->tm_hour+tmnow->tm_min), 1, 8);
			}
		    }
		}

	    if (mode==FM)
		{
		/* In following, the value in status[5] and [6] is
		  1000*log10(pwr) and the bargraph routine wants
		  abs(max volts).  Variable gain_adj comes
		  from uhfa.cfg file and allows compensation for gain variation
		  between units. It is in dB and can be + or -.
		  The 2.0* factor moves the display over 1 bar so that a max
		  signal of about -54 dBm (rf gain full up) lights all s-meter
		  bars  */
 		ix = (int)pwr5n6 + (int)(100.0*gain_adj)
		    -(int)(100.0*(rfg[rfgain].dbgain+tvtr[band].gain_db));
				  /* This last was i-f pwr in db*100 */
		if (dsp_status[17]==2)		/* If status data good */
		    {
		    /* S-Meter bar graph: */
		    setviewport(0,S_METER_POS-19,127,S_METER_POS-9,DO_CLIP);
		    tmp1=2.0*sqrt(upow10(((double)(3703+ix))/1000.0));
		    if (tmp1>32767.0) tmp1=32767.0;
		    bar_graph(20, 0,(unsigned int)tmp1, 0);
 		    }
		/* 94.2 relates to the gain of the radio */
	        /* fm_log_pwr = -94.2+((double)ix)/100.0;  before v3.06 */
		fm_log_pwr = -98.5 + p2db(dp5n6) - scale + gain_adj
		    - (rfg[rfgain].dbgain + tvtr[band].gain_db);
		dp5n6 = 0.0;		/* Start new integration */

		}			/* End mode==FM  */
	    else			/*   not FM */
	      {
/*	      if (   <<<<<<<<<<<<<<<<<<<<<<<<<<<< */	
		/* For the S-meter display */
		dbm_level = KGAIN-rfg[rfgain].dbgain
		    +p2db(pwrdat[biggest_freq])+gain_adj+win_adj
		    -db_fft[fft_bw]+3.0*(double)(power_exp-10)
		    -tvtr[band].gain_db - scale;

		/* Mark the audio power calculation with a 'P' */
		setviewport(113,S_METER_POS+12,122,S_METER_POS+31,NO_CLIP);
		clearviewport();
		/* Strong signals go out-of-range in the FFT output. To
		 * make the S-meter work for these signals, we switch
		 * to the ADC max level.
		 */
		if (dbm_level > -80.0)
		    {

	/* dbm_level=    pwr5n6_dbm = -86.95 + 0.01*((double)pwr5n6) + gain_adj
			-rfg[rfgain].dbgain - tvtr[band].gain_db;
	 */
		    dbm_level=
			2.0*p2db((double)(256*dsp_status[10]+dsp_status[9]))
			-150.15 + gain_adj
			-rfg[rfgain].dbgain - tvtr[band].gain_db;
		    settextjustify(RIGHT_TEXT, TOP_TEXT);
		    settextstyle(0, 0, 1);
		    setcolor(c_text_bold);
	            moveto(8, 0);
		    outtext("P");
		    }		    

		/* S-Meter bar graph: */
		setviewport(0, S_METER_POS-19, 127, S_METER_POS-9, DO_CLIP);

		tmp1=(int)(144.0+dbm_level);
		if (tmp1 < 0)
		    tmp1=0;
		else if (tmp1>113)
		    tmp1=32768;
		else
		    tmp1=smbar[tmp1];
		bar_graph( 20, 0, tmp1, 0);
		} 			/*End RF, not FM */
	    /* Display power below bar graph: */
	    setviewport(1,S_METER_POS+12, 112, S_METER_POS+31, NO_CLIP);
	    clearviewport();
	    settextjustify(1,2);
	    settextstyle(0, 0, 1);
	    setcolor(c_text_bold);
	    sprintf(cc, "%.1lf dBm", dbm_level);
	    /* Rev 3.06 2 -> 3 digits in FM */
	    if (mode==FM)		/* If FM, overwrite pwr, rf or af */
		sprintf(cc, "%.3lf dBm", fm_log_pwr);
            moveto(64, 0);
	    outtext(cc);

	    sprintf(cc,"%d Hz", (int)(df[fft_bw]*(double)biggest_freq));
            moveto(64, 10);
	    outtext(cc);

	    /* Display v2f data */
	    if(v2f_on)  v2f(0);

	    /* After noncohave times, ready to update all spectral displays
	     * Unless diagnostics is blank, print the relative average power
	     * over freq range (281, 2400 Hz).
	     */
	    setviewport(579, 470, 638, 479, DO_CLIP);
	    clearviewport();
	    if (diag_mode>0)
		{
		/* 3*power_exp+30 corrects for scaling in dsp for 8 bit data */
		if (bits_8_16==0)		/* If using 8 bit log data */
		    lpave = p2db(pave_print)-23.692+3.0*(double)power_exp-30.0;
		else
		    lpave = p2db(pave_print)-23.692;
		settextjustify(RIGHT_TEXT, TOP_TEXT);
		settextstyle(0, 0, 1);
		setcolor(c_text_bold);
		sprintf(cc, "%.2lfdB", lpave);
		moveto(58, 0);
		outtext(cc);
		pave_print=0.0;		/* Accumulate over noncohave */
		}
	    if ((mode!=EME2 && do_data_save) || (mode==EME2 && eme_record))
	        {
		/* The uhfa_1.dat file gets too large for a floppy.
		 * It will be written to the default drive, if the path
		 * is to a floppy.
		 */
		if (fdata_path[0]=='A' || fdata_path[0]=='B')
		    do_t_error(50, "W50 Floppy Path", 0);		
		strcpy(cc, "UHFA_1  .DAT");
		/* If path exists, hang file name on the end of it */
		if (strlen(fdata_path)>1 &&
			fdata_path[0]!='A' && fdata_path[0]!='B')
		    {
		    strcpy(&cc[100], fdata_path);
		    strcat(&cc[100], &cc[0]);
		    }
		else
		    strcpy(&cc[100], &cc[0]);

	        if (NULL == ( data_file=fopen(&cc[100], "a+t") ))
		  do_t_error(47, "E47 DAT File", 200);
		else
		  {
		  setviewport(10,80,101,90,DO_CLIP);
		  clearviewport();
		  setcolor(c_text_note);
		  settextstyle(0, 0, 1);
		  settextjustify(1,2);
		  moveto(50,1);  outtext("RECORDING");
		  setcolor(c_text_bold);
		  if (mode==EME2)
		    {
		    if((eme_state > -1) && eme_record)
		      {
		      fprintf(data_file,
			"HA %d %d %d %02d%02d %d\n",
			tmnow->tm_year+1900, tmnow->tm_mon+1, tmnow->tm_mday,
			tmnow->tm_hour, tmnow->tm_min, tmnow->tm_sec);
	              fprintf(data_file, "HE %d %d %d 0 0 %E %E ",
		        rfgain, do_window, fft_bw, paveave, pave);
		      for(i=lt_i2; i<=lt_i4; i++)
			 fprintf(data_file, "%E ", pd16[i]);
		      fprintf(data_file, "\n");
		      }
		    }
		else		/* if mode not EME2 */
		    {
		    fprintf(data_file,
		      "HA %d %d %d %02d%02d %d\n",
		      tmnow->tm_year+1900, tmnow->tm_mon+1, tmnow->tm_mday,
		      tmnow->tm_hour, tmnow->tm_min, tmnow->tm_sec);
	            fprintf(data_file,
		      "HB %d %d %d %d %d %d %d %d %d %d\n",
		      fft_bw, rfgain, noncohave, do_window,
		      256*dsp_deltap_hi+dsp_deltap_lo,
		      4*mode+16*bits_8_16, 
		      0, 0, 0, 16);		/* Spares */
	    	    for (i=0; i<32; i++)
		      fprintf(data_file,
		        "%E %E %E %E %E %E %E %E\n",
		        pwrdat[8*i],pwrdat[8*i+1],pwrdat[8*i+2],pwrdat[8*i+3],
		        pwrdat[8*i+4],pwrdat[8*i+5],pwrdat[8*i+6],
		        pwrdat[8*i+7]);
		    }

		  fclose(data_file);
	          }
	        }			/* End data save to disk file */
	    else
		{
		/* Clear 'Recording' Note */
		setviewport(10,80,101,90,DO_CLIP);
		clearviewport();
		}

	    if(mode==LHL7 || mode==PUA43)
		 lhl7_timing();	/* Receive trig */
	    /* Dump non-coherent integrator to start a new line */
	    for  (i=0; i<512; i++)
		pwrdat[i]=1.0e-100;	/* Dump integrator; Rev 3.19 was 0 */
	    }				/* End display update */

	    /* Mark the display for data lost */
	    setviewport(X2R-2, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
	    if (data_lost && waterfall)
		{
		putpixel(0, scroll_line, 15);	/* Mark waterfall */
		putpixel(1, scroll_line, 15);
		data_lost=FALSE;	/* For next time */
		}
	}

void prep_display(void)
	{int i, x, y;

	/* If waterfall, put data at top, else below frequency dial */
	if (text_area)
	    {
	    setviewport(X1L+1, Y1T+33, X1R-1, Y1B-1, DO_CLIP);
	    clearviewport();
	    setcolor(c_outline);
	    for(i=0; i<=6; i++)
		{
		y = i*16;
		line(1, y, 639, y);	/* Horizontal grid lines */
		}
	    for(i=1; i<=9; i++)
		{
		x = (int)(53.229 * (double)i);
		line(x, 1, x, 96);		/* Vertical lines */
		}
	    }
	else
	    {
	    setviewport(X1L+1, Y1T, X1R-1, Y1B+1, DO_CLIP);
	    clearviewport();
	    setcolor(c_outline);
	    for(i=0; i<=8; i++)
		{
		y = i*16;
		line(0, y, 639, y);	/* Horizontal grid lines */
		}
	    for(i=1; i<=9; i++)
		{
		x = (int)(53.229 * (double)i);
		line(x, 0, x, 130);		/* Vertical lines */
		}
	    }

	/* For information, put the noise bandwidth in the upper right */
	if (waterfall)			/* Don't use for simple display */
	    {
	    sprintf(cc, "%.1f Hz", df[fft_bw]*noise_bw[do_window]);
	    if (text_area)
		setviewport(X1R-62, Y1T+53, X1R-3, Y1T+62, DO_CLIP);
	    else
		setviewport(X1R-62, Y1T+20, X1R-3, Y1T+29, DO_CLIP);
	    clearviewport();
	    settextjustify(RIGHT_TEXT, TOP_TEXT);
	    setcolor(c_text);
	    moveto(58, 1);	outtext(cc);
	    setcolor(c_text_bold);

	    if (beacon)
		show_beacon();
	    }

	set_disp_offset(0);		/* Update display, no dsp command */
	set_db_div(0);			/* Update display only */

	setcolor(c_text_bold);		/* Default */
	setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
	if(waterfall && ++scroll_line > 321) /* Scroll Waterfall */
		{
		scroll_line=312;
		getimage(0,  10, 510, 170, big_area_ptr1);
		getimage(0, 171, 510, 321, big_area_ptr2);
		setviewport(X2L+1, Y2B-30, X2R-1, Y2B-10, DO_CLIP);
		clearviewport();
		setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
		putimage(0,   0, big_area_ptr1, COPY_PUT);
		putimage(0, 161, big_area_ptr2, COPY_PUT);
		}
	}

void normalize(int npoints)
	{
	int i;
	double dnorm_big;

	/* Accumulate noise power to find frequency response */
	if (npoints==512)
	    /* For 512 points of 8 bit data from dsp: */
	    for (i=0; i<npoints; i++)
	    	dnorm[i] += (double)raw_data[i];
	else
	    /* For 256 points of 16 bit data from dsp: */
	    for (i=0; i<npoints; i++)
		dnorm[i] += (double)
		    ( (raw_data[2*i]+(raw_data[2*i+1]<<8)) >>7);

	if (--normalize_on == 1)	/* Used as flag and as a counter */
	    {
	    setviewport(0,90,101,100,DO_CLIP);
	    clearviewport();
	    settextstyle(0, 0, 1);
	    settextjustify(1,2);
	    moveto(50,1);
	    if (normalize_num<100.0) outtext("Fast Norm");
	    else outtext("Prec Norm");

	    dnorm_big=-32768.0;
	    for (i=0; i<npoints; i++)
		{
		dnorm[i]=dnorm[i]/(3.2*normalize_num);
		if (dnorm[i]>dnorm_big)  dnorm_big=dnorm[i];
		}
	    for (i=0; i<npoints; i++)
		{
		dnorm[i] -= dnorm_big;
		}
	    }
	}

/* Curve fit to noise response, y(x) where x is in steps of 1 over isize points
 * and y is dB response. Curve fit is done twice. First with all weights
 * set to 1. Then, a second fit is done where the weights are 1/resid^2
 * where resid is the difference between the measured point and the first
 * curve fit. This prevents a signal from pulling the curve fit out of line.
 * The resulting 4th order polynomial is curve_fit[i], i over (0,3); the
 * fitted curve to be used in normalization is global lt_dnorm[].
 * Revised to (lt_i1, lt_i5) 15 Dec 00 RSL
 */
void lt_normalize(void)
    {
    #define POLY_ORD 4

    double curve_fit[10];
    struct MAT      *At, *B, *At_Ai_At, *Z;
    int             i, j, l;
    int		    isize;		/* Number of points to fit */
    double          *x, *y, *w, *yf;
    double          **at, **b, **z;
    double          yfit, xpow;
    double          resid, rsum, rsqsum, sigma, weight;
    char            pline[100];
    char            *px, *py;

    isize=lt_i5-lt_i1+1;

    /* Allocate memory for arrays */
    w = (double*)calloc(isize, sizeof(double));
    if(!w)
	{
	do_t_error(41, "E41 Mem Alloc", 200);
	return;
	}
    x = (double*)calloc(isize, sizeof(double));
    if(!x)
	{
	do_t_error(41, "E41 Mem Alloc", 200);
	return;
	}
    y = (double*)calloc(isize, sizeof(double));
    if(!y)
	{
	do_t_error(41, "E41 Mem Alloc", 200);
	return;
	}
    yf = (double*)calloc(isize, sizeof(double));
    if(!yf)
	{
	do_t_error(41, "E41 Mem Alloc", 200);
	return;
	}
    for(i=0; i<isize; i++)
       {x[i]=(double)(i+lt_i1);            /* Integer, like 49, 50, ... */
        y[i]=p2db(lt_pwrdat[i+lt_i1]);     /* Measured power, in dB */
        w[i]=1.0;
       }

    /* Fourth order curve fit:    */
    if(fit_curve(curve_fit, x, y, w, isize, POLY_ORD) == 0)
	do_t_error(11, "E11 Curve Fit", 200);

    /* Calculate the least squares fit y values and the squared error */
    rsum=0.0;
    rsqsum=0.0;
    for(i=0;  i<isize;  i++)
        {
        yfit = curve_fit[0];
        xpow = x[i];
        for(j=1;  j<=POLY_ORD;  j++)
            {
            yfit += curve_fit[j]*xpow;
            xpow = xpow * x[i];
            }
        yf[i]=yfit;
        resid=y[i]-yfit;
        rsum += resid;
        rsqsum += resid*resid;
        }
    sigma=sqrt(rsqsum/(double)isize);
    weight=1.0/(sigma*sigma);		/* Use in loop */
    for(i=0;  i<isize;  i++)
        {
        resid=y[i]-yf[i];
        if (fabs(resid) > sigma)    /* Cut the weight on potential sigs */
            w[i]=1.0/(resid*resid);
        else
            w[i]=weight;
        }

    /* Compute again with weighted y[i]     */
    if(fit_curve(curve_fit, x, y, w, isize, POLY_ORD) == 0)
	do_t_error(11, "E11 Curve Fit", 200);

    /* Calculate the least squares y values and fill in normalization values */
    rsqsum=0.0;
    for(i=0;  i<isize;  i++)
        {
        yfit = curve_fit[0];
        xpow = x[i];
        for(j=1;  j<=POLY_ORD;  j++)
            {
            yfit += curve_fit[j]*xpow;
            xpow = xpow * x[i];
            }
	lt_dnorm[i+lt_i1]=yfit;
        }
    if (mat_err)  do_t_error(11, "E11 Curve Fit", 200);
    free(w);
    free(x);
    free(y);
    free(yf);
    }

/* pe(int ci)  -  A debug function to dump the current program execution
 * location onto the screen. Also dumps some variables if needed.
 * Generally call with pe('A'); etc to print a character 'A'. Set
 * dt1,dt2...dt5 to variables in memory to dump them when pe(1000);
 * is called.
 */
void pe(int ci)
    {
    static int ii=0;
    static int jj=0;

    if (ci==1000)
      {
      setviewport(140, 250, 630, 262, DO_CLIP);
      clearviewport();
      settextstyle(0, 0, 1);
      settextjustify(LEFT_TEXT, TOP_TEXT);
      moveto(1,1);
      sprintf(cc, "%9.3g %9.3g %9.3g %9.3g %9.3g", dt1, dt2, dt3, dt4, dt5);
      outtext(cc);
      return;
      }

    setviewport(140+10*ii, 350+jj, 150+10*ii, 362+jj, DO_CLIP);
    clearviewport();
    settextstyle(0, 0, 1);
    settextjustify(LEFT_TEXT, TOP_TEXT);
    moveto(1,1);
    sprintf(cc, "%c", (char)ci);
    outtext(cc);
    if (++ii > 45) 
	{ii=0;
	jj += 7;
	if (++jj > 100) jj=0;
	}
    }

/* do_transmit()  -  Here because the transmit/receive toggle was put into
 * the transmit position.  We will stay here until receive is again commanded.
 * There is no check sum in transmit. The check for data valid is only
 * that dsp_status[17]==2.
 */
void do_transmit(void)
	{
	int i;
	/* Too many status updates per second creates display problems */
	int alternate;
	int stop_now;
	int new_char, last_char;
	int esc_count;
	int nt;				/* Tone repeats for FSK tones */
	int code_mode;
	long int li;
	time_t t0, t0_last;			/* Time stuff for beacon */
	struct tm *tp;
	long int tL;
	int ri;

	code_mode=(mode==CW || mode==LHL7 || mode==PUA43 || mode==LTI);
	stop_now=FALSE;
	alternate=0;
	esc_count=0;			/* Start fresh R1.7 */

	if ((mode==LHL7 || mode==PUA43) && clock_been_set==FALSE)
	    do_t_error(12, "W12 Clock Set", 0);  /* Note, but continue */

	if (code_mode) goto cdmd;

 ppt:	/* First clear excess status input */
	while (ASYNCINSTAT() >60 )
		ASYNCIN();

	for(;;)				/* Wait for ESC_CHAR */
		{if ( ASYNCINSTAT() > 0 )
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR)  esc_count++;
			 break;
			}
		 while ( kbhit() )
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();

		    /* Check for PTT off via fast serial RI line */
		    if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
			{
			ptt=FALSE;
			if (transmit) t_r_toggle();	/* Go to receive */
			return;
			}
		    if (!transmit)  return; 	/* Back to Receive */
		    }
	/*	if (code_mode && transmit)
		    morse(SEND_CW, 0); */   /* Send ch, if available */
		}
	if(alternate++ <= 5)   goto ppt;

	alternate=0;
	for(;;)				/* Wait for 'T' */
		{last_char =new_char;
		 if(ASYNCINSTAT() > 0)
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR) esc_count++;	
			 if (esc_count==2)	/* Waiting for ESC_CHAR, 'T' */
				{esc_count=0;	/*    but got 2xESC_CHAR     */
				 goto ppt;	/* So try again              */
				}
			 if ((last_char==ESC_CHAR) && (new_char=='T'))
				break;
			}

		 while ( kbhit() )
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();
		    if (!transmit)  return;	/* Back to Receive */
		    }
	 /*   	 if (code_mode && transmit)
		      morse(SEND_CW, 0); */  /* Send ch, if available */

		/* Check for PTT off via fast serial RI line */
		if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
		    {
		    ptt=FALSE;
		    if (transmit) t_r_toggle();	/* Go to receive */
		    return;
		    }
		}

	/* Waiting for an almost full buffer eliminates the need for
	 * kbhit, PTT, etc calls in the last few status gets
	 */
	for(;;)				/* Wait for a buf full */
		{
		if ( ASYNCINSTAT() > 29 )	/* Almost full */
			 break;

		while ( kbhit() )
		    {
		    stop_now=do_cmd();
		    if (stop_now)  quit();
		    if (!transmit)  return;	/* Back to Receive */
		    }
	 /*   	if (code_mode && transmit)
		      morse(SEND_CW, 0); */   /* Send ch, if available */

		/* Check for PTT off via fast serial RI line */
		if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
		    {
		    ptt=FALSE;
		    if (transmit) t_r_toggle();	/* Go to receive */
		    return;
		    }
		}

	incount=-1;
	i=0;  esc_count=0;
	for(;;)				/* Get xmit_status */
		{if( ASYNCINSTAT() > 0 )
			{new_char = ASYNCIN();
			 ++incount;
			 if (new_char==ESC_CHAR) esc_count++;
			 if (esc_count==2)
				{esc_count=0;
				 goto t_skip_char;
				}
			 dsp_xmit_status[i]=new_char;
			 if (++i > 9)		/* 10 bytes */
				{if (esc_count==1) ASYNCIN();
				 break;
				}
 t_skip_char:		}
		/* Check for PTT off via fast serial RI line */
		if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
		    {
		    ptt=FALSE;
		    if (transmit) t_r_toggle();	/* Go to receive */
		    return;
		    }
		if (!transmit)  return;
		}

	i=0; esc_count=0;
	for(;;)				/* Wait for dsp_status[] */
		{if( ASYNCINSTAT() > 0 )
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR) esc_count++;
			 if (esc_count==2)
				{esc_count=0;
				 goto t_dstat1;
				}
			 dsp_status[i] = new_char;
			 if (++i > 17) 
				{if (esc_count==1) ASYNCIN();
				 break;
				}
 t_dstat1:		}
		/* Check for PTT off via fast serial RI line */
		if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
		    {
		    ptt=FALSE;
		    if (transmit) t_r_toggle();	/* Go to receive */
		    return;
		    }
		if (!transmit)   return;
		}

	if(dsp_status[17]==2)		/* Valid status data? */
	    {
	    if  ( ((dsp_status[16] & 0X10)==0) && ptt)  /* PTT to receive */
		{
		ptt=0;
		t_r_toggle();		/* Back to receive */
		return;
		}
	    diagnostics(1);		/* Update diagnostics */
	    }
	else
	    {
	    do_t_error(12, "E12 DSP XStatus", 20);


#if 0
 setviewport(130, 400, 638, 410, DO_CLIP);
 clearviewport();
 settextjustify(LEFT_TEXT, TOP_TEXT);
 moveto(0, 1);
 sprintf(cc, "%2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X ",
	dsp_status[0],dsp_status[1],dsp_status[2],dsp_status[3],dsp_status[4],
	dsp_status[5],dsp_status[6],dsp_status[7],dsp_status[8],dsp_status[9]);
 outtext(cc);
 moveto(270, 1);
 sprintf(cc, "%2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X",
	dsp_status[10],dsp_status[11],dsp_status[12],dsp_status[13],dsp_status[14],
	dsp_status[15],dsp_status[16],dsp_status[17],dsp_status[18]);

 outtext(cc);
#endif

	    }

	t0=update_clock(1) + dsp_clock_base - (long int)b_offset_sec;
	if (t0 != t0_last)
	    {
	    print_time();
	    t0_last=t0;
	    if (beacon)
	       {
	       tp=gmtime(&t0);		/* Get struct that includes .tm_sec */
	       tL=(long int)tp->tm_sec + 60L*(long int)tp->tm_min
			+ 3600L*(long int)tp->tm_hour;
	       ri = (int)(tL % (long int)(b_transmit_sec + b_receive_sec));
	       if (ri >= b_transmit_sec)
		  {
		  /* Time to receive */
		  if (transmit)    t_r_toggle();	/* Go to receive */
		  return;
		  }
	       }
	    if (xnoise)
		{
		setviewport(72, 278, 81, 288, NO_CLIP);
		if (xnoise_disp_toggle)
		    {
		    settextjustify(LEFT_TEXT, TOP_TEXT);
		    settextstyle(0, 0, 1);
		    setcolor(c_text_note);
		    moveto (0, 1);
		    outtext("N");
		    }
		else
		    clearviewport();
		xnoise_disp_toggle ^= 1;	/* For next time */
		}
	    }	
	goto ppt;

	/* Now the loop for codes requiring input CW, LHL-7, PUA43, LTI */
cdmd:
	/* First clear status input */
	while (ASYNCINSTAT() >0 )
		ASYNCIN();
	while ( kbhit() )
	    {
	    do_cmd();			/* Execute allowable kbd commands */
	    if (transmit==0) return;	/* Back to Receive */
	    }
	/* Check for PTT off via fast serial RI line */
	if ((0X4000 & ASYNCSTAT()) && hardware_ptt && ptt)
	    {
	    ptt=FALSE;
	    if (transmit) t_r_toggle();	/* Go to receive */
	    return;
	    }
	morse(SEND_CW, 0); /* Send ch, if available */
	if (!transmit) return;		/* In case of '!' */
	t0=update_clock(1) + dsp_clock_base - (long int)b_offset_sec;
	if (t0 != t0_last)
	    {
	    print_time();
	    t0_last=t0;
	    if (beacon)
	       {
	       tp=gmtime(&t0);		/* Get struct that includes .tm_sec */
	       tL=(long int)tp->tm_sec + 60L*(long int)tp->tm_min
			+ 3600L*(long int)tp->tm_hour;
	       ri = (int)(tL % (long int)(b_transmit_sec + b_receive_sec));
	       if (ri >= b_transmit_sec)
		  {
		  /* Time to receive */
		  if (transmit)    t_r_toggle();	/* Go to receive */
		  return;
		  }
	       }
	    if (xnoise)
		{
		setviewport(72, 278, 81, 288, NO_CLIP);
		if (xnoise_disp_toggle)
		    {
		    settextjustify(LEFT_TEXT, TOP_TEXT);
		    settextstyle(0, 0, 1);
		    setcolor(c_text_note);
		    moveto (0, 1);
		    outtext("N");
		    }
		else
		    clearviewport();
		xnoise_disp_toggle ^= 1;	/* For next time */
		}
	    }	
	goto cdmd;
	}

/* eme_timing()  can be called anytime.
 * For transmit initiation, checks to see if millisec has just
 * passed a multiple of 5000. If so, transmit is started
 * The eme_state is set to 0 indicating transmit. If 10 minutes
 * has passed, an ID is transmitted.
 * Ends transmit pulse after 2 seconds.
 * For receive, waits until 2.56 sec has elapsed and triggers a fft
 * measurement.
 *
 * eme_state: 0  Transmit pulse started, receive continuing of last pulse
 *            1  Transmit pulse on, receive and screen update completed
 *            3  After 2.56 sec, Receive FFT started, waiting for transmit
 *            4  Special case: Initial wait for transmit pulse
 *           -1  Special case: EME2 selected, but in idle, no transmitting
 */
void eme_timing(void)
	{
	int i;
	unsigned char c, *idptr;	/* For sending CW ID */

	if (eme_state == -1) return;

	i=(int)update_clock(1);		/* integer seconds */

	if ( (i%540)==0 )
	    {
	    if (!transmit) t_r_toggle(); /* To transmit */
	    idptr=&code_id[0];
	    while ((c=*idptr++) != 0)
		morse_ch(c,23);		/* Send ID at 25 wpm */
	    if (transmit) t_r_toggle(); /* To receive */
	    while( ((int)update_clock(1))%5 ) ;	/* Wait out 5 sec interval */
	    return;
	    }
	i=i%5;				/* Remainder after div by 5 */

	/* The following may get retriggered 1 second into the 5 sec
	 * sequence. This happens if a screen update finished before the
	 * end of the 5th sec. This is OK as it will still be caught
	 * at the end of the 2nd second to turn the pulse off. The only
	 * problem is when processing is too slow to finish before
	 * the end of the 2nd second.
	 */
	if (i==0 && eme_state>0)	/* Every 5 seconds */
	    {
	    /* Put dsp into transmit, offset by doppler */
	    eme_state=0;		/* Mark the activity */
	    if (!transmit) t_r_toggle();  /* To transmit */
	    dsp_cmd('T', 0X0011, 2, fgainl[xmit_power], fgainh[xmit_power]);
	    }
	/* Normally, here with eme_state==1, waiting for i=2.
	   If eme_state is 0, the update failed, but we need to move on.
	 */
	if ( (eme_state==0 || eme_state==1) && i==2 )
	    {
	    /* Put dsp into receive */
	    eme_state=2;		/* Mark the activity */
	    dsp_cmd('T', 0X0001, 2, fgainl[xmit_power], fgainh[xmit_power]);

	    if (transmit) t_r_toggle();	/* Into receive */
	    /* Transmit pulse is finished - at 2.56 sec, trigger receive */
	    /* The delay is shortened by T/R sequencing delays */
	    delay(dly_emet2r);		/* Sit here during eme_state=2 (400) */
 /*<<<<<< Clear FFT??????? */
	    fft_fcn |= 0X0400;		/* Trigger FFT for data collection */
	    dsp_cmd('F', fft_fcn, fft_fcn>>8, 3, 4);
	    fft_fcn &= 0XFBFF;		/* So next guy doesn't trigger */

	    beep(5);			/* Little click */
	    integrate_count=0;		/* Keep display timing in sync */
	    eme_state=3;		/* Receive/process data & next xmit */
	    diagnostics(1);		/* Update screen */
	    }
	}

/* lhl7_timing()  can be called anytime, during receive.
 * Checks to see if sec has just passed a multiple of 2.
 * If so, another receive fft sequence is started. Note that DSP only needs
 * a start trigger and then it finishes up and waits for trigger by itself.
 *
 * lhl7_state keeps track of where we are.  This requires a call to this
 * routine at least once per sec. Also used in exactly the same way for PUA43.
 *   lhl7_state=0   Waiting for start of trigger sequence
 *   lhl7_state=1   Triggered, but during first second
 * Omit data collection at 58 seconds, to allow the timing to catch up if
 * needed, and reset dsp. ver 3.29
 */
void lhl7_timing(void)
	{
	int i, ri;
	static long int  t_last=-1L;
	long int       t_new;
	struct tm      *tp;

	if (transmit) return;

	t_new=update_clock(1);
	if (t_new == t_last)   return;

	t_last=t_new;			/* For next time */
        t_new += dsp_clock_base;	/* Reuse t_new */
	tp=gmtime(&t_new);		/* Get struct that includes .tm_sec */
	ri = tp->tm_sec % 2;		/* Remainder after div by 2 */

	if (lhl7_state==0 && ri==0 && tp->tm_sec<58)	/* Every 2 seconds */
	    {
	    fft_fcn |= 0X0400;		/* Software trigger DSP */
	    /* 4*tp->tm_sec is DSP time tag */
	    dsp_cmd('F', fft_fcn, fft_fcn>>8, 4*tp->tm_sec, 4);
	    fft_fcn &= 0XFAFF;		/* Rev 3.28, was FBFF */

	    setviewport(90, 470, 127, 479, DO_CLIP);
	    clearviewport();
	    if(!eme_doppler)				/*<<<<<< */
		{
		settextstyle(0, 0, 1);
		settextjustify(RIGHT_TEXT, TOP_TEXT);
		moveto(36, 0);
		sprintf(cc, "R:%d", tp->tm_sec); /* Show trig sec */
		outtext(cc);
		}
	    if (pua43_tick) beep(tick_time);	/* Click if wanted */
            if (lpt_port != 0)		/* Hardware trig out */		
               {lpt_dat |= 1;
		outp(lpt_port, lpt_dat);
                delayu(ntau);
		lpt_dat &= 0xFE;
                outp(lpt_port, lpt_dat);
               }
	    lhl7_state=1;		/* This prevents a retrigger */
	    }
	if (tp->tm_sec==59) /* One sec early (3.29)*/
	    {
	    /* Clear/reset triggered measurement.  */
	    fft_fcn &= 0XFBFF;		/* Clear trigger, preserve others */
	    fft_fcn |= 0X0100;		/* OR in a clear of current meas */
	    dsp_cmd('F', fft_fcn, fft_fcn>>8, 0XFF, 4);
	    fft_fcn &= 0XFEFF;		/* AND off the clear (bit 8) */
	    }
	if (tp->tm_sec==0)		/* Start of a new minute */
	    {
	    code_min=tp->tm_min;	/* Used to randomize pua43 code */
	    code_min += (60*tp->tm_hour);
	    }
	if (ri==1)			/* On odd min, reset lhl7_state */
	    {
	    lhl7_state=0;		/* Only one trig per 2 sec */
	    }
	}
	
void print_stars(void)
	{
	 unsigned int cnt;
	 for(cnt = 0; cnt < 79; cnt ++)
		  printf("*");
	 }

/* Display "RECEIVE" or "TRANSMIT" */
void show_tr(int tr, int color)
	{
	char  ct[10];
	if(tr)
	    sprintf(ct, "%s", "TRANSMIT");
	else
	    sprintf(ct, "%s", "RECEIVE");
	if (color==2)
	    setcolor(c_text_note);	/* Red */
	else if (color==1)
	    setcolor(c_text);		/* Beige */
	else
	    setcolor(c_text_bold);	/* White */
	setviewport(0,60,101,70,DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(1,2);
	moveto(50,1);	outtext(ct);
	setcolor(c_text_bold);
	}

/* t_r_toggle()   - Responds to hitting t/r toggling key.  Adds sequencing
 * for external antenna relays and power amps. Bits are controlled by
 * uint shift_reg with  0=CW on, 1=trans, 2=recv, 3=PA enable,
 * 4=Ant relay transmit.
 */
void t_r_toggle(void)
	{
	static unsigned int save_rfgain;
	double n, dph_sign;
	unsigned int cmd1, cmd2, save_dph, beep_ph;
	
	cw_on = 0;			/* Start with cw key up */
	transmit ^= 1U;			/* Toggle transmit/receive */

	if (transmit)
		{
		save_rfgain=rfg[rfgain].gainbyte;
	 	rfg[rfgain].gainbyte=0;	/* Disconnect receiver from antenna */
		show_tr(1, 0);		/* Transmit, White */

 /* Bob-There are 3 new_freq()'s - Is one at beginning adequate?? */
		new_freq();		/* T & R freq may be different */
		/* Erase bargraphs  if not active */
		if (diag_mode==2 &&
		   (mode==CW || mode==LTI || mode==LHL7 ||
				 mode==EME2 || mode==PUA43))
		    {
		    setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
		    clearviewport();
		    }
		/* If noise is being added for signal simulation
		   we need a value. S/N refers to a 10 Hz BW (arbitrary):
		 */
		if (xnoise)
		    {
		    /* 10*log10(24000/10)=33.802  Bandwidth
		       20*log10(0.9)=-0.915       CW voltage reduction in dsp
		     */
		    n=32767.0*3.4641*fgain[xmit_power] /
			pow(10.0, ( (xsnr-33.802+0.915) / 20.0));
		    if (n>32767.0)
			{
		    	do_t_error(50, "W50 S/N value", 20);
			n=32767.0;
			}
		    dsp_cmd('N',  near_int(n),  near_int(n) >> 8,  0,  0);
		    }
		else
		    {
		    dsp_cmd('N',  0,  0,  0,  0); /* Turn off noise gen */
		    }
		if (mode==LTI)
		    {
		    morse(SHOW_BUF, 0);	/* Display buffer over Yellow trace */
		    }
		dsp_cmd(10, 0, 0, 0, 4);	/* Set audio gain to 0 */
		if (hardware)
		    {  
		    shift_reg = 0X0010;    /*(shift_reg & 0X00E0) | 0X0010;*/
		    send_shift_data(R10MHZ); /* Ant ry to transmit, recv off */
		    beep(10); delay(dly_ant2amp);
		    shift_reg=0X0018;    /* |= 0X0008;*/
		    send_shift_data(R10MHZ);	/* Enable power amp */
		    show_tr(1, 1);		/* Transmit, Beige */
		    delay(dly_amp2xmit);
		    shift_reg=0X001A;    /*  |= 0X0002; */
		    send_shift_data(R10MHZ);	/* Hardware into transmit */
		    delay(10);
		    }
		else if (lpt_port != 0)	/* Alternate T/R control (Rev 3.39) */
		    {
		    lpt_dat |= 2; outp(lpt_port, lpt_dat); /* Ant Ry */
		    dsp_cmd(10, 0, 0, 0, 4);	/* Set audio gain to 0 */
		    beep(10);
		    delay(dly_ant2amp);
		    lpt_dat |= 4; outp(lpt_port, lpt_dat); /* Amp */
		    show_tr(1, 1);		/* Transmit, Beige */
		    delay(dly_amp2xmit);
		    lpt_dat |= 8; outp(lpt_port, lpt_dat); /* RF ready */
		    }
		else			/* Audio Processor w/o LPT */
		    {
		    dsp_cmd(10, 0, 0, 0, 4);	/* Set audio gain to 0 */
		    beep(10);
		    delay(dly_ant2amp);
		    delay(dly_amp2xmit);
		    }
		show_tr(1, 2);		/* Transmit, Red */
		dsp_cmd('T', 0X0003, 2,         /* Xmit, t status, key up */ 
		    fgainl[xmit_power], fgainh[xmit_power] );
		if (mode==USB || mode==LSB || mode==FM)
		    {
		    draw_clip();		/* Bar graph for clipper */
		    if (mode != FM && xmit_power > 95)
			do_t_error(17, "W17 Excess Pwr", 0);
		    }

		/* Finally, for FM there can be an initial tone beep, or a
		 * continual CTCSS tone:     */
		if (mode==FM)
		    {
		    if (vfo[cur_vfo].t_fm_use==FMTONE_TONE) /* On & not DTMF */
			{
			ttn_freq1=vfo[cur_vfo].t_fm_freq;
			ttn_level1=vfo[cur_vfo].t_fm_level;
			ttn_level2=0;	/* Only one tone */
			set_ttn(2);	/* Set freq and level */
			ttn_control=5;  set_ttn(1); /* Tone plus voice */
			if (vfo[cur_vfo].t_fm_dur < 9999)  /* Not continuous */
			    {
			    delay(vfo[cur_vfo].t_fm_dur);
			    ttn_control=4;  set_ttn(1);  /* To voice only */
			    }
			}
		    else
			{		/* Turn off tone, voice only */
			ttn_control=4;  set_ttn(1);
			}
		    }
		}
	else				/* if receive */
		{
		show_tr(1, 1);		/* Transmit, Beige */
		if ((beep_use & 1) && (mode==USB || mode==LSB))  /* End beep */
		    {
		    if(beep_use & 2)	/* Use side tone */	
			{
			beep_ph=(unsigned int)(1.36533333*beep_freq);
			dsp_cmd('H', beep_ph, beep_ph>>8,
			    fgainl[sidetone_lev], fgainh[sidetone_lev]);
			}
		    else
			{
			beep_ph=(unsigned int)(1.36533333*beep_freq);
			dsp_cmd('H', beep_ph, beep_ph>>8,
			    0, 0);
			}
		    save_dph=dsp_dph;
		    if (dsp_usb) dph_sign=1.0;  else dph_sign=-1.0;
		    dsp_dph= (unsigned int) ((int)dsp_dph +
			(int)(dph_sign*1.36533333*beep_freq) );
		    /* In 7 cmd, byte 2 is strobe on shift regs. Set to 0 */
		    dsp_cmd(7, 0, 1, 0XFF & dsp_dph, 0XFF & (dsp_dph >> 8) );
		    /* Set mode to CW to send beep */
		    cmd1=1;		/* 1=CW */
		    /* Set level per mike gain, and beep_level  */
		    if (   (mode==USB && tvtr[band].sb>0)
			|| (mode==LSB && tvtr[band].sb<0)
			|| band==AUDIO_BAND)
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			else
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
		    dsp_cmd('M', cmd1, cmd2,
			 fgainl[mike_mult], fgainh[mike_mult]);
		    dsp_cmd('T', 0X0013, 2,	    /* Key down */
			fgainl[xmit_power+beep_level-100],
			fgainh[xmit_power+beep_level-100] );
		    delay(beep_dur);	/* In millisecs */
		    dsp_cmd('T', 0X0013, 2,	    /* Key up */
			fgainl[xmit_power],
			fgainh[xmit_power] );
		    cmd1=0;		/* 0=SSB, cmd2 is already set above */
		    dsp_cmd('M', cmd1, cmd2,
			 fgainl[mike_mult], fgainh[mike_mult]);
		    dsp_dph=save_dph;
		    dsp_cmd(7, 0, 1, 0XFF & dsp_dph, 0XFF & (dsp_dph >> 8) );

		    /* Restore CW settings for sidetone */
		    beep_ph=(unsigned int)(1.365333333*(double)cwoffset);
		    dsp_cmd('H', beep_ph, beep_ph>>8,
			fgainl[sidetone_lev], fgainh[sidetone_lev]);
		    }		/* End Hermann Beep */

		if (mode==FM) ttn_control=4;  set_ttn(1);  /* Clear display */

		/* Get back out of transmit to allow receiving command */
		dsp_cmd('T', 0X0002, 2,        /*Rcv, t status, key up  */ 
		    fgainl[xmit_power], fgainh[xmit_power] );
		new_freq();		/* T & R freq may be different */
		if (hardware)
		    {
		    shift_reg = 0X0018;
		    send_shift_data(R10MHZ);	/* Hardware xmit off */
		    delay(dly_xmit2amp);
		    shift_reg = 0X0010;
		    send_shift_data(R10MHZ);	/* Amp off */
		    show_tr(1, 0);		/* Transmit, White */
		    delay(dly_amp2ant);
		    shift_reg = 0X0000;
		    send_shift_data(R10MHZ);	/* Ant relay to receive */

		    delay(2);
	 	    rfg[rfgain].gainbyte=save_rfgain;  /* Restore rcvr gain */
		    shift_reg = 0X0004;
 /* Bob-are these needed or can new_freq() do the last  of these?? */
		    send_shift_data(R10MHZ);	/* Hardware into receive */
		    }
		/* LPT port can be used to control an external transceiver
		 * if DSP-10 hdw is not present, i.e., the audio processor */
		else if (lpt_port != 0)	/* Alternate T/R control (Rev 3.37) */
		    {
		    lpt_dat &= 0xF7;  outp(lpt_port, lpt_dat); /* RF off */
		    delay(dly_xmit2amp);
		    lpt_dat &= 0xFB;  outp(lpt_port, lpt_dat); /* Amp off */
		    show_tr(1, 0);		/* Transmit, White */
		    delay(dly_amp2ant);
		    lpt_dat &= 0xFD;  outp(lpt_port, lpt_dat); /* AntRy off */
		    }
		else			/* Audio processor w/o LPT */
		    {
		    delay(dly_xmit2amp);
		    delay(dly_amp2ant);
		    }
		delay(20);  set_afgain(0);	/* Pop over? Restore AF gain */
		show_tr(0, 0);		/* Receive, White */
		error_on_tr=data_omit_tr;	/* Prevent error after T/R change */
		draw_s_meter();
		}			/* End of 'if receive'  */

	new_freq();			/* T & R freq may be different */
	}

/* rit_toggle()  -  Turns RIT on and off.  When turned back on the old
 * offset is restored.  Not restored after shutdown.
 */
void rit_toggle(void)
	{
	rit_on ^= 1;			/* Toggle rit on/off */
	if (rit_on)
		{
		rit = rit_save;
		}
	else
		{
		rit_save = rit;
		rit = 0;
		}
	new_freq();
	}

/* new_vfo(int) - Selects a vfo, now separate from tvtr[].
 * Call with the desired vfo number (0,20) in nvfo. This function will
 * change cur_vfo. More info, see new_band( ) below.
 * vfo[] includes an offset for FM, but this is picked up directly
 * from vfo[].offset.      V3.28     */
void new_vfo(int nvfo)
	{
	static char *v[]={"A", "B", "C", "D", "9", "E", "F", "G", "H", "I",
	    "J", "K", "L", "M", "N", "O", "1", "2", "3", "4", "5"};

	/* See if a transverter will be available, and cancel the new VFO
	 * if it doesn't exist   */
	if (tvtr[vfo[nvfo].tvtr].lo_type==999)
	    {
	    do_t_error(14, "W14 No Tvtr", 0);
	    return;			/* No transverter for this VFO */
	    }
	if (nvfo<=20) v_m=1;   else v_m=0;	/* VFO or Memory indicator */

	/* Save old freq and mode, as they may have changed. */
	if (v_m)			/* VFO's track the dial settings */
	    {
	    vfo[cur_vfo].last_tfreq=(double)tfreqL;
	    vfo[cur_vfo].last_mode=mode;
	    }

	/* Make nvfo the current vfo, cur_vfo.  */
	cur_vfo=nvfo;

	/* Display V or M in upper left corner    */
	setviewport(1, 3, 48, 13, DO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(0,0);
	if (v_m)
	    {
	    sprintf(cc, "V:%s", v[cur_vfo]);
	    outtext(cc);
	    }
	else
	    if (mem_incl_fr_md)
		{
		sprintf(cc, "M:%d", cur_mem);
		outtext(cc);
		}
 
	tfreqL=(long int)(0.5+vfo[cur_vfo].last_tfreq); /* The new VFO freq */
	mode=vfo[cur_vfo].last_mode;	/* The new vfo mode */
	set_mode(0);
	new_band(vfo[cur_vfo].tvtr);  	/* new_freq() is in new_band() */
	}

/* new_band(int nband)  -  Changes transverter parameters to nband.
 * Before V3.28 a tvtr[] and a VFO were the same. In prep for adding memories
 * these two are split apart. Now the tvtr[] defines the hardware of a
 * transverter. vfo[] selects a tvtr[], sets the frequency, frequency offset
 * and mode.  The number of each is the same (21) but more than one vfo[] can
 * be assigned to a tvtr[]. Later memories will also be assigned to tvtr[].
 */
void new_band(int nband)
	{
	setviewport(49, 3, 113, 13, DO_CLIP);
	clearviewport();
	setcolor(c_text_note);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(64, 0);
	sprintf(cc, "%s", tvtr[nband].name);
	outtext(cc);
	setcolor(c_text_bold);

	band=nband;
	new_freq();  

	if (diag_mode==3) set_cur_loc(-1);	/* Update doppler EME data */

	setcolor(c_text_bold);
	setviewport(2,100,104,110,DO_CLIP);
	clearviewport();
	if (band==AUDIO_BAND && eme_doppler)	/* Only for audio processor */
	    {
	    sprintf(cc, "Fd=%dMHz", near_int(0.000001*f_eme) );
	    settextstyle(0,0,1);
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    moveto(0, 0);
	    outtext(cc);
	    }
	}

/* Enter new frequency from a little box. V2.09 24 Jan 02 
 */
void freq_box(void)
	{
	static struct edit_info   el1;
	char eline1[20];
	double ed_freq;

	lb_create(&lb_newfreq, "FREQUENCY MHZ", 180, 175, 366, 260, 0, MODAL);
	lb_open(&lb_newfreq, 1);

	/* Fixed text in modal box:     */
	lb_line(&lb_newfreq, 5, BOLD, "FREQ: Backspace & Type", 0);
	lb_line(&lb_newfreq, 6, BOLD, "ENTER=Close Dialog", 0);

	/* The field to be edited:    */
	ed_freq=tvtr[band].lo_freq-((double)tvtr[band].sb)*(double)tfreqL;
	if (mode==CW)
	    ed_freq += (double)cwoffset;
	ed_freq *= 0.000001;		/* Convert to MHz */

	sprintf(eline1, "%.6lf", ed_freq);
	lb_create_el(&el1, 14, 26, &eline1[0],
		ED_ACTIVE, 0, strlen(eline1) );
	lb_line(&lb_newfreq, 2, EDIT, "", &el1);

	lb_update(&lb_newfreq);

	/* Now edit the active, cursor on, field:  */
	lb_edit_line(&lb_newfreq, &el1);

	/* As we exit, the variables must be changed to agree with the
	 * indicated states of the buttons and edit fields:
	 */
	ed_freq = 1000000.0*atof(eline1);
	if (mode==CW)
	    ed_freq -= (double)cwoffset;
	tfreqL = (long)(0.5+ed_freq) + (long)tvtr[band].sb*tvtr[band].lo_freq;
	lb_close(&lb_newfreq, 1);
	new_freq();
	}

/* The first conversion synthesizer freq is set by the long int in the
 * call to send_shift_data. The dsp lo is set by global uint's dsp_deltap_lo
 * and dsp_deltap_hi.  These are sent to the dsp by the '7' command.
 * For transmit the above frequencies are calculated here, based on
 *      global transmit		   1=transmit, 0=receive
 *      global tfreqL		   Transmit carrier freq in Hz at 2 meters
 *					(except FM, where it is rcvr freq
 *					 with .control for transmit offset)
 *	global tvtr[band].lo_freq  Offset of displayed freq due to transverter
 *	global tvtr[band].sb	   -1 if i-f=rf-lo, else +1
 *		>>global tvtr[band].control  FM offset (FM Only)<< Out 3.28
 *	global vfo[cur_vfo].tr_offset  FM offset (FM Only)      << New 3.28
 *	global mode		
 * For receive, all the above are used, plus
 *	global rit_on		1 if RIT is on, else 0
 *	global rit		Offset in Hz, +RIT raises the receiver freq
 *	global cwoffset		Receive is usb and the displayed frequency
 *				  is higher by cwoffset than the zero-beat.
 * The displayed receive frequency includes RIT which is also displayed
 * separately. When the receive frequency is not obvious, such as with Doppler
 * correction or random stir, it is displayed below the transmit freq.
 *
 * DSP lo freq ranges from 12.501 to 17.500 kHz as main synthesizer
 * goes in 5 KHz steps.
 */
void new_freq(void)
	{
	int i, x;
	unsigned int cmd2;
	double synth_freq, dt_freq;
	double et_freq;			/* Receive frequency for display */
	long int    disp_MHz;		/* MHz part of display */
	long int    disp_Hz;
	long int    disp_f;
	long int    aL, bL, xfreqL;

	double      freq_eme2_base;

	static long int   int_freqL=0L;
	static long int   synth_freqL=0L;
	static long int   n_synthL=0L;

	static int  last_time_transmit=FALSE;
	int         lock_synth;		/* TRUE=Don't change synth */
	static long int  last_n_synthL = 0;

	/* These are variables for time keeping for EME doppler */
	time_t t0;
	double eme_dop;
	/* And time keeping for LTI randomization  */
	long int       t_new;
	struct tm      *tp;

	/* If in transmit, don't change the synthesizer.
	 * But, retuning in these modes (while
	 * transmitting) can move one out of passband--so limit excursion */
	if (!transmit)			/* In receive */
	    {
	    lock_synth=FALSE;
	    last_time_transmit=FALSE;
	    }
	else 				/* In transmit */
	    {
	    lock_synth=last_time_transmit;	/* Lock if return to here */
	    last_time_transmit=TRUE;
	    }
	
	if (rit_on)
		setcolor(c_text_bold);
	else
		setcolor(c_text);
	setviewport(0,303,126,312,DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(2,2);
	moveto(126,0);
	sprintf(cc, "%d", rit);
	outtext(cc);
	settextjustify(0,2);
	moveto(2,0);
	outtext("RIT");
	setcolor(c_text_bold);

	if (!waterfall)
	    {
	    setviewport(X2R-47, Y2T+20, X2R-1, Y2T+40, DO_CLIP);
	    clearviewport();
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    moveto(23, 1);
	    outtext("RIT");
	    moveto(23, 11);
	    if (rit_on)
		outtext(cc);
	    else
		outtext("Off");
	    }

	/* Calculate 'xfreqL', the carrier frequency needed at 2 meters
	 * Note that the transmit frequency, kept in global tfreqL is the base
	 * from which all frequencies are derived. It should NEVER be
	 * modified here. Thus the need for xfreqL.
	 */
	xfreqL = tfreqL;			/* Base for all freqs */
	if (!transmit && rit_on)
	    xfreqL += (long int)rit;	/* For receive only */
	if (transmit && mode==CW)
	    xfreqL += (long int)cwoffset;
	if (transmit && (mode==LHL7 || mode==PUA43))
	    xfreqL += (long int)fsk;
	if (transmit && mode==FM)	/* FM Offset */
	    xfreqL += 1000L*(long int)vfo[cur_vfo].tr_offset;  //v3.28

	/* In EME2 the randomizing must occur once per 5 sec period. It must
	 * be the same value for T & R. random_freqL is global and survives
	 * this function call.
	 */
	if (transmit && mode==EME2 && eme_state==0)
	    {
	    if (eme2_format & 2)
		random_freqL=(long int)
		    (eme_freq_range*0.000030518*(double)(rand()-16384));
	    else
		random_freqL=0L;
	    }

	/* Base freq, 323, 600 or 900 Hz */
	freq_eme2_base=df[fft_bw]*(double)lt_i3;

	if (mode==EME2)
	    {
	    fsk=(int)random_freqL+(int)freq_eme2_base;
	    set_fsk(2);
	    if(transmit)
		xfreqL += random_freqL+(long int)freq_eme2_base;
	    else
		xfreqL += random_freqL;
	    }

	/* Several conditions require EME doppler. This, in turn, needs
	 * UTC. For all users this is calculated here.
	 */
	if(!transmit && ((mode==EME2 && (eme2_format&1)) || eme_doppler))
	    {
	    if (ut_available)
		{
		if (ms_available)
		    t0=update_clock(1)+dsp_clock_base;
		else
		    (void)time(&t0);
		/* Convert to TDT for Moon calculations */
		cnvut2tdt(&t0, &tdt);	/* t0 is local, tdt is global struct */
		eme_dop=update_moon(&tdt, 1);	/* 1 for display update  */
		}
	    else
		{
		do_t_error(15, "E15 No UTime", 200);
		eme_dop=0.0;
		}
	    }

/*>>>>>TEMP */
	if(transmit && ((mode==EME2 && (eme2_format&1)) || eme_doppler))
	    {
	    if (ut_available)
		{
		if (ms_available)
		    t0=update_clock(1)+dsp_clock_base;  /* Get clock time */
		else
		    (void)time(&t0);
		/* Convert to TDT for Moon calculations */
		cnvut2tdt(&t0, &tdt);	/* t0 is local, tdt is global struct */
		update_moon(&tdt, 1);	/* 1 for display update  */
		}
	    else
		do_t_error(15, "E15 No UTime", 200);
	    }

	/* To use the EME doppler, (a)must be receive, (b)must be activated
	 * if EME2 mode, or eme_doppler must be true if other modes.
	 */
	if (!transmit && diag_mode==3 && 
	    ((mode==EME2 && (eme2_format & 1)) || eme_doppler))
	        xfreqL += (long int)eme_dop;

	/* LTI mode gets randomized every minute, both T & R. This is
	 * updated here using data from lti_freq().
	 */
	if (mode==LTI)
	    {
	    t_new=update_clock(1) + dsp_clock_base;
	    tp=gmtime(&t_new);		/* Need sec of min */
	    code_min=tp->tm_min;	/* Randomize (like pua43) */
	    code_min += (60*tp->tm_hour); /* lti_freq() needs this */
	    fsk=(int)(lti_freq()+0.5);  /* Set code freq */
	    if (transmit)
		{
		xfreqL += (long int)fsk;  /* Includes lt_i3 cntr and random */
		set_fsk(2);		/* Display fsk */
		}
	    else
		{
		xfreqL += (long int)lti_frandom; /* random only */
		set_fsk(0);		/* Clear, no display */
		}
	    }

	/* Receive frequency for case of offsets such as Doppler or random */
	setviewport(9, 162, 119, 171, DO_CLIP);
	clearviewport();
	settextjustify(CENTER_TEXT,TOP_TEXT);
	settextstyle(0, 0, 1);
	setcolor(c_text_bold);
	moveto(55,1);

	/* et_freq is receive freq for display only. Shown during receive,
	 * below xmit freq, if there is a modification from transmit freq */
	if (!transmit &&
		(mode==EME2 || (rit_on && rit!=0) ||
		eme_doppler || (mode==LTI && lti_freq_range>0.4)))
	    {
	    /* et_freq is for display purposes only */
	    if (band != AUDIO_BAND)		/* For rf processing */
		{
		et_freq=tvtr[band].lo_freq-
		    ((double)tvtr[band].sb)*(double)xfreqL;
		sprintf(cc, "%.6lf", 0.000001*et_freq);
		}
	    else				/* Audio Processor */
		{
		et_freq=(double)xfreqL;
		sprintf(cc, "%.0lf Hz", et_freq);
		}
	    outtext(cc);
	    }

	/* Display an R if rcvr freq is randomized */
	show_er();

	/* The insides of the frequency display box for waterfall:    */
	setviewport(2,124,125,156,DO_CLIP);
	clearviewport();
	settextjustify(1,2);
	settextstyle(0, 0, 1);
	setcolor(c_text_bold);
	moveto(62,2);

	if (band != AUDIO_BAND)			/* For rf processing */
	    {
	    /* Correct for reference frequency error, first:  */
	    xfreqL=(long int)(0.5+ref_offset*(double)xfreqL);
	    if (!lock_synth)	/* Don't change synth if still in transmit */
		{
		/* int_freqL is desired carrier freq at 2-meters, quantized to
		 * the next lower 5 kHz.  The differences for USB and LSB
		 * are due to birdie avoidance at 9600 and 19200 Hz. 
		 */
		if (mode==FM)
		    int_freqL=5000L*((xfreqL+2500L)/5000L);
		else if (dsp_usb)
		    int_freqL=5000L*((xfreqL+250L)/5000L);
		else			/* if dsp I-F is LSB */
		    int_freqL=5000L*((xfreqL+4000L)/5000L);

		synth_freqL=int_freqL-19665000L;
		n_synthL=synth_freqL/5000L;
		}
	    /* dff is a number, close to 1.0 to correct for crystal freq
	     * errors in EZ-Kit 24.567 MHz osc. Default is 1.0 and is
	     * set at startup from ezkit_dfreq in .CFG file.
	     * Added 0.5 and dff 29 July 01 v2.04. RSL
	     */
	    dsp_dph=(unsigned int)
		(0.5+(dff*1.365333333*(double)(int_freqL-xfreqL+15000UL)));
	    dsp_deltap_lo=0XFF & dsp_dph;
	    dsp_deltap_hi=0XFF & (dsp_dph >> 8);
	    /* In 7 cmd, byte 2 is strobe on shift regs. Set here to 0 */
	    dsp_cmd(7, 0, 1, dsp_deltap_lo, dsp_deltap_hi);

	    if(dsp_dph<13653 || dsp_dph>27307)
		do_t_error(16, "E16 3rd LO", 20);

	    if(current_info_box == &lb_freq)
		freq_data(2, synth_freqL, n_synthL, xfreqL,
		    xfreqL-synth_freqL, int_freqL-xfreqL+15000UL, dsp_dph);
	    setviewport(2,124,125,156,NO_CLIP);  /* The small frequency box */
	    clearviewport();
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    settextstyle(0, 0, 1);
	    setcolor(c_text_bold);
	    moveto(62,2);

	    if (n_synthL != last_n_synthL)	/* If need new synth freq */
		{
		bL=( (unsigned long)( (double)n_synthL / 64.0 ) ) & 0X7FFUL;
		aL=(n_synthL - 64UL * bL) & 0X7FUL;
		send_shift_data( (aL<<1) | (bL<<8) );  /* To dsp to prog PLL */
		last_n_synthL=n_synthL;
		}

	    /* Now find the freq to actually display.  Note that in cw the
	     * displayed freq is that which receives a signal producing
	     * a pitch of cwoffset.
	     * In FM the display is the receive freq in receive, transmit if
	     * in transmit 
	     */
	    dt_freq=tvtr[band].lo_freq-((double)tvtr[band].sb)*(double)tfreqL;
/*>>>>	    if (!transmit) dt_freq += (double)rit;  */
	    if (mode==CW)
	        dt_freq += (double)cwoffset;
	    if (mode==FM && transmit)
		dt_freq += -((double)tvtr[band].sb)*
			1000.0*(double)vfo[cur_vfo].tr_offset;  //v3.28
	    disp_MHz=(long int)(0.000001*(dt_freq+0.0000005));
	    disp_Hz=(long int)(dt_freq-1.0e6*(double)disp_MHz);

	    if (waterfall)
		{
		sprintf(cc, "%ld", disp_MHz);
		outtext(cc);
		settextstyle(0, 0, 2);
		if (mode==FM)
		    {
		    sprintf(cc, ".%3.3ld", disp_Hz/1000L);
		    moveto(62,15);  outtext(cc);
		    settextstyle(0, 0, 1);
		    }
		else
		    {
		    sprintf(cc, ".%3.3ld", disp_Hz/1000L);
		    moveto(30,15);  outtext(cc);
		    sprintf(cc, "%3.3ld", disp_Hz-1000L*(disp_Hz/1000L));
		    moveto(95,15);  outtext(cc);
		    settextstyle(0, 0, 1);
		    }
		}
	    else			/* Large frequency dial */
		{
		setviewport(0,122,127,158,DO_CLIP);	/* Small Freq Box */
		clearviewport();	/* Remove it */

		/* 128, 639   d=511  */
		setviewport(X2L+53, Y2T+8, X2L+460, Y2T+39, DO_CLIP);
		clearviewport();
		for(i=0; i<5; i++)
		    {
		    x = 43+i*80;
		    line(x, 35, x, 24);
		    }
		for(i=0; i<25; i++)
		    line(11+i*16, 35, 11+i*16, 29);
		/* The frequency cursor: */
		x=3+(int)((xfreqL-143493750)/12500);
		setcolor(c_plot_marker);
		line(x, 34, x, 29);
		setlinestyle(SOLID_LINE, 0, THICK_WIDTH);
		line(x, 28, x, 23);
		/* Frequency Display: */
		disp_f=1000000L*disp_MHz+disp_Hz;

		setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
		setcolor(c_text_bright);
		settextjustify(CENTER_TEXT, TOP_TEXT);
		settextstyle(DEFAULT_FONT, HORIZ_DIR, 3);
		if (mode==FM)
		    {
		    if (disp_MHz < 1000)
			moveto(170, 0);
		    else if (disp_MHz>=1000 && disp_MHz<10000)
			moveto(158, 0);
		    else if (disp_MHz>=10000 && disp_MHz<100000)
			moveto(146, 0);
		    else 			/* disp_MHz>=100000 */
			moveto(134, 0);
		    sprintf(cc, "%ld.", disp_MHz);
		    outtext(cc);
		    moveto(252, 0);
		    disp_f=disp_Hz/1000;
		    sprintf(cc, "%3.3ld", disp_f);
		    outtext(cc);

		    /* Now print FM Offset */
		    setviewport(X2L+1, Y2T+30, X2L+49, Y2T+40, DO_CLIP);
		    clearviewport();
		    settextjustify(CENTER_TEXT, TOP_TEXT);
		    settextstyle(0, 0, 1);
		    if (vfo[cur_vfo].tr_offset)  setcolor(c_text_bold); //3.28
		    else  setcolor(c_text);
		    moveto(24, 1);
		    sprintf(cc, "%d", vfo[cur_vfo].tr_offset);  //3.28
		    outtext(cc);
		    setcolor(c_text_bold);
		    }			/* End mode==FM */
		else
		    {
		    if (disp_MHz < 1000)
			moveto(143, 0);
		    else if (disp_MHz>=1000 && disp_MHz<10000)
			moveto(131, 0);
		    else if (disp_MHz>=10000 && disp_MHz<100000)
			moveto(119, 0);
		    else 			/* disp_MHz>=100000 */
			moveto(107, 0);
		    sprintf(cc, "%ld.", disp_MHz);
		    outtext(cc);
		    moveto(223, 0);
		    disp_f=disp_Hz/1000;
		    sprintf(cc, "%3.3ld", disp_f);
		    outtext(cc);
		    moveto(305, 0);
		    disp_f=disp_Hz-1000*disp_f;
		    sprintf(cc, "%3.3ld", disp_f);
		    outtext(cc);

		    /* Now clear any FM Offset */
		    setviewport(X2L+1, Y2T+30, X2L+49, Y2T+40, DO_CLIP);
		    clearviewport();
		    }
		setcolor(c_text_bold);
		}
	    }
	else				/* if band==AUDIO_BAND (15=af input) */
	    {				/* For audio processing */
	    dsp_dph=(unsigned int)   /*  Rev 3.38 was tfreqL */
			(0.5+(dff*1.365333333*(double)xfreqL));

#if 0
	    if (transmit)
		dsp_dph=0;

    /* THis needs some work for other modes??? */



	    if (transmit && (mode==CW))
		dsp_dph= (unsigned int)
                   (0.5+(dff*1.365333333 * (double)cwoffset ));
#endif


	    dsp_deltap_lo=0XFF & dsp_dph;
	    dsp_deltap_hi=0XFF & (dsp_dph >> 8);
	    /* In 7 cmd, byte 2 is strobe on shift regs. Set here to 0 */
	    dsp_cmd(7, 0, 1, dsp_deltap_lo, dsp_deltap_hi);

	    disp_Hz=tfreqL;
	    outtext("Offset Hz");
	    settextstyle(0,0,2);
	    sprintf(cc, "%ld", disp_Hz);
	    moveto(62,15);  outtext(cc);
	    settextstyle(0, 0, 1);
	    }				/* End audio input */
	}				/* End new_freq() */

/* The af gain is set in the dsp by a multiplicative constant and a shift
 * value that ends up in the se register.  Up to 50, se=0.  Each increase
 * of 1 in se is 6 dB of added gain.
 * There are two separate software gain controls for FM and another for
 * other modes.
 */
void set_afgain(int dbstep)
	{
	int gain, afg, ifg;
	int se;				/* Shift value for dsp se register */
	static int afgain_save=0;

	if (dbstep==999)		/* Mute toggle */
	    {
	    if (mute)			/* Toggling from mute to non-mute */
		{
		mute=FALSE;
		if (mode==FM)  fm_afgain=afgain_save;
		else           afgain=afgain_save;
		}
	    else			/* Toggling from non-mute to mute */
		{
		mute=TRUE;
		if (mode==FM)
		    {
		    afgain_save=fm_afgain;
		    fm_afgain=0;
		    }
		else
		    {
		    afgain_save=afgain;
		    afgain=0;
		    }
		}
	    dbstep=0;			/* Stop any further gain changes */
	    }
	if (mode==FM)
	    {
	    fm_afgain += dbstep;
	    if (fm_afgain>100)  fm_afgain=100;
	    if (fm_afgain<0)  fm_afgain=0;
	    gain=fm_afgain;		/* Local copy */
	    }
	else
	    {
	    afgain += dbstep;
	    if (afgain>100)  afgain=100;
	    if (afgain<0)  afgain=0;
	    gain=afgain;
	    }
	setviewport(63,393,126,402,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	if (mute)
	    sprintf(cc, "MUTE");
	else
	    sprintf(cc, "%d", gain);
	outtext(cc);
	if(gain==0)
	    {
	    se=0;
	    afg=0;
	    }
	else if(gain>50)
	    {
	    se=(gain-45)/6;
	    afg=95+(gain-45)%6;
	    }
	else
	    {
	    se=0;
	    afg=50+gain;
	    }
	dsp_cmd(10, fgainl[afg], fgainh[afg], se, 4);
	}

/* Up to 18 dB of "software gain" is available in the DSP, implemented as
 * shifting of 0,1,2, or 3 bit positions. This is communicated as
 * mgain_shift.  Be sure to use with ver 2.5 or later.
 */
void set_mike_gain(void)
	{
	setviewport(103,258,127,267,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(24, 1);
	sprintf(cc, "%d", mike_gain);
	outtext(cc);
	if (mike_gain >= 95)
	    {
	    mike_mult = mike_gain;
	    mgain_shift = 0xD0;
	    }
	else if (mike_gain >= 89 && mike_gain < 95)
	    {
	    mike_mult = mike_gain + 6;
	    mgain_shift = 0x80;
	    }
	else if (mike_gain >= 83 && mike_gain < 89)
	    {
	    mike_mult = mike_gain + 12;
	    mgain_shift = 0x40;
	    }
	else
	    {
	    mike_mult = mike_gain + 18;
	    mgain_shift = 0x00;
	    }
	set_mode(1);			/* Send new mike_gain to dsp */
	}

void set_cwoffset(void)
	{
	int sidetone_ph;

	setviewport(34, 288, 100, 297, DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(33, 1);
	sprintf(cc, "%d %d", sidetone_lev, cwoffset);
	outtext(cc);
	setcolor(c_text_bold);

	sidetone_ph=(unsigned int)(1.365333333*(double)cwoffset);
	dsp_cmd('H', sidetone_ph, sidetone_ph>>8,
	   fgainl[sidetone_lev], fgainh[sidetone_lev]);
	new_freq();
	}

void set_eme_doppler(void)
	{
	setviewport(1, 162, 127, 171, DO_CLIP);
	clearviewport();
	if (mode==FM) return;
	show_er();
	new_freq();
	}

void show_er(void)
	{
	/* Display an E on the right, if eme doppler being corrected */
	setviewport(119, 162, 127, 171, DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	setcolor(c_text_bold);
	if (eme_doppler || mode==EME2)
	    {
	    settextstyle(0,0,1);
	    if (diag_mode!=3)
		setcolor(c_text);
	    moveto(4, 1);
	    outtext("E");
	    setcolor(c_text_bold);	/* Always leave this way */
	    }

	/* Display an R on left, if randomization of freq being used */
	setviewport(1, 162, 9, 171, DO_CLIP);
	clearviewport();
	if ((mode==LTI && lti_freq_range>0.4) ||
		 (mode==EME2 && eme_freq_range>0.4))
	    {
	    moveto(4, 1);
	    outtext("R");
	    }
	}

/* set_fsk(do_what)
 * If do_what==0  Clears the display space above the small freq box.
 * If do_what==1  Displays the current fsk, sets the DSP and does a new_freq().
 * If do_what==2  Displays the current fsk & sets the DSP only.
 */
void set_fsk(int do_what)
	{
	int sidetone_ph;

	setviewport(1, 110, 127, 120, DO_CLIP);
	clearviewport();
	if (do_what==0)  return;	/* Just clear the box */
	settextstyle(0,0,1);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(63, 1);
	sprintf(cc, "FSK=%d", fsk);
	outtext(cc);
	setcolor(c_text_bold);
	sidetone_ph=(unsigned int)(1.365333333*(double)fsk);
	dsp_cmd('H', sidetone_ph, sidetone_ph>>8, 
	    fgainl[sidetone_lev], fgainh[sidetone_lev]);
	if (do_what==2)   return;	/* To be called from new_freq() */
	new_freq();
	}

/* FM Squelch and Transmit deviation.
 * Added deviation to this, Rev 3.12  RSL
 */
void set_fm_squelch(void)
	{
	unsigned int sq, dv;

	if (mode != FM) return;
	setviewport(34, 288, 84, 297, DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(1,2);
	setcolor(c_text);
	moveto(25, 1);
	sprintf(cc, "%d", fm_squelch);
	outtext(cc);
	setcolor(c_text_bold);
	/* fm_squelch (0, 99)  */
	sq=3*(99-fm_squelch)*(99-fm_squelch);	/* 29403,0 for 0,99 setting */

	/* FM Deviation from fm_dev the peak dev in kHz. Sent as sqrt of a
	 * multiplier times af level at modulator. 83 is 5 kHz, 181 is
	 * max value (181^2=32761) equal to 24 kHz. That much deviation will
	 * not begin to fit through the crystal filter. The value of fm_dev
	 * is limited in .cfg read to (0.0, 9.99).   */
	dv = (unsigned int)near_int(sqrt(fm_dev*1377.8));

	dsp_cmd('S', sq, sq>>8, dv, 4); 
	}

/* v2f()  -  Controls the "V2F" function in the DSP. This takes the observed
 * average power and converts it to a tone. Higher tones are more power.
 * Driven by variables:
 *   v2f_tc    Smoothing time constant in seconds (double)
 *   v2f_gain  Scaling factor for tone in Hz/dB (double)
 *   v2f_noise 1=Noise source, 0=Signal source
 *   v2f_pkave For signal source, 1=peak signal, 0=average signal
 * Byte 1 of the DSP command (following ESC and 'K') controls the function:
 *    Bit 0  1=Use V2F tone and stop regular audio
 *    Bit 1  0=Use noise source,  1=Use signal source (FFT bins)
 *    Bit 2  0=Command using bits 0, 1 and 7;  1=Load parameters per bits 3-6
 *    Bits 3 to 6  Parameter select
 *    Bit 7  0=For signal source, use average;  1=Use peak over bins.
 * Byte 2 is not used.
 */
void v2f(int do_what)
	{
	double	omt_lf, t_lf;
	long int  omt_LI, t_LI;		/* 32 bit RC filter parameters */
	unsigned int k_UI;		/* Use LS 8 bits as gain mult */
	unsigned int uint_1;		/* Build 8 bit control (Byte 1) */
	unsigned int uint_3n4;		/* Gen var for filling bytes 3 and 4 */
	double cur_lev;

	/* do_what ==0 is update of display info, falls through */

	if (do_what==1)			/* Turn on V2F function in DSP */
	    {
	    v2f(2);			/* Load all parameters */
	    v2f_on=TRUE;
	    if (v2f_noise)  uint_1 = 0X01; /* Do V2F with noise */
	    else            uint_1 = 0X03; /* Do V2F with signal */
	    if (v2f_pkave)  uint_1 |= 0X80; /* Make that peak signal */
	    dsp_cmd('K', uint_1, 2, 3, 4);  /* Send command to turn on */
	    }
	else if (do_what==2)		/* Update V2F display and control */
	    {
	    /* v2f_gain is sent as an 16-bit value in 'K' cmd position 3 and 4.
	     * v2f_gain ranges (0,3844) Hz/dB with 0.1173 Hz/dB resolution  */
	    uint_3n4=(int)(v2f_gain/0.117301);
	    dsp_cmd('K', 0X2C, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));

	    /* The DSP "RC" algorithm multiplies the old power average by
	     * t_LI (almost 1 in 1.31 format) and adding omt_LI times the new
	     * power measurement. omt_LI is a small number compared with 1.
	     * omt(as a decimal) from EMRFD is 1/(0.5+(tc/ts)) where tc
	     * is the RC time constant and ts is the sample period, 1/9600
	     * for noise and 1/18.75 for signals.
	     */
	    if(v2f_noise)
		omt_lf=2147483648.0*(1.0/(0.5+(v2f_tc/0.00010416666666667)));
	    else
		omt_lf=2147483648.0*(1.0/(0.5+(v2f_tc/0.05333333333333)));
	    omt_lf=floor(omt_lf);	/* Convert to an integer value */
	    t_lf=2147483648.0-omt_lf;
	    /* Convert both values to 1.31 format for the DSP */
	    omt_LI=(long int)(0.5+omt_lf);
	    t_LI=(long int)(0.5+t_lf);
	    /* Being 32 bit numbers, these require two commands per number */
	    uint_3n4=0XFFFF&(omt_LI>>16); /* Work on high order of omt */
	    dsp_cmd('K', 0X04, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));
	    uint_3n4=0XFFFF&omt_LI;	/* Work on low order of omt */
	    dsp_cmd('K', 0X0C, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));
	    uint_3n4=0XFFFF&(t_LI>>16);	/* Work on high order of t */
	    dsp_cmd('K', 0X14, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));
	    uint_3n4=0XFFFF&t_LI;	/* Work on low order of t */
	    dsp_cmd('K', 0X1C, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));

	    /* basef_v2f ranges from to 32767, sent as an 16-bit value in
	     * 'K' cmd position 3 and 4. Resolution is 0.146484 Hz.   */
	    uint_3n4=(int)(v2f_basef/0.146484);
	    dsp_cmd('K', 0X34, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));

	    /* FFT bins (0, 512) are sent as 16 bit numbers. These are for
	     * signal use, where either average or peak values are
	     * measured across the range of bins.                    */
	    dsp_cmd('K', 0X3C, 2, 0XFF&v2f_bin_low, 0XFF&(v2f_bin_low>>8));
	    dsp_cmd('K', 0X44, 2, 0XFF&v2f_bin_high, 0XFF&(v2f_bin_high>>8));

	    /* The af level is set in 1 dB steps by v2f_af_level */
	    dsp_cmd('K', 0X4C, 2, fgainl[v2f_af_level], fgainh[v2f_af_level]);

	    /* Reactivate (in case of change to base command) and display,
	     * if active */
	    if (v2f_on)
		{
		if (v2f_noise)  uint_1 = 0X01; /* Do V2F with noise */
		else            uint_1 = 0X03; /* Do V2F with signal */
		if (v2f_pkave)  uint_1 |= 0X80; /* Make that peak signal */
		dsp_cmd('K', uint_1, 2, 3, 4);  /* Send command to turn on */
		}
	    else
		return;	/* No display of data */
	    }
	else if (do_what==3)		/* Turn V2F function off in DSP */
	    {
	    v2f_on=FALSE;
	    dsp_cmd('K', 0, 2, 3, 4);
	    setviewport(0, 100, 107, 110, NO_CLIP);
	    clearviewport();
	    return;
	    }
	else if (do_what==6)		/* Set dsp tone to current voltage */
	    {
	    /* The current measurement is in [7] (LS) and [8] (MS byte).
	     * Return it as the new offset.
	     */
	    dsp_cmd('K', 0X24, 0, dsp_status[7], dsp_status[8] );
	    /* Display difference in dB, not values: */
	    v2f_base_db=(double)(256*dsp_status[8]+dsp_status[7])/409.6;
	    }
	disp_v2f();			/* Update Display */
	}

/* set_ttn()  -  Controls the tone generation for transmit.
 * Driven by variables:
 *   ttn_control Tone (Bits 0,1) and voice (Bit 2) on. 
 *   ttn_freq1 and 2: Tone frequency in Hz (double)
 *   ttn_level1 and 2: Tone levels in 0,99 dB
 * Byte 1 of the DSP command (following ESC and 'X') controls the function:
 *    Bit 0  1=Use tone 1
 *    Bit 1  0=Use tone 2
 *    Bit 2  0=Include microphone audio
 *    Bits 3 to 5  Parameter select
 *    Bit 6  NU
 *    Bit 7  0=Control command;   1=Load parameters in bytes 3,4 per bits 3-5
 * Byte 2 is not used.
 */
void set_ttn(int do_what)
	{
	unsigned int uint_1;
	unsigned int uint_3n4;		/* Gen var for filling bytes 3 and 4 */
	double fgn1, fgn2;
	static int fgn1i, fgn2i;
	double cur_lev;

	if (do_what==1)			/* Set tones on/off in DSP */
	    {
	    /* ttn_control is ready to go for byte 1 */
	    dsp_cmd('X', ttn_control, 2, 3, 4);  // Send command to turn on/off
	    }
	else if (do_what==2)		/* Update V2F display and control */
	    {
	    /* The tone frequencies are in Hz. dff is a factor, close to
	     * 1.0 that corrects for DSP clock errors. */
	    uint_3n4=(unsigned int)(0.5+(dff*1.365333333*ttn_freq1));
	    dsp_cmd('X', 0X80, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));
	    uint_3n4=(unsigned int)(0.5+(dff*1.365333333*ttn_freq2));
	    dsp_cmd('X', 0X88, 2, 0XFF&uint_3n4, 0XFF&(uint_3n4>>8));

	    /* The tone level is set in 1 dB steps by ttn_level1 and 2.
	     * The levels need to be adjusted for their frequency. In the DSP
	     * they are injected after the "preemphasis" filter. This
	     * reduces the level at low frequencies.
	     *   fgain[i]=10^((i-100)/20.0), i.e., fgain[85]=0.17783  */
	    if (ttn_freq1 > 160.0)
		fgn1=fgain[ttn_level1]*ttn_freq1/2500.0;
	    else
		fgn1=fgain[ttn_level1]*0.064;
	    fgn1i=near_int(fgn1*32768.0);

	    if (ttn_freq2 > 160.0)
		fgn2=fgain[ttn_level2]*ttn_freq2/2500.0;
	    else
		fgn2=fgain[ttn_level2]*0.064;
	    fgn2i=near_int(fgn2*32768.0);

	    dsp_cmd('X', 0X90, 2, fgn1i & 0XFF, (fgn1i >> 8) & 0XFF);
	    dsp_cmd('X', 0X98, 2, fgn2i & 0XFF, (fgn2i >> 8) & 0XFF);
	    }
	/* Display the frequencies and levels above the s-meter. They will
	 * disappear with the tone off command.      */
	setviewport(0, 100, 107, 120, NO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	if(ttn_control&1)
	    {
	    moveto(1, 1);
	    sprintf(cc, "  %6.1lf %d", ttn_freq1, ttn_level1);
	    outtext(cc);
	    }
	if(ttn_control&2)
	    {
	    moveto(1, 10);
	    sprintf(cc, "  %6.1lf %d", ttn_freq2, ttn_level2);
	    outtext(cc);
	    }
	}

/* Used for testing to display a few bytes */
void xx(int pos, int x1, int x2, int x3, int x4)
	{
	setviewport(150, 390, 600, 440, NO_CLIP);
	if (pos==0) clearviewport();
	settextstyle(0,0,1);
	setcolor(c_text);
 	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(1, 1+pos);
	sprintf(cc, "%.2X   %.2X   %.2X   %.2X", x1, x2, x3, x4);
	outtext(cc);
	}

void disp_v2f(void)
	{
	setviewport(0, 100, 107, 110, NO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(1, 1);
	outtext("Tone");
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(106, 1);
	sprintf(cc, "%.3lfdB",
	   -v2f_base_db + (double)(256*dsp_status[8]+dsp_status[7])/409.6);
	outtext(cc);
	}

#if 0
/* i2mfp(i)  -  Returns 8-bit mini foating point for a 16-bit integer. High
 * 8 bits of return value are 0.
 *  Bit  7       Sign - 1 if mantissa is >= 0
 *  Bits 6,5,4   Mantissa, 3 bit positive number, 0 to 7 (8/16, 9/16, ...15/15)
 *  Bits 3,2,1,0 Exponent, 0 to 15
 */
int i2mfp(int i)
	{
	double dman;
	int sign, exp, man, vrtn;

	if (i >= 0) sign=1;		/* Find sign of i */
	else sign = -1;
	i *= sign;			/* Make i positive */
	dman=frexp((double)i, &exp); /* Convert to base 2 mantissa & exp */
	/* dman is (0.5, 1.0) and we want an integer that divides this range
	 * into 1/16ths, that we call man */
	if (dman>0.999) dman=0.999;	/* If 1.0 we want to produce 15/16 */
	man=(int)(16.0*(dman-0.5));	/* 3-bits (0,7) positive */
	vrtn=0X00FF & (man<<4 | exp);	/* Mini float w/o sign */
	if (sign<0) vrtn |= 0X0080;	/* OR in minus sign */
	return vrtn;
	}
#endif

void set_xmit_power(void)
	{
	setviewport(63,278,126,287,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	sprintf(cc, "%d", xmit_power);
	outtext(cc);
	if (transmit)			/* Update dsp */
	    {
	    dsp_cmd('T', 0X0003 | (cw_on<<4), 2, 
		fgainl[xmit_power], fgainh[xmit_power]);
	    if ((mode==USB || mode==LSB) && xmit_power > 95)
			do_t_error(17, "W17 Excess Pwr", 50);
	    }
	}

void set_rfgain(void)
	{
	setviewport(63,403,126,412,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	sprintf(cc, "%d", rfg[rfgain].dbdisplay);
	outtext(cc);
	send_shift_data(R10MHZ);
	}

void set_cw_speed(void)
	{
	setviewport(66,268,127,277,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(60, 1);
	outtext(t[cw_speed].description);
	}

void set_contrast(void)
	{
	setviewport(63,233,126,242,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	sprintf(cc, "%d", contrast);
	outtext(cc);
	dis_gain=0.0025+0.0035*(double)(contrast);
	contrast_line_count=20;
	}

void set_brightness(void)
	{
	setviewport(63,243,126,252,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	sprintf(cc, "%d", bright);
	outtext(cc);
	dis_offset=200.0-3.0*(double)bright;
	contrast_line_count=20;
	}

/* Set window has evolved into a general setup of the screen in the spectral
 * display area! It also sets the constants for varying conditions and keeps
 * the variables consistant.
 */
void set_window(void)
	{
	int i;
	if (do_window==NO_WINDOW)
	    {
	    sprintf(cc, "%s", "Non");
	    win_adj_loss=0.0;		/* Coherent loss of window fcn, dB */
	    win_offset=0;		/* 3 dB steps in dsp sptrm output */
	    win_adj_residual=0.0;	/* After 3 dB steps */
	    }
	else if (do_window==TUKEY25)
	    {
	    sprintf(cc, "%s", "Tuk");
	    win_adj_loss=2.3;		/* Book says 1.11 dB ? */
	    win_offset=1;
	    win_adj_residual=-0.5;
	    }
	else if (do_window==HAMMING)
	    {
	    sprintf(cc, "%s", "Ham");
	    win_adj_loss=5.35;
	    win_offset=2;
	    win_adj_residual=-0.7;
	    }
	else  /* do_window==BH92 */
	    {
	    sprintf(cc, "%s", "B-h");
	    win_adj_loss=8.87;
	    win_offset=3;
	    win_adj_residual=-0.2;
	    }
	setviewport(63,378,90,387,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(15, 1);
	outtext(cc);
	setcolor(c_text_bold);
	dsp_cmd('W', do_window, 2, 3, 4);

	/* In EME2 or LTI, a change of fft_bw or do_window needs fresh data */
	if (mode==EME2 || mode==LTI)
	    {
/* USE LTERM_INT_UPDATE() */
	    for(i=0; i<256; i++)  lt_dnorm[i]=0.0;
	    for(i=0; i<21;  i++)  eme_pwr_sum[i]=0.0;
	    eme_pulse_count=0;
	    set_bins();			/* Set lt_i1,... */
	    }
	fft_fcn &= 0XFFF0;		/* Clear last 4 bits */
	if (fft_bw==0)			/* 1200 Hz BW (2400 samp/sec)*/
	    {
	    sprintf(cc, "%s", "1200");
	    fft_fcn |= 0X0001;
	    bin_low=(int)(0.5+freq_low/2.34375);
	    }
	else if (fft_bw==1)		/* 2400 Hz BW (4800 samp/sec)*/
	    {
	    sprintf(cc, "%s", "2400");
	    fft_fcn |= 0X0002;
	    bin_low=(int)(0.5+freq_low/4.6875);
	    }
	else if (fft_bw==2)		/* 4800 Hz BW (9600 samp/sec)*/
	    {
	    sprintf(cc, "%s", "4800");
	    fft_fcn |= 0X0004;
	    bin_low=(int)(0.5+freq_low/9.375);
	    }
	else				/* fft_bw==3, 4800*2  BW */
	    {
	    sprintf(cc, "%s", "4800W");
	    fft_fcn |= 0X0004;
	    bin_low=(int)(0.5+freq_low/9.375);
	    }
	setviewport(91,378,126,387,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	setcolor(c_text_bold);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(35, 1);
	outtext(cc);

	set_display_width();
	dsp_cmd('F', fft_fcn&0XFF, (fft_fcn>>8)&0XFF, 3, 4);

	if (fft_fcn & 0X0080)  sprintf(cc, "%s", "W");
	else                   sprintf(cc, "%s", "N");
	setviewport(55,313,64,322,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(4, 0);
	outtext(cc);
	setcolor(c_text_bold);

	/* For information, put the noise bandwidth in the upper right */
	if (waterfall)			/* Don't use for simple display */
	    {
	    sprintf(cc, "%.1f Hz", df[fft_bw]*noise_bw[do_window]);
	    if (text_area)
		setviewport(X1R-62, Y1T+53, X1R-3, Y1T+62, DO_CLIP);
	    else
		setviewport(X1R-62, Y1T+20, X1R-3, Y1T+29, DO_CLIP);
	    clearviewport();
	    settextjustify(RIGHT_TEXT, TOP_TEXT);
	    setcolor(c_text);
	    moveto(58, 1);	outtext(cc);
	    setcolor(c_text_bold);
	    }

	set_noncohave(0);		/* Altered by fft_bw ? */
	}

/* set_if_bw(what)  -  Switches between two filters coming from the
 * I and Q channels (or discriminator output for FM). The narrower 2.8 kHz
 * filter excludes the foldback at 4.8 kHz. The wider 4 kHz is better for
 * spectral display, and possibly for some applications, for the audio
 * response, but it does allow slight wraparound on very strong signals
 * just above 4.8 kHz. The issue comes about since the sample rate is
 * decimated to 9.6 kHz following this filter.
 *
 * what==-1   Toggles between the two values
 * what==0    Sets the filter to Narrow 2.8 kHz
 * what==1    Sets the filter to Wide 4 kHz
 */
void set_if_bw(int what)
    {
    if(what==-1)
	{
	/* Toggles I-Q (bit 7) & Interpolation LP filter (13)*/
	if(fft_fcn & 0X0080)   fft_fcn &= 0XDF7F;
	else fft_fcn |= 0X2080;
	}
    else if(what==0)
	fft_fcn &= 0XDF7F;
    else
	fft_fcn |= 0X2080;
    set_window();
    }

/* set_mode(do_what)  -  Arrive with global int mode=0 , 1, 2,...
 * do_what is 1 if save_fine_freqL is valid.
 *            2 increments the mode until an active one is found
 *              relative to mode_mask.  Note that CW (mode=0) is always
 *              available. Change to this mode and exit.
 *            0 if came from a new VFO, or cold start. No old mode.
 *            3 Just update display   v3.09
 * The mike gain comes from the 'M' command to the DSP, as well as the mode.
 * This also OR's mgain_shift into cmd2, where bits 6&7 are x1 to x8 gain.
 */
void set_mode(int do_what)
	{
	int cmd1, cmd2;
	int mode_test;
	int last_mode;
 	char cm[20];		/* cc doesn't work here as too many users */
	/* save_fine_freqL is used when going into and out of the fm mode
	   to allow fm to go in 5 kHz steps, but return to the 1 Hz freq */
	static long	save_fine_freqL=0;
	static char *mname[]=
	    {"   CW", "  USB", "  LSB", "   FM",
	     " LHL7", "PUA43", " EME2", "  LTI"};

	if (do_what==3)
	  {
	  /* String comes from array */
	  setviewport(84,288,126,297,NO_CLIP);   /* Area for mode on left */
	  clearviewport();
	  settextstyle(0,0,1);
	  settextjustify(RIGHT_TEXT, TOP_TEXT);
	  moveto(42, 1);
	  sprintf(cm, "%s", mname[mode]);
	  outtext(cm);			/* Print the current mode */

	  /* Clear the time in the lower left that is used for code modes */
	  setviewport(90, 470, 127, 479, DO_CLIP);
	  clearviewport();

	  /* The CW data is a little wide! Print now to have room */
	  if (!waterfall)				/* Slide-rule dial */
	    {
	    setviewport(X2L+1, Y2T+20, X2L+49, Y2T+30, DO_CLIP);
	    clearviewport();
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    moveto(24, 1);
	    outtext(mname[mode]+2);		/* Squeeze it in */
	    }
	  return;			/* Just update annotation */
	  }

	/* We may be about to change the mode. Sometimes we need to know
	 * which mode we came from:         */
	if (do_what != 0) last_mode=mode;
	dsp_cmd(10, 0, 0, 0, 4);	/* Set audio gain to 0 */
	set_fsk(0);			/* Clears FSK annotation, if any */

	if (do_what==0)			/* Came from new VFO or cold start */
	    {
	    save_fine_freqL=tfreqL;	/* Initialize */
	    }

	if (do_what==2)			/* Mode change, rotates through them */
	    {
	    while (1)
		{
	        if (++mode>7)  mode=0;	/* Maximum installed mode */
		mode_test=near_int(pow(2.0, (double)mode));
		if (mode_test & mode_mask)  break;	/* An active mode */
		}

	    /* Data modes only when ms clock available:   */
	    if (!ms_available && mode>=4)
		{
		mode=0;
		do_t_error(25, "W25 ms Unavail", 0);
		}
	    if (!waterfall && mode>=4) mode=0;
	    do_what=1;			/* From here on, it is a 1 */
	    }				/* End do_what==2 */

	/* Fall through here for any do_what. Variable 'mode' is the new mode
	 * we are going to.
	 */

	/* String comes from array */
	setviewport(84,288,126,297,NO_CLIP);   /* Area for mode on left */
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(42, 1);
	sprintf(cm, "%s", mname[mode]);
	outtext(cm);			/* Print the current mode */

	/* text_area means top 32 pixels of spectrum plot is for text data */
	if ((waterfall && mode==CW) || mode==LHL7 || mode==PUA43)
	    text_area=TRUE;
	else
	    text_area=FALSE;


#if 0
	/* First clear old data from Mode line  */
	setviewport(34, 288, 126, 297, DO_CLIP);
	clearviewport();
#endif



	/* If there are Alt A info boxes running, they get closed when we
	 * change modes:     */
	if (mode != EME2  &&  current_info_box==&lb_eme2) /* If displayed */
	    eme2_update(0);		/* Close */
	if (mode != LHL7  &&  current_info_box==&lb_lhl7)
	    lhl7_update(0, 0);
	if (mode != PUA43  &&  current_info_box==&lb_pua43)
	    pua43_update(0, 0);
	if (mode != LTI  &&  current_info_box==&lb_lterm_int)
	    lterm_int_update(0);

	/* fft_bw must be 2 for LHL7 & FM. Save old BW if new fft_bw = 2 */
	if ((mode==FM || mode==LHL7) && fft_bw != 2)
	    {
	    fft_bw_save=fft_bw;
	    fft_bw=2;
	    }
	if (mode!=LHL7 && mode!=FM && (last_mode==FM || last_mode==LHL7))
	    fft_bw=fft_bw_save;

	/* do_window must be Hamming in FM. Save old
	   Also, same treatment for i-f BW as we want 4 kHz for FM.  */
	if (mode==FM)
	    {
	    do_window_save=do_window;
	    do_window=HAMMING;		/* Hamming window function */
	    if_bw_save = (fft_fcn & 0X0080) >> 7;	/* 0 or 1 */
	    set_if_bw(1);		/* Always wide for FM */
	    }
	if (mode!=FM && last_mode==FM)
	    {
	    do_window=do_window_save;
	    set_if_bw(if_bw_save);	/* argument=0,1 narrow,wide */
	    }

	fft_fcn &= 0XE080;		/* Retain I-Q & Interp filts */
	if (mode==LHL7)
	  {
	  integrate_count=0;
	  fft_fcn |= 0X0244;  /* Software trig, 9.6kHz, 1024pt, 3 data/2 sec */
	  }

	/* EME2 and PUA43 allow variable fft bandwidth, fft_bw==2 or 3 use the
	 * DSP data.  */
	if (mode==EME2 || mode==PUA43)
	  {
	  integrate_count=0;
	  if(fft_bw==0)       fft_fcn |= 0X0241;  /* Trig, 1024 pt, 1200 Hz */
	  else if(fft_bw==1)  fft_fcn |= 0X0242;  /* Trig, 1024 pt, 2400 Hz */
	  else                fft_fcn |= 0X0244;  /* Trig, 1024 pt, 4800 Hz */
	  }
	else
	  {
	  if(fft_bw==0)       fft_fcn |= 0X0041;  /* 1024 pt, 1200 Hz */
	  else if(fft_bw==1)  fft_fcn |= 0X0042;  /* 1024 pt, 2400 Hz */
	  else                fft_fcn |= 0X0044;  /* 1024 pt, 4800 Hz FFT BW */
	  }

	if (mode==EME2)
	    {
	    eme2_update(-1);
	    eme_pulse_count=0;
	    }
	if (mode==LTI)
	    {
	    lterm_int_update(-1);
	    eme_pulse_count=0;
	    }

	/* Long term integration forces 16 bit data. Others use what is set */
	if (mode==EME2 || mode==LTI)
	    {
	    bits_8_16 = 1;
	    set_8_16(9);
	    set_bins();			/* Also set up i0,... values */
	    }
	else				/* Go back to last value */
	    {
	    bits_8_16 = 0;
	    set_8_16(9);
	    }

	/* Now program DSP for fft_fcn */
	dsp_cmd('F', fft_fcn&0XFF, (fft_fcn>>8)&0XFF, 0, 4);

	set_window();			/* Update display */

	if (mode==FM )  /* Use 5 kHz resolution */
	    {
	    save_fine_freqL=tfreqL;
	    tfreqL = 5000*((tfreqL+2500)/5000);
	    }
	if (last_mode==FM && do_what>0 && save_fine_freqL>0L)
	    {
	    tfreqL=save_fine_freqL;	/* Get old freq to 1 Hz */
	    }
	if (mode==FM)
	    set_agc(-1);		/* Turn off AGC for FM */
	else
	    set_agc(agc_decay);
	if (narrow)			/* Not used now; always 1 */
	    {
	    display_width=FULL_FREQ;	/* All except LHL7, EME2, LTI */
	    switch (mode)
		{
		case CW: {
			cmd1=1;		/* 1=CW */

			/* set_cwoffset(); Done below */
			print_medium(0, -1);	/* Initialize */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			break;
			}
		case USB: {
			cmd1=0;		/* 0=SSB */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			break;
			}
		case LSB: {
			cmd1=0;		/* 0=SSB */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			else
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
                        /* cmd2=2 | mgain_shift;   2=Binaural */
			break;
			}
		case FM: {
			cmd1=2;		/* 2=FM */
			cmd2=mgain_shift;		/* 0=no ssb */
			set_fm_squelch();
			break;
			}
		case LHL7: {
			cmd1=1;		/* 1=CW */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			d_nx2=0;
			set_code_nx2();		/* Update screen settings */
			lhl7_update(0, -1);
			fsk=600;  set_fsk(1);
			morse(INIT_CW, 0);
			print_medium_1(0, -1);	/* Initialize */
			print_medium_2(0, -1);	/* Initialize */
			print_big(0, -1, 0);	/* Initialize */
			sync_flag=TRUE; 	/* Time align displays */
			/* HALF_FREQ is not generally available */
			display_width=HALF_FREQ;
			break;
			}
		case PUA43: {
			cmd1=0x01;		/* 80=Triggered 2 sec, 1=CW */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			print_big(0, -1, 0);	/* Initialize */
			print_super_row(0, -1, 0);	/* Initialize */
			pua43_update(0, -1);	/* Also initialize */
			fsk=0; set_fsk(1);
			sync_flag=TRUE;
			eme_pulse_count=0;	/* New start for Noise Blank */
			break;
			}
		case EME2: {
			cmd1=0x01;		/* For now: 1=CW */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			eme2_update(-1);	/* Also initialize */
			set_bins();
			fsk=(int)(0.5+df[fft_bw]*(double)lt_i3);
			set_fsk(0);		/* No display, use Alt-A Box*/
			display_width=HALF_FREQ;
			break;
			}
		case LTI: {
			cmd1=1;		/* 1=CW */
			if (tvtr[band].sb>0  || band==AUDIO_BAND)
			    {
			    cmd2=1 | mgain_shift;  /* 1=LSB (USB in dsp) */
			    dsp_usb=TRUE;
			    }
			else
			    {
			    cmd2=mgain_shift;	/* 0=USB (LSB in dsp) */
			    dsp_usb=FALSE;
			    }
			print_medium(0, -1);	/* Initialize */
			fsk=(int)(0.5+lti_freq());
			set_fsk(0);
			display_width=HALF_FREQ;
			lterm_int_update(-1);	/* Also initialize */
			break;
			}

		/* AM=7, ... */
		default: break;
		}
	    }
	else				/* else if wideband modes */
	    {
#if 0
	    /* This needs to be integrated into things.
	     * The most useful are binaural and USB left, LSB right.
	     */
	    if (binaural && iq)
		{
 /*		sprintf(cm, "%s",  "WB IQ"); */
		cmd1=0;			/* Not used in wideband */
		cmd2=0X96 | mgain_shift;  /* ?????  Wide, I-Q, Binaural */
		}
	    else if (!binaural && iq)
		{
 /*		sprintf(cm, "%s",  "WM  I"); */
		cmd1=0;			/* Not used in wideband */
		cmd2=0X94;		/* Wide, I-Q, Monaural */
		}
	    else if (binaural && !iq)
		{
 /*		sprintf(cm, "%s",  "B SSB"); */
		cmd1=0;			/* Not used in wideband */
		cmd2=0X9A;		/* Wide, SSB, Binaural */
		}
	    else if (!binaural && !iq)
		{
/*		sprintf(cm, "%s",  "M USB"); */
		cmd1=0;			/* Not used in wideband */
		cmd2=0X98;		/* Wide, SSB, Monaural */
		}
#endif
 /* 	    sprintf(cm, "%s", " ");*/   /* Temp before reinstalling wb stuff */
	    }				/* End 'if wideband modes' */

	new_freq();			/* Get cw offset, etc */

	/* In CW and LTI the sidetone is at the same pitch and level
	 * even though the FSK tone changes in LTI with random freq.
	 */
	if (mode==CW || mode==LTI) set_cwoffset();

	/* Note: when these commands get to dsp they produce the reverse
	   sideband.  This is because of upper side conversion at 19.68 MHz.
	   Notation, USB, LSB here refers to signal at 2 meters */
	dsp_cmd('M', cmd1, cmd2, fgainl[mike_mult], fgainh[mike_mult]);
	delay(50);			/* Stop pop after mode change */
	set_afgain(0);			/* Set but don't change */
	set_noncohave(0);		/* Mode affects these, sometimes */
	set_display_width();
	set_eme_doppler();

	/* Clear the time in the lower left that is used for code modes */
	setviewport(90, 470, 127, 479, DO_CLIP);
	clearviewport();

	/* The CW data is a little wide! Print now to have room */
	if (!waterfall)				/* Slide-rule dial */
	    {
	    setviewport(X2L+1, Y2T+20, X2L+49, Y2T+30, DO_CLIP);
	    clearviewport();
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    moveto(24, 1);
	    outtext(mname[mode]+2);		/* Squeeze it in */
	    }
	/* The vfo[] needs to reflect the current mode */
	vfo[cur_vfo].last_mode=mode;
	}

/* set_code_nx2()  -  sets/displays the number of 2 second periods being
 * used in codes LHL7. The variable d_nx2 determines the action:
 *   d_nx2==-1  Reduce code_nx2 to the next lower value, if there is one.
 *   d_nx2==0   No change to code_nx2. Redisplay and alter spec_ave if needed.
 *   d_nx2==1   Increase code_nx2 to the next higher value, if there is one.
 * Allowable values of code_nx2 are 1, 2, 4, 10 and 30 with the times being
 * twice these values.
 */
void set_code_nx2(void)
	{
	setviewport(34, 288, 84, 297, DO_CLIP);
	clearviewport();

	if (mode != LHL7) return;

	if (d_nx2 == 0) ;		/* Display, but don't change */
	else if (d_nx2 == -1)		/* Reduce code_nx2 to next lower */
	    {
	    if (code_nx2==1) ;		/* Just redisplay */
	    else if (code_nx2==2) code_nx2=1;
	    else if (code_nx2==5) code_nx2=2;
	    else if (code_nx2==10) code_nx2=5;
	    else code_nx2=10;
	    }
	else if (d_nx2 == 1)
	    {
	    if (code_nx2==30) ;		/* Just redisplay */
	    else if (code_nx2==1) code_nx2=2;
	    else if (code_nx2==2) code_nx2=5;
	    else if (code_nx2==5) code_nx2=10;
	    else code_nx2=30;
	    }
	settextstyle(0,0,1);
	setcolor(c_text);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(25, 1);
	sprintf(cc, "%d s", 2*code_nx2);
	outtext(cc);
	setcolor(c_text_bold);

	set_noncohave(2);		/* Keep this tracking */
	}

void set_data_save(void)
	{
	do_data_save ^= 1;	
	setviewport(10,80,101,90,DO_CLIP);
	clearviewport();
	setcolor(c_text_note);
	settextstyle(0, 0, 1);
	settextjustify(1,2);
	moveto(50,1);
	if (do_data_save)
	    outtext("RECORDING");
	setcolor(c_text_bold);
	}

void normalize_plot(int k)
	{
	int i;

	for (i=0; i<512; i++)
		dnorm[i]=0.0;
	setviewport(0,90,101,100,DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(1,2);
	moveto(50,1);
	if (k==0)			/* Clear normalization */
	    {
	    normalize_on=0;
	    }
	else if (k==1)			/* Add fast normalization */
	    {
	    normalize_on=25;		/* Count down to 1 */
	    normalize_num=25.0;
	    outtext("Wait...");
	    }
	else
	    {
	    normalize_on=1000;		/* Big countdown for precision */
	    normalize_num=1000.0;
	    outtext("Wait...");
	    }
	}

/* do_auto_disp(k)  -  corrects for analog gain variations so that the
 * colors in the spectral display don't drift.  The down side to this
 * is that strong signals tend to push the noise down.  So it works very
 * well for weak signals in noise.
 */
void do_auto_disp(int k)
	{
	int i;

	setviewport(63,423,126,432,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	if (k==0)			/* Stop color agc */
	    {
	    auto_disp_on=FALSE;
	    setcolor(c_text);
	    outtext("OFF");
	    setcolor(c_text_bold);
	    }
	else				/* Turn on auto_disp */
	    {
	    auto_disp_on=-1;		/* A flag to initialize base value */
	    setcolor(c_text_bold);
	    outtext("ON");
	    }
	}

/* set_8_16(k)  -  Programs the DSP for 8 or 16 bit data (either 512 bytes
 * or 256 words of logged spectral data).  Changes the resolution of the
 * data coming to the PC from the DSP.  If k=0, resets bits_8_16=0, the
 * value for 8 bit data.  If k=1 it toggles 8/16.  If k=9 it just programs
 * the DSP to the current value of bits_8_16.
 * See also set_disp_offset() for more info.
 */
void set_8_16(int k)
	{
	if (k==0)  bits_8_16=0;		/* Reset to 8 bit on 0 */
	else if (k==1)  bits_8_16 ^= 1;		/*   or toggle on 1 */
	/* Fall through on 9 */

	dsp_cmd(9, bits_8_16, power_exp,
	     LOG_OFFSET&0XFF, (LOG_OFFSET>>8)&0XFF);

	setviewport(70,368,90,377,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(CENTER_TEXT, TOP_TEXT);
	moveto(10, 1);
	if (bits_8_16&2)
	   setcolor(c_text_bold);	/* Indicates FFT data not coming */
	else
	   setcolor(c_text);		/* Normal FFT data */
	if (bits_8_16&1)
	    outtext("16");
	else
	    outtext("8");
	setcolor(c_text_bold);

	set_disp_offset(1);  /* << CHECK - This probably does same dsp cmds*/
	}

/* set_affilt_sw()  -  Arrive with do_af_filt=0 for no af filter.
 */
void set_affilt_sw(void)
	{
	annotate_af_filt();
	dsp_cmd('A', do_af_filt, do_LMS+1, 3, 4);
	}

/* set_affilt_select()  -  Arrive with af_coef (0, 7).
 * Sets the filter number (byte 1) and the gain inherent in the FIR coeffs
 * Variable kfilt is from the configuration file and adjusts whether
 * the noise level is constant as the bandwidth changes or the center
 * freq sine-wave gain is constant.
 *
 * filter_mask is a bit-by-bit mask for the 8 DSP audio filters.
 */
void set_affilt_select(void)
	{
	unsigned int byte2;
	int filter_test;

	/* First insure that a valid mask exists and then rotate through the
	 * filters until an active one is found */
	if (filter_mask==0 || (filter_mask&0XFF)==0)	/* Invalid situation */
	    af_coef=0;			/* So, use the first filter anyway */
	else
	    {
	    while (1)
		{
		filter_test=near_int(pow(2.0, (double)af_coef));
		if (filter_test & filter_mask)  break;	/* An active mode */
		if (++af_coef > 7) af_coef=0;
		}
	    }
	/* Following the DSP FIR there is a gain reduction in steps of 2.
	 * The number of steps is byte2. */
	byte2=0X000F & (unsigned)(kfilt*(double)filt[af_coef].fil_gain);
	annotate_af_filt();
	dsp_cmd('B', af_coef, byte2, 3, 4);
	}

/* set_delay_right(do_what)   Sets amount of delay put into right channel
 * in dsp.  do_what=0 means toggle delay on and off.
 */
void set_delay_right(int do_what)
	{
	int del;

	del=(int)(0.001+pow(2.0, (double)(delay_right+1)));
	if (do_what==0)
	    {
	    delay_state ^= 1;
	    }
	setviewport(63,333,126,342,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(64, 1);
	if (delay_state)
	    {
	    dsp_cmd('E', del, del>>8, 3, 4);
	    sprintf(cc, "%d", del);
	    outtext(cc);
	    }
	else
	    {
	    dsp_cmd('E', 1, 0, 3, 4);	/* Delay off is a delay of 1 */
	    outtext("Off");
	    }
	}

void select_denoise(void)
	{
	do_LMS += 1;
	if (do_LMS>1)  do_LMS = -1;
	set_denoise();			/* This updates the screen */
	dsp_cmd('A', do_af_filt, do_LMS+1, 3, 4);
	}

void set_denoise(void)
	{
	unsigned int autonotch_beta;

	unsigned int byte2;
	unsigned int dcynum;

	/* The beta number used in the DSP is the 0.5*(denoise_level)^2.
	   As denoise_level ranges (0, 100) beta ranges (1, 5202)
	   which as 1.15 is (0.0, 0.159). The square root compression
	   puts most of the range towards 0.0, e.g. 50 -> 0.041.
	 */

	/* The decay in the DSP is the number sent in byte 2, doubled
	   and subtracted from 32735. As denoise_decay ranges (0, 100)
	   the number sent, dcynum, ranges (254, 0) in steps of 2. In the
	   DSP the range is (32227, 32735) corresponding to 1.15 values
	   of (0.983, 0.999). The DSP decay is per W9GR and decays only
	   one of the coefficients per processing cycle, so the effective
	   decay rate is modified by 31 or 61, depending on the length.
	   The effective range is (0.99973 to 0.99998) for 61 length.
	   W9GR recommended a value around 1-0.00004=0.99996, or a
	   denoise_decay value of 75.
	 */
	dcynum=2*(int)(0.5+1.27*(double)(100-denoise_decay));
	byte2=dcynum | denoise_del_len;	/* Byte 2 */

	/* As autonotch_level ranges (0, 100) autonotch_beta
	   ranges (15, 1500). */
	autonotch_beta=(unsigned)
	    (1500.0/pow(1.0472,(double)(100-autonotch_level)));

	/* This is made wide on left to insure clearing the old decay data */
 	setviewport(95,323,127,332,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(32, 0);
	if (do_LMS==1)	/* De-Noise */
	    {
	    sprintf(cc, "%d", denoise_level);
	    outtext(cc);
	    setviewport(2,323,96,332,DO_CLIP);
	    clearviewport();
	    settextjustify(LEFT_TEXT, TOP_TEXT);
	    moveto(0, 0);
	    outtext("LMS Denoise");

	    setviewport(87,323,103,332,DO_CLIP);
	    clearviewport();
	    settextjustify(LEFT_TEXT, TOP_TEXT);
	    setcolor(c_text);
	    moveto(0, 0);
	    sprintf(cc, "%d", denoise_decay);
	    outtext(cc);
	    setcolor(c_text_bold);
	    }
	else if (do_LMS== -1)	/* Auto-Notch */
	    {
	    sprintf(cc, "%d", autonotch_level);
	    outtext(cc);
	    setviewport(2,323,96,332,DO_CLIP);
	    clearviewport();
	    settextjustify(LEFT_TEXT, TOP_TEXT);
	    moveto(0, 0);
	    outtext("LMS Notch");
	    }
	else
	    {
	    outtext("Off");
	    setviewport(2,323,96,332,DO_CLIP);
	    clearviewport();
	    settextjustify(LEFT_TEXT, TOP_TEXT);
	    moveto(0, 0);
	    outtext("LMS");
	    }
	/* Add 2 to beta to prevent beta in DSP going negative */
	dsp_cmd('C',2+denoise_level,byte2,autonotch_beta, autonotch_beta>>8);
	}


/* set_agc(int decay)  -  Sets the AGC decay rate in the DSP, or turns
 * AGC off by setting decay to 0.
 * Special case: No AGC in FM.
 */
void set_agc(int decay)
	{
	static int agcd[]={0, 32, 24, 16, 12, 8, 6, 4, 3, 2, 1};

	setviewport(63,413,126,422,DO_CLIP);
	clearviewport();
	setcolor(c_text_bold);
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(64, 1);

	if (decay == -1)		/* Special case--AGC off for FM */
	    {
	    outtext("Off");
	    dsp_cmd('V', 2, 3, 0, 0);
	    return;			/* Do not alter last_agc */
	    }
	    
	if (decay>0)
	    {
	    sprintf(cc, "%d", decay);
	    outtext(cc);
	    /* AGC on raises gain by 48 dB in DSP--compensate here.
	     * This does not affect fm_afgain, as there is no AGC in FM.
	     */
	    if (last_agc==0)  set_afgain(-48);
	    }
	else
	    {
	    outtext("Off");
	    if (last_agc > 0)  set_afgain(48);
	    }
	last_agc=decay;
	/* AGC Settings: */
	dsp_cmd('V', 2, 3, agcd[decay]&0XFF, (agcd[decay]>>8)&0XFF);
	}

void annotate_af_filt(void)
	{
	if (do_af_filt==0)
	   sprintf(cc, "%s", "None");
	else
	   sprintf(cc, "%s", filt[af_coef].name);
	setcolor(c_text_bold);
	setviewport(69,313,126,322,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(57, 0);
	outtext(cc);

	sprintf(cc, "%d", 1+af_coef);
	setcolor(c_text);
	settextjustify(LEFT_TEXT, TOP_TEXT);
	moveto(0, 0);
	outtext(cc);
	setcolor(c_text_bold);
	}

void set_dot(void)
	{dot_type += 1;
	 if  (dot_type>2) dot_type=0;
	}

/* disp_toggle()  As a resuly of the Alt Y command sets the display format
 * to either Large freq display (!waterfall), or to waterfall.
 */
void disp_toggle(void)
	{
	int i, x;

	setviewport(0, 232, 70, 252, DO_CLIP);	/* For Contrast & Bright */
	clearviewport();
	settextjustify(0,2);
	settextstyle(0,0,1);
	if (waterfall)
	    {
	    setcolor(c_text_bold);
	    moveto(2,1);	outtext("Contrast");
	    moveto(2,11);	outtext("Bright");
	    setviewport(0, 0, 639, 479, DO_CLIP);
	    line(X2L, Y2T, X2R, Y2T);
	    /* Print freq ticks at top of waterfall */
	    for(i=1; i<=9; i++)
		{
		x = 129 + (int)(53.229 * (double)i);
		line(x, Y2T-2, x, Y2T);		/* Vertical ticks */
		}
	    /* Waterfall area */
	    setviewport(X2L+1, Y2T+1, X2R-1, Y2B-1, DO_CLIP);
	    clearviewport();
	    scroll_line=0;
	    setcolor(c_outline_bright);		/* Freq display border */
	    setviewport(0, 122, 127, 158, DO_CLIP);
	    setcolor(c_outline_bright); /* Freq display border */
	    rectangle(1,0,126,36);	/* Leave off corner pixels */
	    rectangle(0,1,127,35);
	    setcolor(c_text_bold);
	    new_freq();
	    if(mode==CW)
		{
		text_area=TRUE;
		print_medium(0, -1);	/* Initialize */
		}
	    if(mode==LHL7 || mode==PUA43)
		{
		text_area=TRUE;
		print_medium_1(0, -1);	/* Initialize */
		print_big(0, -1, 0);
		print_medium_2(0, -1);
		}
	    }
	else				/* Slide rule dial display */
	    {
	    setcolor(c_text);		/* Keep first uses viewport above */
	    moveto(2,1);	outtext("Contrast");
	    moveto(2,11);	outtext("Bright");
	    if (mode==CW)
		{
		morse(ADD_CH, 32);		/* Force a redraw of cw text */
		morse(ADD_CH, 8);		/* Back it out */
		}
	    if (mode==LHL7 || mode==PUA43 || mode==EME2 || mode==LTI)
		{
		mode=USB;
		set_mode(1);
		}
	    setcolor(c_text_bold);
	    draw_panel();
	    new_freq();
	    diag_mode=0;
	    switch (mode)
		{
		case CW: {
			sprintf(cc, "%s", "   CW");
			break;
			}
		case USB: {
			sprintf(cc, "%s", "  USB");
			break;
			}
		case LSB: {
			sprintf(cc, "%s", "  LSB");
			break;
			}
		case FM: {
			sprintf(cc, "%s", "   FM");
			break;
			}
		default: break;
		}
	    setviewport(X2L+1, Y2T+15, X2L+49, Y2T+45, DO_CLIP);
	    clearviewport();
	    settextstyle(0, 0, 1);
	    settextjustify(CENTER_TEXT, TOP_TEXT);
	    moveto(24, 10);
	    outtext(cc+2);
	    text_area=FALSE;
	    }			/* End !waterfall */
	}
	    
/* set_display_width()  -  Arrive with global display_width 0 or 1,
 * i.e., FULL_FREQ or HALF_FREQ.
 */
void set_display_width(void)
	{
	if (display_width)
		{if      (fft_bw==0) mark_frequency(500);
		 else if (fft_bw==1) mark_frequency(1000);
		 else if (fft_bw==2) mark_frequency(2000);
		 else                mark_frequency(4000);
		}
	else
		{if      (fft_bw==0) mark_frequency(1000);
		 else if (fft_bw==1) mark_frequency(2000);
		 else if (fft_bw==2) mark_frequency(4000);
		 else                mark_frequency(8000);
		}
	}

/* set_db_div(changed) is used to change the setting (changed==1) and
 * to redraw the display (changed==0).
 */
void set_db_div(int changed)
	{
	int i, yy, kdb;

	if (db_div_set==0)
		{
		kdb_div=3.2;
		kdb=10;
		}
	else if (db_div_set==1)
		{
		kdb_div=6.4;
		kdb=5;
		}
	else if (db_div_set==2)
		{
		kdb_div=16.0;
		kdb=2;
		}
	else                            /* if(db_div_set==3) */
		{
		kdb_div=32.0;
		kdb=1;
		}
	if (text_area)
	    setviewport(X1L+1, Y1T+35, X1L+68, Y1T+44, DO_CLIP);
	else
	    setviewport(X1L+1, Y1T+2, X1L+68, Y1T+11, DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(0,2);
	setcolor(c_text);
	moveto(1, 1);
	sprintf(cc, "%d dB/", kdb);
	outtext(cc);
	setcolor(c_text_bold);

	if (changed)
	    {
	    setviewport(102, Y1T, X1L-1, Y1B-8, DO_CLIP);
	    clearviewport();
	    settextjustify(1,1);
	    for (i=0; i<=7; i++)
		{
		yy =  1 + i*16;
		moveto(14, yy);
		sprintf(cc, "%d", kdb*(8 -i));
		outtext(cc);		/* dB level markings */
		}
	    setcolor(c_text_bold);
	    set_disp_offset(1);
	    }
	}

void set_disp_offset(int update_dsp)
	{
	if ( near_int(koffset) != 0)
		{
		if (text_area)
		    setviewport(X1R-60, Y1T+35, X1R-4, Y1T+44, DO_CLIP);
		else
		    setviewport(X1R-60, Y1T+2, X1R-4, Y1T+11, DO_CLIP);
		clearviewport();
		settextstyle(0,0,1);
		settextjustify(RIGHT_TEXT, TOP_TEXT);
		setcolor(c_text);
		moveto(55, 1);
		sprintf(cc, "%d dB", near_int(koffset) );
		outtext(cc);
		setcolor(c_text_bold);
		}
	/* power_exp is sent to the dsp to scale the power data before it is
	 * log converted to 8 bits.  The total dynamic range of the 8 bit data
	 * is 80 dB (lsb=80/256) and this factor keeps it centered in the
	 * 80 dB display area.
	 * In the following we add 99.0 to the offset to prevent (int)
	 * confusion at 0.  It is removed by an extra 33 in front.
	 * Note: not used except with 10 dB/div.  Therefore you can still
	 * overload if you offset more than -50 dB in 5 dB/div.
	 * Limit to this amount back in command.
	 * Variable win_offset is the approximate coherent loss of the window
	 * function, in 3 dB steps. This gives the power of 2 for scaling
	 * the dsp power spectrum data to almost correct for windowing.
	 * The remainder of the correction is win_adj_residual.
	 */
	if (db_div_set==0)		/* 0 --> 10 dB/div  */
	    {
	    power_exp = 43 + win_offset - ((int)(99.0+koffset))/3;
	    win_adj=win_adj_residual;	/* Part not included above */
	    }
	else				/* 5, 2, or 1 dB/div */
	    {
	    power_exp=10;
	    win_adj=win_adj_loss;	/* Coherent loss factor for window */
	    }

	if (update_dsp)
	    dsp_cmd(9, bits_8_16, power_exp, LOG_OFFSET, (LOG_OFFSET>>8));
	}

/* show_1cur()  -  For keyboard command, shows location of the center
 * frequency on the spectral displays. Shows error if not valid.
 * RSL 22 Dec 02
 */
void show_1cur(void)
	{
	int nc;

	switch (mode)
	    {
	    case CW:
	    case USB:
		nc=(int)((double)cwoffset/df[fft_bw]);
		break;
	    case EME2:
	    case LTI:
		nc=(1+display_width)*lt_i3;
		break;
	    case LHL7:
		nc=7*16*(1+display_width);
		break;
	    case PUA43:
		/* Display the CW ID/tune freq; there is no useful center. */
		nc=b_id43[fft_bw];
		break;
	    default:
		do_t_error(27, "W27 No Center", 50);
		return;
	    }
	/* Now display the lines. The called function will hold control
	 * until any key is hit. */
	vert_cursor1(nc);
	}

/* show_2cur()  -  For keyboard command, shows location of the band edge
 * frequencies on the spectral displays. Shows error if not valid.
 * RSL 22 Dec 02
 */
void show_2cur(void)
	{
	int nc1, nc2;

	switch (mode)
	    {
	    case EME2:
	    case LTI:
		nc1=(1+display_width)*lt_i2;
		nc2=(1+display_width)*lt_i4;
		break;
	    case LHL7:
		nc1=4*16*(1+display_width);
		nc2=10*16*(1+display_width);
		break;
	    case PUA43:
		nc1=b0[fft_bw];
		nc2=nc1+168;
		break;
	    default:
		do_t_error(28, "W28 No Band", 50);
		return;
	    }
	/* Now display the lines. The called function will hold control
	 * until any key is hit. */
	vert_cursor2(nc1, nc2);
	}

/* SET_NOCOHAVE(WHY) - Sets the amount of non-coherent averaging of 
 * spectral data. Why is the reason for being here, so that scrolling
 * through the modes does not leave a wierd value from EME2, or the like.
 * 
 * why==0  ==> here from a mode change.
 * why==1  ==> here from changing amount of noncohave.
 * why==2  ==> special case for LHL7, change in code_nx2 may need new noncohave
 */
void set_noncohave(int why)
	{
	double x;
	static int ncsave=-1;		/* -1 = never been set */
	static nclhlsave=-1;

	/* LHL7 mode allows a number of different tone lengths, all multiples
	 * of 2 sec. The noncoherent averaging time needs to be a multiple
	 * of 18 to be consistent with the basic 2 second multiple, and also
	 * needs to be less than the tone length. The following trees implement
	 * these constraints. Note LHL7 uses only fft_bw=2 (4800 Hz max)   */
	if (mode==LHL7)
	    {
	    if (why==0)			/* Mode change */
		{
		if (nclhlsave>0)
		    noncohave=nclhlsave;
		else
		    noncohave=3;
		}
	    else if (why==1)
		{
		if (noncohave==3 && d_noncoh==-1) return;  /* Never less */
	        switch(code_nx2)
		   {
		   case 30:
		    if (noncohave==3 && d_noncoh==1) {noncohave=6; break;}
		    if (noncohave==6 && d_noncoh==-1){ noncohave=3; break;}
		    if (noncohave==6 && d_noncoh==1) {noncohave=9; break;}
		    if (noncohave==9 && d_noncoh==-1) {noncohave=6; break;}
		    if (noncohave==9 && d_noncoh==1) {noncohave=15; break;}
		    if (noncohave==15 && d_noncoh==-1) {noncohave=9; break;}
		    if (noncohave==15 && d_noncoh==1) {noncohave=30; break;}
		    if (noncohave==30 && d_noncoh==-1) {noncohave=15; break;}
		    if (noncohave==30 && d_noncoh==1) {noncohave=45; break;}
		    if (noncohave==45 && d_noncoh==-1) {noncohave=30; break;}
		    if (noncohave==45 && d_noncoh==1) {noncohave=90; break;}
		    if (noncohave==90 && d_noncoh==-1) {noncohave=45; break;}
		    if (noncohave==90 && d_noncoh==1) break;
		   case 10:
		    if (noncohave==3 && d_noncoh==1) {noncohave=6; break;}
		    if (noncohave==6 && d_noncoh==-1) {noncohave=3; break;}
		    if (noncohave==6 && d_noncoh==1) {noncohave=15; break;}
		    if (noncohave==9) {noncohave=6; break;} /*Chng code_nx2*/
		    if (noncohave==15 && d_noncoh==-1) {noncohave=6; break;}
		    if (noncohave==15 && d_noncoh==1) {noncohave=30; break;}
		    if (noncohave==30 && d_noncoh==-1) {noncohave=15; break;}
		    if (noncohave==30 && d_noncoh==1) break;
		    if (noncohave>30) {noncohave=30; break;}
		   case 5:
		    if (noncohave==3 && d_noncoh==1) {noncohave=15; break;}
		    if (noncohave==6 || noncohave==9) {noncohave=3; break;}
		    if (noncohave==15 && d_noncoh==-1) {noncohave=3; break;}
		    if (noncohave==15 && d_noncoh==1) break;
		    if (noncohave>15) {noncohave=15; break;}
		   case 2:
		    if (noncohave==3 && d_noncoh==1) {noncohave=6; break;}
		    if (noncohave==6 && d_noncoh==-1) {noncohave=3; break;}
		    if (noncohave==6 && d_noncoh==1) break;
		    if (noncohave>6) {noncohave=6; break;}
		   case 1:
		    noncohave=3;
		    break;
		   }			/* End, switch(code_nx2) */
		nclhlsave=noncohave;
		}			/* End, why==1 */
	    else if (why==2)		/* Adjust noncohave to legal value */
		{
	        switch(code_nx2)
		   {
		   case 30:
		    break;		/* They are all valid */
		   case 10:
		    if (noncohave<=6) break;
		    if (noncohave==9) {noncohave=6; break;}
		    if (noncohave<=30) break;
		    if (noncohave>30) {noncohave=30; break;}
		   case 5:
		    if (noncohave==3 || noncohave==15) break;
		    if (noncohave==6 || noncohave==9) {noncohave=3; break;}
		    noncohave=15;
		    break;
		   case 2:
		    if (noncohave==3 || noncohave==6) break;
		    noncohave=6;
		    break;
		   case 1:
		    noncohave=3;
		    break;
		   }			/* End, switch(code_nx2) */
		nclhlsave=noncohave;
		}			/* End, why==2 */
	    }				/* End LHL7 */
	else if (mode==EME2)
	    {
	    /*  if (why==0) */      noncohave=3;
	    }
	else if (mode==PUA43)
	    {
	    noncohave=3;
	    }
	else				/* All other modes,  CW, USB, etc */
	    {
	    if (why==0 && ncsave>0) noncohave=ncsave;
	    if (d_noncoh==-1)
		{
		noncohave--;
		if (noncohave<1) noncohave=1;
		}
	     else if (d_noncoh==1)
		{
		noncohave++;
		if (noncohave>1667) noncohave=1667;
		}
		ncsave=noncohave;	/* Needed if mode is changed */
	    }				/* End, other modes */

	d_noncoh=0;			/* In case reenter for mode change */

	/* Print as integer if not xx.5 */
	x=fft_per[fft_bw]*(double)noncohave;
	if ((int)(x+.01) == (int)(x+0.51))
	    sprintf(cc, "%d", (int)(x+0.01) );
	else
	    sprintf(cc, "%.1f", x);
	setviewport(92,368,126,377,DO_CLIP);
	clearviewport();
	settextstyle(0,0,1);
	settextjustify(RIGHT_TEXT, TOP_TEXT);
	moveto(33, 1);	outtext(cc);
	}

/*
 * draw_grid()  - Assumes that BGI graphics are initialized
 * and draws grid for plotting of SA levels and threshold crossings.
 * Adds annotation for readability.
 * Set up for VGA 640 x 480 pixels ONLY
 */
void draw_grid(void)
	{
	unsigned int x, i;
	int xx, yy;

	new_freq();

	setviewport(0,0,639,479,DO_CLIP);
	setcolor(c_outline_bright);
	line(3,19,101,19); line(3,53,101,53);
	line(3,19,3,53); line(101,19,101,53);
	setcolor(c_text_bright);
	settextstyle(0,0,1);
	settextjustify(1,2);
	moveto(51,23); outtext(call_str);
	moveto(51,32); outtext("DSP TRANSCVR");
	moveto(51,43); outtext("W7PUA DSP10");

	setcolor(c_outline_bright);		/* Freq display border */
	rectangle(1,122,126,158);		/* Leave off corner pixels */
	rectangle(0,123,127,157);

	setcolor(c_text_bold);
	moveto(51,61);	outtext("RECEIVE");
	if (!hardware)
	    {
	    moveto(51, 71);
	    outtext("No RF Hdw");
	    }

	if (band==AUDIO_BAND && eme_doppler)	/* Only for audio processor */
	    {
	    moveto(51, 101);
	    sprintf(cc, "Fd=%dMHz", near_int(0.000001*f_eme) );
	    outtext(cc);
	    }
	for(i=1; i<=9; i++)
		{
		x = X2L + 1 + (int)(53.229 * (double)i);
		line(x, Y2T-2, x, Y2T);		/* Vertical ticks */
		}
	setcolor(c_outline_bold);
	line(X2L, Y2T, X2R, Y2T);		/* Lower spectral box */
	line(X2L, Y2B, X2R, Y2B);
	line(X2L, Y2T, X2L, Y2B);
	line(X2R, Y2T, X2R, Y2B);

	/* Now status info along left side */
	draw_s_meter();

	setviewport(0,0,639,479,DO_CLIP);

	settextjustify(0,2);
	settextstyle(0,0,1);
	setcolor(c_text);
	line(1,229,127,229);
	setcolor(c_text_bold);
	moveto(2,233);	outtext("Contrast");
	moveto(2,243);	outtext("Bright");
	setcolor(c_text);
	line(1,255,127,255);
	setcolor(c_text_bold);
	moveto(2,258);  outtext("MikeGain");
	moveto(2,268);  outtext("CW Speed");
	moveto(2,278);  outtext("Xmit Pwr");
	moveto(2,288);	outtext("Mode");
	setcolor(c_text);
	line(1,299,127,299);
	moveto(2,303);	outtext("RIT");
	setcolor(c_text_bold);
	moveto(2,313);	outtext("Filter");
	moveto(2,323);  outtext("DeNoise");
	moveto(2,333);  outtext("Binaural");

/*	setcolor(c_text);
	moveto(2,333);  outtext("ANL"); */
/*	moveto(2,243);  outtext(""); */		/* Empty line */

	setcolor(c_text);
	line(1,354,127,354);
/*	moveto(2,358);  outtext("BinAve"); */   /* Not used now */
	setcolor(c_text_bold);
	moveto(2,368);	outtext("SpecAve");
	moveto(2,378);	outtext("SpecAnl");

	setcolor(c_text);
	line(1,389,127,389);
	setcolor(c_text_bold);
	moveto(2,393);	outtext("AF Gain");
	moveto(2,403);	outtext("RF Gain");
	moveto(2,413);	outtext("AGC");
	moveto(2,423);  outtext("AutDisp");

	setcolor(c_text);
	line(1,434,127,434);

	setcolor(c_text_bold);
	setviewport(1,1,639,479,DO_CLIP);
	settextstyle(0,0,1);
	settextjustify(2,2);
	moveto(126, 233);
	sprintf(cc, "%d", contrast);
	outtext(cc);
	moveto(126, 243);
	sprintf(cc, "%d", bright);
	outtext(cc);
	moveto(126, 258);
	sprintf(cc, "%d", mike_gain);
	outtext(cc);
	moveto(126, 268);
	outtext(t[cw_speed].description);
	moveto(126, 278);
	sprintf(cc, "%d", xmit_power);
	outtext(cc);
/*	setcolor(c_text);
	moveto(126, 358);
	sprintf(cc, "%d", 1); */		/* BinAve - not used now */
	outtext(cc);
	setcolor(c_text_bold);
/*	moveto(126, 368);
	sprintf(cc, "%d", noncohave);
	outtext(cc);
 */

	moveto(126, 413);
	sprintf(cc, "%d", rfg[rfgain].dbdisplay);
	outtext(cc);
/*	setcolor(c_text);
	moveto(126, 423);
	sprintf(cc, "%d", 100);
	outtext(cc);
	setcolor(c_text_bold);
*/
	if(!waterfall)   draw_panel();
	sa_graph();			/* Put spectrum analyzer graph up */
	set_db_div(1);			/* Add numbers on left */

	normalize_plot(0);		/* Clear normalization */
	}

/* Draw the upper right portion of the screen for the spectrum analyzer
 * display.  Separated so that this same area can be used for a modal
 * dialog box.  Box outline  and markings only.  Grid is drawn during
 * the plotting of the data.
 */
void sa_graph(void)
	{
	int i, yy, kdb;

	setviewport(X1L-12, Y1T, X1R, Y1B, DO_CLIP);	
	setcolor(c_outline);
	line(12,         0,       12+X1R-X1L, 0);
	line(12,         Y1B-Y1T, 12+X1R-X1L, Y1B-Y1T);
	line(12,         0,       12,         Y1B-Y1T);
	line(12+X1R-X1L, 0,       12+X1R-X1L, Y1B-Y1T);

	settextstyle(0, 0, 1);
	settextjustify(1,1);
	/* db_div_set:  0=10db/div, 1=5db, 2=2db, 3=1db */
	if (db_div_set==0) kdb=10;
	else if (db_div_set==1) kdb=5;
	else if (db_div_set==2) kdb=2;
	else kdb=1;
	for (i=0; i<=8; i++)
		{
		yy = i*16;
		moveto(0, yy);
		sprintf(cc, "%d", kdb*(8 -i));
		outtext(cc);		/* dB level markings */
		}
	}

void mark_frequency(int fmax)
	{
	int i, k;

	setcolor(c_text);
	settextstyle(0, 0, 1);
	settextjustify(1, 2);
	setviewport(178, 134, 638, 144, DO_CLIP);
	clearviewport();
	moveto(430, 0);
	if (fmax<10000)
		{
		k=fmax/4;
		outtext("Hz");
		}
	else
		{
		k=fmax/4000;
		outtext("kHz");
		}
	for(i=0; i<4; i++)
		{
		moveto(57+i*107, 0);
		sprintf(cc, "%d", k*(i+1));
		outtext(cc);			/* Freq markings */
		}
	setcolor(c_text_bold);
	}

/* draw_panel() is used for the conventional panel display.  The lower
 * waterfall window is divided into two parts, the lower 2/3 for help
 * info and the frequency display in the top 1/3.  This display is never
 * used with the waterfall display.
 */
void draw_panel(void)
	{
	int i, x;

	/* 128, 639   d=511  */
	setviewport(X2L+1, Y2T, X2R-1, Y2T+95, DO_CLIP);
	clearviewport();
	setcolor(c_outline);
	setlinestyle(SOLID_LINE, 0, THICK_WIDTH);
	rectangle(50, 2, 461, 55);	/* Big Frequency Box */
	setcolor(c_outline_bold);
	line(55, 41, 455, 41);
	setlinestyle(SOLID_LINE, 0, NORM_WIDTH);
	line(0, 0, 46,0);  line(0, 3, 46, 3);	/* Trim lines at top */
	line(465, 0, 510, 0);  line(465, 3, 510, 3);

	for(i=0; i<5; i++)
	    {
	    x = 95+i*80;
	    line(x, 43, x, 32);
	    tiny_char(1,   x-9, 51);   /* Print with bottom left at coords */
	    tiny_char(4,   x-3, 51);
	    tiny_char(i+4, x+3, 51);
	    }
	for(i=0; i<25; i++)
	    line(63+i*16, 43, 63+i*16, 37);

	/* Now put the general help screen at the bottom: */
	setcolor(c_text_bold);
	settextstyle(DEFAULT_FONT, HORIZ_DIR, 1);

	settextjustify(0,2);		/* Left and top */
	/* Works out to be 224 pixels high: */
	setviewport(X2L+1, Y2T+96, X2R-1, Y2B-12, DO_CLIP);
	getimage(0,  0, 510, 160, big_area_ptr1);
	getimage(0, 161, 510, 224, big_area_ptr2);
	clearviewport();
	line(1, 5, 509, 5);		/* Half into "first" line (blank) */
	line(1, 223, 509, 223);
 /* 63 characters:
          123456789012345678901234567890|23456789012345678901234567890123 */
 moveto(3, 3);			/* Line 1-1 */
 /* Blank */
 moveto(3, 13);			/* Line 1-2 */
 outtext("               DSP-10 KEYBOARD COMMANDS - Summary");
 moveto(3, 23);			/* Line 1-3 */
 outtext("       FREQUENCY              |                 GAIN");
 moveto(3, 33);			/* Line 1-4 */
 outtext("1 Hz   Ctrl-Alt-Sh F9 & F10   |   AF Gain               F7 & F8");

 moveto(3, 43);			/* Line 1-5 */
 if(kbd_alt1 && kbd_alt3)
 outtext("10 Hz Down            Sh F9   |   RF Gain          Scrl F7 & F8");
 else if (!kbd_alt1 && kbd_alt3)
 outtext("10 Hz Down               F9   |   RF Gain          Scrl F7 & F8");
 else if (kbd_alt1 && !kbd_alt3)
 outtext("10 Hz Down            Sh F9   |   RF Gain            Sh F7 & F8");
 else
 outtext("10 Hz Down               F9   |   RF Gain            Sh F7 & F8");

 moveto(3, 53);			/* Line 1-6 */
 if (kbd_alt1)
 {
 outtext("10 Hz Up              Sh F10  |   Mike Gain      Alt-Sh F7 & F8");
 moveto(3, 63);			/* Line 1-7 */
 outtext("100 Hz              F9 & F10  |");
 }
 else
 {
 outtext("10 Hz Up                 F10  |   Mike Gain      Alt-Sh F7 & F8");
 moveto(3, 63);			/* Line 1-7 */
 outtext("100 Hz           Sh F9 & F10  |");
 }

 moveto(3, 73);			/* Line 1-8 */
 outtext("1 kHz           Alt F9 & F10  |   CW Send Speed     Alt F7 & F8");
 moveto(3, 83);			/* Line 1-9 */
 outtext("10 kHz         Ctrl F9 & F10  |   Require Scroll Lock On:");
 moveto(3, 93);			/* Line 1-10 */
 outtext("100 kHz    Ctrl-Alt F9 & F10  |   AGC Release        Scrl G & H");
 moveto(3, 103);		/* Line 1-11 */
 outtext("1 MHz       Ctrl-Sh F9 & F10  |   Transmit Power     Scrl O & P");
 moveto(3, 113);		/* Line 1-12 */
 outtext("                              |   CW Sidetone Lev    Scrl S & D");
 moveto(3,123);			/* Line 1-13 */
 outtext("FM Squelch        Alt S & D   |   ");
 moveto(3,133);			/* Line 1-14 */
 outtext("                              |");
 moveto(3,143);			/* Line 1-15 */
 outtext("                              |       TRANSMIT/RECEIVE");
 moveto(3, 153);		/* Line 1-16 */
 outtext("RIT On/Off            Alt O   |   T/R Toggle               Home");
 moveto(3, 163);		/* Line 1-17 */
 outtext("RIT Down/Up 10Hz  Alt R & T   |");
 moveto(3, 173);		/* Line 1-18 */
 outtext("RIT Zero              Alt Z   |          AF FILTERS");
 moveto(3, 183);		/* Line 1-19 */
 outtext("Mode change           Alt M   |   Filters On/Off             F3");
 moveto(3, 193);		/* Line 1-20 */
 outtext("Help Screens             F1   |   Filter Select              F4");

 moveto(3, 203);		/* Line 1-21 */
 if (kbd_alt3)
 outtext("Display Format       Scrl Y   |   DeNoise & Notch On/Off  Sh F3");
 else
 outtext("Display Format        Alt Y   |   DeNoise & Notch On/Off  Sh F3");

 moveto(3, 213);		/* Line 1-22 */
 outtext("Exit to DOS     Scrl-Alt F4   |   De-Noise Gain    Ctrl F3 & F4");
	}

void draw_s_meter(void)
	{
	/* Bar graph     POS-19 to POS-9.
	 * Annotation    POS-7 to POS+1
	 * Title         POS+2 to POS+12
	 * Strength      POS+12 to POS+22
	 * Freq          POS+22 to POS+31 (228 absolute)
	 */
	setviewport(0, S_METER_POS-7, 127, S_METER_POS+31, DO_CLIP);
	clearviewport();
	/* Place S-Meter annotation, ready for bar graph */
	settextjustify(CENTER_TEXT, TOP_TEXT);
	settextstyle(0, 0, 1);
	setcolor(c_text);
	moveto(64,9);	outtext("Signal Level");
	/* bar_graph()  bars are every 6 pixels, starting at 20 */
	/* Annotation 3 wide, 4 high, specified by lower left corner */
	micro_char(1, 21, 4); micro_char(5, 44, 4); micro_char(9, 68, 4);
	micro_char(2, 79, 4); micro_char(0, 83, 4);
	micro_char(4, 91, 4); micro_char(0, 95, 4);
	micro_char(6, 103, 4); micro_char(0, 107, 4);
	}

void draw_clip(void)
	{
	/* Bar graph     POS-19 to POS-9.
	 * Annotation    POS-6 to POS+2
	 * Title         POS+6 to POS+16
	 */
	setviewport(0, S_METER_POS-7, 127, S_METER_POS+31, DO_CLIP);
	clearviewport();

	/* Place Audio Clipping Meter annotation, ready for bar graph */
	settextjustify(CENTER_TEXT, TOP_TEXT);
	settextstyle(0, 0, 1);
	setcolor(c_text);
	moveto(64,9);	outtext("Clipping, dB");
	/* bar_graph()  bars are every 6 pixels, starting at 20 */
	micro_char(1, 21, 4); micro_char(6, 51, 4);
	micro_char(1, 84, 4); micro_char(2, 88, 4);
	}

/* set_bins()  -  Sets up the global bin numbers  lt_i1, lt_i2,... used
 * in both EME2 and LTI long term integration.
 */
void set_bins(void)
	{
	int mid;

	/* For EME2 (center freq 323, 600, 900  and 886 Hz):
	 *   FFT  low disp  low detail  center  high detail  high disp
	 *  1200     90        128       138        148         186
	 *  2400     80        118       128        138         176
	 *  4800     48         86        96        106         144
	 *  4800D     7         37        47         57          97
	 */
	if (mode==EME2)
	    {
	    if (fft_bw==0)
		{
		lt_i1=90;
		lt_i2=128;
		lt_i3=138;
		lt_i4=148;
		lt_i5=186;
		}
	    else if (fft_bw==1)
		{
		lt_i1=80;
		lt_i2=118;
		lt_i3=128;
		lt_i4=138;
		lt_i5=176;}
	    else if (fft_bw==2)
		{
		lt_i1=48;
		lt_i2=86;
		lt_i3=96;
		lt_i4=106;
		lt_i5=144;
		}
	    else
		{
		lt_i1=7;
		lt_i2=37;
		lt_i3=47;
		lt_i4=57;
		lt_i5=97;
		}
	    }
	else if (mode==LTI)
	    {
	    lt_i1=(int)(0.5+(lterm_int_flow/df[fft_bw]));
	    if (lt_i1<1 || lt_i1>234) lt_i1=1;

	    lt_i5=(int)(0.5+(lterm_int_fhigh/df[fft_bw]));
	    if (lt_i5>254 || lt_i5<21) lt_i5=254;

	    /* Be sure there is enough room for 21 detailed points */
	    if ((lt_i5-lt_i1) <= 21)
		{
		lt_i1=1;
		lt_i5=254;
		}

	    /* lt_i3 is the desired center. Be  sure it is inside the range */
	    lt_i3=(int)(0.5+(lterm_int_center/df[fft_bw]));
	    if (lt_i3<=(lt_i1+10) || lt_i3>=(lt_i5-10))
		 lt_i3=(lt_i1+lt_i5)/2;

	    lt_i2=lt_i3-10;		/* Set limits for 21 detailed pts */
	    lt_i4=lt_i3+10;
	    }
	}

/* fsk2sec() sends the selected fsk tone for 1.8 seconds. This gives
   enough time for changing tone and getting back before next 2 sec start.
 */
void fsk2sec(void)
	{
	dsp_cmd('T', 0X0011, 2,		/* Xmit, no stat, key dwn */ 
		fgainl[xmit_power], fgainh[xmit_power]);
	delay(450);
	update_clock(0);
	delay(450);
	update_clock(0);
	delay(450);
	update_clock(0);
	delay(450);
	update_clock(0);
	dsp_cmd('T', 0X0001, 2,		/* Npw key up */
		fgainl[xmit_power], fgainh[xmit_power]);
	}

/* update_clock(int fcn)  -  can be called often to
 * add time to the 'millisec' clock. 
 * If (fcn==0) the return value is 0 unless an
 * integral second was passed by the update.
 * If (fcn==2) there is no 2.6 sec EME offset. This allows clock set during
 * LHL7 and PUA43.
 * The time returned is in seconds, suitable for localtime() etc, after
 * adding dsp_clock_base.
 */
long int  update_clock(int fcn)
	{
	long int          cl_diff, milli_new, m;
	static long int   cl_old=0;
	static long int   m_last=0;

	cl_ms=clock();			/* Returns milliseconds */
	cl_diff=cl_ms-cl_old;		/* Integer millisecs */
        cl_old=cl_ms;			/* To keep accumulating millisec */
	millisec += (clock_cor_factor*(double)cl_diff); /* Update dsp clock */
	/* millisec_offset, a double, moves the tick times around in a
	   second and also offsets the time by a few seconds to bring
	   computer set time to correct time */
	milli_new=(long int)(millisec+millisec_offset);

	/* Offset by a eme delay, if appropriate */
	if ((!transmit) && (fcn!=2) && eme_doppler
		 && (mode==LHL7 || mode==PUA43))
	    milli_new -= 2600;

	m=milli_new/1000L;		/* Whole seconds in milli_new */
        if ( m != m_last || fcn==1 )
	   {m_last=m;
	    return m;
	   }
	else
	   {
	    return 0L;
	   }
	}

/* set_clock(cmd)  -  Uses line at screen bottom to set/change the internal
   dsp-10 clock.
   cmd=0  ==> initialization of clock.
   cmd=1  ==> changing of clock.
   This function is modal and cannot be left running or the other functions
   will stop.
   Note: clock() starts fresh with startup, but will rollover after 594 hours
   with errors at that point. Work on that later! [Maybe update_clock()<0 ? ]
   Added '^' function with v3.09.
 */
void set_clock(int cmd)
	{
	char  c3[50];
	double         dsp_clock;
	unsigned int   ichrr;
	long int       t_new, ttt;
	struct tm      *tp;

	if (ms_available==0) return;

	/* Set dsp10 clock to computer clock.
           Then for dsp10 use millisec + a base offset to convert
           to a long int that is display time.
	 */
	if (cmd==0)		/* Initialization stuff, done at startup:   */
	   {dsp_clock_base=time(NULL);
	    /* In update _clock(), need to initialize cl_old. This also sets
	     * millisec, which we will reset at the next line. v3.09
	     */
	    update_clock(1);
	    millisec=0.0;
	    return;
	   }
	else if (cmd==1)	/* Setclock */
	   {setviewport(129, 390, 638, 479, DO_CLIP); /* Clear clock space */
	    getimage(0,  0, 510, 89, big_area_ptr1);  /* For later restore */
	    clearviewport();
	    rectangle(2, 2, 508, 87);

	    clock_been_set=TRUE;
	    settextjustify(LEFT_TEXT,TOP_TEXT);
	    moveto(20, 6);
	    outtext("------------------ DSP-10 CLOCK SET ------------------");
	    moveto(20, 18);
	    sprintf(cc," Back  1 sec     (      |      Forward  1 sec      )");
	    cc[17]=ky_clock_m1s;  cc[51]=ky_clock_p1s;
	    outtext(cc);
	    moveto(20, 28);
	    sprintf(cc," Back 10 mSec    {      |      Forward 10 mSec     }");
	    cc[17]=ky_clock_m10ms;  cc[51]=ky_clock_p10ms;
	    outtext(cc);
	    moveto(20, 38);
	    sprintf(cc," Rate slower     <      |      Rate faster         >");
	    cc[17]=ky_clock_mrate;  cc[51]=ky_clock_prate;
	    outtext(cc);
	    moveto(20, 56);
	    sprintf(cc," Exit by * or <Enter>             Re-Sync to PC by ^");
	    cc[9]=ky_clock_exit;  cc[51]=ky_clock_resync;
	    outtext(cc);

	    setviewport(132, 466, 636, 475, DO_CLIP); /* Clear bottom line */
	    clearviewport();		/* Just the bottom line */
	    settextjustify(0,2);

	    while (1)
		{if ( (t_new=update_clock(2)) != 0L)
        	     {if (lpt_port != 0)
                          outp(lpt_port, 1);
		      /* Removed req on beep for lpt_port==0, rev 3.35  */
                      beep(tick_time);	/* Do this once per second */
                      if (lpt_port != 0)
                          {delayu(ntau);	/* Really ntau+time_tick */
                           outp(lpt_port, 0);
                          }
                      ttt=dsp_clock_base+t_new;
		      if (time_type)
			tp=localtime(&ttt);	/* time_type==1 for local */
		      else
			tp=gmtime(&ttt);	/*          ==0 for gmt */
                      strcpy(c3, asctime(tp));	/* Rearrange the characters */
                      c3[19]=0;
                      sprintf(cc,
			  "DSP-10 time=%s   Clock rate=%.7lf   Offset=%.3lf",
			  &c3[11], clock_cor_factor, 0.001*millisec_offset);
		      clearviewport();
                      moveto(5, 0); outtext(cc);
                     }
                 if (kbhit() != 0)
                     {ichrr = 0X7F & _bios_keybrd(_KEYBRD_READ);
                      if      (ichrr == ky_clock_m10ms) millisec_offset-=10.0;
                      else if (ichrr ==  ky_clock_p10ms) millisec_offset+=10.0;
                      else if (ichrr ==  ky_clock_m1s) millisec_offset-=1000.0;
                      else if (ichrr ==  ky_clock_p1s) millisec_offset+=1000.0;
                      else if (ichrr ==  ky_clock_mrate)
			 clock_cor_factor -= 0.0000001;
                      else if (ichrr ==  ky_clock_prate)
			 clock_cor_factor += 0.0000001;
                      else if (ichrr ==  ky_clock_exit || ichrr == 13)
			  {setviewport(129, 390, 638, 469, DO_CLIP);
			   clearviewport();
			   putimage(0, 0, big_area_ptr1, COPY_PUT);
			   return;
			  }
		      else if (ichrr ==  ky_clock_resync)  /*  to PC clock */
			  {dsp_clock_base=time(NULL);	/* Get PC Clock */
			   update_clock(1);		/* For cl_old */
			   millisec=0.0;		/* New clock */
			   millisec_offset=0.0;
			  }
                      /* Ignore everything else */
                     }
              }				/* End while(1) */
	   }				/* End cmd==1 */
	}				/* End fcn */

/* Diagnostics usually uses the bottom line on the display to display info
 * about the inner workings of the transceiver.
 * Apr99 Added Moon/Sun track as 'diagnostic' option.
 * The data depends on the value of diag_mode:
 *	0  No diagnostic info is shown.
 *	1  The 20 bytes of status_buf data are printed in hex.
 *	2  Two bar graphs of A/D lvel and D/A level are shown.
 *	3  Moon data
 *	4  Sun data
 */
void diagnostics(int do_what)
	{
	int i, j, x;
	double del;
	unsigned int del_bar;
	if (do_what==0)			/* do_what==0 for toggle diag_mode */
		{
		diag_mode++;

		if ((diag_mode) > 4)  diag_mode=0;

		setviewport(129, 470, 639, 479, DO_CLIP); /* Clear main line */
		clearviewport();
		settextjustify(0,2);
		moveto(10, 0);
		sprintf(cc, "%.1d", diag_mode);
		outtext(cc);
		setviewport(0, 470, 40, 479, DO_CLIP); /* Clear left line */
		clearviewport();

		if (diag_mode==3 || diag_mode==4)
		    set_cur_loc(-1);		/* Update screen at bottom */
		set_eme_doppler();		/* Update eme mark */
		return;
		}

	/* PC and DSP Versions need to be compatible */
	if (dsp_status[0] < 0X25)
	    do_t_error(18, "W18 DSP Version", 0);
	if (transmit && (mode==USB || mode==LSB || mode==FM)) /* Show clip */
	    {
	    /* Show transmit audio clipping in all voice modes */
	    setviewport(0,S_METER_POS-19,127,S_METER_POS-9,DO_CLIP);
	    clearviewport();
	    /* Fix up the numbers for 1 dB per bar on bargraph. Using 8190 in
	     * place of 8192 suppresses first bar with no clipping  */
	    del = 0.000122070313*(double)
		(8190+256*dsp_xmit_status[3]+dsp_xmit_status[2]);
	    /* del is the ratio of input to output voltage for clipper.
	     * del is 1.0 to 4.0.  To get 12 bars lit up at 4.0, we need to
	     * decrease 6.02 dB/bar to 1.0 dB/bar. 3.9 brings up all red
	     * just before running out of clipping.      */
	    if (del<0 || del>3.9)
		del_bar = 32767;		/* Overload-all red segments */
	    else
		del_bar = (int)pow(del, 6.02059999);  /* 1 dB/segment */
	    /* Clipping bar graph: */
	    setviewport(0,S_METER_POS-19,127,S_METER_POS-9,DO_CLIP);
	    bar_graph(20, 0, del_bar, 0 );
	    }

	/* do_what==1 for display according to diag_mode */
	if (diag_mode>0)
	  {
	  setviewport(0, 470, 40, 479, DO_CLIP);
	  clearviewport();
	  settextstyle(0, 0, 1);
	  settextjustify(LEFT_TEXT, TOP_TEXT);
	  moveto(0, 0);
	  sprintf(cc, "%d", ASYNCINSTAT() );  /* Always put loading info out */
	  outtext(cc);
	  }
	if (diag_mode==0)
	  {
	  setviewport(0, 470, 40, 479, DO_CLIP); /* Clear left corner */
	  clearviewport();
	  setviewport(129, 470, 639, 479, DO_CLIP); /* Clear main line */
	  clearviewport();
	  for(j=1; j<=9; j++)
	      {
	      x = (int)(53.229 * (double)j);
	      line(x, 0, x, 5);		/* Frequency Ticks */
	      }
	  return;
	  }
	else if (diag_mode==1)
	    {setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
	     clearviewport();
	     settextjustify(0,2);
	     for(i=0; i<19; i++)
		{
		moveto(2+i*20, 0);
		sprintf(cc, "%2.2X", dsp_status[i]);
		outtext(cc);
		}
	    moveto(389, 0);
	    if (transmit)
		outtext("T");
	    else
		outtext("R");
	    moveto(401, 0);
	    outtext(version);		/* 2.12, etc */

	    /* This is disabled because there is no data here to show:
	    if(transmit)
		{
		setviewport(150, 30, 450, 50, DO_CLIP);
		clearviewport();
		settextstyle(0, 0, 1);
		settextjustify(0, 2);
		for(i=0; i<10; i++)
		    {
		    moveto(10+i*20, 0);
		    sprintf(cc, "%2.2X", dsp_xmit_status[i]);
		    outtext(cc);
		    if (4*(i/4)==i)  outtext(" ");
		    }
		}
	    */

	    if ((!transmit) && auto_disp_on)
		{moveto(400, 0);
		 if (ave_spectral_pwr<1e-12)
			ave_spectral_pwr=1e-12;
		 sprintf(cc, "%7.2lf", p2db(ave_spectral_pwr));
		 outtext(cc);
		}
	    }
	else if (diag_mode==2)	/* Show Bar Graphs */
	    {
	    setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
	    clearviewport();

	    moveto(10, 0);
	    outtext("A/D Level:");
	    bar_graph( 100, 0, 256*dsp_status[10]+dsp_status[9], 0 );

	    moveto(220, 0);
	    outtext("D/A Level:");
	    if (!transmit)
	        bar_graph( 300, 0, 256*dsp_status[12]+dsp_status[11], 0 );

	    /* Show the DAC level, which in the DSP was reduced by 0.5 to
	     * allow overload to be a separate bit. Style 2 of the
	     * bargraph multiplies the display back x2 and shows o'load.
	     * For DSP convenience, this is not done in other modes and
	     * for consistency, the scaling is changed here.     */
	    if (transmit)
		{
		if (mode==USB || mode==USB)
		    bar_graph(300, 0, 256*dsp_status[12]+dsp_status[11], 2);
		else
		    bar_graph(300,0, (256*dsp_status[12]+dsp_status[11])/2, 2);
		}
	    if ((!transmit) && auto_disp_on)
		{moveto(400,0);
		 if (ave_spectral_pwr<1e-12)
			ave_spectral_pwr=1e-12;
		 sprintf(cc, "%7.2lf", p2db(ave_spectral_pwr));
		 outtext(cc);
		}
	    }
	}

/* Preliminaries and tables - this is preliminary, but it must follow
 * get_mem() so that arrays to be initialized exist.
 * Initializes radiofunctions.
 * The ASYNCINIT startup must proceed this function.
 * If amt==1, total initialization
 *    amt==2, update controls as needed after DOS return
 */
void initialize(int amt)
	{
	int i, ifg;
	double fg;
	long  a, b, fr;
	unsigned int xu;
	long int li1, li2;
	size_t sl;			/* length of string */

	if (amt==1)
	  {
	  srand(1);			/* Seed random num gen */
	  set_clock(0);			/* Synchronize to PC clock */
	  }
	/* Correction ratio, dff, derived from the .cfg variable ezkit_dfreq */
	/* The 15375 assumes error measurement at 375 Hz LTI, 145.0 MHz */
	dff = (15375.0-ezkit_dfreq)/15375.0;	/* dff is used internally */

	if (amt==1)
	  {
	  /* Following vector converts (0,255), representing 0 to
	      80 dB into a power ratio (1.0, 10^8).  Use vector for speed */
	  for (i=0; i<256; i++)		/* Convert log to power */
		power10[i]=pow(10.0, 0.03125*(double)i );

	  /* Gain function in 1 dB steps for use in dsp */
	  /* fgain[ ] is the voltage, normalized to 1.0 */
	  for(i=1; i<=100; i++)		/* Setup gain fcn as 2 bytes */
		{fg = pow(10.0, (double)(i-100) / 20. );
		 fgain[i] = fg;
		 ifg = near_int(32767.0*fg);
		 fgainl[i] = ifg & 0XFF;
		 fgainh[i] = (ifg >> 8) & 0XFF;
		}

	  for(i=0; i<515; i++)  raw_data[i]=0;
	  for(i=0; i<256; i++)  pd16[i]=1.0e-100;  /* Rev 3.19; was 0 */
	  for(i=0; i<515; i++)  dnorm[i]=0.0;
	  for(i=0; i<256; i++)  lt_dnorm[i]=0.0;
	  for(i=0; i<21;  i++)  eme_pwr_sum[i]=0.0;

	  /* Setup reference divider in synthesizer */
	  send_shift_data(R10MHZ);	/* R1MHZ, R5MHZ, R6_4MHZ or R10MHZ */
	  delay(250);
	  }				/* End if (amt==1) */

	if (!waterfall)
	    {
	    draw_panel();
	    setviewport(0,124,127,161,DO_CLIP);	/* Small Freq Box */
	    clearviewport();
	    }

	setviewport(0,60,101,70,DO_CLIP);
	clearviewport();
	settextstyle(0, 0, 1);
	settextjustify(1,2);
	setcolor(c_text_bold);
	moveto(50,1);
	outtext("RECEIVE");

	if (amt==1)			/* First startup */
	  {
	  print_big(0, -1, 0);		/* Initialize scroll printing */
	  print_medium(0, -1);
	  print_medium_1(0, -1);
	  print_medium_2(0, -1);
	  lhl7_update(0, -1);		/* Init statics */
	  pua43_update(0, -1);
	  morse(INIT_CW, 0);
	  }

	set_contrast();
	set_brightness();
	contrast_line_count=0;		/* Don't show "Contrast Line" */
	do_auto_disp(0);		/* Start with auto display off */
	set_disp_offset(1);
	do_window_save=do_window;
	if (amt==1)
	  set_mode(0);			/* Full set */
	else
	  set_mode(3);			/* Just display */
	set_window();
	set_afgain(0);
	set_rfgain();
	if (mode==FM)
	    set_agc(-1);
	else
	    set_agc(agc_decay);
	last_agc=agc_decay;
	set_affilt_sw();
	set_affilt_select();
	set_denoise();
	set_delay_right(delay_right);
	set_display_width();
	set_eme_doppler();
	if (amt==1)
	    {
	    new_vfo(cur_vfo);
	    /* Following is a kludge to make  sure that hardware is properly
	     * properly initialized.  Someday find out why needed!! */
	    tfreqL -= 1000000;	new_freq();
	    delay(10);
	    tfreqL += 1000000;	new_freq();
	    }
	if (amt==1)
	    {
	    /* Calibrate ntau delay to 20 useconds */
	    li1=biostime(0,0);
	    for (li2=0L; li2<1000000; li2++)  delayu(20);
	    li1=biostime(0,0) - li1;
	    ntau=(unsigned int)(20*(1+(int)( 503.0 / (double)li1 )));
	    }

	/* For signal simulation, need to initialize panel annotation: */
	if (xfade_rate < 0.0)  xfade_rate=0.0;
	settextjustify(LEFT_TEXT, TOP_TEXT);
	settextstyle(0, 0, 1);
	setcolor(c_text_note);

	setviewport(72, 278, 81, 288, NO_CLIP);
	clearviewport();
	moveto (0, 1);
	if (xnoise)
	    outtext("N");

 /*	setviewport(87, 278, 96, 288, NO_CLIP);  - Not yet implemented
	clearviewport();
	moveto (0, 1);
	if (xfade>0)
	    outtext("F");
  */

	setcolor(c_text_bold);

	/* Make sure .cfg has legal beacon parameters */
	if (b_transmit_sec < 0 || b_transmit_sec==1 || b_transmit_sec==2)
	     b_transmit_sec=0;
	if (b_receive_sec < 1)   b_receive_sec=1;
	if (b_offset_sec < 0)   b_offset_sec=0;

	/* Make sure that the file paths are either empty (default
	 * directory) or correct. Only minor checking done. */
	sl = strlen(fscreen_path);
	if (sl>0)			/* Make drive caps */
	    {
	    if(fscreen_path[0]=='a')
		fscreen_path[0]='A';
	    if(fscreen_path[0]=='b')
		fscreen_path[0]='B';
	    }
	/* Look for single 'A' or 'B'  */
	if (sl==1)
	    {
	    if(fscreen_path[0]=='A' || fscreen_path[0]=='B')
		{
		fscreen_path[1]=':';
		fscreen_path[2]='\\';
		fscreen_path[3]=0;
		sl=3;
		}
	    else
		{
		fscreen_path[0]=0;
		sl=0;
		}
	    }
	/* Make sure it ends in a backslash, called '\\' in C  */
	if (sl > 0  &&  fscreen_path[sl-1] != '\\')
	    {
	    fscreen_path[sl]='\\';
	    fscreen_path[sl+1]=0;
	    }

	sl = strlen(fdata_path);
	if (sl>0)			/* Make drive caps */
	    {
	    if(fdata_path[0]=='a')
		fdata_path[0]='A';
	    if(fdata_path[0]=='b')
		fdata_path[0]='B';
	    }
	/* Look for single 'A' or 'B'  */
	if (sl==1)
	    {
	    if(fdata_path[0]=='A' || fdata_path[0]=='B')
		{
		fdata_path[1]=':';
		fdata_path[2]='\\';
		fdata_path[3]=0;
		sl=3;
		}
	    else
		{
		fdata_path[0]=0;
		sl=0;
		}
	    }
	/* Make sure it ends in a backslash, called '\\' in C  */
	if (sl > 0  &&  fdata_path[sl-1] != '\\')
	    {
	    fdata_path[sl]='\\';
	    fdata_path[sl+1]=0;
	    }
	if (amt==1)
	  error_on_tr=7;		/* Prevent error on initial data */
	}

/* Print frequency data in a text box. 20 characters wide, 14 lines
 * high.
 * 
 */
void freq_data(int what, long int synth_fL, long int n_synL,
       long int fL,  long int if1L,  long int if2L, unsigned int dsp_dp)
    {
    static char cb[350];

	if (what==0)		/* Close FREQ, but no reset */
	   {
	   lb_close(&lb_freq, 0);
	   current_info_box=0;	/* Mark as none */
	   return;
	   }
	else if (what==1)		/* Open FREQ */
	   {
	   if (current_info_box != &lb_freq) /* If not open */
		{
		/* Only one at a time:     */
		if(current_info_box)  lb_close(current_info_box, 0);
		lb_create(&lb_freq, "INTERNAL FREQS",
		     474, 148, 638, 300, 0, NON_MODAL);
	    	current_info_box=&lb_freq;

		lb_open(&lb_freq, 0);

		/* Fixed text in FREQ box:     */
		lb_line(&lb_freq, 13, TXT, "  Close Box=Alt A", 0);
		}
	    lb_update(&lb_freq);	/* Draw the empty box */
	   }
	else if (what==2)
	   {
	    lb_open(&lb_freq, 0);
	    sprintf(cb+50,  " Carrier= %9ld", fL);
	    lb_line(&lb_freq, 2, BOLD, cb+50, 0);
	    sprintf(cb+75,  " LO#1   = %9ld", synth_fL);
	    lb_line(&lb_freq, 3, BOLD, cb+75, 0);
	    sprintf(cb+100, " LO#2   = %9ld", 19680000L);
	    lb_line(&lb_freq, 4, BOLD, cb+100, 0);
	    sprintf(cb+125, " IF#1   = %9ld", if1L);
	    lb_line(&lb_freq, 5, BOLD, cb+125, 0);
	    sprintf(cb+150, " IF#2 (DSP)  %6ld", if2L);
	    lb_line(&lb_freq, 6, BOLD, cb+150, 0);

	    sprintf(cb+175, " N-Synth     %6ld", n_synL);
	    lb_line(&lb_freq, 7, BOLD, cb+175, 0);

	    if (mode==CW)
		sprintf(cb+200, " CW Offset    %5d", cwoffset);
	    else if (mode==LTI || mode==LHL7 || mode==PUA43)
		sprintf(cb+200, " FSK          %5d", fsk);
	    else
		sprintf(cb+200, " No CW Offset or FSK");
	    lb_line(&lb_freq, 8, BOLD, cb+200, 0);

	    if (rit_on)
		sprintf(cb+225, " RIT          %5d", rit);
	    else
		sprintf(cb+225, " No RIT");
	    lb_line(&lb_freq, 9, BOLD, cb+225, 0);

	    sprintf(cb+250, " DSP, HL: 0X%.2X 0X%.2X",
		 (0XFF & (dsp_dp >> 8)), (0XFF & dsp_dp) );
	    lb_line(&lb_freq, 10, BOLD, cb+250, 0);

	    lb_update(&lb_freq);
	   }
	else return;			/* Safety on what */
    }

/* The following preset values are entered for now, but may be overwritten
 * by the .CFG file if the associated entries are present.
 * The tvtr[] structure provides one preset for each of the 16 VHF/UHF/uWave
 * bands plus 5 more 'Memory' channels that can be used on any band by the
 * UHFA.CFG file.  These are invoked by Ctrl-Alt x where x is either a band
 * from A to P or a memory channel 1 to 5.
 */
void presets(void)
	{
	int i;

	strcpy(tvtr[0].name, "50 MHz");
	tvtr[0].lo_type=0;
	tvtr[0].sb=-1;
	tvtr[0].lo_freq=-94000000.0;	/* - sign for upconversion */
	tvtr[0].chain_mult=1;
	strcpy(tvtr[0].init_str, "  ");
	tvtr[0].gain_db=20.0;
	tvtr[0].control=0;
	tvtr[0].bus_adr=0;
	tvtr[0].last_tfreq=0.0;
	tvtr[0].last_mode=0;

	/* 2 Meters is a dummy transverter with LO frequency of 0.0 Hz */
	strcpy(tvtr[1].name, "144 MHz");
	tvtr[1].lo_type=0;
	tvtr[1].sb=-1;
	tvtr[1].lo_freq=0.0;
	tvtr[1].chain_mult=1;
	strcpy(tvtr[1].init_str, "  ");
	tvtr[1].gain_db=0.0;
	tvtr[1].control=0;
	tvtr[1].bus_adr=0;
	tvtr[1].last_tfreq=0.0;
	tvtr[1].last_mode=0;

	for (i=2; i<9; i++)
	    tvtr[i].lo_type=999;	/* Transverter not installed */

	strcpy(tvtr[9].name, "10 GHz");
	tvtr[9].lo_type=0;
	tvtr[9].sb=-1;
	tvtr[9].lo_freq=10224000000.0;
	tvtr[9].chain_mult=1;
	strcpy(tvtr[9].init_str, "  ");
	tvtr[9].gain_db=28.0;
	tvtr[9].control=0;
	tvtr[9].bus_adr=0;
	tvtr[9].last_tfreq=0.0;
	tvtr[9].last_mode=0;

	for (i=10; i<15; i++)
	    tvtr[i].lo_type=999;	/* Transverter not installed */

	strcpy(tvtr[AUDIO_BAND].name, "Audio In");
	tvtr[AUDIO_BAND].lo_type=0;
	tvtr[AUDIO_BAND].sb=1;
	tvtr[AUDIO_BAND].lo_freq=0.0;
	tvtr[AUDIO_BAND].chain_mult=1;
	strcpy(tvtr[AUDIO_BAND].init_str, "  ");
	tvtr[AUDIO_BAND].gain_db=0.0;
	tvtr[AUDIO_BAND].control=0;
	tvtr[AUDIO_BAND].bus_adr=0X00;
	tvtr[AUDIO_BAND].last_tfreq=0.0;
	tvtr[AUDIO_BAND].last_mode=USB;

	for (i=16; i<21; i++)
	    tvtr[i].lo_type=999;	/* Transverter not installed */

	vfo[0].tvtr=0;			/* Predefined association */
	vfo[0].tr_offset=0;
	vfo[0].last_tfreq=144125000.0;
	vfo[0].last_mode=USB;

	vfo[1].tvtr=1;
	vfo[1].tr_offset=0;
	vfo[1].last_tfreq=144200000.0;
	vfo[1].last_mode=USB;

	for (i=2; i<9; i++)
	    vfo[i].tvtr=999;		/* Transverter not installed */

	vfo[9].tvtr=9;
	vfo[9].tr_offset=0;
	vfo[9].last_tfreq=144100000.0;
	vfo[9].last_mode=USB;

	for (i=10; i<15; i++)
	    vfo[i].tvtr=999;		/* Transverter not installed */

	vfo[AUDIO_BAND].tvtr=AUDIO_BAND;
	vfo[AUDIO_BAND].tr_offset=0;
	vfo[AUDIO_BAND].last_tfreq=0.0;
	vfo[AUDIO_BAND].last_mode=LSB;

	vfo[16].tvtr=1;
	vfo[16].tr_offset=0;
	vfo[16].last_tfreq=144100000.0;
	vfo[16].last_mode=CW;

	vfo[17].tvtr=1;
	vfo[17].tr_offset=0;
	vfo[17].last_tfreq=144200000.0;
	vfo[17].last_mode=USB;

	vfo[18].tvtr=1;
	vfo[18].tr_offset=0;
	vfo[18].last_tfreq=144250000.0;
	vfo[18].last_mode=USB;

	vfo[19].tvtr=1;
	vfo[19].tr_offset=0;
	vfo[19].last_tfreq=146580000.0;
	vfo[19].last_mode=FM;

	vfo[20].tvtr=1;
	vfo[20].tr_offset = -600;
	vfo[20].last_tfreq=146940000.0;
	vfo[20].last_mode=FM;

	for (i=0; i<=20; i++)		/* Init FM tone setup to 'none' */
	    {
	    vfo[i].t_fm_use=0;		/* xmit tone not used */
	    vfo[i].t_fm_freq=0.0;
	    vfo[i].t_fm_dur=0;
	    vfo[i].t_fm_level=0;
	    vfo[i].par1=0;		/* Unused parameters */
	    vfo[i].par2=0;
	    }

	/* Initialize the locations to non-existant except 0 which is at
	 * lat=0 and lon=0.  Non-existant is time_zone=999.
	 */
	for (i=0; i<=9; i++)
	    {
	    loc[i].lat=0.0;
	    loc[i].lon=0.0;
	    loc[i].ht=0.0;
	    loc[i].time_zone=999;
	    loc[i].dst=0;
	    strcpy(loc[i].nm, "");
	    strcpy(loc[i].name, "");
	    }
	loc[0].time_zone=0;
	strcpy(loc[0].nm, "(0,0)");
	strcpy(loc[0].name, "Lat=0 Lon=0");

	/* Initialize filters */
	strcpy(filt[0].name, "MTCH12");
	strcpy(filt[0].file_name, " ");
	filt[0].fil_type=3;
	filt[0].param1=0.0;
	filt[0].fclow=550;
	filt[0].fchigh=650;
	filt[0].fil_gain=6;

	strcpy(filt[1].name, "6CW200");
	strcpy(filt[1].file_name, " ");
	filt[1].fil_type=1;
	filt[1].param1=40.0;
	filt[1].fclow=500;
	filt[1].fchigh=700;
	filt[1].fil_gain=3;

	strcpy(filt[2].name, "6CW300");
	strcpy(filt[2].file_name, " ");
	filt[2].fil_type=1;
	filt[2].param1=70.0;
	filt[2].fclow=450;
	filt[2].fchigh=750;
	filt[2].fil_gain=3;

	strcpy(filt[3].name, "6CW450");
	strcpy(filt[3].file_name, " ");
	filt[3].fil_type=1;
	filt[3].param1=70.0;
	filt[3].fclow=375;
	filt[3].fchigh=825;
	filt[3].fil_gain=3;

	strcpy(filt[4].name, "7CW600");
	strcpy(filt[4].file_name, " ");
	filt[4].fil_type=1;
	filt[4].param1=70.0;
	filt[4].fclow=400;
	filt[4].fchigh=1000;
	filt[4].fil_gain=2;

	strcpy(filt[5].name, "SSB-N");
	strcpy(filt[5].file_name, " ");
	filt[5].fil_type=1;
	filt[5].param1=70.0;
	filt[5].fclow=250;
	filt[5].fchigh=2500;
	filt[5].fil_gain=0;

	strcpy(filt[6].name, "SSB-M");
	strcpy(filt[6].file_name, " ");
	filt[6].fil_type=1;
	filt[6].param1=70.0;
	filt[6].fclow=200;
	filt[6].fchigh=2900;
	filt[6].fil_gain=0;

	strcpy(filt[7].name, "8CW300");
	strcpy(filt[7].file_name, " ");
	filt[7].fil_type=1;
	filt[7].param1=60.0;
	filt[7].fclow=650;
	filt[7].fchigh=950;
	filt[7].fil_gain=3;
	}

void print_time(void)
	{int i, tmin;
	 time_t t0;
	 struct tm *tp;

	 if (ms_available)		/* use update_clock() */
	    t0=update_clock(1) + dsp_clock_base;
	 else				/* if not avail, use regular PC time */
	    time(&t0);

	 if (time_type)			/* time_type==1 --> use local time */
	    tp=localtime(&t0);
	 else
	    tp=gmtime(&t0);		/* time_type==0 --> use GMT */

	 /* Un-indent--or else qualify the passage of a second */
	     strcpy(cc, asctime(tp));	/* Rearrange the characters */
	     cc[0]=cc[8]; cc[1]=cc[9]; cc[2]=32;	/* Get Date */
	     cc[3]=cc[4]; cc[4]=cc[5]; cc[5]=cc[6]; cc[6]=32;
	     cc[7]=cc[22]; cc[8]=cc[23]; cc[9]=0;
	     setviewport(0,437,127,457,DO_CLIP);
	     clearviewport();
	     setcolor(c_text_bold);
	     settextstyle(0, 0, 1);
	     settextjustify(1, 2);
	     moveto(63,1);
	     outtext(cc);
	     /* Now create string with 'hhmm:ss XXX' where XXX is GMT, etc. */
	     cc[0]=cc[11];  cc[1]=cc[12];	/* Get hh */
	     for(i=14; i<=18; i++)		/* Get mm:ss */
	         cc[i-12]=cc[i];
	     cc[7]=32;			/* Space */
	     for (i=0; i<=3; i++)
	        cc[i+8]=time_str[i];	/* XXX */
	     cc[12]=0;			/* End of string */
	     moveto(63,11);
	     outtext(cc);

	     /* If the .cfg variable scsave is >0, do a screen save every
	      * scsave minutes.
	      */
	     if (scsave>0 && tp->tm_sec==0)  /* Wait for top of minute */
		{
		tmin = 60*tp->tm_hour + tp->tm_min; /* Minute of the day */
		if (tmin % scsave == 0) /* Check remainder */
		/* Rev 3.50 Removed BMP option */
		    do_gif_file();
		}

	     /* Now update Moon &/| Sun data, Process if GMT is available */
	     if(ut_available)		/* We can update Moon/Sun data */
	       {
	       cnvut2tdt(&t0, &tdt);
	       if (diag_mode==3)
		  (void)update_moon(&tdt, 1);  /* Update */
	       else if (diag_mode==4)
		  update_sun(&tdt, 1);	/* Update (0 for "if it's time") */
	       }
	}

/* get_dsp_ptrs() - Used at initialization time to fill in a global table of
 * addresses, dsp_ptr[i]. These come from the DSP. At DSP reset the
 * pointer to the addresses is in DSP status_buf bytes 7 (low) and 8 (high).
 */
void get_dsp_ptrs(void)
	{
	int i, j, k;

/*	dsp_cmd('F', 0X44, 0, 0, 4); */	/* Make FFT free running */
	dsp_cmd('F', 0X44, 0X80, 0, 4);	/* Make FFT free running <<<<<<<<< */

	delay(50);
	dsp_cmd('R', 32, 2, 3, 4); 	/* Immediate Fresh Start */
	/* Need a delay here, since the serial receiver is not working
	 * until the reset is finished.
	 */
	delay(1000);
	ASYNCCLEAR();			/* Clear the serial buffer */

	/* bit 1=1 stops the serial data stream and changes to ESC 'B' */
	bits_8_16 |= 0X0002;
	dsp_cmd(9, bits_8_16, 0, 0, 0);

	/* First, get the address of the ptr table:   */
	if(!get_status_bytes('B', 10.0))	/* Fill in dsp_status */
	    {
	    printf("\nTimeout error in obtaining DSP pointer table. Either\n");
	    printf("UHF3.EXE is prior to 2.0 or DSP is not responding.\n");
	    printf("Exiting to DOS.\n");
	    exit(1);
	    }
	/* Form the address from the 2 bytes: */
	if (dsp_status[17]==2)
	    {
	    table_adr=256*dsp_status[8]+dsp_status[7];
	    printf("\nTable address = %.4X, Getting pointers...\n", table_adr);
	    }
	else
	    {
	    printf("\nByte 17 error in obtaining DSP table address. Filter\n");
	    printf("design will not be available. Hit Enter to continue\n");
	    table_adr=0;
	    getch();
	    return;
	    }

	/* We now have a pointer to the dsp pointer table. Get them:

	/* Set up the FFT for software trigger (that comes from 5 cmd)
	 * and no data collection to speed things up.
	 */
	fft_fcn &= 0XB5FF;		/* Clear bits 9, 11, 14 */
 /*<<<<<<<<<<  Want to add bit 8 to clear current collection??? */
	fft_fcn |= 0X4200;		/* Software trigger, No FFT data */
	dsp_cmd('F', fft_fcn, fft_fcn>>8, 3, 4);

	/* The process is to request an address, get an address, ...
	 * continuing on 38 times: */
	for(j=0; j<38; j++)
	    {
	    k=-1;
	    do
		{
		/* cmd 5: Get pm(). Adr in positions 1 & 2, tag in 3, 4 nu */
		dsp_cmd(5, table_adr+j, (table_adr+j)>>8, j+1, 4);
	        delay(20);
		if (!get_status_bytes('B', 1.0))  /* Fill in dsp_status */
		  {
		  if (k>=0)
		    {
		    printf("\nRetry in obtaining DSP pointer table entry.\n");
		    printf("Attempt %d\n", k);
		    }
		  break;
		  }
		dsp_ptr[j]= 256*dsp_status[8]+dsp_status[7];
		}
	    while (dsp_status[13] != (j+1) && (k++)<2);
	    }

	if (dsp_ptr[37] != 0X1234 || dsp_status[18] != 0X56)
	    {
	    printf("\nError in obtaining dsp_ptr[37] (Validity check).");
	    printf("\n Read: %X %2.2X (1234 56 proper)", dsp_ptr[37],
			dsp_status[18]);
	    printf("\nFilter design will not be available.");
	    printf("\nHit Enter to continue\n");
	    table_adr=0;
	    getch();
	    return;
	    }
	printf("Reading low program memory...\n");
	/* Get pm for adr=0,1,2,3,4 */
	for(j=0; j<=4; j++)
	    {
	    k=-1;
	    do
		{
		/* cmd 5: Get pm(). Adr in positions 1 & 2, tag in 3, 4 nu */
		dsp_cmd(5, j, 0, j+100, 4);
	        delay(20);
		if (!get_status_bytes('B', 1.0))  /* Fill in dsp_status */
		  {
		  if (k>=0)
		    {
		    printf("\nRetry in obtaining DSP pointer table entry.\n");
		    printf("Attempt %d\n", k);
		    }
		  break;
		  }
		printf("pm[%d]=", j);
		printf(" %2.2X %2.2X %2.2X\n", dsp_status[8], dsp_status[7],
			dsp_status[18]);
		if (j==2)
		    {
		    if (dsp_status[7]&0X40)
			printf("EPROM version of UHF3.EXE being used\n");
		    else
			printf("Loaded version of UHF3.EXE being used\n");
		    if (dsp_status[18]&0X01)
			printf("DSPx version of UHF3.EXE being used\n");
		    else
			printf("EZ-Kit version of UHF3.EXE being used\n");
		    if (dsp_status[18]&0X08)
			printf("AIC23 CODEC being used\n");
		    else
			printf("AD1847 CODEC being used\n");
		    }
		}
	    while (dsp_status[13] != (j+100) && (k++)<2); /* Data tag */
	    }

	/* UHF3 code:
        loc 0  jump start;
	loc 1  .var/pm/ram/abs=1  tbl;	.init	tbl:^ptr_table;
	{Configuration - allows the PC to know what hardware is being used.
	    Bits 3 to 0 are not used.
	    CODEC Bits 6,5,4   000=1847  001=AIC23
	    Board Bits 9,8,7   000=EZKIt 001=DSPx  (presently tracks CODEC)
	    EPROM Bit 10        0=No EPROM  1=EPROM load  (always EZKit)
	    Bits 15 to 11       Not assigned
	 To be sure this is in location 2 of PM, the ax0=<data> instruction
	 is used. See UHF3A.DSP for more info.
	} */

#if 0
				        {     15                  0<-Data bits}
 ax0 = 0X0000;				{ 0100 00000 0 000 000 00000000 }
 endif
 if (CODEC_1847 & EZKIT & EPROM)
 ax0 = 0X0400;				{ 0100 00000 1 000 000 00000000 }
 endif
 if CODEC_AIC23
 ax0 = 0X0090;				{ 0100 00000 0 001 001 00000000 }
 endif
	rti;				{ PM location 03 }
 endif
	if ((pm[2] & 
#endif



	bits_8_16 &= 0X0001;		/* Restore FFT data */
	dsp_cmd(9, bits_8_16, 0, 0, 0);

	/* Restore FFT functions to normal free run */
	fft_fcn &= 0XB5FF;		/* Clear bits 9, 11, 14 */
	dsp_cmd('F', fft_fcn, fft_fcn>>8, 3, 4);
	}

/* get_status_bytes()  -  Reads serial data stream until status bytes
 * are found. Fills these to dsp_status[].
 * The DSP inserts a byte after the ESC character to indicate the
 * nature of the data that follows. Here it is called type and
 *    type=='A' status and spectral data
 *    type=='B' status data only
 */
int get_status_bytes(int type, double timeout)
	{
	int new_char, last_char, i, j, esc_count;
	time_t t0;

	time(&t0);
/* dstat, below can go?? */

	/* We are now looking for a sequence of bytes:
	 *   ESC type b0 b1 b2 ... b18
	 * where bi are DSP status_buf bytes.
	 */
	esc_count=0;
pp:	for(;;)					/* Wait for 1st character */
		{
		if (difftime(time(NULL), t0) > timeout) {beep(500); return 0;}
		if ( ASYNCINSTAT() > 0 )	/* Have ch, see if ESC_CHAR */
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR)  esc_count++;
			 break;			/* Break the for(;;) */
			}
		}
	for(;;)			/* Wait here for ESC_CHAR B to come */
		{last_char=new_char;
		 if (difftime(time(NULL), t0) > timeout)  return 0;
		 if(ASYNCINSTAT() > 0)
			{new_char = ASYNCIN();
		 	 if (new_char==ESC_CHAR)  esc_count++;	
			 if (esc_count==2)	/* Waiting for ESC_CHAR,type */
				{esc_count=0;	/*    but got 2xESC_CHAR     */
				 goto pp;	/* So try again              */
				}
			 if ((last_char==ESC_CHAR) && (new_char==type))
				break;
			}
		}
	/* Now get dsp_status bytes */
	i=0; esc_count=0;
	for(;;)
		{
		if (difftime(time(NULL), t0) > timeout)  return 0;
		if( ASYNCINSTAT() > 0 )
			{new_char = ASYNCIN();
			 if (new_char==ESC_CHAR) esc_count++;
			 if (esc_count==2)
				{esc_count=0;
				 goto dstat1;
				}
			 dsp_status[i] = new_char;
			 /* If status is ESC, needs to be entered to table */
			 if (++i > 18)  break;
 dstat1:			}
		}
	return 1;
	}

/* get_dsp_mem()  -  The DM() or PM() item that is in the dsp_ptr[] table
 * can be retrieved. Offset is the number of addresses above the base.
 *   what==0  Request of DSP to get DM()
 *   what==1  Request of DSP to get PM()
 *   what==2  Acquire DM() from status bytes
 *   what==3  Acquire DM() from status bytes
 *
 * The status of this process is carried in the global int dsp_data_index.
 *   dsp_data_index==0   No data being acquired
 *   dsp_data_index>0    This index is currently requested from DSP
 *   dsp_data_index==-1  Request was cancelled by timeout
 *
 * The process can fail due to data being lost and the request can be cancelled
 * by timeouts. This generates an error.  dsp_data_index==-1 indicates that a
 * re-request is needed.
 *
 * In using this function, keep in mind that it is inherently very slow. When
 * FFT data is being acquired, it allows one data item every 0.6 sec. Normally,
 * almost no data is needed. NOT FINISHED
 */
void get_dsp_mem(int what, int* tbl, int item, int offset)
	{
#if 0
	static int last_index=1;
	static int* ptr_last_table;
	static int last_item;
	static int last_offset;
#endif
	if(what==2570) tbl[1]=item+offset; /*JUNK TO STOP WARNINGS */
	if (dsp_data_index==-1) tbl[1]=5432;/*MORE SUCH JUNK*/
	}

/* set_dsp_mem() - Downloads from PC to DSP either DM(), for what==0, or PM()
 * for what==1. DM is 16 bit and PM is 24 bit.  NOT FINISHED
 */
void set_dsp_mem(int what, int adr, long int data)
	{

 if (what==adr+data +2345) return;/* Dummy stuff */
	if (what==0)			/* Download data memory */
	    {
	    dsp_cmd('3', adr, adr>>8, (int)data, (int)(data>>8));
	    }
	else if (what==1)		/* Download program memory */
	    {
	    }
	}

/**** UPDATE_MOON() *************************************************
 *
 * do_now=0    Update if minute has changed and print on screen
 * do_now=1    Update immediately, and print data on screen
 * do_now=2    Update immediately, but don't print data on screen
 *
 * returns the roundtrip doppler dop00+dop11 (double)
 */
double update_moon(struct tm *ptm, int do_now)
    {
    int j;
    static int last_min;
    static int last_day;
    struct latlon ll;
    struct radec  rd;
    struct hadec  hd, hd1;
    struct azel   ae;			/* Az-El coordinates of Moon */
    double rmoon;			/* Range loc[0] to Moon center */
    double gst;
    double pm;
    double mrange;
    double um_freq;			/* Freq used in doppler calc */
    double dop00, dop11;

    if (band != AUDIO_BAND)		/* If RF processor use um_freq */
	um_freq = (double)tfreqL - (double)tvtr[band].sb * tvtr[band].lo_freq;

    if ((ptm->tm_yday != last_day) || (do_now==1) )
	{
	last_day=ptm->tm_yday;	/* Won't do this for another day */
	cur_env.day=(double) ptm->tm_mday;
	cur_env.month=1+ptm->tm_mon;
	cur_env.year=1900+ptm->tm_year;
	julday(&cur_env);		/* Update Julian day */
	nutat(&cur_env);		/* Once a day find nutation */
	obliq(&cur_env);		/*  and obliquity of ecliptic */
	}
    /* Calc the moon apparent position and Doppler, if appropriate:  */
    if ((last_min != ptm->tm_min) || (do_now>0) )	/* Appropriate? */
	{
	last_min=ptm->tm_min;	/* Don't do this for another min */

	/* We calculate doppler twice, once for home loc[0] and
	 * then for loc[cur_loc] to get dx location doppler */

	j=cur_loc;				/* Now for DX location */
	if (band==AUDIO_BAND)		/* Audio processor, use f_eme */
	    dop11=doppler(ptm, &ae, &rmoon, f_eme, &loc[j]);
	else				/* Full radio, use tfreqL */
	    dop11=doppler(ptm, &ae, &rmoon, um_freq, &loc[j]);

	j=0;
	if (band==AUDIO_BAND)		/* Audio processor, use f_eme */
	    dop00=doppler(ptm, &ae, &rmoon, f_eme, &loc[j]);
	else				/* Full radio, use tfreqL */
	    dop00=doppler(ptm, &ae, &rmoon, um_freq, &loc[j]);

	if (do_now<2)
	    {
	    setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
	    clearviewport();
	    settextjustify(0,2);
	    sprintf(cc, "Moon: %s-%s AZ=%.2lf EL=%.2lf %dHz %.2lfdB",
	        loc[0].nm, loc[cur_loc].nm, r2d(ae.az), r2d(ae.el),
	        near_int(dop00+dop11), -40.0*log10(rmoon/REM_MEAN));
	    moveto(0, 0);
	    outtext(cc);
	    }
	}
    return  dop00+dop11;
    }

/* update_sun()  -  If the diag_mode==4 this will print the az and el of the
 * Sun on the bottom diagnostic line.  Earth location is loc[0] always.
 * Also slight variation in Sun intensity due to distance != 1 AU.
 */
void update_sun(struct tm *ptm, int do_now)
    {
    static int last_min;
    static int last_day;
    struct latlon ll;
    struct radec  rd;
    struct hadec  hd, hd1;
    struct azel   ae;
    double gst;
    double sun_dist;

    if ((ptm->tm_yday != last_day) || do_now)
	{
	last_day=ptm->tm_yday;	/* Won't do this for another day */
	cur_env.day=(double) ptm->tm_mday;
	cur_env.month=1+ptm->tm_mon;
	cur_env.year=1900+ptm->tm_year;
	julday(&cur_env);		/* Update Julian day */
	nutat(&cur_env);		/* Once a day find nutation */
	obliq(&cur_env);		/*  and obliquity of ecliptic */
	}
    if ((last_min != ptm->tm_min) || do_now)
	{
	last_min=ptm->tm_min;	/* Don't do this for another min */
	gst=cnvlt2gst( (double)ptm->tm_hour +
	    ((((double)ptm->tm_min) + ((double)ptm->tm_sec/60.0))/60.0),
	    &cur_env, &loc[cur_loc], &ut);
	sun(cur_env.dj, ut, &ll.lon, &sun_dist);
	ll.lat=0.0;			/* Easy for the Sun */
	/* Now a couple of minor corrections.  These are not generally
	 * needed.  They maintain an accuracy of better than 1 sec in
	 * angle.  First for nutation from cur_env.dp [from nutat() ].
	 * Then a fixed correction of 0.00569 deg that is the change in Sun
	 * position during the time that the light was travelling to the Earth,
	 * called aberation.
	 */
	ll.lon += (cur_env.dp-d2r(0.00569));
	cnvll2radec(&ll, &rd, &cur_env);	/* Get RA and Dec */
	/* Convert RA to hour angle: */
	hd.ha=cnvra2ha(rd.ra, gst, loc[cur_loc].lon);
	hd.dec=rd.dec;
	/* A tiny correction for parallax (we are not at Earth center):  */
	parallax(loc[cur_loc].lat, &hd, d2r(0.0024428)/sun_dist, &hd1);
	cnvhadec2azel(&hd1, &loc[cur_loc], &ae);
	    
	setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
	clearviewport();
	settextjustify(0,2);
	sprintf(cc, "Sun: %s AZ=%.2lf EL=%.2lf %.2lfdB",
	    loc[cur_loc].nm, r2d(ae.az), r2d(ae.el), -20.0*log10(sun_dist));
	moveto(0, 0);
	outtext(cc);
	}
    }

/* set_cur_loc()  -  Not all loc[i] need exist.  This moves through  to
 * those that do.  Note the data is initialized so that loc[0] always
 * exists so this routine can't hang. loc[i].time_zone is not used in
 * the program now except to keep track of the active loc[i].
 * If updn==-1 there is no change in loc[i], but just update screen.
 */
void set_cur_loc(int updn)
    {
    time_t t0;

    if (updn==1)				/* Go to next */
	{
	if (++cur_loc > 9)  cur_loc=0;
	while (loc[cur_loc].time_zone == 999)
	    {
	    if (++cur_loc > 9)  cur_loc=0;
	    }
	}
    else if (updn==0)				/* Go to previous */
	{
	if (--cur_loc < 0)  cur_loc=9;
	while (loc[cur_loc].time_zone == 999)
	    {
	    if (--cur_loc < 0)  cur_loc=9;
	    }
	}
    /* Now update bottom line of screen */
    if (ut_available)
	{
	if (ms_available)
	    t0=update_clock(1)+dsp_clock_base;
	else
	    (void)time(&t0);
	cnvut2tdt(&t0, &tdt);

	if (diag_mode==3)
	    (void)update_moon(&tdt, 1);	/* 1 for 'do_now'  */
	else if (diag_mode==4)
	    update_sun(&tdt, 1);
	}
    if(!ut_available && (diag_mode==3 || diag_mode==4))
	{
	setviewport(129, 470, 579, 479, DO_CLIP); /* Clear line */
	clearviewport();
	settextjustify(0,2);
	moveto(0, 0);
	outtext("Moon/Sun data unavailable; need TZ in DOS enviornment");
	}
    }

/* Hardware specific code to load both the LMX1501 synthesizer as well
 * as the 16 bit control shift register.  lsynth is a 19 bit data
 * word to program the synthesizer.  It comes in two forms, one to
 * set up the ref divider and another to set the frequency.  Both are treated
 * identically here.
 *
 * The 19+16=35 bits are sent out as three 16 bit commands with the first 13
 * bits being passed through the shift registers:
 *	word_first	MS 3 bits of synthesizer data
 *	word_mid	LS 16 bits of synthesizer data
 * Only 16 bits are sent per command to ensure that the shift reg is
 * loaded before the next command is sent asking for more.
 */
void send_shift_data(unsigned long lsynthx)
	{int i;
	 unsigned int  sr;
	 unsigned int  word_first, word_mid;

	 word_first = (unsigned int) (0X7UL & (lsynthx>>16));  /* 3 bits */
	 word_mid   = (unsigned int) (0XFFFFUL & lsynthx);     /* 16 bits */
	 /* 'Word_last' comes from shift_reg (low 8) and
	    rfg[i].gainbyte (high 8)                  */

	 /* Now clock in bits, 16 at a time */
	 dsp_cmd(8, word_first & 0XFF, (word_first >> 8) & 0X00FFU, 3, 4);
	 dsp_cmd(8, word_mid & 0XFF,   (word_mid >> 8)   & 0X00FFU, 3, 4);
	 dsp_cmd(8, shift_reg & 0XFF,  rfg[rfgain].gainbyte & 0XFF, 3, 4);

	 /* The 7 cmd sets the enable gate to load from the shift regs.
	  * To syncronize frequency chages, it also sets the DSP LO */
	/* Revised 12 Mar 04 Rev 3.17 to leave the PF7 line from the DSP
	   high when not used. This prevents accidental enabling. */
	 dsp_cmd(7, 0, 2, dsp_deltap_lo, dsp_deltap_hi);  /* Start low 3.17 */
 delay(10);
	 dsp_cmd(7, 1, 2, dsp_deltap_lo, dsp_deltap_hi);  /* Do enable */
#if 0
	 /* Close the gate in prep for next shift & load */
	 dsp_cmd(7, 0, 2, dsp_deltap_lo, dsp_deltap_hi);
#endif
	}

/* do_t_error(err, text*, tau)  -  Displays error err for about 15 seconds or
 * until replaced by next error.  Sounds beep for tau millisecgonds.
 * err==err+1000 clears err if the current error is err.
 * err==9999 clears all errors.
 * Similar to do_error, but prints text, and no return.
 */
void do_t_error(int err, char *ce, int tau)
	{
	static int current_err=0;

	if (beep_on_error && tau>0)  beep(tau);
	if ((err>0) && (err<1000))
	    display_error_count=25;	/* About 15 seconds */

	if ((err==9999) || ((err>1000) && (current_err==(err-1000))))
	    current_err=0;
	if (err<1000)
	    current_err=err;
	setviewport(0,457,127,467,DO_CLIP);
	clearviewport();
	setcolor(c_text_note);
	settextstyle(0, 0, 1);
	settextjustify(1, 2);
	moveto(63,1);
	if (current_err>0)  outtext(ce);
	setcolor(c_text_bold);
	}

/* near_int(x)  -  Returns the integer nearest x.
 */
int near_int(double x)
	{
	if (x <= -32768.0)  return -32768;
	if (x >= 32767.0)  return 32767;
	if (x >= 0.0)  return (int)(x+0.5);
	return (int)(x-0.5);
	}

/*****************************************************************************
 *
 * double upow10(double x)  -  Returns 10^x.
 * Wraps around the library pow(10.0, x) to not create an error message.
 * The D_VBIG response is intended as the best one can
 * do for a real-time process.  This function is not intended for
 * math computation, in general.
 * RSL 2 March 00
 * Still problems. Reduced max |x| to 20  RSL 16 Jan 00
 */
double upow10(double x)
    {
    /* This seems to still lead to overflow in pow(10.0, x)  */
    /* if (x > DBL_MAX_10_EXP-12) return D_VBIG;  */

    if      (x>20.0)  x=20.0;
    else if (x<-20.0) x=-20.0;

    return pow(10.0, x);
    }
    
/*****************************************************************************
 *
 * double p2db(double p)  -  Returns 10*log10(p), accurate to 7 places.
 * Replaces library log10() as it does not create error interrupt.
 * About the same speed as 10*log10(p), depending on the value of p,
 * but less accurate.
 * Data from Abramowitz and Stegun, scaled from ln(x).
 * RSL 2 March 00
 */
double p2db(double p)
    {
    double t1, t2, tpow;
    double db_scale;                    /* dB due to scaling into range */
    double d;                           /* Power in dB */
    int i;

    /* 10log(0) d= -2956.5  A response for 0 or negative */
    if (p < D_TINY) return -2956.5;
    db_scale=0.0;
    while (p<0.316227766)
        {p *= 10.0;
         db_scale -= 10.0;
        }

    while (p>3.16227766)
        {p *= 0.1;
         db_scale += 10.0;
        }

    /* p is in range .316 to 3.16 */
    t1=(p-1.0)/(p+1.0);
    t2=t1*t1;
    d=8.68591718*t1;
    t1 *= t2;
    d  += 2.89335524*t1;
    t1 *= t2;
    d  += 1.77522071*t1;
    t1 *= t2;
    d  += 0.94376476*t1;
    t1 *= t2;
    d  += (db_scale + 1.91337714*t1);

    return d;
    }

/* get_kbd_int()  -  Returns an integer from keyboard entry.  Assumes text
 * mode (not graphics.)  Ends on 'Enter' and responds to Backspace
 */
int  get_kbd_int(void)
	{
	char  c, *cptr;
	cptr=cc;
	for(;;)
	    {
	    if (kbhit()) 
		{
		c=(char)(_bios_keybrd(_KEYBRD_READ) & 0XFF);
		if ( (c==8) && (cptr>cc) )
		    {
		    putch(8);
		    putch(32);
		    putch(8);
		    cptr--;
		    }
		else if (c==13)
		    {
		    *cptr=0;
		    putch(13);
		    putch(10);
		    break;
		    }
		else if (c>=32 && c<127)
		    {
		    putch((int)c);
		    *cptr++=c;
		    }
		if ( (cptr-cc)>49 )
		    {
		    *cptr=0;
		    break;
		    }
		}
	    }
	return atoi(cc);
	}	

/* keyident() - Utility that is available before graphics is started.
 * Gives information on key that has been hit, for use in setting up
 * header file for keyboard definition.
 */
int keyident(void)
	{
	unsigned int stat, s, ichr;
	long lkey;

	stat = _bios_keybrd(_KEYBRD_SHIFTSTATUS);
	s = stat;
	/* Make left and right shift keys equals (bits 0 and 1) and
	 * mask out Caps Lock, Num Lock and Insert (bits 5, 6, and 7).     */
	if (stat&2)   stat |= 1;
	stat &= 0X1D;
	if ((stat&1) > 0) printf("SHIFT, ");
	if ((stat&4) > 0) printf("CTRL, ");
	if ((stat&8) > 0) printf("ALT, ");
	if ((stat&16) > 0) printf("ScrlLk, ");
	ichr = _bios_keybrd(_KEYBRD_READ);
	lkey = (long)ichr + (((long)stat)<<16);
	printf(" ichr=%X  stat=%X.\n    Hex k_ value for CFG file: 0X%lX",
		ichr, s, lkey);
	printf("  Hex ky_ value: 0X%X\n", (0XFF & ichr));
	if (stat==0 && (ichr&0XFF)==0X0D)
	    return 1;
	else
	    {
	    printf("\nHit 'Enter' to proceed (others give key ident)\n");
	    return 0;
	    }
	}

void get_mem(void)			/* Get memory for screen scroll */
	{
	int hc;

	/* This is part of sign-on and set-up! */
	if ( (big_area_ptr1 = (char far *)farmalloc((size_t)60000) ) == NULL)
		{printf("Unable to get memory for bit image (top). ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (big_area_ptr2 = (char far *)farmalloc((size_t)50000) ) == NULL)
		{printf("Unable to get memory for bit image (bottom). ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (power10 = (double *)calloc(256, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for power10[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (pwrdat = (double *)calloc(515, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for pwrdat[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (dnorm = (double *)calloc(515, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for dnorm[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (lt_dnorm = (double *)calloc(256, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for lt_dnorm[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (lt_pwrdat = (double *)calloc(256, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for lt_pwrdat[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (pd16 = (double *)calloc(256, sizeof(double)) ) == NULL)
		{printf("Unable to get memory for pd16[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}
	if ( (cwmbuf = (char *)calloc(500, sizeof(char)) ) == NULL)
		{printf("Unable to get memory for cwmbuf[]. ");
		 printf("'Enter' to continue\n");
		 getch();
		 exit(0);
		}

	if (startup_detail)
	    {
	    printf("big_area_ptr1 = %p\n", big_area_ptr1);
	    printf("big_area_ptr2 = %p\n", big_area_ptr2);
	    printf("power10[]     = %p\n", power10);
	    printf("pwrdat[]      = %p\n", pwrdat);
	    printf("dnorm[]       = %p\n", dnorm);
	    printf("lt_dnorm[]    = %p\n", lt_dnorm);
	    printf("lt_pwrdat[]   = %p\n", lt_pwrdat);
	    printf("pd16[]        = %p\n", pd16);
	    printf("cwmbuf[]      = %p\n", cwmbuf);
	    }
	hc=heapcheck();
	if (hc==_HEAPCORRUPT)
	    {
	    printf("Heap has been corrupted\n");
	    exit(0);
	    }
	if (hc==_HEAPEMPTY)
	    {
	    printf("No heap\n");
	    exit(0);
	    }
	if (startup_detail && hc==_HEAPOK)      printf("Heap is verified\n");
	}
